@@ -1,6 +1,4 @@
 /*
- * $Id: clock.c 16797 2009-05-28 22:56:29Z rmanfredi $
- *
  * Copyright (c) 2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -36,35 +34,33 @@
 
 #include "common.h"
 
-RCSID("$Id: clock.c 16797 2009-05-28 22:56:29Z rmanfredi $")
-
 #include "clock.h"
 
-#include "lib/cq.h"
-#include "lib/misc.h"
-#include "lib/halloc.h"
-#include "lib/walloc.h"
-
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
+#include "lib/aging.h"
+#include "lib/entropy.h"
+#include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/misc.h"
 #include "lib/stats.h"
 #include "lib/tm.h"
+#include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
-#define REUSE_DELAY	10800		/**< 3 hours */
-#define ENOUGH_DATA	30			/**< Update skew when we have enough data */
-#define MIN_DATA	15			/**< Minimum amount of points for update */
-#define MAX_SDEV	60.0		/**< Maximum dispersion we tolerate */
+#define REUSE_DELAY	1800		/**< 30 minutes */
+#define ENOUGH_DATA	15			/**< Update skew when we have enough data */
+#define MIN_DATA	10			/**< Minimum amount of points for update */
 #define CLEAN_STEPS	3			/**< Amount of steps to remove off-track data */
 
 struct used_val {
 	host_addr_t addr;			/**< The IP address */
-	cevent_t *cq_ev;			/**< Scheduled cleanup event */
 	int precision;				/**< The precision used for the last update */
 };
 
-static GHashTable *used;		/**< Records the IP address used */
+static aging_table_t *used;		/**< Records the IP address used */
 
 /**
  * This container holds the data points (clock offset between the real UTC
@@ -91,32 +87,19 @@ static statx_t *datapoints;
 
 /**
  * Dispose of the value from the `used' table.
+ *
+ * This is an aging_table freeing callback.
  */
 static void
-val_free(struct used_val *v)
+val_free(void *key, void *value)
 {
-	g_assert(v);
+	struct used_val *v = value;
+
+	g_assert(v != NULL);
 	g_assert(is_host_addr(v->addr));
+	g_assert(key == &v->addr);
 
-	cq_cancel(callout_queue, &v->cq_ev);
-	wfree(v, sizeof *v);
-}
-
-/**
- * Called from callout queue when it's time to destroy the record.
- */
-static void
-val_destroy(cqueue_t *unused_cq, gpointer obj)
-{
-	struct used_val *v = obj;
-
-	(void) unused_cq;
-	g_assert(v);
-	g_assert(is_host_addr(v->addr));
-
-	g_hash_table_remove(used, &v->addr);
-	v->cq_ev = NULL;
-	val_free(v);
+	WFREE(v);
 }
 
 /**
@@ -125,56 +108,36 @@ val_destroy(cqueue_t *unused_cq, gpointer obj)
 static struct used_val *
 val_create(const host_addr_t addr, int precision)
 {
-	struct used_val *v = walloc(sizeof *v);
+	struct used_val *v;
 
 	g_assert(is_host_addr(addr));
 
+	WALLOC(v);
 	v->addr = addr;
 	v->precision = precision;
-	v->cq_ev = cq_insert(callout_queue, REUSE_DELAY * 1000, val_destroy, v);
 
 	return v;
 }
 
-/**
- * Accepted an update due to a lower precision entry, reschedule the
- * expiration timeout.
- */
-static void
-val_reused(struct used_val *v, int precision)
-{
-	v->precision = precision;
-	cq_resched(callout_queue, v->cq_ev, REUSE_DELAY * 1000);
-}
-
 /**
  * Called at startup time to initialize local structures.
  */
 void
 clock_init(void)
 {
-	used = g_hash_table_new(host_addr_hash_func, host_addr_eq_func);
+	used = aging_make(REUSE_DELAY,
+		host_addr_hash_func, host_addr_eq_func, val_free);
+
 	datapoints = statx_make();
 }
 
-static void
-used_free_kv(gpointer unused_key, gpointer val, gpointer unused_x)
-{
-	struct used_val *v = val;
-
-	(void) unused_key;
-	(void) unused_x;
-	val_free(v);
-}
-
 /**
  * Called at shutdown time to cleanup local structures.
  */
 void
 clock_close(void)
 {
-	g_hash_table_foreach(used, used_free_kv, NULL);
-	g_hash_table_destroy(used);
+	aging_destroy(&used);
 	statx_free(datapoints);
 }
 
@@ -190,7 +153,7 @@ clock_adjust(void)
 	double min;
 	double max;
 	int i;
-	guint32 new_skew;
+	uint32 new_skew;
 	int k;
 
 	/*
@@ -198,29 +161,37 @@ clock_adjust(void)
 	 */
 
 	n = statx_n(datapoints);
+
+	g_assert(n > 1);		/* To be able to compute the standard deviation */
+
 	avg = statx_avg(datapoints);
 	sdev = statx_sdev(datapoints);
 
+	entropy_harvest_many(VARLEN(n), VARLEN(avg), VARLEN(sdev), NULL);
+
 	/*
 	 * Incrementally remove aberration points.
 	 */
 
 	for (k = 0; k < CLEAN_STEPS; k++) {
 		double *value = statx_data(datapoints);
+		int old_n = statx_n(datapoints);
+
+		g_assert(value != NULL);	/* Since n > 1, there are data points */
 
 		if (GNET_PROPERTY(clock_debug) > 1)
-			g_message("CLOCK before #%d: n=%d avg=%.2f sdev=%.2f",
+			g_debug("CLOCK before #%d: n=%d avg=%g sdev=%g",
 				k, n, avg, sdev);
 
 		statx_clear(datapoints);
 
 		/*
-		 * Remove aberration points: keep only the sigma range around the
-		 * average.
+		 * Remove aberration points: keep only a 2*sigma range around the
+		 * mean (about 95% of the values for a Normal distribution).
 		 */
 
-		min = avg - sdev;
-		max = avg + sdev;
+		min = avg - 2.0 * sdev;
+		max = avg + 2.0 * sdev;
 
 		for (i = 0; i < n; i++) {
 			double v = value[i];
@@ -236,39 +207,40 @@ clock_adjust(void)
 		 */
 
 		n = statx_n(datapoints);
+
+		if (n < 2)
+			break;		/* Not enough data to compute standard deviation */
+
 		avg = statx_avg(datapoints);
 		sdev = statx_sdev(datapoints);
 
 		if (GNET_PROPERTY(clock_debug) > 1)
-			g_message("CLOCK after #%d: kept n=%d avg=%.2f sdev=%.2f",
+			g_debug("CLOCK after #%d: kept n=%d avg=%g sdev=%g",
 				k, n, avg, sdev);
 
-		if (sdev <= MAX_SDEV || n < MIN_DATA)
+		if (n < MIN_DATA || old_n == n)
 			break;
 	}
 
 	/*
-	 * If standard deviation is too large still, we cannot update our
-	 * clock, collect more points.
-	 *
 	 * If we don't have a minimum amount of data, don't attempt the
 	 * update yet, continue collecting.
 	 */
 
-	if (sdev > MAX_SDEV || n < MIN_DATA) {
+	if (n < MIN_DATA) {
 		if (GNET_PROPERTY(clock_debug) > 1)
-			g_message("CLOCK will continue collecting data");
+			g_debug("CLOCK will continue collecting data");
 		return;
 	}
 
 	statx_clear(datapoints);
 
-	new_skew = GNET_PROPERTY(clock_skew) + (gint32) avg;
+	new_skew = (int32) avg;
 
 	if (GNET_PROPERTY(clock_debug))
-		g_message("CLOCK with n=%d avg=%.2f sdev=%.2f => SKEW old=%d new=%d",
-			n, avg, sdev, (gint32) GNET_PROPERTY(clock_skew),
-			(gint32) new_skew);
+		g_debug("CLOCK with n=%d avg=%g sdev=%g => SKEW old=%d new=%d",
+			n, avg, sdev, (int32) GNET_PROPERTY(clock_skew),
+			(int32) new_skew);
 
 	gnet_prop_set_guint32_val(PROP_CLOCK_SKEW, new_skew);
 }
@@ -283,7 +255,7 @@ void
 clock_update(time_t update, int precision, const host_addr_t addr)
 {
 	time_t now;
-	gint32 delta;
+	int32 delta;
 	struct used_val *v;
 
 	g_assert(used);
@@ -298,26 +270,34 @@ clock_update(time_t update, int precision, const host_addr_t addr)
 	 * end is running NTP.
 	 */
 
-	if ((v = g_hash_table_lookup(used, &addr))) {
+	if (NULL != (v = aging_lookup(used, &addr))) {
 		if (precision && precision >= v->precision)
 			return;
-		val_reused(v, precision);
+		(void) aging_lookup_revitalise(used, &addr);
+		v->precision = precision;
 	} else {
 		v = val_create(addr, precision);
-		g_hash_table_insert(used, &v->addr, v);
+		aging_insert(used, &v->addr, v);
 	}
 
+	entropy_harvest_small(
+		VARLEN(update), VARLEN(precision), VARLEN(addr), NULL);
+
 	now = tm_time();
-	delta = delta_time(update, (now + (gint32) GNET_PROPERTY(clock_skew)));
+	delta = delta_time(update, now);
 
 	statx_add(datapoints, (double) (delta + precision));
-	statx_add(datapoints, (double) (delta - precision));
 
-	if (GNET_PROPERTY(clock_debug) > 1)
-		g_message("CLOCK skew=%d delta=%d +/-%d [%s] (n=%d avg=%.2f sdev=%.2f)",
-			(gint32) GNET_PROPERTY(clock_skew),
+	if (precision != 0)
+		statx_add(datapoints, (double) (delta - precision));
+
+	if (GNET_PROPERTY(clock_debug) > 1) {
+		g_debug("CLOCK skew=%d delta=%d +/-%d [%s] (n=%d avg=%g sdev=%g)",
+			(int32) GNET_PROPERTY(clock_skew),
 			delta, precision, host_addr_to_string(addr),
-			statx_n(datapoints), statx_avg(datapoints), statx_sdev(datapoints));
+			statx_n(datapoints), statx_avg(datapoints),
+			statx_n(datapoints) >= 2 ? statx_sdev(datapoints) : 0.0);
+	}
 
 	if (statx_n(datapoints) >= ENOUGH_DATA)
 		clock_adjust();
@@ -332,7 +312,7 @@ clock_loc2gmt(time_t stamp)
 	if (GNET_PROPERTY(host_runs_ntp))
 		return stamp;
 
-	return stamp + (gint32) GNET_PROPERTY(clock_skew);
+	return stamp + (int32) GNET_PROPERTY(clock_skew);
 }
 
 /**
@@ -344,7 +324,7 @@ clock_gmt2loc(time_t stamp)
 	if (GNET_PROPERTY(host_runs_ntp))
 		return stamp;
 
-	return stamp - (gint32) GNET_PROPERTY(clock_skew);
+	return stamp - (int32) GNET_PROPERTY(clock_skew);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
