@@ -1,6 +1,4 @@
 /*
- * $Id: downloads.h 17312 2010-02-27 18:53:49Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -42,6 +40,7 @@
 
 #include "fileinfo.h"
 
+#include "lib/array.h"
 #include "if/core/downloads.h"
 #include "if/core/search.h"			/* For gnet_host_vec_t */
 
@@ -58,66 +57,72 @@ void download_info_change_all(fileinfo_t *old_fi, fileinfo_t *new_fi);
 void download_orphan_new(const char *file, filesize_t size,
 		const struct sha1 *sha1, fileinfo_t *fi);
 void download_queue(struct download *d,
-	const char *fmt, ...) G_GNUC_PRINTF(2, 3);
+	const char *fmt, ...) G_PRINTF(2, 3);
 void download_stop(struct download *, download_status_t,
-	const char *, ...) G_GNUC_PRINTF(3, 4);
+	const char *, ...) G_PRINTF(3, 4);
 void download_stop_v(struct download *d, download_status_t new_status,
     const char * reason, va_list ap);
 void download_push_ack(struct gnutella_socket *);
-void download_forget(struct download *, gboolean unavailable);
-gboolean download_start_prepare(struct download *d);
-gboolean download_start_prepare_running(struct download *d);
+void download_forget(struct download *, bool unavailable);
+bool download_start_prepare(struct download *d);
+bool download_start_prepare_running(struct download *d);
 void download_send_request(struct download *);
 void download_connected(struct download *);
 void download_close(void);
-gboolean download_server_nopush(const struct guid *,
-			const host_addr_t addr, guint16 port);
+bool download_server_nopush(const struct guid *,
+			const host_addr_t addr, uint16 port);
 void download_free_removed(void);
 void download_redirect_to_server(struct download *d,
-		const host_addr_t addr, guint16 port);
-void download_actively_queued(struct download *d, gboolean queued);
+		const host_addr_t addr, uint16 port);
+void download_actively_queued(struct download *d, bool queued);
+struct download *download_pick_another_waiting(const struct download *d);
+void download_switch(struct download *od, struct download *nd, bool on_error);
 
+void download_server_publishes_in_dht(const struct guid *);
+
+bool download_send_udp_push(
+	const struct array packet, host_addr_t addr, uint16 port);
 void download_add_push_proxy(const struct guid *guid,
-	host_addr_t addr, guint16 port);
+	host_addr_t addr, uint16 port);
 void download_add_push_proxies(const struct guid *,
 	gnet_host_t *proxies, int proxy_count);
 void download_proxy_dht_lookup_done(const struct guid *);
-void download_no_push_proxies(const struct guid *);
 void download_found_server(const struct guid *,
-	const host_addr_t addr, guint16 port);
+	const host_addr_t addr, uint16 port);
+void download_attach_socket(struct download *d, struct gnutella_socket *s);
 
 void download_move_start(struct download *d);
 void download_move_progress(struct download *d, filesize_t copied);
 void download_move_done(struct download *d, const char *pathname,
-		guint elapsed);
+		uint elapsed);
 void download_move_error(struct download *d);
 
-guint extract_retry_after(struct download *d, const header_t *header);
-gboolean is_faked_download(const struct download *d);
+uint extract_retry_after(const struct download *d, const header_t *header);
+bool is_faked_download(const struct download *d);
 
 struct download *download_find_waiting_unparq(const host_addr_t addr,
-					guint16 port);
+					uint16 port);
 void download_set_socket_rx_size(unsigned rx_size);
 
 void download_proxy_newstate(struct download *d);
 void download_proxy_sent(struct download *d);
 void download_proxy_failed(struct download *d);
 
-gboolean download_known_guid(const struct guid *guid,
-	host_addr_t *addr, guint16 *port, sequence_t **proxies);
+bool download_known_guid(const struct guid *guid,
+	host_addr_t *addr, uint16 *port, sequence_t **proxies);
 void download_got_push_proxies(const struct guid *guid,
-	const gnet_host_vec_t *proxies);
+	const gnet_host_vec_t *proxies, bool g2);
 
 struct download * download_browse_start(
-	const char *hostname, host_addr_t addr, guint16 port,
+	const char *hostname, host_addr_t addr, uint16 port,
 	const struct guid *, const gnet_host_vec_t *proxies,
-	gnet_search_t search, guint32 flags);
+	gnet_search_t search, uint32 flags);
 
 struct download * download_thex_start(const char *uri,
 	const struct sha1 *sha1, const struct tth *tth, filesize_t filesize,
-	const char *hostname, host_addr_t addr, guint16 port,
+	const char *hostname, host_addr_t addr, uint16 port,
 	const struct guid *, const gnet_host_vec_t *proxies,
-	guint32 flags);
+	uint32 flags);
 
 void download_abort_browse_host(struct download *d, gnet_search_t sh);
 void download_got_eof(struct download *d);
@@ -126,18 +131,29 @@ void download_rx_done(struct download *d);
 void download_data_received(struct download *d, ssize_t received);
 void download_maybe_finished(struct download *d);
 
-gboolean download_handle_http(const char *url);
-gboolean download_is_stalled(struct download *);
-gboolean download_is_alive(const struct download *);
-gboolean download_is_completed_filename(const char *name);
+bool download_handle_http(const char *url);
+bool download_is_stalled(const struct download *);
+bool download_is_alive(const struct download *);
+bool download_is_active(const struct download *);
+bool download_is_completed_filename(const char *name);
 
-gboolean download_remove(struct download *d);
+bool download_sha1_is_rare(const struct sha1 *sha1);
+
+bool download_remove(struct download *d);
 void download_abort(struct download *d);
 
 void download_got_fw_node_info(const struct guid *guid,
-	host_addr_t addr, guint16 port, const char *fwinfo);
+	host_addr_t addr, uint16 port, const char *fwinfo);
 
 const char *server_host_info(const struct dl_server *server);
+const char *download_status_to_string(const struct download *d);
+
+static inline const char *
+download_host_info(const struct download *d)
+{
+	download_check(d);
+	return server_host_info(d->server);
+}
 
 #endif /* _core_downloads_h_ */
 
