@@ -1,6 +1,4 @@
 /*
- * $Id: gdht.c 17381 2010-03-18 08:40:15Z rmanfredi $
- *
  * Copyright (c) 2008, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,8 +33,6 @@
 
 #include "common.h"
 
-RCSID("$Id: gdht.c 17381 2010-03-18 08:40:15Z rmanfredi $")
-
 #include "gdht.h"
 #include "fileinfo.h"
 #include "extensions.h"
@@ -56,8 +52,12 @@ RCSID("$Id: gdht.c 17381 2010-03-18 08:40:15Z rmanfredi $")
 #include "lib/base32.h"
 #include "lib/bstr.h"
 #include "lib/endian.h"
+#include "lib/hikset.h"
 #include "lib/sha1.h"
+#include "lib/str.h"
+#include "lib/stringify.h"
 #include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 #define MAX_PROXIES		32		/**< Max push-proxies we collect from a PROX */
@@ -65,8 +65,8 @@ RCSID("$Id: gdht.c 17381 2010-03-18 08:40:15Z rmanfredi $")
 /**
  * Hash table holding all the pending lookups by KUID.
  */
-static GHashTable *sha1_lookups;	/* KUID -> struct sha1_lookup * */
-static GHashTable *guid_lookups;	/* KUID -> struct guid_lookup * */
+static hikset_t *sha1_lookups;	/* KUID -> struct sha1_lookup * */
+static hikset_t *guid_lookups;	/* KUID -> struct guid_lookup * */
 
 typedef enum {
 	SHA1_LOOKUP_MAGIC = 0x5fd660bfU
@@ -103,7 +103,7 @@ struct guid_lookup {
 	const kuid_t *id;		/**< ID being looked for (atom) */
 	const guid_t *guid;		/**< Servent's GUID (atom) */
 	host_addr_t addr;		/**< Servent's address */
-	guint16 port;			/**< Servent's port */
+	uint16 port;			/**< Servent's port */
 	unsigned nope:1;		/**< Was looking for a NOPE instead of a PROX */
 };
 
@@ -115,7 +115,7 @@ guid_lookup_check(const struct guid_lookup *glk)
 }
 
 static void gdht_guid_found(const kuid_t *kuid,
-	const lookup_val_rs_t *rs, gpointer arg);
+	const lookup_val_rs_t *rs, void *arg);
 
 /**
  * Convert a SHA1 to the proper Kademlia key for lookups.
@@ -136,48 +136,64 @@ gdht_kuid_from_sha1(const sha1_t *sha1)
 const kuid_t *
 gdht_kuid_from_guid(const guid_t *guid)
 {
-	SHA1Context ctx;
 	struct sha1 digest;
 
-	SHA1Reset(&ctx);
-	SHA1Input(&ctx, guid->v, GUID_RAW_SIZE);
-	SHA1Result(&ctx, &digest);
+	SHA1_COMPUTE(guid->v, &digest);
 
 	return kuid_get_atom((const kuid_t *) &digest);
 }
 
+/**
+ * Is IP:port pointing back at us?
+ */
+static bool
+gdht_is_our_ip_port(const host_addr_t addr, uint16 port)
+{
+	return is_my_address_and_port(addr, port) ||
+		local_addr_cache_lookup(addr, port);
+}
+
+/**
+ * Was result published by ourselves
+ */
+static bool
+gdht_published_by_ourselves(const lookup_val_rc_t *rc)
+{
+	return gdht_is_our_ip_port(rc->addr, rc->port);
+}
+
 /**
  * Free SHA1 lookup context.
  */
 static void
-gdht_free_sha1_lookup(struct sha1_lookup *slk, gboolean do_remove)
+gdht_free_sha1_lookup(struct sha1_lookup *slk, bool do_remove)
 {
 	sha1_lookup_check(slk);
 
 	if (do_remove)
-		g_hash_table_remove(sha1_lookups, slk->id);
+		hikset_remove(sha1_lookups, slk->id);
 
 	kuid_atom_free(slk->id);
 	atom_guid_free(slk->fi_guid);
-	wfree(slk, sizeof *slk);
+	WFREE(slk);
 }
 
 /**
  * Free GUID lookup context.
  */
 static void
-gdht_free_guid_lookup(struct guid_lookup *glk, gboolean do_remove)
+gdht_free_guid_lookup(struct guid_lookup *glk, bool do_remove)
 {
 	guid_lookup_check(glk);
 
 	if (do_remove) {
 		download_proxy_dht_lookup_done(glk->guid);
-		g_hash_table_remove(guid_lookups, glk->id);
+		hikset_remove(guid_lookups, glk->id);
 	}
 
 	kuid_atom_free(glk->id);
 	atom_guid_free(glk->guid);
-	wfree(glk, sizeof *glk);
+	WFREE(glk);
 }
 
 /*
@@ -187,28 +203,68 @@ gdht_free_guid_lookup(struct guid_lookup *glk, gboolean do_remove)
 static const char *
 value_infostr(const lookup_val_rc_t *rc)
 {
-	static char info[60];
+	static char info[64];
 
-	gm_snprintf(info, sizeof info, "DHT %s v%u.%u (%lu byte%s)",
+	str_bprintf(ARYLEN(info), "DHT %s v%u.%u (%lu byte%s) [%s]",
 		dht_value_type_to_string(rc->type), rc->major, rc->minor,
-		(gulong) rc->length, 1 == rc->length ? "" : "s");
+		(ulong) PLURAL(rc->length), vendor_to_string(rc->vcode));
 
 	return info;
 }
 
+/**
+ * Callback when SHA1 lookup is initiated.
+ *
+ * @return TRUE if OK, FALSE if lookup must be aborted.
+ */
+static bool
+gdht_sha1_looking(const kuid_t *kuid, void *arg)
+{
+	struct sha1_lookup *slk = arg;
+	fileinfo_t *fi;
+
+	sha1_lookup_check(slk);
+	g_assert(slk->id == kuid);		/* They are atoms */
+
+	fi = file_info_by_guid(slk->fi_guid);	/* NULL if fileinfo was removed */
+	if (fi != NULL) {
+		return file_info_dht_query_starting(fi);
+	} else {
+		return FALSE;		/* No need to query */
+	}
+}
+
 /**
  * Callback when SHA1 lookup is unsuccessful.
  */
 static void
-gdht_sha1_not_found(const kuid_t *kuid, lookup_error_t error, gpointer arg)
+gdht_sha1_not_found(const kuid_t *kuid, lookup_error_t error, void *arg)
 {
 	struct sha1_lookup *slk = arg;
+	fileinfo_t *fi;
 
 	sha1_lookup_check(slk);
 	g_assert(slk->id == kuid);		/* They are atoms */
 
+	fi = file_info_by_guid(slk->fi_guid);	/* NULL if fileinfo was removed */
+	if (fi != NULL) {
+		bool launched;
+
+		switch (error) {
+		case LOOKUP_E_CANCELLED:
+		case LOOKUP_E_EMPTY_ROUTE:
+			launched = FALSE;
+			break;
+		default:
+			launched = TRUE;
+			break;
+		}
+
+		file_info_dht_query_completed(fi, launched, FALSE);
+	}
+
 	if (GNET_PROPERTY(dht_lookup_debug) > 1)
-		g_message("DHT ALOC lookup for %s failed: %s",
+		g_debug("DHT ALOC lookup for %s failed: %s",
 			kuid_to_string(kuid), lookup_strerror(error));
 
 	gdht_free_sha1_lookup(slk, TRUE);
@@ -224,42 +280,46 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 	extvec_t exv[MAX_EXTVEC];
 	int exvcnt;
 	int i;
-	gboolean firewalled = FALSE;
+	bool firewalled = FALSE;
 	struct tth tth;
-	gboolean has_tth = FALSE;
+	bool has_tth = FALSE;
 	guid_t guid;
-	guint16 port = 0;
-	gboolean tls = FALSE;
+	uint16 port = 0;
+	bool tls = FALSE;
 	filesize_t filesize = 0;
-	guint32 flags = 0;
+	filesize_t available = 0;
+	uint32 flags = 0;
 	char host[MAX_HOSTLEN];
 	const char *hostname = NULL;
+	bool has_valid_guid = FALSE;
 
 	g_assert(DHT_VT_ALOC == rc->type);
 
 	ext_prepare(exv, MAX_EXTVEC);
-	memset(guid.v, 0, GUID_RAW_SIZE);
+	ZERO(&guid.v);
 
 	exvcnt = ext_parse(rc->data, rc->length, exv, MAX_EXTVEC);
 
 	for (i = 0; i < exvcnt; i++) {
 		extvec_t *e = &exv[i];
-		guint16 paylen;
+		uint16 paylen;
 
 		switch (e->ext_token) {
 		case EXT_T_GGEP_client_id:
-			if (GUID_RAW_SIZE == ext_paylen(e))
+			if (GUID_RAW_SIZE == ext_paylen(e)) {
 				memcpy(guid.v, ext_payload(e), GUID_RAW_SIZE);
+				has_valid_guid = TRUE;
+			}
 			break;
 		case EXT_T_GGEP_firewalled:
 			if (1 == ext_paylen(e)) {
-				guint8 fw = peek_u8(ext_payload(e));
+				uint8 fw = peek_u8(ext_payload(e));
 				firewalled = fw != 0;
 			}
 			break;
 		case EXT_T_GGEP_length:
 			{
-				guint64 fs;
+				uint64 fs;
 				ggept_status_t ret;
 
 				ret = ggept_filesize_extract(e, &fs);
@@ -284,17 +344,29 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 			{
 				ggept_status_t ret;
 
-				ret = ggept_hname_extract(e, host, sizeof host);
+				ret = ggept_hname_extract(e, ARYLEN(host));
 				if (GGEP_OK == ret)
 					hostname = host;
 			}
 			break;
+		case EXT_T_GGEP_avail:		/* Length available (for partial file) */
+			{
+				uint64 fs;
+				ggept_status_t ret;
+
+				ret = ggept_filesize_extract(e, &fs);
+				if (GGEP_OK == ret)
+					available = 0 == fs ? 1 : fs;
+				else
+					available = 1;		/* Force "partial" status */
+			}
+			break;
 		default:
 			if (GNET_PROPERTY(ggep_debug) > 1 && e->ext_type == EXT_GGEP) {
 				paylen = ext_paylen(e);
 				g_warning("%s: unhandled GGEP \"%s\" (%d byte%s)",
 					value_infostr(rc),
-					ext_ggep_id_str(e), paylen, paylen == 1 ? "" : "s");
+					ext_ggep_id_str(e), PLURAL(paylen));
 			}
 			break;
 		}
@@ -308,10 +380,10 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 	if (port) {
 		if (port != rc->port && GNET_PROPERTY(download_debug))
 			g_warning("%s: port mismatch: creator's was %u, "
-				"%sALOC is %u for %s",
+				"%sALOC is %u for %s%s",
 				value_infostr(rc), rc->port,
 				firewalled ? "firewalled " : "",
-				port, fi->pathname);
+				port, available != 0 ? "partial " : "", fi->pathname);
 	} else {
 		port = rc->port;
 	}
@@ -322,9 +394,22 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 
 	if (!firewalled && !host_is_valid(rc->addr, port)) {
 		if (GNET_PROPERTY(download_debug))
-			g_warning("discarding %s from %s for %s: invalid IP:port",
+			g_warning("discarding %s from %s for %s%s: invalid IP:port",
 				value_infostr(rc), host_addr_port_to_string(rc->addr, port),
-				fi->pathname);
+				available != 0 ? "partial " : "", fi->pathname);
+		goto cleanup;
+	}
+
+	/*
+	 * Make sure firewalled servents list their GUID.
+	 */
+
+	if (firewalled && !has_valid_guid) {
+		if (GNET_PROPERTY(download_debug))
+			g_warning("discarding %s from %s for %s%s: "
+				"firewalled host, no GUID",
+				value_infostr(rc), host_addr_port_to_string(rc->addr, port),
+				available != 0 ? "partial " : "", fi->pathname);
 		goto cleanup;
 	}
 
@@ -332,11 +417,39 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 	 * Discard hostile sources.
 	 */
 
-	if (hostiles_check(rc->addr)) {
-		if (GNET_PROPERTY(download_debug))
-			g_warning("discarding %s from %s for %s: hostile IP",
+	if (hostiles_is_bad(rc->addr)) {
+		if (GNET_PROPERTY(download_debug)) {
+			hostiles_flags_t hflags = hostiles_check(rc->addr);
+			g_warning("discarding %s from %s for %s%s: hostile IP (%s)",
 				value_infostr(rc), host_addr_port_to_string(rc->addr, port),
-				fi->pathname);
+				available != 0 ? "partial " : "", fi->pathname,
+				hostiles_flags_to_string(hflags));
+		}
+		goto cleanup;
+	}
+
+	/*
+	 * Rule out on GUID collision: alt-loc cannot bear our own GUID.
+	 *
+	 * This is done after checking for hostile sources of course since
+	 * anything can happen with hostiles
+	 */
+
+	if (has_valid_guid && guid_eq(&guid, GNET_PROPERTY(servent_guid))) {
+		has_valid_guid = FALSE;
+
+		/*
+		 * Make sure the server address is not ours, otherwise we don't count
+		 * that as a GUID collision.
+		 */
+
+		if (!gdht_is_our_ip_port(rc->addr, port))
+			gnet_stats_inc_general(GNR_OWN_GUID_COLLISIONS);
+
+		if (GNET_PROPERTY(download_debug))
+			g_warning("discarding %s from %s for %s%s: host bears our GUID",
+				value_infostr(rc), host_addr_port_to_string(rc->addr, port),
+				available != 0 ? "partial " : "", fi->pathname);
 		goto cleanup;
 	}
 
@@ -346,10 +459,11 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 
 	if (filesize != 0 && fi->size != 0 && fi->size != filesize) {
 		if (GNET_PROPERTY(download_debug))
-			g_warning("discarding %s from %s for %s: "
-				"we have size=%lu, ALOC says %lu",
+			g_warning("discarding %s from %s for %s%s: "
+				"we have size=%s, ALOC says %s",
 				value_infostr(rc), host_addr_port_to_string(rc->addr, port),
-				fi->pathname, (gulong) fi->size, (gulong) filesize);
+				available != 0 ? "partial " : "", fi->pathname,
+				filesize_to_string(fi->size), filesize_to_string2(filesize));
 		goto cleanup;
 	}
 
@@ -361,12 +475,13 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 		if (GNET_PROPERTY(download_debug)) {
 			char buf[TTH_BASE32_SIZE + 1];
 
-			base32_encode(buf, sizeof buf, tth.data, sizeof tth.data);
+			base32_encode(ARYLEN(buf), ARYLEN(tth.data));
 
-			g_warning("discarding %s from %s for %s: "
+			g_warning("discarding %s from %s for %s%s: "
 				"we have TTH root %s, ALOC says %s",
 				value_infostr(rc), host_addr_port_to_string(rc->addr, port),
-				fi->pathname, tth_base32(fi->tth), buf);
+				available != 0 ? "partial " : "", fi->pathname,
+				tth_base32(fi->tth), buf);
 		}
 		goto cleanup;
 	}
@@ -376,8 +491,9 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 	 */
 
 	if (GNET_PROPERTY(download_debug) > 1)
-		g_message("adding %s%ssource %s (GUID %s) from DHT ALOC for %s",
+		g_debug("adding %s%s%ssource %s (GUID %s) from DHT ALOC for %s",
 			firewalled ? "firewalled " : "", tls ? "TLS " : "",
+			available != 0 ? "partial " : "",
 			host_addr_port_to_string(rc->addr, port),
 			guid_to_string(&guid), fi->pathname);
 
@@ -393,35 +509,43 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 		fi->sha1,
 		has_tth ? &tth : NULL,
 		tm_time(),
-		deconstify_gpointer(fi),
+		deconstify_pointer(fi),
 		flags);
 
 	/* FALL THROUGH */
 
 cleanup:
-	if (exvcnt)
+	/*
+	 * Regardless of whether this ALOC matches the file we queried originally
+	 * we can flag the server as publishing in the DHT.
+	 */
+
+	if (has_valid_guid) {
+		download_server_publishes_in_dht(&guid);
+	}
+	if (exvcnt) {
 		ext_reset(exv, MAX_EXTVEC);
+	}
 }
 
 /**
  * Callback when SHA1 lookup is successful.
  */
 static void
-gdht_sha1_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
+gdht_sha1_found(const kuid_t *kuid, const lookup_val_rs_t *rs, void *arg)
 {
 	struct sha1_lookup *slk = arg;
 	fileinfo_t *fi;
 	size_t i;
-	gboolean seen_foreign = FALSE;
+	bool seen_foreign = FALSE;
 
 	g_assert(rs);
 	sha1_lookup_check(slk);
 	g_assert(slk->id == kuid);		/* They are atoms */
 
 	if (GNET_PROPERTY(dht_lookup_debug) > 1)
-		g_message("DHT ALOC lookup for %s returned %lu value%s",
-			kuid_to_string(kuid), (gulong) rs->count,
-			1 == rs->count ? "" : "s");
+		g_debug("DHT ALOC lookup for %s returned %zu value%s",
+			kuid_to_string(kuid), PLURAL(rs->count));
 
 	fi = file_info_by_guid(slk->fi_guid);
 
@@ -442,9 +566,7 @@ gdht_sha1_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 
 	for (i = 0; i < rs->count; i++) {
 		lookup_val_rc_t *rc = &rs->records[i];
-		if (is_my_address_and_port(rc->addr, rc->port))
-			continue;
-		if (local_addr_cache_lookup(rc->addr, rc->port))
+		if (gdht_published_by_ourselves(rc))
 			continue;
 		seen_foreign = TRUE;		/* ALOC not published by ourselves */
 		gdht_handle_aloc(rc, fi);
@@ -456,9 +578,11 @@ gdht_sha1_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 	 */
 
 	if (seen_foreign) {
-		gnet_stats_count_general(GNR_DHT_SUCCESSFUL_ALT_LOC_LOOKUPS, 1);
+		gnet_stats_inc_general(GNR_DHT_SUCCESSFUL_ALT_LOC_LOOKUPS);
 	}
 
+	file_info_dht_query_completed(fi, TRUE, seen_foreign);
+
 cleanup:
 	gdht_free_sha1_lookup(slk, TRUE);
 }
@@ -467,14 +591,14 @@ cleanup:
  * Launch a SHA1 lookup in the DHT to collect alternate locations.
  */
 void
-gdht_find_sha1(const fileinfo_t *fi)
+gdht_find_sha1(fileinfo_t *fi)
 {
 	struct sha1_lookup *slk;
 
 	file_info_check(fi);
 	g_assert(fi->sha1);
 
-	slk = walloc(sizeof *slk);
+	WALLOC(slk);
 	slk->magic = SHA1_LOOKUP_MAGIC;
 	slk->id = gdht_kuid_from_sha1(fi->sha1);
 	slk->fi_guid = atom_guid_get(fi->guid);
@@ -485,7 +609,7 @@ gdht_find_sha1(const fileinfo_t *fi)
 	 * request.
 	 */
 
-	if (g_hash_table_lookup(sha1_lookups, slk->id)) {
+	if (hikset_contains(sha1_lookups, slk->id)) {
 		if (GNET_PROPERTY(dht_lookup_debug))
 			g_warning("DHT already has pending search for %s (%s) for %s",
 				kuid_to_hex_string(slk->id),
@@ -496,19 +620,21 @@ gdht_find_sha1(const fileinfo_t *fi)
 	}
 
 	if (GNET_PROPERTY(dht_lookup_debug))
-		g_message("DHT will be searching ALOC for %s (%s) for %s",
+		g_debug("DHT will be searching ALOC for %s (%s) for %s",
 			kuid_to_hex_string(slk->id), kuid_to_string(slk->id), fi->pathname);
 
-	gm_hash_table_insert_const(sha1_lookups, slk->id, slk);
-	ulq_find_value(slk->id, DHT_VT_ALOC, 
-		gdht_sha1_found, gdht_sha1_not_found, slk);
+	hikset_insert_key(sha1_lookups, &slk->id);
+	file_info_dht_query_queued(fi);
+
+	ulq_find_value(slk->id, DHT_VT_ALOC,
+		gdht_sha1_found, gdht_sha1_looking, gdht_sha1_not_found, slk);
 }
 
 /**
  * Callback when GUID lookup is unsuccessful.
  */
 static void
-gdht_guid_not_found(const kuid_t *kuid, lookup_error_t error, gpointer arg)
+gdht_guid_not_found(const kuid_t *kuid, lookup_error_t error, void *arg)
 {
 	struct guid_lookup *glk = arg;
 
@@ -516,10 +642,9 @@ gdht_guid_not_found(const kuid_t *kuid, lookup_error_t error, gpointer arg)
 	g_assert(glk->id == kuid);		/* They are atoms */
 
 	if (GNET_PROPERTY(dht_lookup_debug) > 1)
-		g_message("DHT PROX lookup for GUID %s failed: %s",
+		g_debug("DHT PROX lookup for GUID %s failed: %s",
 			guid_to_string(glk->guid), lookup_strerror(error));
 
-	download_no_push_proxies(glk->guid);
 	gdht_free_guid_lookup(glk, TRUE);
 }
 
@@ -533,7 +658,7 @@ gdht_handle_prox(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 	int exvcnt;
 	int i;
 	guid_t guid;
-	guint16 port = 0;
+	uint16 port = 0;
 	gnet_host_t proxies[MAX_PROXIES];
 	int proxy_count = 0;
 
@@ -541,13 +666,13 @@ gdht_handle_prox(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 	guid_lookup_check(glk);
 
 	ext_prepare(exv, MAX_EXTVEC);
-	memset(guid.v, 0, GUID_RAW_SIZE);
+	ZERO(&guid.v);
 
 	exvcnt = ext_parse(rc->data, rc->length, exv, MAX_EXTVEC);
 
 	for (i = 0; i < exvcnt; i++) {
 		extvec_t *e = &exv[i];
-		guint16 paylen;
+		uint16 paylen;
 
 		switch (e->ext_token) {
 		case EXT_T_GGEP_client_id:
@@ -580,11 +705,11 @@ gdht_handle_prox(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 
 				while (
 					bstr_unread_size(bs) > 0 &&
-					UNSIGNED(proxy_count) < G_N_ELEMENTS(proxies)
+					UNSIGNED(proxy_count) < N_ITEMS(proxies)
 				) {
 					host_addr_t a;
-					guint16 p;
-					guint8 len;
+					uint16 p;
+					uint8 len;
 
 					if (!bstr_read_u8(bs, &len))
 						break;
@@ -605,18 +730,21 @@ gdht_handle_prox(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 					 * Discard hostile sources.
 					 */
 
-					if (hostiles_check(a)) {
-						if (GNET_PROPERTY(download_debug))
+					if (hostiles_is_bad(a)) {
+						if (GNET_PROPERTY(download_debug)) {
+							hostiles_flags_t flags = hostiles_check(a);
 							g_warning("discarding proxy %s in %s from %s "
-								"for GUID %s: hostile IP",
+								"for GUID %s: hostile IP (%s)",
 								host_addr_port_to_string(a, p),
 								value_infostr(rc),
 								host_addr_port_to_string2(rc->addr, rc->port),
-								guid_to_string(glk->guid));
+								guid_to_string(glk->guid),
+								hostiles_flags_to_string(flags));
+						}
 						continue;
 					} else {
 						if (GNET_PROPERTY(download_debug))
-							g_message("new push-proxy for %s is at %s",
+							g_debug("new push-proxy for %s is at %s",
 								guid_to_string(glk->guid),
 								host_addr_port_to_string(a, p));
 					}
@@ -632,7 +760,7 @@ gdht_handle_prox(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 				paylen = ext_paylen(e);
 				g_warning("%s: unhandled GGEP \"%s\" (%d byte%s)",
 					value_infostr(rc),
-					ext_ggep_id_str(e), paylen, paylen == 1 ? "" : "s");
+					ext_ggep_id_str(e), PLURAL(paylen));
 			}
 			break;
 		}
@@ -684,7 +812,7 @@ gdht_handle_prox(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 	 * since we last heard about it.
 	 */
 
-	if (!host_addr_equal(glk->addr, rc->addr) || port != glk->port)
+	if (!host_addr_equiv(glk->addr, rc->addr) || port != glk->port)
 		download_found_server(glk->guid, rc->addr, port);
 
 	/*
@@ -692,12 +820,13 @@ gdht_handle_prox(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 	 */
 
 	if (GNET_PROPERTY(download_debug) > 0)
-		g_message("adding %d push-prox%s (GUID %s) from DHT PROX for %s (%s)",
-			proxy_count, 1 == proxy_count ? "y" : "ies",
+		g_debug("adding %d push-prox%s (GUID %s) from DHT PROX for %s (%s)",
+			PLURAL_Y(proxy_count),
 			guid_to_string(glk->guid),
 			host_addr_port_to_string(rc->addr, port),
 			host_addr_port_to_string(glk->addr, glk->port));
 
+	download_server_publishes_in_dht(glk->guid);
 	download_add_push_proxies(glk->guid, proxies, proxy_count);
 
 	/* FALL THROUGH */
@@ -717,19 +846,19 @@ gdht_handle_nope(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 	int exvcnt;
 	int i;
 	guid_t guid;
-	guint16 port = 0;
+	uint16 port = 0;
 
 	g_assert(DHT_VT_NOPE == rc->type);
 	guid_lookup_check(glk);
 
 	ext_prepare(exv, MAX_EXTVEC);
-	memset(guid.v, 0, GUID_RAW_SIZE);
+	ZERO(&guid.v);
 
 	exvcnt = ext_parse(rc->data, rc->length, exv, MAX_EXTVEC);
 
 	for (i = 0; i < exvcnt; i++) {
 		extvec_t *e = &exv[i];
-		guint16 paylen;
+		uint16 paylen;
 
 		switch (e->ext_token) {
 		case EXT_T_GGEP_guid:
@@ -748,7 +877,7 @@ gdht_handle_nope(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 				paylen = ext_paylen(e);
 				g_warning("%s: unhandled GGEP \"%s\" (%d byte%s)",
 					value_infostr(rc),
-					ext_ggep_id_str(e), paylen, paylen == 1 ? "" : "s");
+					ext_ggep_id_str(e), PLURAL(paylen));
 			}
 			break;
 		}
@@ -787,10 +916,10 @@ gdht_handle_nope(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 	 */
 
 	if (GNET_PROPERTY(download_debug) > 0)
-		g_message("adding %s (NOPE creator) as push-proxy for %s (%s)",
+		g_debug("adding %s [%s] (NOPE creator) as push-proxy for %s (%s)",
 			host_addr_port_to_string(rc->addr, port),
-			guid_to_string(glk->guid),
-			host_addr_port_to_string(glk->addr, glk->port));
+			vendor_to_string(rc->vcode), guid_to_string(glk->guid),
+			host_addr_port_to_string2(glk->addr, glk->port));
 
 	download_add_push_proxy(glk->guid, rc->addr, port);
 
@@ -805,12 +934,12 @@ cleanup:
  * Callback when GUID lookup is successful (PROX or NOPE values).
  */
 static void
-gdht_guid_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
+gdht_guid_found(const kuid_t *kuid, const lookup_val_rs_t *rs, void *arg)
 {
 	struct guid_lookup *glk = arg;
 	size_t i;
-	gboolean prox = FALSE;
-	gboolean nope = FALSE;
+	bool prox = FALSE;
+	bool nope = FALSE;
 	size_t other = 0;
 
 	g_assert(rs);
@@ -818,9 +947,8 @@ gdht_guid_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 	g_assert(glk->id == kuid);		/* They are atoms */
 
 	if (GNET_PROPERTY(dht_lookup_debug) > 1) {
-		g_message("DHT PROX lookup for GUID %s returned %lu value%s",
-			guid_to_string(glk->guid), (gulong) rs->count,
-			1 == rs->count ? "" : "s");
+		g_debug("DHT PROX lookup for GUID %s returned %zu value%s",
+			guid_to_string(glk->guid), PLURAL(rs->count));
 	}
 
 	/*
@@ -833,6 +961,8 @@ gdht_guid_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 
 	for (i = 0; i < rs->count; i++) {
 		lookup_val_rc_t *rc = &rs->records[i];
+		if (gdht_published_by_ourselves(rc))
+			continue;
 		switch (rc->type) {
 		case DHT_VT_PROX:
 			prox = TRUE;
@@ -849,15 +979,14 @@ gdht_guid_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 	}
 
 	if (GNET_PROPERTY(dht_lookup_debug)) {
-		g_message("DHT PROX %s lookup for GUID %s returned %lu other value%s",
+		g_debug("DHT PROX %s lookup for GUID %s returned %zu other value%s",
 			(prox || nope) ? "successful" : "failed",
-			guid_to_string(glk->guid),
-			(gulong) other, 1 == other ? "" : "s");
+			guid_to_string(glk->guid), PLURAL(other));
 	}
 
 	if (other > 0) {
 		/* Was looking for SHA1(GUID), found some other SHA1 */
-		gnet_stats_count_general(GNR_DHT_SHA1_DATA_TYPE_COLLISIONS, 1);
+		gnet_stats_inc_general(GNR_DHT_SHA1_DATA_TYPE_COLLISIONS);
 	}
 
 	/*
@@ -872,11 +1001,9 @@ gdht_guid_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 
 	/* If we got at least one NOPE back, count a successful NOPE lookup */
 
-	gnet_stats_count_general(
-		nope ?
-			GNR_DHT_SUCCESSFUL_NODE_PUSH_ENTRY_LOOKUPS :
-			GNR_DHT_SUCCESSFUL_PUSH_PROXY_LOOKUPS,
-		1);
+	gnet_stats_inc_general(nope ?
+		GNR_DHT_SUCCESSFUL_NODE_PUSH_ENTRY_LOOKUPS :
+		GNR_DHT_SUCCESSFUL_PUSH_PROXY_LOOKUPS);
 
 	gdht_free_guid_lookup(glk, TRUE);
 }
@@ -885,7 +1012,7 @@ gdht_guid_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
  * Launch a GUID lookup in the DHT to collect push proxies for a server.
  */
 void
-gdht_find_guid(const guid_t *guid, const host_addr_t addr, guint16 port)
+gdht_find_guid(const guid_t *guid, const host_addr_t addr, uint16 port)
 {
 	struct guid_lookup *glk;
 
@@ -893,7 +1020,7 @@ gdht_find_guid(const guid_t *guid, const host_addr_t addr, guint16 port)
 	g_assert(!guid_is_blank(guid));
 	g_assert(host_addr_initialized(addr));
 
-	glk = walloc(sizeof *glk);
+	WALLOC(glk);
 	glk->magic = GUID_LOOKUP_MAGIC;
 	glk->id = gdht_kuid_from_guid(guid);
 	glk->guid = atom_guid_get(guid);
@@ -906,7 +1033,7 @@ gdht_find_guid(const guid_t *guid, const host_addr_t addr, guint16 port)
 	 * request.
 	 */
 
-	if (g_hash_table_lookup(guid_lookups, glk->id)) {
+	if (hikset_contains(guid_lookups, glk->id)) {
 		if (GNET_PROPERTY(dht_lookup_debug))
 			g_warning("DHT already has pending search for %s (GUID %s) for %s",
 			kuid_to_hex_string(glk->id),
@@ -917,11 +1044,11 @@ gdht_find_guid(const guid_t *guid, const host_addr_t addr, guint16 port)
 	}
 
 	if (GNET_PROPERTY(dht_lookup_debug))
-		g_message("DHT will be searching PROX for %s (GUID %s) for %s",
+		g_debug("DHT will be searching PROX for %s (GUID %s) for %s",
 			kuid_to_hex_string(glk->id),
 			guid_to_string(guid), host_addr_port_to_string(addr, port));
 
-	gm_hash_table_insert_const(guid_lookups, glk->id, glk);
+	hikset_insert_key(guid_lookups, &glk->id);
 
 	/*
 	 * We're looking for ANY value here, but we really expect PROX or NOPE
@@ -948,19 +1075,20 @@ gdht_find_guid(const guid_t *guid, const host_addr_t addr, guint16 port)
 void
 gdht_init(void)
 {
-	sha1_lookups = g_hash_table_new(sha1_hash, sha1_eq);
-	guid_lookups = g_hash_table_new(sha1_hash, sha1_eq);
+	sha1_lookups = hikset_create(
+		offsetof(struct sha1_lookup, id), HASH_KEY_FIXED, KUID_RAW_SIZE);
+	guid_lookups = hikset_create(
+		offsetof(struct guid_lookup, id), HASH_KEY_FIXED, KUID_RAW_SIZE);
 }
 
 /**
  * Hash table iterator to free a struct sha1_lookup
  */
 static void
-free_sha1_lookups_kv(gpointer unused_key, gpointer val, gpointer unused_x)
+free_sha1_lookups_kv(void *val, void *unused_x)
 {
 	struct sha1_lookup *slk = val;
 
-	(void) unused_key;
 	(void) unused_x;
 
 	gdht_free_sha1_lookup(slk, FALSE);
@@ -970,11 +1098,10 @@ free_sha1_lookups_kv(gpointer unused_key, gpointer val, gpointer unused_x)
  * Hash table iterator to free a struct guid_lookup
  */
 static void
-free_guid_lookups_kv(gpointer unused_key, gpointer val, gpointer unused_x)
+free_guid_lookups_kv(void *val, void *unused_x)
 {
 	struct guid_lookup *glk = val;
 
-	(void) unused_key;
 	(void) unused_x;
 
 	gdht_free_guid_lookup(glk, FALSE);
@@ -986,13 +1113,11 @@ free_guid_lookups_kv(gpointer unused_key, gpointer val, gpointer unused_x)
 void
 gdht_close(void)
 {
-	g_hash_table_foreach(sha1_lookups, free_sha1_lookups_kv, NULL);
-	g_hash_table_destroy(sha1_lookups);
-	sha1_lookups = NULL;
+	hikset_foreach(sha1_lookups, free_sha1_lookups_kv, NULL);
+	hikset_free_null(&sha1_lookups);
 
-	g_hash_table_foreach(guid_lookups, free_guid_lookups_kv, NULL);
-	g_hash_table_destroy(guid_lookups);
-	guid_lookups = NULL;
+	hikset_foreach(guid_lookups, free_guid_lookups_kv, NULL);
+	hikset_free_null(&guid_lookups);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
