@@ -1,7 +1,5 @@
 /*
- * $Id: guid.h 12496 2006-11-26 09:22:49Z cbiere $
- *
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2003, 2011, Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -30,7 +28,7 @@
  * Globally Unique ID (GUID) manager.
  *
  * @author Raphael Manfredi
- * @date 2002-2003
+ * @date 2002-2003, 2011
  */
 
 #ifndef _core_guid_h_
@@ -46,17 +44,29 @@
  */
 
 void guid_init(void);
-gboolean guid_is_gtkg(
-	const gchar *xuid, guint8 *majp, guint8 *minp, gboolean *relp);
-gboolean guid_is_requery(const gchar *xuid);
-void guid_random_muid(gchar *muid);
-void guid_ping_muid(gchar *muid);
-void guid_query_muid(gchar *muid, gboolean initial);
-gboolean guid_query_muid_is_gtkg(const gchar *guid,
-	gboolean oob, guint8 *majp, guint8 *minp, gboolean *relp);
-void guid_query_oob_muid(
-	gchar *muid, const host_addr_t addr, guint16 port, gboolean initial);
-void guid_oob_get_addr_port(const gchar *guid,
-	host_addr_t *addr, guint16 *port);
+void guid_close(void);
+
+bool guid_is_banned(const guid_t *guid);
+void guid_add_banned(const guid_t *guid);
+
+bool guid_is_gtkg(const guid_t *xuid,
+	uint8 *majp, uint8 *minp, bool *relp);
+bool guid_is_requery(const guid_t *xuid);
+void guid_random_muid(guid_t *muid);
+void guid_ping_muid(guid_t *muid);
+void guid_query_muid(guid_t *muid, bool initial);
+bool guid_query_muid_is_gtkg(const guid_t *guid,
+	bool oob, uint8 *majp, uint8 *minp, bool *relp);
+void guid_query_oob_muid(guid_t *muid,
+	const host_addr_t addr, uint16 port, bool initial);
+void guid_oob_get_addr_port(const guid_t *guid,
+	host_addr_t *addr, uint16 *port);
+
+struct hikset;
+const guid_t *guid_unique_atom(const struct hikset *hik, bool gtkg);
+
+void guid_free_atom2(void *guid, void *unused);
 
 #endif /* _core_guid_h_ */
+
+/* vi: set ts=4 sw=4 cindent: */
