@@ -1,6 +1,4 @@
 /*
- * $Id: hcache.h 16961 2009-09-07 19:36:42Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -42,6 +40,19 @@
 #include "if/core/hcache.h"
 #include "lib/gnet_host.h"
 
+/**
+ * Host cache addition notification structure.
+ *
+ * Anyone waiting on "hcache_add" will be notified when a new host is
+ * going to be added to the cache.  This structure defines the data given
+ * to the waiting callback.
+ */
+struct hcache_new_host {
+	hcache_type_t type;			/**< Type of cache to which host is added */
+	host_addr_t addr;			/**< Host address */
+	uint16 port;				/**< Host port */
+};
+
 /*
  * Global Functions
  */
@@ -51,39 +62,34 @@ void hcache_shutdown(void);
 void hcache_close(void);
 void hcache_retrieve_all(void);
 
-void hcache_timer(time_t now);
-
 const char *host_type_to_string(host_type_t type);
 const char *hcache_type_to_string(hcache_type_t type);
 
-gboolean hcache_add(
-    hcache_type_t type, const host_addr_t addr, guint16 port,
-    const char *what);
+bool hcache_add(
+    hcache_type_t type, const host_addr_t addr, uint16 port, const char *what);
 
-gboolean hcache_add_caught(
-    host_type_t type, const host_addr_t addr, guint16 port,
-    const char *what);
+bool hcache_add_caught(
+    host_type_t type, const host_addr_t addr, uint16 port, const char *what);
 
-gboolean hcache_add_valid(
-    host_type_t type, const host_addr_t addr, guint16 port,
-    const char *what);
+bool hcache_add_valid(
+    host_type_t type, const host_addr_t addr, uint16 port, const char *what);
 
-gboolean hcache_node_is_bad(const host_addr_t addr);
+bool hcache_node_is_bad(const host_addr_t addr);
+bool hcache_addr_within_net(const host_addr_t addr, host_net_t net);
 
 void hcache_prune(hcache_type_t type);
-void hcache_purge(const host_addr_t addr, guint16 port);
+void hcache_purge(hcache_class_t class, const host_addr_t addr, uint16 port);
 
-guint hcache_size(host_type_t type);
-gboolean hcache_is_low(host_type_t type);
+uint hcache_size(host_type_t type);
+bool hcache_is_low(host_type_t type);
 
 int hcache_fill_caught_array(
-	host_type_t type, gnet_host_t *hosts, int hcount);
+	host_net_t net, host_type_t type, gnet_host_t *hosts, int hcount);
 
-gboolean hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port);
-gboolean hcache_find_nearby(host_type_t type,
-	host_addr_t *addr, guint16 *port);
-
-void hcache_store_if_dirty(host_type_t type);
+bool hcache_get_caught(host_type_t type, host_addr_t *addr, uint16 *port);
+bool hcache_find_nearby(host_type_t type,
+	host_addr_t *addr, uint16 *port);
 
 #endif /* _core_hcache_h_ */
 
+/* vi: set ts=4 sw=4 cindent: */
