@@ -1,6 +1,4 @@
 /*
- * $Id: hosts.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,47 +33,61 @@
 
 #include "common.h"
 
-RCSID("$Id: hosts.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
-#include "sockets.h"
 #include "hosts.h"
-#include "nodes.h"
-#include "share.h"			/* For files_scanned and kbytes_scanned. */
-#include "routing.h"
-#include "gmsg.h"
-#include "pcache.h"
-#include "whitelist.h"
-#include "settings.h"
 #include "bogons.h"
+#include "gmsg.h"
+#include "hostiles.h"
+#include "nodes.h"
+#include "pcache.h"
+#include "routing.h"
+#include "settings.h"
+#include "share.h"			/* For files_scanned and kbytes_scanned. */
+#include "sockets.h"
+#include "udp.h"
 #include "uhc.h"
+#include "whitelist.h"
+
+#include "g2/gwc.h"
 
 #include "if/gnet_property_priv.h"
+#include "if/dht/dht.h"		/* For dht_fill_random() */
 
+#include "lib/aging.h"
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
 #include "lib/parse.h"
 #include "lib/stringify.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
+#include "lib/xmalloc.h"
 
 #include "lib/override.h"	/* Must be the last header included */
 
 #define HOST_PINGING_PERIOD		30		/**< Try pinging every 30 seconds */
+#define HOST_DHT_MAX			50		/**< Get that many random hosts */
+#define HOST_CONNECT_FREQ		120		/**< At most 1 connection per 2 mins */
 
-gboolean host_low_on_pongs = FALSE;		/**< True when less than 12% full */
+bool host_low_on_pongs = FALSE;		/**< True when less than 12% full */
 
-static gboolean in_shutdown = FALSE;
+static bool in_shutdown = FALSE;
+static aging_table_t *node_connects;
+
+#define HOST_GNET	0		/* Gnutella */
+#define HOST_G2		1		/* G2 */
 
 /*
  * Avoid nodes being stuck helplessly due to completely stale caches.
- * @return TRUE if an UHC may be contact, FALSE if it's not permissable.
+ * @return TRUE if an UHC may be contacted, FALSE if it's not permissable.
  */
-static gboolean
-host_cache_allow_bypass(void)
+static bool
+host_cache_allow_bypass(int which)
 {
-	static time_t last_try;
+	static time_t last_try[2];
+	uint count = which == HOST_GNET ? node_count() : node_g2_count();
 
-	if (node_count() > 0)
+	g_assert(UNSIGNED(which) < N_ITEMS(last_try));
+
+	if (count != 0)
 		return FALSE;
 
 	/* Wait at least 2 minutes after starting up */
@@ -87,10 +99,42 @@ host_cache_allow_bypass(void)
 	 * or downtime.
 	 */
 
-	if (last_try && delta_time(tm_time(), last_try) < 12 * 3600)
+	if (last_try[which] && delta_time(tm_time(), last_try[which]) < 12 * 3600)
 		return FALSE;
 
-	last_try = tm_time();
+	last_try[which] = tm_time();
+	return TRUE;
+}
+
+/**
+ * Attempt Gnutella host connection.
+ * @return TRUE if OK, FALSE if attempt was throttled
+ */
+static bool
+host_gnutella_connect(host_addr_t addr, uint16 port)
+{
+	if (aging_lookup(node_connects, &addr))
+		return FALSE;
+
+	node_add(addr, port, 0);
+	aging_record(node_connects, WCOPY(&addr));
+
+	return TRUE;
+}
+
+/**
+ * Attempt G2 host connection.
+ * @return TRUE if OK, FALSE if attempt was throttled
+ */
+static bool
+host_g2_connect(host_addr_t addr, uint16 port)
+{
+	if (aging_lookup(node_connects, &addr))
+		return FALSE;
+
+	node_g2_add(addr, port, 0);
+	aging_record(node_connects, WCOPY(&addr));
+
 	return TRUE;
 }
 
@@ -104,23 +148,31 @@ host_cache_allow_bypass(void)
 void
 host_timer(void)
 {
-    guint count;
-	int missing;
+	uint count, g2_count, max_nodes, max_g2_nodes;
+	int missing, g2_missing;
 	host_addr_t addr;
-	guint16 port;
+	uint16 port;
 	host_type_t htype;
-	guint max_nodes;
+	bool empty_cache = FALSE, empty_g2_cache = FALSE;
 
-	if (in_shutdown || !GNET_PROPERTY(online_mode))
+	if (
+		in_shutdown ||
+		!GNET_PROPERTY(online_mode) ||
+		GNET_PROPERTY(net_buffer_shortage)
+	)
 		return;
 
-	max_nodes = (GNET_PROPERTY(current_peermode) == NODE_P_LEAF) ?
-		GNET_PROPERTY(max_ultrapeers) : GNET_PROPERTY(max_connections);
-	count = node_count();
+	max_nodes = node_outdegree();	/* Gnutella, depending on leaf/ultra mode */
+	max_g2_nodes = GNET_PROPERTY(max_g2_hubs);
+	count = node_count();			/* Established + connecting */
+	g2_count = node_g2_count();		/* Established + connecting */
 	missing = node_keep_missing();
+	g2_missing = node_g2_hubs_missing();
 
-	if (GNET_PROPERTY(host_debug) > 1)
-		g_message("host_timer - count %u, missing %u", count, missing);
+	if (GNET_PROPERTY(host_debug) > 1) {
+		g_debug("%s(): count=%u, missing=%u, missing G2=%u",
+			G_STRFUNC, count, missing, g2_missing);
+	}
 
 	/*
 	 * If we are not connected to the Internet, apparently, make sure to
@@ -136,7 +188,7 @@ host_timer(void)
 		last_try = tm_time();
 
 		if (GNET_PROPERTY(host_debug))
-			g_message("host_timer - not connected, trying to connect");
+			g_debug("%s(): not connected, trying to connect", G_STRFUNC);
 	}
 
 	/*
@@ -145,30 +197,30 @@ host_timer(void)
 	 * than quick_connect_pool_size   This is the "greedy mode".
 	 */
 
-	if (count >= GNET_PROPERTY(quick_connect_pool_size)) {
-		if (GNET_PROPERTY(host_debug) > 1)
-			g_message("host_timer - count %u >= pool size %u",
-				count, GNET_PROPERTY(quick_connect_pool_size));
+	if (
+		(int) (count + g2_count - max_nodes - max_g2_nodes) >=
+			(int) GNET_PROPERTY(quick_connect_pool_size)
+	) {
+		if (GNET_PROPERTY(host_debug) > 1) {
+			g_debug("%s(): count %u + %u - %u - %u >= pool size %u",
+				G_STRFUNC, count, g2_count, max_nodes, max_g2_nodes,
+				GNET_PROPERTY(quick_connect_pool_size));
+		}
 		return;
 	}
 
 	if (count < max_nodes)
 		missing -= whitelist_connect();
 
-	if (GNET_PROPERTY(host_debug) && missing > 0)
-		g_message("host_timer - missing %d host%s",
-			missing, missing == 1 ? "" : "s");
-
 	/*
 	 * If we are under the number of connections wanted, we add hosts
 	 * to the connection list
 	 */
 
-	htype = (GNET_PROPERTY(current_peermode) == NODE_P_NORMAL) ?
-        HOST_ANY : HOST_ULTRA;
+	htype = HOST_ULTRA;
 
 	if (
-        GNET_PROPERTY(current_peermode) == NODE_P_ULTRA &&
+        settings_is_ultra() &&
         GNET_PROPERTY(node_normal_count) < GNET_PROPERTY(normal_connections) &&
         GNET_PROPERTY(node_ultra_count) >=
 			(GNET_PROPERTY(up_connections) - GNET_PROPERTY(normal_connections))
@@ -179,14 +231,31 @@ host_timer(void)
 	if (hcache_size(htype) == 0)
 		htype = HOST_ANY;
 
+	empty_cache = 0 == hcache_size(htype);
+	empty_g2_cache = 0 == hcache_size(HOST_G2HUB);
+
+	if (GNET_PROPERTY(host_debug)) {
+		if (missing > 0) {
+			g_debug("%s(): missing %d host%s%s", G_STRFUNC,
+				PLURAL(missing), empty_cache ? " [empty caches]" : "");
+		}
+
+		if (g2_missing > 0) {
+			g_debug("%s(): missing %d G2 host%s%s", G_STRFUNC,
+				PLURAL(g2_missing), empty_g2_cache ? " [empty caches]" : "");
+		}
+	}
+
+
     if (!GNET_PROPERTY(stop_host_get)) {
         if (missing > 0) {
 			static time_t last_try;
             unsigned fan, max_pool, to_add;
 
             max_pool = MAX(GNET_PROPERTY(quick_connect_pool_size), max_nodes);
-            fan = (missing * GNET_PROPERTY(quick_connect_pool_size))/ max_nodes;
-            to_add = GNET_PROPERTY(is_inet_connected) ? fan : (guint) missing;
+            fan = (missing * GNET_PROPERTY(quick_connect_pool_size))/ max_pool;
+			fan = MAX(1, fan);
+            to_add = GNET_PROPERTY(is_inet_connected) ? fan : (uint) missing;
 
 			/*
 			 * Every so many calls, attempt to ping all our neighbours to
@@ -210,31 +279,89 @@ host_timer(void)
                 to_add = max_pool - count;
 
             if (GNET_PROPERTY(host_debug) > 2) {
-                g_message("host_timer - connecting - "
+                g_debug("%s(): connecting - "
 					"add: %d fan:%d miss:%d max_hosts:%d count:%d extra:%d",
-					 to_add, fan, missing, max_nodes, count,
+					 G_STRFUNC, to_add, fan, missing, max_nodes, count,
 					 GNET_PROPERTY(quick_connect_pool_size));
             }
 
             missing = to_add;
 
-			while (hcache_size(htype) && missing-- > 0) {
-				if (hcache_get_caught(htype, &addr, &port)) {
-					node_add(addr, port, 0);
+			if (missing > 0 && (0 == connected_nodes() || host_low_on_pongs)) {
+				gnet_host_t host[HOST_DHT_MAX];
+				int hcount;
+				int i;
+
+				hcount = dht_fill_random(host,
+					MIN(UNSIGNED(missing), N_ITEMS(host)));
+
+				missing -= hcount;
+
+				for (i = 0; i < hcount; i++) {
+					addr = gnet_host_get_addr(&host[i]);
+					port = gnet_host_get_port(&host[i]);
+					if (!hcache_node_is_bad(addr)) {
+						if (GNET_PROPERTY(host_debug) > 3) {
+							g_debug("%s(): UHC pinging and connecting "
+								"to DHT node at %s", G_STRFUNC,
+								host_addr_port_to_string(addr, port));
+						}
+						/* Try to use the host as an UHC before connecting */
+						udp_send_ping(NULL, addr, port, TRUE);
+						if (!host_gnutella_connect(addr, port)) {
+							missing++;	/* Did not use entry */
+						}
+					} else {
+						missing++;	/* Did not use entry */
+					}
 				}
 			}
 
-			if (missing > 0 && host_cache_allow_bypass()) {
+			while (hcache_size(htype) && missing-- > 0) {
+				if (hcache_get_caught(htype, &addr, &port)) {
+					if (!(hostiles_is_bad(addr) || hcache_node_is_bad(addr))) {
+						if (!host_gnutella_connect(addr, port)) {
+							missing++;	/* Did not use entry */
+						}
+					} else {
+						missing++;	/* Did not use entry */
+					}
+				}
+			}
+
+			if (
+				missing > 0 &&
+				(empty_cache || host_cache_allow_bypass(HOST_GNET))
+			) {
 				if (!uhc_is_waiting()) {
 					if (GNET_PROPERTY(host_debug))
-						g_message("host_timer - querying UDP host cache");
+						g_debug("%s(): querying UDP host cache", G_STRFUNC);
 					uhc_get_hosts();	/* Get new hosts from UHCs */
-				} else if (
-					GNET_PROPERTY(bootstrap_debug) > 2 ||
-					GNET_PROPERTY(host_debug)
-				) {
-					g_message("BOOT host_timer - waiting for reply from %s",
-						uhc_is_waiting() ? "UDP host cache" : "web cache");
+				}
+			}
+		}
+
+		if (g2_missing > 0) {
+			while (hcache_size(HOST_G2HUB) && g2_missing-- > 0) {
+				if (hcache_get_caught(HOST_G2HUB, &addr, &port)) {
+					if (!(hostiles_is_bad(addr) || hcache_node_is_bad(addr))) {
+						if (!host_g2_connect(addr, port)) {
+							g2_missing++;	/* Did not use entry */
+						}
+					} else {
+						g2_missing++;	/* Did not use entry */
+					}
+				}
+			}
+
+			if (
+				g2_missing != 0 &&
+				(empty_g2_cache || host_cache_allow_bypass(HOST_G2))
+			) {
+				if (!gwc_is_waiting()) {
+					if (GNET_PROPERTY(host_debug))
+						g_debug("%s(): querying GWC for G2", G_STRFUNC);
+					gwc_get_hosts();	/* Get new hosts from a random GWC */
 				}
 			}
 		}
@@ -258,12 +385,14 @@ void
 host_init(void)
 {
 	pcache_init();
+	node_connects = aging_make(HOST_CONNECT_FREQ,
+		host_addr_hash_func, host_addr_eq_func, wfree_host_addr);
 }
 
 /**
  * Check whether host's address is usable: routable and not bogus.
  */
-gboolean
+bool
 host_address_is_usable(const host_addr_t addr)
 {
 	if (!host_addr_is_routable(addr))
@@ -281,8 +410,8 @@ host_address_is_usable(const host_addr_t addr)
  * i.e. that it has a valid port and that its IP address is not private
  * nor bogus.
  */
-gboolean
-host_is_valid(const host_addr_t addr, guint16 port)
+bool
+host_is_valid(const host_addr_t addr, uint16 port)
 {
 	if (!port_is_valid(port))
 		return FALSE;
@@ -296,7 +425,7 @@ host_is_valid(const host_addr_t addr, guint16 port)
  * When `connect' is true, attempt to connect if we are low in Gnet links.
  */
 void
-host_add(const host_addr_t addr, guint16 port, gboolean do_connect)
+host_add(const host_addr_t addr, uint16 port, bool do_connect)
 {
 	if (!do_connect || !hcache_add_caught(HOST_ANY, addr, port, "pong"))
 		return;
@@ -304,13 +433,6 @@ host_add(const host_addr_t addr, guint16 port, gboolean do_connect)
 	/*
 	 * If we are under the number of connections wanted, we add this host
 	 * to the connection list.
-	 *
-	 * Note: we're not using `node_count()' for the comparison with
-	 * `up_connections' but connected_nodes().	The node_add() routine also
-	 * compare `node_count' with `max_connections' to ensure we don't
-	 * launch too many connections, but comparing here as well may help
-	 * avoid useless call to connected_nodes() and/or node_add().
-	 *				--RAM, 20/09/2001
 	 */
 
 
@@ -321,7 +443,8 @@ host_add(const host_addr_t addr, guint16 port, gboolean do_connect)
 		host_is_nearby(addr) &&
 		node_remove_worst(TRUE)
 	) {
-		/* If we are above the max connections, delete a non-nearby
+		/*
+		 * If we are above the max connections, delete a non-nearby
 		 * connection before adding this better one
 		 */
 		node_add(addr, port, 0);
@@ -334,7 +457,7 @@ host_add(const host_addr_t addr, guint16 port, gboolean do_connect)
  * may be unsuitable for Gnet connections.
  */
 void
-host_add_semi_pong(const host_addr_t addr, guint16 port)
+host_add_semi_pong(const host_addr_t addr, uint16 port)
 {
 	g_assert(host_low_on_pongs);	/* Only used when low on pongs */
 
@@ -344,12 +467,12 @@ host_add_semi_pong(const host_addr_t addr, guint16 port)
 /* ---------- Netmask heuristic by Mike Perry -------- */
 struct network_pair
 {
-	guint32 mask;
-	guint32 net;
+	uint32 mask;
+	uint32 net;
 };
 
 struct network_pair *local_networks = NULL;
-guint32 number_local_networks;
+uint32 number_local_networks;
 
 /**
  * frees the local networks array
@@ -357,7 +480,7 @@ guint32 number_local_networks;
 static void
 free_networks(void)
 {
-	G_FREE_NULL(local_networks);
+	XFREE_NULL(local_networks);
 }
 
 /**
@@ -369,13 +492,13 @@ parse_netmasks(const char *str)
 {
 	char **masks = g_strsplit(str, ";", 0);
 	char *p;
-	guint32 mask_div;
+	uint32 mask_div;
 	int i;
 
 	free_networks();
 
-    if (!masks)
-        return;
+	if (!masks)
+		return;
 
 	for (i = 0; masks[i]; i++)
 		/* just count */ ;
@@ -387,17 +510,17 @@ parse_netmasks(const char *str)
 		return;
     }
 
-	local_networks = g_malloc(i * sizeof *local_networks);
+	XMALLOC_ARRAY(local_networks, i);
 
 	for (i = 0; masks[i]; i++) {
 		/* Network is of the form ip/mask or ip/bits */
-		if ((p = strchr(masks[i], '/')) && *p) {
+		if ((p = vstrchr(masks[i], '/')) && *p) {
 			*p++ = '\0';
 
-			if (strchr(p, '.')) {
+			if (vstrchr(p, '.')) {
 				/* get the network address from the user */
 				if (!string_to_ip_strict(p, &local_networks[i].mask, NULL))
-					g_warning("parse_netmasks(): Invalid netmask: \"%s\"", p);
+					g_warning("%s(): invalid netmask: \"%s\"", G_STRFUNC, p);
 			}
 			else {
 				int error;
@@ -405,10 +528,10 @@ parse_netmasks(const char *str)
 				mask_div = parse_uint32(p, NULL, 10, &error);
 				mask_div = MIN(32, mask_div);
 				if (error)
-					g_warning("parse_netmasks(): "
-						"Invalid CIDR prefixlen: \"%s\"", p);
+					g_warning("%s(): invalid CIDR prefixlen: \"%s\"",
+						G_STRFUNC, p);
 				else
-					local_networks[i].mask = (guint32) -1 << (32 - mask_div);
+					local_networks[i].mask = (uint32) -1 << (32 - mask_div);
 			}
 		}
 		else {
@@ -417,7 +540,7 @@ parse_netmasks(const char *str)
 		}
 		/* get the network address from the user */
 		if (!string_to_ip_strict(masks[i], &local_networks[i].net, NULL))
-			g_warning("parse_netmasks(): Invalid netmask: \"%s\"", masks[i]);
+			g_warning("%s(): invalid netmask: \"%s\"", G_STRFUNC, masks[i]);
 	}
 
 	g_strfreev(masks);
@@ -426,20 +549,20 @@ parse_netmasks(const char *str)
 /**
  * @returns true if the address is inside one of the local networks
  */
-gboolean
+bool
 host_is_nearby(const host_addr_t addr)
 {
-	guint i;
+	uint i;
 
-	if (NET_TYPE_IPV4 == host_addr_net(addr)) {
+	if (host_addr_is_ipv4(addr)) {
 		for (i = 0; i < number_local_networks; i++) {
-			guint32 m_mask = local_networks[i].mask;
-			guint32 m_ip = local_networks[i].net;
+			uint32 m_mask = local_networks[i].mask;
+			uint32 m_ip = local_networks[i].net;
 
 			if ((host_addr_ipv4(addr) & m_mask) == (m_ip & m_mask))
 				return TRUE;
 		}
-	} else if (NET_TYPE_IPV6 == host_addr_net(addr)) {
+	} else if (host_addr_is_ipv6(addr)) {
 		/* XXX: Implement this! */
 	}
 	return FALSE;
@@ -462,6 +585,7 @@ host_close(void)
 {
 	pcache_close();
 	free_networks();
+	aging_destroy(&node_connects);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
