@@ -1,6 +1,4 @@
 /*
- * $Id: hsep.c 17220 2009-11-10 18:48:45Z rmanfredi $
- *
  * Copyright (c) 2004, Thomas Schuerger & Jeroen Asselman
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -95,25 +93,28 @@
 
 #include "common.h"
 
-RCSID("$Id: hsep.c 17220 2009-11-10 18:48:45Z rmanfredi $")
-
-#include "gmsg.h"
-#include "routing.h"
-#include "nodes.h"
 #include "hsep.h"
-#include "uploads.h"
-#include "share.h"
+
 #include "features.h"
+#include "gmsg.h"
+#include "nodes.h"
+#include "routing.h"
+#include "settings.h"
+#include "share.h"
+#include "uploads.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
+#include "lib/cstr.h"
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
+#include "lib/override.h"
+#include "lib/pslist.h"
+#include "lib/random.h"
 #include "lib/stringify.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
-#include "lib/override.h"
 
 /** global HSEP table */
 static hsep_triple hsep_global_table[HSEP_N_MAX + 1];
@@ -142,7 +143,7 @@ hsep_fire_global_table_changed(time_t now)
 	/* do nothing if we don't have any listeners */
 
 	if (event_subscriber_active(hsep_global_table_changed_event)) {
-		hsep_triple table[G_N_ELEMENTS(hsep_global_table)];
+		hsep_triple table[N_ITEMS(hsep_global_table)];
 
 		/*
 		 * Make a copy of the global HSEP table and give that
@@ -150,10 +151,10 @@ hsep_fire_global_table_changed(time_t now)
 		 * listeners.
 		 */
 
-		hsep_get_global_table(table, G_N_ELEMENTS(table));
+		hsep_get_global_table(table, N_ITEMS(table));
 
 		event_trigger(hsep_global_table_changed_event,
-		    T_NORMAL(hsep_global_listener_t, (table, G_N_ELEMENTS(table))));
+		    T_NORMAL(hsep_global_listener_t, (table, N_ITEMS(table))));
 	}
 }
 
@@ -162,18 +163,18 @@ hsep_fire_global_table_changed(time_t now)
  * triple is given. Returns TRUE if monotony is ok, FALSE otherwise.
  */
 
-static gboolean
+static bool
 hsep_check_monotony(hsep_triple *table, unsigned int triples)
 {
-	gboolean error = FALSE;
-	guint i, j;
+	bool error = FALSE;
+	uint i, j;
 
 	g_assert(table);
 
 	for (i = 1; i < triples; i++) {
 
 		/* if any triple is not >= the previous one, error will be TRUE */
-		for (j = 0; j < G_N_ELEMENTS(table[0]); j++)
+		for (j = 0; j < N_ITEMS(table[0]); j++)
 			error |= table[i - 1][j] > table[i][j];
 
 		if (error)
@@ -202,11 +203,11 @@ hsep_check_monotony(hsep_triple *table, unsigned int triples)
 static void
 hsep_sanity_check(void)
 {
-	const GSList *sl;
-	hsep_triple sum[G_N_ELEMENTS(hsep_global_table)];
+	const pslist_t *sl;
+	hsep_triple sum[N_ITEMS(hsep_global_table)];
 	unsigned int i, j;
 
-	memset(sum, 0, sizeof sum);
+	ZERO(&sum);
 
 	g_assert(1 == hsep_own[HSEP_IDX_NODES]);
 
@@ -215,8 +216,10 @@ hsep_sanity_check(void)
 	 * sum up all the connections' triple values.
 	 */
 
-	for (sl = node_all_nodes() ; sl; sl = g_slist_next(sl)) {
-		struct gnutella_node *n = sl->data;
+	PSLIST_FOREACH(node_all_gnet_nodes(), sl) {
+		gnutella_node_t *n = sl->data;
+
+		node_check(n);
 
 		/* also consider unestablished connections here */
 
@@ -230,22 +233,22 @@ hsep_sanity_check(void)
 
 		/* check if values are monotonously increasing (skip first) */
 		g_assert(
-			hsep_check_monotony(cast_to_gpointer(n->hsep->table[1]),
-				G_N_ELEMENTS(n->hsep->table[1]) - 1)
+			hsep_check_monotony(cast_to_pointer(n->hsep->table[1]),
+				N_ITEMS(n->hsep->table[1]) - 1)
 		);
 
 		/* sum up the values */
 
-		for (i = 0; i < G_N_ELEMENTS(sum); i++) {
-			for (j = 0; j < G_N_ELEMENTS(sum[0]); j++)
+		for (i = 0; i < N_ITEMS(sum); i++) {
+			for (j = 0; j < N_ITEMS(sum[0]); j++)
 				sum[i][j] += n->hsep->table[i][j];
 		}
 	}
 
 	/* check sums */
 
-	for (i = 0; i < G_N_ELEMENTS(sum); i++) {
-		for (j = 0; j < G_N_ELEMENTS(sum[0]); j++)
+	for (i = 0; i < N_ITEMS(sum); i++) {
+		for (j = 0; j < N_ITEMS(sum[0]); j++)
 			g_assert(hsep_global_table[i][j] == sum[i][j]);
 	}
 }
@@ -261,17 +264,17 @@ hsep_dump_table(void)
 
 	printf("HSEP: Reachable nodes (1-%d hops): ", HSEP_N_MAX);
 
-	for (i = 1; i < G_N_ELEMENTS(hsep_global_table); i++)
+	for (i = 1; i < N_ITEMS(hsep_global_table); i++)
 		printf("%s ", uint64_to_string(hsep_global_table[i][HSEP_IDX_NODES]));
 
 	printf("\nHSEP: Reachable files (1-%d hops): ", HSEP_N_MAX);
 
-	for (i = 1; i < G_N_ELEMENTS(hsep_global_table); i++)
+	for (i = 1; i < N_ITEMS(hsep_global_table); i++)
 		printf("%s ", uint64_to_string(hsep_global_table[i][HSEP_IDX_FILES]));
 
 	printf("\nHSEP:   Reachable KiB (1-%d hops): ", HSEP_N_MAX);
 
-	for (i = 1; i < G_N_ELEMENTS(hsep_global_table); i++)
+	for (i = 1; i < N_ITEMS(hsep_global_table); i++)
 		printf("%s ", uint64_to_string(hsep_global_table[i][HSEP_IDX_KIB]));
 
 	printf("\n");
@@ -297,8 +300,8 @@ hsep_dump_table(void)
 static unsigned int
 hsep_triples_to_send(const hsep_triple *table, unsigned int triples)
 {
-	guint i, j, last;
-	gboolean changed = FALSE;
+	uint i, j, last;
+	bool changed = FALSE;
 
 	g_assert(table);
 
@@ -311,7 +314,7 @@ hsep_triples_to_send(const hsep_triple *table, unsigned int triples)
 
 	for (i = last; i-- > 0; triples--) {
 
-		for (j = 0; j < G_N_ELEMENTS(table[0]); j++)
+		for (j = 0; j < N_ITEMS(table[0]); j++)
 	   		changed |= table[i][j] != table[last][j];
 
 		if (changed)
@@ -326,7 +329,7 @@ hsep_triples_to_send(const hsep_triple *table, unsigned int triples)
  * Initializes HSEP.
  */
 
-void
+void G_COLD
 hsep_init(void)
 {
 	header_features_add(FEATURES_CONNECTIONS,
@@ -344,9 +347,9 @@ hsep_init(void)
  */
 
 void
-hsep_add_global_table_listener(GCallback cb, frequency_t t, guint32 interval)
+hsep_add_global_table_listener(callback_fn_t cb, frequency_t t, uint32 interval)
 {
-	hsep_triple table[G_N_ELEMENTS(hsep_global_table)];
+	hsep_triple table[N_ITEMS(hsep_global_table)];
 	hsep_global_listener_t func = (hsep_global_listener_t) cb;
 
 
@@ -360,12 +363,12 @@ hsep_add_global_table_listener(GCallback cb, frequency_t t, guint32 interval)
 	 * given callback call constraints.
 	 */
 
-	hsep_get_global_table(table, G_N_ELEMENTS(table));
-	func(table, G_N_ELEMENTS(table));
+	hsep_get_global_table(table, N_ITEMS(table));
+	func(table, N_ITEMS(table));
 }
 
 void
-hsep_remove_global_table_listener(GCallback cb)
+hsep_remove_global_table_listener(callback_fn_t cb)
 {
 	event_remove_subscriber(hsep_global_table_changed_event, cb);
 }
@@ -383,13 +386,15 @@ hsep_remove_global_table_listener(GCallback cb)
 void
 hsep_reset(void)
 {
-	const GSList *sl;
-	guint i;
+	const pslist_t *sl;
+	uint i;
 
-	memset(hsep_global_table, 0, sizeof hsep_global_table);
+	ZERO(&hsep_global_table);
 
-	for (sl = node_all_nodes(); sl; sl = g_slist_next(sl)) {
-		struct gnutella_node *n = sl->data;
+	PSLIST_FOREACH(node_all_gnet_nodes(), sl) {
+		gnutella_node_t *n = sl->data;
+
+		node_check(n);
 
 		/* also consider unestablished connections here */
 
@@ -398,12 +403,12 @@ hsep_reset(void)
 
 		g_assert(n->hsep);
 
-		memset(n->hsep->table, 0, sizeof n->hsep->table);
-		memset(n->hsep->sent_table, 0, sizeof n->hsep->sent_table);
+		ZERO(&n->hsep->table);
+		ZERO(&n->hsep->sent_table);
 
 		/* this is what we know before receiving the first message */
 
-		for (i = 1; i < G_N_ELEMENTS(hsep_global_table); i++) {
+		for (i = 1; i < N_ITEMS(hsep_global_table); i++) {
 			n->hsep->table[i][HSEP_IDX_NODES] = 1;
 			hsep_global_table[i][HSEP_IDX_NODES]++;
 		}
@@ -422,11 +427,10 @@ hsep_reset(void)
  */
 
 void
-hsep_connection_init(struct gnutella_node *n, guint8 major, guint8 minor)
+hsep_connection_init(gnutella_node_t *n, uint8 major, uint8 minor)
 {
-	static const hsep_ctx_t zero_hsep;
 	time_t now = tm_time();
-	guint i;
+	uint i;
 
 	g_assert(n);
 
@@ -434,15 +438,14 @@ hsep_connection_init(struct gnutella_node *n, guint8 major, guint8 minor)
 		printf("HSEP: Initializing node %s\n",
 			host_addr_port_to_string(n->addr, n->port));
 
-	n->hsep = walloc(sizeof *n->hsep);
-	*n->hsep = zero_hsep; /* Initializes everything to 0 */
+	WALLOC0(n->hsep);
 	n->hsep->last_sent = now;
 	n->hsep->major = major;
 	n->hsep->minor = minor;
 
 	/* this is what we know before receiving the first message */
 
-	for (i = 1; i < G_N_ELEMENTS(hsep_global_table); i++) {
+	for (i = 1; i < N_ITEMS(hsep_global_table); i++) {
 		n->hsep->table[i][HSEP_IDX_NODES] = 1;
 		hsep_global_table[i][HSEP_IDX_NODES]++;
 	}
@@ -462,8 +465,8 @@ hsep_connection_init(struct gnutella_node *n, guint8 major, guint8 minor)
 void
 hsep_timer(time_t now)
 {
-	const GSList *sl;
-	gboolean scanning_shared;
+	const pslist_t *sl;
+	bool scanning_shared;
 	static time_t last_sent = 0;
 
 	/* update number of shared files and KiB */
@@ -477,10 +480,12 @@ hsep_timer(time_t now)
 			hsep_notify_shared(0UL, 0UL);
 	}
 
-	for (sl = node_all_nodes(); sl; sl = g_slist_next(sl)) {
-		struct gnutella_node *n = sl->data;
+	PSLIST_FOREACH(node_all_gnet_nodes(), sl) {
+		gnutella_node_t *n = sl->data;
 		int diff;
 
+		node_check(n);
+
 		/* only consider established connections here */
 		if (!NODE_IS_ESTABLISHED(n))
 			continue;
@@ -516,7 +521,7 @@ hsep_timer(time_t now)
  * zero and the CAN_HSEP attribute is cleared.
  */
 void
-hsep_connection_close(struct gnutella_node *n, gboolean in_shutdown)
+hsep_connection_close(gnutella_node_t *n, bool in_shutdown)
 {
 	unsigned int i, j;
 
@@ -530,9 +535,9 @@ hsep_connection_close(struct gnutella_node *n, gboolean in_shutdown)
 	if (in_shutdown)
 		goto cleanup;
 
-	for (i = 1; i < G_N_ELEMENTS(hsep_global_table); i++) {
+	for (i = 1; i < N_ITEMS(hsep_global_table); i++) {
 
-		for (j = 0; j < G_N_ELEMENTS(hsep_global_table[0]); j++) {
+		for (j = 0; j < N_ITEMS(hsep_global_table[0]); j++) {
 			hsep_global_table[i][j] -= n->hsep->table[i][j];
 			n->hsep->table[i][j] = 0;
 		}
@@ -550,14 +555,14 @@ hsep_connection_close(struct gnutella_node *n, gboolean in_shutdown)
 
 cleanup:
 	n->attrs &= ~NODE_A_CAN_HSEP;
-	wfree(n->hsep, sizeof *n->hsep);
+	WFREE(n->hsep);
 	n->hsep = NULL;
 }
 
 static inline void
 hsep_fix_endian(hsep_triple *messaget, size_t n)
 {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#if IS_LITTLE_ENDIAN
 	(void) messaget;
 	(void) n;
 #else
@@ -570,11 +575,11 @@ hsep_fix_endian(hsep_triple *messaget, size_t n)
 	 */
 
 	for (i = 0; i < n; i++) {
-		for (j = 0; j < G_N_ELEMENTS(messaget[0]); j++) {
+		for (j = 0; j < N_ITEMS(messaget[0]); j++) {
 			poke_le64(&messaget[i][j], messaget[i][j]);
 		}
 	}
-#endif	/* LITTLE ENDIAN */
+#endif	/* IS_LITTLE_ENDIAN */
 }
 
 /**
@@ -583,7 +588,7 @@ hsep_fix_endian(hsep_triple *messaget, size_t n)
  */
 
 void
-hsep_process_msg(struct gnutella_node *n, time_t now)
+hsep_process_msg(gnutella_node_t *n, time_t now)
 {
 	unsigned int i, j, k, max, msgmax, length;
 	hsep_triple *messaget;
@@ -597,7 +602,7 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 
 	/* note the offset between message and local data by 1 triple */
 
-	messaget = cast_to_gpointer(n->data);
+	messaget = cast_to_pointer(n->data);
 
 	if (length == 0) {   /* error, at least 1 triple must be present */
 		if (GNET_PROPERTY(hsep_debug) > 1)
@@ -651,8 +656,8 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	}
 
 	if (GNET_PROPERTY(hsep_debug) > 1) {
-		printf("HSEP: Received %d %s from node %s (msg #%u): ", max,
-		    max == 1 ? "triple" : "triples",
+		printf("HSEP: Received %d triple%s from node %s (msg #%u): ",
+			PLURAL(max),
 			host_addr_port_to_string(n->addr, n->port),
 			hsep->msgs_received + 1);
 	}
@@ -664,16 +669,16 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	for (k = 0, i = 1; k < max; k++, i++) {
 
 		if (GNET_PROPERTY(hsep_debug) > 1) {
-			char buf[G_N_ELEMENTS(messaget[0])][32];
+			char buf[N_ITEMS(messaget[0])][32];
 
-			for (j = 0; j < G_N_ELEMENTS(buf); j++)
+			for (j = 0; j < N_ITEMS(buf); j++)
 				uint64_to_string_buf(messaget[k][j], buf[j], sizeof buf[0]);
 
-			STATIC_ASSERT(3 == G_N_ELEMENTS(buf));
+			STATIC_ASSERT(3 == N_ITEMS(buf));
 			printf("(%s, %s, %s) ", buf[0], buf[1], buf[2]);
 		}
 
-		for (j = 0; j < G_N_ELEMENTS(hsep_global_table[0]); j++) {
+		for (j = 0; j < N_ITEMS(hsep_global_table[0]); j++) {
 			hsep_global_table[i][j] += messaget[k][j] - hsep->table[i][j];
 			hsep->table[i][j] = messaget[k][j];
 		}
@@ -691,9 +696,9 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	if (k > 0)
 		k--;
 
-	for (/* NOTHING */; i < G_N_ELEMENTS(hsep_global_table); i++) {
+	for (/* NOTHING */; i < N_ITEMS(hsep_global_table); i++) {
 
-		for (j = 0; j < G_N_ELEMENTS(hsep_global_table[0]); j++) {
+		for (j = 0; j < N_ITEMS(hsep_global_table[0]); j++) {
 			hsep_global_table[i][j] += messaget[k][j] - hsep->table[i][j];
 			hsep->table[i][j] = messaget[k][j];
 		}
@@ -722,9 +727,9 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
  */
 
 void
-hsep_send_msg(struct gnutella_node *n, time_t now)
+hsep_send_msg(gnutella_node_t *n, time_t now)
 {
-	hsep_triple tmp[G_N_ELEMENTS(n->hsep->sent_table)], other;
+	hsep_triple tmp[N_ITEMS(n->hsep->sent_table)], other;
 	unsigned int i, j, msglen, msgsize, triples, opttriples;
 	gnutella_msg_hsep_t *msg;
 	hsep_ctx_t *hsep;
@@ -733,6 +738,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	g_assert(n->hsep);
 
 	hsep = n->hsep;
+	ZERO(&other);
 
 	/*
 	 * If we are a leaf, we just need to send one triple,
@@ -742,9 +748,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	 * it contains only our own triple, which is correct.
 	 */
 
-	triples = NODE_P_LEAF == GNET_PROPERTY(current_peermode)
-				? 1
-				: G_N_ELEMENTS(tmp);
+	triples = settings_is_leaf() ? 1 : N_ITEMS(tmp);
 
 	/*
 	 * Allocate and initialize message to send.
@@ -755,7 +759,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 
 	{
 		gnutella_header_t *header;
-		
+
 		header = gnutella_msg_hsep_header(msg);
 		message_set_muid(header, GTA_MSG_HSEP_DATA);
 		gnutella_header_set_function(header, GTA_MSG_HSEP_DATA);
@@ -774,8 +778,8 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	}
 
 	for (i = 0; i < triples; i++) {
-		for (j = 0; j < G_N_ELEMENTS(other); j++) {
-			guint64 val;
+		for (j = 0; j < N_ITEMS(other); j++) {
+			uint64 val;
 
 			val = hsep_own[j] + (0 == i ? 0 : other[j]) +
 				hsep_global_table[i][j] - hsep->table[i][j];
@@ -792,7 +796,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 		goto charge_timer;
 	}
 
-	memcpy(cast_to_gchar_ptr(msg) + GTA_HEADER_SIZE,
+	memcpy(cast_to_char_ptr(msg) + GTA_HEADER_SIZE,
 		tmp, triples * sizeof tmp[0]);
 
 	/* store the table for later comparison */
@@ -805,27 +809,27 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	 */
 
 	/* optimize number of triples to send */
-	opttriples = hsep_triples_to_send(cast_to_gpointer(tmp), triples);
+	opttriples = hsep_triples_to_send(cast_to_pointer(tmp), triples);
 
 	if (GNET_PROPERTY(hsep_debug) > 1) {
-		printf("HSEP: Sending %d %s to node %s (msg #%u): ", opttriples,
-		    opttriples == 1 ? "triple" : "triples",
+		printf("HSEP: Sending %d triple%s to node %s (msg #%u): ",
+			PLURAL(opttriples),
 			host_addr_port_to_string(n->addr, n->port),
 			hsep->msgs_sent + 1);
 	}
 
 	for (i = 0; i < opttriples; i++) {
 		if (GNET_PROPERTY(hsep_debug) > 1) {
-			char buf[G_N_ELEMENTS(hsep_own)][32];
+			char buf[N_ITEMS(hsep_own)][32];
 
-			for (j = 0; j < G_N_ELEMENTS(buf); j++) {
-				guint64 v;
+			for (j = 0; j < N_ITEMS(buf); j++) {
+				uint64 v;
 
 				v = hsep_own[j] + hsep_global_table[i][j] - hsep->table[i][j];
 				uint64_to_string_buf(v, buf[j], sizeof buf[0]);
 			}
 
-			STATIC_ASSERT(3 == G_N_ELEMENTS(buf));
+			STATIC_ASSERT(3 == N_ITEMS(buf));
 			printf("(%s, %s, %s) ", buf[0], buf[1], buf[2]);
 		}
 	}
@@ -866,7 +870,7 @@ charge_timer:
  */
 
 void
-hsep_notify_shared(guint64 own_files, guint64 own_kibibytes)
+hsep_notify_shared(uint64 own_files, uint64 own_kibibytes)
 {
 	/* check for change */
 	if (
@@ -875,7 +879,7 @@ hsep_notify_shared(guint64 own_files, guint64 own_kibibytes)
 	) {
 
 		if (GNET_PROPERTY(hsep_debug)) {
-			g_message("HSEP: Shared files changed to %s (%s KiB)",
+			g_debug("HSEP: Shared files changed to %s (%s KiB)",
 			    uint64_to_string(own_files), uint64_to_string2(own_kibibytes));
 		}
 
@@ -904,7 +908,7 @@ hsep_get_global_table(hsep_triple *buffer, unsigned int maxtriples)
 {
 	g_assert(buffer);
 
-	maxtriples = MIN(maxtriples, G_N_ELEMENTS(hsep_global_table));
+	maxtriples = MIN(maxtriples, N_ITEMS(hsep_global_table));
 	memcpy(buffer, hsep_global_table, maxtriples * sizeof buffer[0]);
 
 	return maxtriples;
@@ -920,14 +924,14 @@ hsep_get_global_table(hsep_triple *buffer, unsigned int maxtriples)
  */
 
 unsigned int
-hsep_get_connection_table(const struct gnutella_node *n,
+hsep_get_connection_table(const gnutella_node_t *n,
     hsep_triple *buffer, unsigned int maxtriples)
 {
 	g_assert(n);
 	g_assert(n->hsep);
 	g_assert(buffer);
 
-	maxtriples = MIN(maxtriples, G_N_ELEMENTS(n->hsep->table));
+	maxtriples = MIN(maxtriples, N_ITEMS(n->hsep->table));
 	memcpy(buffer, n->hsep->table, maxtriples * sizeof buffer[0]);
 
 	return maxtriples;
@@ -950,7 +954,7 @@ hsep_close(void)
  * FALSE otherwise.
  */
 
-gboolean
+bool
 hsep_has_global_table_changed(time_t since)
 {
 	return delta_time(hsep_last_global_table_change, since) > 0;
@@ -972,10 +976,10 @@ hsep_has_global_table_changed(time_t since)
 void
 hsep_get_non_hsep_triple(hsep_triple *tripledest)
 {
-	const GSList *sl;
-	guint64 other_nodes = 0;      /* # of non-HSEP nodes */
-	guint64 other_files = 0;      /* what non-HSEP nodes share (files) */
-	guint64 other_kib = 0;        /* what non-HSEP nodes share (KiB) */
+	const pslist_t *sl;
+	uint64 other_nodes = 0;      /* # of non-HSEP nodes */
+	uint64 other_files = 0;      /* what non-HSEP nodes share (files) */
+	uint64 other_kib = 0;        /* what non-HSEP nodes share (KiB) */
 
 	g_assert(tripledest);
 
@@ -984,10 +988,12 @@ hsep_get_non_hsep_triple(hsep_triple *tripledest)
 	 * sum up what they share (PONG-based library size).
 	 */
 
-	for (sl = node_all_nodes() ; sl; sl = g_slist_next(sl)) {
-		struct gnutella_node *n = sl->data;
+	PSLIST_FOREACH(node_all_gnet_nodes(), sl) {
+		gnutella_node_t *n = sl->data;
 		gnet_node_status_t status;
 
+		node_check(n);
+
 		if ((!NODE_IS_ESTABLISHED(n)) || n->attrs & NODE_A_CAN_HSEP)
 			continue;
 
@@ -1018,11 +1024,11 @@ const char *
 hsep_get_static_str(int row, int column)
 {
 	const char *ret = NULL;
-	hsep_triple hsep_table[G_N_ELEMENTS(hsep_global_table)];
+	hsep_triple hsep_table[N_ITEMS(hsep_global_table)];
 	hsep_triple other[1];
-	guint64 v;
+	uint64 v;
 
-	hsep_get_global_table(hsep_table, G_N_ELEMENTS(hsep_table));
+	hsep_get_global_table(hsep_table, N_ITEMS(hsep_table));
 	hsep_get_non_hsep_triple(other);
 
     switch (column) {
@@ -1031,7 +1037,7 @@ hsep_get_static_str(int row, int column)
 			static char buf[UINT64_DEC_BUFLEN];
 
 			v = hsep_table[row][HSEP_IDX_NODES] + other[0][HSEP_IDX_NODES];
-			uint64_to_string_buf(v, buf, sizeof buf);
+			uint64_to_string_buf(v, ARYLEN(buf));
 			ret = buf;
 		}
 		break;
@@ -1041,7 +1047,7 @@ hsep_get_static_str(int row, int column)
 			static char buf[UINT64_DEC_BUFLEN];
 
 			v = hsep_table[row][HSEP_IDX_FILES] + other[0][HSEP_IDX_FILES];
-			uint64_to_string_buf(v, buf, sizeof buf);
+			uint64_to_string_buf(v, ARYLEN(buf));
 			ret = buf;
 		}
 		break;
@@ -1053,9 +1059,8 @@ hsep_get_static_str(int row, int column)
 			/* Make a copy because concurrent usage of short_kb_size()
 	 	 	 * could be hard to discover. */
 			v = hsep_table[row][HSEP_IDX_KIB] + other[0][HSEP_IDX_KIB];
-			g_strlcpy(buf,
-				short_kb_size(v, GNET_PROPERTY(display_metric_units)),
-				sizeof buf);
+			cstr_bcpy(ARYLEN(buf),
+				short_kb_size(v, GNET_PROPERTY(display_metric_units)));
   			ret = buf;
 		}
 		break;
@@ -1071,10 +1076,10 @@ hsep_get_static_str(int row, int column)
 int
 hsep_get_table_size(void)
 {
-	hsep_triple hsep_table[G_N_ELEMENTS(hsep_global_table)];
+	hsep_triple hsep_table[N_ITEMS(hsep_global_table)];
 
-	hsep_get_global_table(hsep_table, G_N_ELEMENTS(hsep_table));
-	return G_N_ELEMENTS(hsep_table);
+	hsep_get_global_table(hsep_table, N_ITEMS(hsep_table));
+	return N_ITEMS(hsep_table);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
