@@ -1,6 +1,4 @@
 /*
- * $Id: huge.h 17205 2009-11-08 21:01:48Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Ch. Tronche & Raphael Manfredi
  *
  * Started by Ch. Tronche (http://tronche.com/) 28/04/2002
@@ -21,7 +19,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -43,6 +41,7 @@
 #include "common.h"
 
 struct gnutella_node;
+struct gnutella_host;
 struct shared_file;
 struct header;
 struct sha1;
@@ -58,18 +57,22 @@ void huge_close(void);		/**< Call this when servent is shutdown */
 
 struct tth;
 
-gboolean request_sha1(struct shared_file *);
-gboolean sha1_is_cached(const struct shared_file *sf);
-gboolean huge_update_hashes(struct shared_file *sf,
+void request_sha1(struct shared_file *);
+bool sha1_is_cached(const struct shared_file *sf);
+bool huge_update_hashes(struct shared_file *sf,
 	const struct sha1 *sha1, const struct tth *tth);
 
-gboolean huge_improbable_sha1(const char *buf, size_t len);
-gboolean huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
+bool huge_improbable_sha1(const char *buf, size_t len);
+bool huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
 	const struct gnutella_node *n);
-gboolean huge_tth_extract32(const char *buf, size_t len, struct tth *tth,
+bool huge_tth_extract32(const char *buf, size_t len, struct tth *tth,
 	const struct gnutella_node *n);
 void huge_collect_locations(
-	const struct sha1 *sha1, const struct header *header);
+	const struct sha1 *sha1, const struct header *header,
+	const struct gnutella_host *origin);
+bool huge_cached_is_uptodate(const char *path, filesize_t size, time_t mtime);
+
+void huge_sha1_cache_prune(void);
 
 #endif	/* _core_huge_h_ */
 
@@ -82,3 +85,4 @@ void huge_collect_locations(
  * indent-tabs-mode: nil ***
  * End: ***
  */
+/* vi: set ts=4 sw=4 cindent: */
