@@ -1,6 +1,4 @@
 /*
- * $Id: inet.c 16687 2009-05-07 19:28:22Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,8 +33,6 @@
 
 #include "common.h"
 
-RCSID("$Id: inet.c 16687 2009-05-07 19:28:22Z rmanfredi $")
-
 #include "inet.h"
 #include "nodes.h"				/* For node_become_firewalled() */
 #include "sockets.h"
@@ -47,11 +43,22 @@ RCSID("$Id: inet.c 16687 2009-05-07 19:28:22Z rmanfredi $")
 
 #include "lib/aging.h"
 #include "lib/cq.h"
+#include "lib/stacktrace.h"
+#include "lib/str.h"
 #include "lib/tm.h"
+#include "lib/unsigned.h"
 #include "lib/wd.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
+/***
+ *** IP network buffer shortage.
+ ***/
+
+#define BUF_SHORTAGE_WINDOW		90		/**< IP buffer shortage monitoring */
+
+static watchdog_t *buf_shortage_wd;
+
 /***
  *** Firewall status management structures.
  ***/
@@ -66,8 +73,8 @@ RCSID("$Id: inet.c 16687 2009-05-07 19:28:22Z rmanfredi $")
 #define FW_STARTUP_GRACE		300		/**< Startup period: we send pongs */
 #define FW_GRACE_INTERVAL		3600	/**< Every hour, new grace period */
 #define FW_PERIODIC_GRACE		120		/**< We send pongs for 2 minutes */
-#define FW_INCOMING_WINDOW		3600	/**< Incoming monitoring window */
-#define FW_SOLICITED_WINDOW		3600	/**< Solicited UDP monitoring window */
+#define FW_INCOMING_WINDOW		1800	/**< Incoming monitoring window */
+#define FW_SOLICITED_WINDOW		1800	/**< Solicited UDP monitoring window */
 
 static time_t fw_time = 0;				/**< When we last became firewalled */
 
@@ -93,6 +100,7 @@ static watchdog_t *solicited_udp_wd;
  */
 
 #define FW_UDP_WINDOW			120		/**< 2 minutes, in most firewalls */
+#define FW_UDP_MAX_PERIODS		8		/**< Maximum # of delay periods */
 
 static aging_table_t *outgoing_udp;		/**< IP addresses to whom we send */
 
@@ -104,14 +112,27 @@ struct ip_record {
 /**
  * States of our small automaton to try to detect reception of unsolicited
  * messages if we do not get them naturally: there are only a few such messages
- * that can only be viewed as unsolicited.
+ * that can truly be viewed as unsolicited.
  *
- * SO if we don't get these, we enter the mode UNSOLICITED_PREPARE in which
+ * So if we don't get these, we enter the UNSOLICITED_PREPARE mode in which
  * we just record the addresses of whom we send things to.  We ignore any
  * unsolicited checks for non-matching addresses at this point.  After
  * FW_UDP_WINDOW seconds have elapsed, we enter UNSOLICITED_CHECK and the
  * first time we get an unsolicited packet, we know...  At which point we
  * move back to UNSOLICTED_OFF to turn off these checks.
+ *
+ * We stay UNSOLICITED_OFF during a period, then turn to UNSOLICITED_PREPARE
+ * followed by UNSOLICITED_CHECK.  In case they are really UDP firewalled,
+ * we would be alternating active checking and passive monitoring every
+ * FW_INCOMING_WINDOW.  To avoid wasting too much CPU time on these active
+ * checks, we maintain a counter of how many periods we have to let go before
+ * switching, as indicated by ``unsolicited_wait_periods''.  This variable
+ * starts at 0, and is then incremented each time we move from UNSOLICITED_OFF
+ * to UNSOLICITED_PREPARE.  It is reset to 0 when we determine we are not
+ * UDP firewalled.
+ *
+ * The variable `unsolicited_checks_in'' counts the number of periods we have
+ * to wait before running active unsolicited checks again.
  */
 
 enum solicited_states {
@@ -122,6 +143,8 @@ enum solicited_states {
 
 static enum solicited_states outgoing_udp_state = UNSOLICITED_OFF;
 static cevent_t *unsolicited_udp_ev;
+static unsigned unsolicited_wait_periods;
+static unsigned unsolicited_checks_in;
 
 /***
  *** External connection status management structures.
@@ -138,7 +161,7 @@ static cevent_t *unsolicited_udp_ev;
 
 static watchdog_t *outgoing_wd;			/**< Watchdog for outgoing activity */
 
-static void inet_set_is_connected(gboolean val);
+static void inet_set_is_connected(bool val);
 
 /**
  * Checks whether a host address is considered being "local".
@@ -147,7 +170,7 @@ static void inet_set_is_connected(gboolean val);
  * @returns TRUE if the IP address is that of the local machine or
  *			a private address. Otherwise FALSE is returned.
  */
-static gboolean
+static bool
 is_local_addr(const host_addr_t addr)
 {
 	static host_addr_t our_addr, our_addr_v6;
@@ -158,12 +181,9 @@ is_local_addr(const host_addr_t addr)
 	 *		 cause blocking of several seconds which is not acceptable
 	 *		 except during the startup phase.
 	 */
-	if (
-		NET_USE_IPV4 == GNET_PROPERTY(network_protocol) ||
-		NET_USE_BOTH == GNET_PROPERTY(network_protocol)
-	) {
+	if (settings_use_ipv4()) {
 		if (!is_host_addr(our_addr)) {
-			static gboolean tried;
+			static bool tried;
 
 			if (!tried) {
 				tried = TRUE;
@@ -175,7 +195,7 @@ is_local_addr(const host_addr_t addr)
 		if (!is_host_addr(our_addr))
 			our_addr = listen_addr();
 		if (!is_host_addr(our_addr)) {
-			static gboolean tried;
+			static bool tried;
 
 			if (!tried) {
 				tried = TRUE;
@@ -187,13 +207,10 @@ is_local_addr(const host_addr_t addr)
 			}
 		}
 	}
-	
-	if (
-		NET_USE_IPV6 == GNET_PROPERTY(network_protocol) ||
-		NET_USE_BOTH == GNET_PROPERTY(network_protocol)
-	) {
+
+	if (settings_use_ipv6()) {
 		if (!is_host_addr(our_addr_v6)) {
-			static gboolean tried;
+			static bool tried;
 
 			if (!tried) {
 				/* This should not change */
@@ -205,7 +222,7 @@ is_local_addr(const host_addr_t addr)
 		if (!is_host_addr(our_addr_v6))
 			our_addr = listen_addr6();
 		if (!is_host_addr(our_addr_v6)) {
-			static gboolean tried;
+			static bool tried;
 
 			if (!tried) {
 				tried = TRUE;
@@ -221,9 +238,9 @@ is_local_addr(const host_addr_t addr)
 	if (is_my_address(addr))
 		return TRUE;
 
-	if (host_addr_equal(addr, ipv4_loopback))
+	if (host_addr_equiv(addr, ipv4_loopback))
 		return TRUE;
-	if (host_addr_equal(addr, ipv6_loopback))
+	if (host_addr_equiv(addr, ipv6_loopback))
 		return TRUE;
 
 	switch (host_addr_net(addr)) {
@@ -238,6 +255,17 @@ is_local_addr(const host_addr_t addr)
 	return FALSE;
 }
 
+/**
+ * Log current stacktrace when debugging, so that we can see which part of
+ * the code caused a state change in the firewalled state.
+ */
+static void
+inet_where(void)
+{
+	if (GNET_PROPERTY(fw_debug) > 1)
+		stacktrace_where_sym_print_offset(stderr, 1);
+}
+
 /***
  *** Firewall status management routines.
  ***/
@@ -256,12 +284,19 @@ inet_firewalled(void)
 
 /**
  * Called when we enter the firewalled status (UDP).
+ *
+ * @param new_env	when TRUE, we become firewalled due to a new environment
  */
 void
-inet_udp_firewalled(void)
+inet_udp_firewalled(bool new_env)
 {
 	gnet_prop_set_boolean_val(PROP_IS_UDP_FIREWALLED, TRUE);
 	node_became_udp_firewalled();
+
+	if (new_env)
+		unsolicited_wait_periods = 0;
+
+	unsolicited_checks_in = unsolicited_wait_periods;
 }
 
 /**
@@ -269,15 +304,14 @@ inet_udp_firewalled(void)
  * Enter the UNSOLICITED_CHECK state.
  */
 static void
-move_to_unsolicited_check(cqueue_t *unused_cq, gpointer unused_data)
+move_to_unsolicited_check(cqueue_t *cq, void *unused_data)
 {
-	(void) unused_cq;
 	(void) unused_data;
 
-	unsolicited_udp_ev = NULL;		/* Event fired */
+	cq_zero(cq, &unsolicited_udp_ev);		/* Event fired */
 
 	if (GNET_PROPERTY(fw_debug))
-		g_message("FW: will be now monitoring UDP for unsolicited messages");
+		g_debug("FW: will be now monitoring UDP for unsolicited messages");
 
 	outgoing_udp_state = UNSOLICITED_CHECK;
 }
@@ -289,12 +323,22 @@ static void
 move_to_unsolicited_prepare(void)
 {
 	if (GNET_PROPERTY(fw_debug))
-		g_message("FW: set for unsolicited traffic detection in %d secs",
+		g_debug("FW: set for unsolicited traffic detection in %d secs",
 			FW_UDP_WINDOW);
 
-	unsolicited_udp_ev = cq_insert(callout_queue, FW_UDP_WINDOW * 1000,
+	unsolicited_udp_ev = cq_main_insert(FW_UDP_WINDOW * 1000,
 		move_to_unsolicited_check, NULL);
 	outgoing_udp_state = UNSOLICITED_PREPARE;
+
+	/*
+	 * Next time, wait more watchdog periods before launching the active
+	 * unsolicited traffic checks.  We cap the number of periods we can
+	 * defer the checks to FW_UDP_MAX_PERIODS.
+	 */
+
+	unsolicited_wait_periods++;
+	unsolicited_wait_periods =
+		MIN(unsolicited_wait_periods, FW_UDP_MAX_PERIODS);
 }
 
 /**
@@ -303,11 +347,13 @@ move_to_unsolicited_prepare(void)
 static void
 move_to_unsolicited_off(void)
 {
-	if (GNET_PROPERTY(fw_debug))
-		g_message("FW: turning off unsolicited traffic detection");
+	if (GNET_PROPERTY(fw_debug)) {
+		g_debug("FW: turning off unsolicited traffic detection "
+			"(wait periods = %u)", unsolicited_wait_periods);
+	}
 
 	outgoing_udp_state = UNSOLICITED_OFF;
-	cq_cancel(callout_queue, &unsolicited_udp_ev);	/* Paranoid */
+	cq_cancel(&unsolicited_udp_ev);		/* Paranoid */
 }
 
 /**
@@ -315,15 +361,16 @@ move_to_unsolicited_off(void)
  * for some amount of time.  We conclude we're no longer able to get
  * solicited UDP traffic.
  */
-static gboolean
-got_no_udp_solicited(watchdog_t *unused_wd, gpointer unused_obj)
+static bool
+got_no_udp_solicited(watchdog_t *unused_wd, void *unused_obj)
 {
 	(void) unused_wd;
 	(void) unused_obj;
 
-	if (GNET_PROPERTY(fw_debug))
-		g_message("FW: got no solicited UDP traffic for %d secs",
+	if (GNET_PROPERTY(fw_debug)) {
+		g_debug("FW: got no solicited UDP traffic for %d secs",
 			FW_SOLICITED_WINDOW);
+	}
 
 	gnet_prop_set_boolean_val(PROP_RECV_SOLICITED_UDP, FALSE);
 	return FALSE;			/* Disarm watchdog */
@@ -337,8 +384,10 @@ inet_udp_got_solicited(void)
 {
 	gnet_prop_set_boolean_val(PROP_RECV_SOLICITED_UDP, TRUE);
 
-	if (wd_wakeup(solicited_udp_wd) && GNET_PROPERTY(fw_debug))
-		g_message("FW: got solicited UDP traffic");
+	if (wd_wakeup(solicited_udp_wd) && GNET_PROPERTY(fw_debug)) {
+		g_debug("FW: got solicited UDP traffic");
+		inet_where();
+	}
 
 	wd_kick(solicited_udp_wd);
 }
@@ -347,15 +396,16 @@ inet_udp_got_solicited(void)
  * This is a callback invoked when no incoming connection has been received
  * for some amount of time.  We conclude we became firewalled.
  */
-static gboolean
-got_no_connection(watchdog_t *unused_wd, gpointer unused_obj)
+static bool
+got_no_connection(watchdog_t *unused_wd, void *unused_obj)
 {
 	(void) unused_wd;
 	(void) unused_obj;
 
-	if (GNET_PROPERTY(fw_debug))
-		g_message("FW: got no connection to port %u for %d secs",
+	if (GNET_PROPERTY(fw_debug)) {
+		g_debug("FW: got no connection to port %u for %d secs",
 			socket_listen_port(), FW_INCOMING_WINDOW);
+	}
 
 	inet_firewalled();
 	return FALSE;			/* Disarm watchdog */
@@ -365,19 +415,21 @@ got_no_connection(watchdog_t *unused_wd, gpointer unused_obj)
  * This is a callback invoked when no unsolicited UDP datagrams have been
  * received for some amount of time.  We conclude we became firewalled.
  */
-static gboolean
-got_no_udp_unsolicited(watchdog_t *unused_wd, gpointer unused_obj)
+static bool
+got_no_udp_unsolicited(watchdog_t *unused_wd, void *unused_obj)
 {
 	(void) unused_wd;
 	(void) unused_obj;
 
-	if (GNET_PROPERTY(fw_debug))
-		g_message("FW: got no unsolicited UDP datagram to port %u for %d secs",
+	if (GNET_PROPERTY(fw_debug)) {
+		g_debug("FW: got no unsolicited UDP datagram to port %u for %d secs",
 			socket_listen_port(), FW_INCOMING_WINDOW);
+	}
 
 	/*
 	 * If we are in the UNSOLICITED_OFF state, move to UNSOLICITED_PREPARE
-	 * for FW_UDP_WINDOW seconds.
+	 * for FW_UDP_WINDOW seconds, provided ``unsolicited_checks_in'' is 0.
+	 * Otherwise, decrease that amount and wait.
 	 *
 	 * Otherwise, move to UNSOLICITED_OFF and state we are firewalled.
 	 */
@@ -385,14 +437,27 @@ got_no_udp_unsolicited(watchdog_t *unused_wd, gpointer unused_obj)
 	STATIC_ASSERT(FW_UDP_WINDOW < FW_SOLICITED_WINDOW);
 
 	if (UNSOLICITED_OFF == outgoing_udp_state) {
-		move_to_unsolicited_prepare();
+		g_assert(uint_is_non_negative(unsolicited_checks_in));
+		if (0 == unsolicited_checks_in)
+			move_to_unsolicited_prepare();
+		else {
+			unsolicited_checks_in--;
+
+			if (GNET_PROPERTY(fw_debug)) {
+				g_debug("FW: will look for unsolicited UDP %s %s period%s",
+					0 == unsolicited_checks_in ? "at" : "in",
+					0 == unsolicited_checks_in ? "next" :
+						str_smsg("%u", unsolicited_checks_in + 1),
+					0 == unsolicited_checks_in ? "" : "s");
+			}
+		}
 	} else {
 		if (GNET_PROPERTY(fw_debug)) {
-			g_message("FW: no unsolicited UDP again for %d secs on port %u "
+			g_debug("FW: no unsolicited UDP again for %d secs on port %u "
 				"=> firewalled", FW_SOLICITED_WINDOW, socket_listen_port());
 		}
 		move_to_unsolicited_off();
-		inet_udp_firewalled();
+		inet_udp_firewalled(FALSE);
 	}
 
 	return TRUE;		/* Let watchdog fire again at next period */
@@ -407,9 +472,23 @@ inet_not_firewalled(void)
 	gnet_prop_set_boolean_val(PROP_IS_FIREWALLED, FALSE);
 	node_proxy_cancel_all();
 
-	if (GNET_PROPERTY(fw_debug))
-		g_message("FW: we're not TCP-firewalled for port %u",
+	if (GNET_PROPERTY(fw_debug)) {
+		g_debug("FW: we're not TCP-firewalled for port %u",
 			socket_listen_port());
+	}
+}
+
+/**
+ * Switch to non TCP-firewalled if we were indeed firewalled.
+ */
+static void
+inet_switch_to_not_firewalled(void)
+{
+	if (GNET_PROPERTY(is_firewalled)) {
+		wd_wakeup(incoming_wd);
+		inet_not_firewalled();
+		inet_where();
+	}
 }
 
 /**
@@ -420,26 +499,63 @@ inet_udp_not_firewalled(void)
 {
 	gnet_prop_set_boolean_val(PROP_IS_UDP_FIREWALLED, FALSE);
 
-	if (GNET_PROPERTY(fw_debug))
-		g_message("FW: we're not UDP-firewalled for port %u",
+	if (GNET_PROPERTY(fw_debug)) {
+		g_debug("FW: we're not UDP-firewalled for port %u",
 			socket_listen_port());
+	}
+
+	unsolicited_wait_periods = 0;
 }
 
 /**
- * Called when we got an incoming connection from another computer at `ip'.
+ * Switch to non UDP-firewalled if we were indeed firewalled.
+ */
+static void
+inet_switch_to_udp_not_firewalled(void)
+{
+	if (GNET_PROPERTY(is_udp_firewalled)) {
+		inet_udp_not_firewalled();
+		inet_where();
+	}
+}
+
+/**
+ * Called when we established UPnP or NAT-PMP router configuration.
+ */
+void
+inet_router_configured(void)
+{
+	if (GNET_PROPERTY(fw_debug)) {
+		g_debug("FW: configured router to forward port %u",
+			socket_listen_port());
+	}
+
+	/*
+	 * Assume mapping will be all-right, which will be the case usually.
+	 *
+	 * In case something gets wrong afterwards or the router lied, we can
+	 * still go back to a firewalled status later on anyway.
+	 */
+
+	inet_switch_to_not_firewalled();
+	inet_switch_to_udp_not_firewalled();
+}
+
+/**
+ * Called when we got an incoming connection from another computer at `addr'.
  */
 void
 inet_got_incoming(const host_addr_t addr)
 {
 	if (is_local_addr(addr)) {
 		if (GNET_PROPERTY(fw_debug))
-			g_message("FW: not counting local connection from %s",
+			g_debug("FW: not counting local connection from %s",
 				host_addr_to_string(addr));
 		return;
 	}
 
 	if (GNET_PROPERTY(fw_debug) > 19)
-		g_message("FW: got TCP connection from %s", host_addr_to_string(addr));
+		g_debug("FW: got TCP connection from %s", host_addr_to_string(addr));
 
 	/*
 	 * If we get an incoming connection from the outside, we're surely
@@ -451,7 +567,8 @@ inet_got_incoming(const host_addr_t addr)
 		!GNET_PROPERTY(is_inet_connected) &&
 		GNET_PROPERTY(fw_debug)
 	) {
-		g_message("FW: got incoming connection => connected");
+		g_debug("FW: got incoming connection => connected");
+		inet_where();
 	}
 
 	if (!GNET_PROPERTY(is_inet_connected))
@@ -461,11 +578,7 @@ inet_got_incoming(const host_addr_t addr)
 	 * We're not firewalled.
 	 */
 
-	if (GNET_PROPERTY(is_firewalled)) {
-		wd_wakeup(incoming_wd);
-		inet_not_firewalled();
-	}
-
+	inet_switch_to_not_firewalled();
 	wd_kick(incoming_wd);
 }
 
@@ -480,18 +593,19 @@ inet_got_incoming(const host_addr_t addr)
  * a reply of some sort (i.e. sent back to us through a masquerated port).
  * When we do get these messages, then this routine can be called explicitly.
  */
-void 
+void
 inet_udp_got_unsolicited_incoming(void)
 {
 	if (outgoing_udp_state != UNSOLICITED_OFF) {
 		if (GNET_PROPERTY(fw_debug))
-			g_message("FW: got unsolicited UDP message => not firewalled");
+			g_debug("FW: got unsolicited UDP message => not firewalled");
 		move_to_unsolicited_off();
+	} else if (GNET_PROPERTY(is_udp_firewalled)) {
+		if (GNET_PROPERTY(fw_debug))
+			g_debug("FW: got unsolicited UDP message");
 	}
 
-	if (GNET_PROPERTY(is_udp_firewalled))
-		inet_udp_not_firewalled();
-
+	inet_switch_to_udp_not_firewalled();
 	wd_kick(incoming_udp_wd);
 }
 
@@ -509,7 +623,8 @@ inet_udp_got_incoming(const host_addr_t addr)
 		!GNET_PROPERTY(is_inet_connected) &&
 		GNET_PROPERTY(fw_debug)
 	) {
-		g_message("FW: got incoming UDP traffic => connected");
+		g_debug("FW: got incoming UDP traffic => connected");
+		inet_where();
 	}
 
 	if (!GNET_PROPERTY(is_inet_connected))
@@ -541,8 +656,7 @@ inet_udp_record_sent(const host_addr_t addr)
 		return;		/* Not currently monitoring for unsolicited UDP */
 
 	if (!aging_lookup_revitalise(outgoing_udp, &addr)) {
-		aging_insert(outgoing_udp,
-			wcopy(&addr, sizeof addr), uint_to_pointer(1));
+		aging_record(outgoing_udp, WCOPY(&addr));
 	}
 }
 
@@ -555,12 +669,12 @@ inet_udp_check_unsolicited(void)
 {
 	if (UNSOLICITED_OFF == outgoing_udp_state) {
 		if (GNET_PROPERTY(fw_debug))
-			g_message("FW: forcing checks for UDP unsolicited traffic");
+			g_debug("FW: forcing checks for UDP unsolicited traffic");
 
 		move_to_unsolicited_prepare();
 	} else {
 		if (GNET_PROPERTY(fw_debug))
-			g_message("FW: already checking UDP unsolicited traffic");
+			g_debug("FW: already checking UDP unsolicited traffic");
 	}
 }
 
@@ -571,14 +685,14 @@ inet_udp_check_unsolicited(void)
  * a non-private IP and are within a "grace period", act as if we were not:
  * we can only know we're not firewalled when we get an incoming connection.
  */
-gboolean
+bool
 inet_can_answer_ping(void)
 {
 	int elapsed;
 
 	/* Leaves don't send pongs */
 	if (!GNET_PROPERTY(is_firewalled))
-		return GNET_PROPERTY(current_peermode) != NODE_P_LEAF;
+		return !settings_is_leaf();
 
 	if (!is_host_addr(listen_addr()) && !is_host_addr(listen_addr6()))
 		return FALSE;		/* We don't know our local IP, we can't reply */
@@ -615,12 +729,12 @@ inet_can_answer_ping(void)
  * Sets our internet connection status.
  */
 static void
-inet_set_is_connected(gboolean val)
+inet_set_is_connected(bool val)
 {
 	gnet_prop_set_boolean_val(PROP_IS_INET_CONNECTED, val);
 
 	if (GNET_PROPERTY(fw_debug))
-		g_message("FW: we're %sconnected to the Internet",
+		g_debug("FW: we're %sconnected to the Internet",
 			val ? "" : "no longer ");
 }
 
@@ -628,8 +742,8 @@ inet_set_is_connected(gboolean val)
  * This callback fires when there was no outgoing activity for the period
  * after the watchdog was started.
  */
-static gboolean
-no_outgoing_connection(watchdog_t *unused_wd, gpointer unused_obj)
+static bool
+no_outgoing_connection(watchdog_t *unused_wd, void *unused_obj)
 {
 	(void) unused_wd;
 	(void) unused_obj;
@@ -672,7 +786,7 @@ inet_connection_succeeded(const host_addr_t addr)
 		!GNET_PROPERTY(is_inet_connected) &&
 		GNET_PROPERTY(fw_debug)
 	) {
-		g_message("FW: outgoing TCP connection succeeded => connected");
+		g_debug("FW: outgoing TCP connection succeeded => connected");
 	}
 
 	if (!GNET_PROPERTY(is_inet_connected))
@@ -693,10 +807,55 @@ inet_read_activity(void)
 	wd_kick(outgoing_wd);
 }
 
+/***
+ *** IP network buffer shortage monitoring.
+ ***/
+
+/**
+ * This callback fires when the IP network buffer shortage is hopefully gone.
+ */
+static bool
+inet_buf_shortage_gone(watchdog_t *unused_wd, void *unused_obj)
+{
+	(void) unused_wd;
+	(void) unused_obj;
+
+	g_message("%s(): assuming network buffer shortage is gone", G_STRFUNC);
+
+	gnet_prop_set_boolean_val(PROP_NET_BUFFER_SHORTAGE, FALSE);
+	return FALSE;						/* Put watchdog back to sleep */
+}
+
+/**
+ * We just got an ENOBUFS error on a socket operation.
+ */
+void
+inet_buf_shortage(void)
+{
+	/*
+	 * Settting PROP_NET_BUFFER_SHORTAGE to TRUE means that we will try to
+	 * avoid TCP connections and socket I/Os, if possible, for a while until
+	 * hopefully more network buffers become available.
+	 */
+
+	gnet_prop_set_boolean_val(PROP_NET_BUFFER_SHORTAGE, TRUE);
+
+	/*
+	 * Wake-up (if not already done) and kick the watchdog.  If no other
+	 * buffer shortage occurs within the monitoring period, we'll consider
+	 * that the buffer shortage condition is gone.
+	 */
+
+	if (wd_wakeup_kick(buf_shortage_wd)) {
+		g_carp("%s(): avoiding networking for a while to let buffers flush",
+			G_STRFUNC);
+	}
+}
+
 /**
  * Initialization code.
  */
-void
+void G_COLD
 inet_init(void)
 {
 	/*
@@ -722,6 +881,13 @@ inet_init(void)
 	if (GNET_PROPERTY(recv_solicited_udp))
 		wd_wakeup(solicited_udp_wd);
 
+	/*
+	 * This watchdog is used to monitor IP network buffer shortages.
+	 */
+
+	buf_shortage_wd = wd_make("IP network buffer shortage",
+		BUF_SHORTAGE_WINDOW, inet_buf_shortage_gone, NULL, FALSE);
+
 	/*
 	 * Monitoring watchdog for outgoing connections.
 	 */
@@ -742,11 +908,12 @@ inet_init(void)
 /**
  * Shutdown cleanup.
  */
-void
+void G_COLD
 inet_close(void)
 {
 	aging_destroy(&outgoing_udp);
-	cq_cancel(callout_queue, &unsolicited_udp_ev);
+	cq_cancel(&unsolicited_udp_ev);
+	wd_free_null(&buf_shortage_wd);
 	wd_free_null(&outgoing_wd);
 	wd_free_null(&incoming_wd);
 	wd_free_null(&incoming_udp_wd);
