@@ -1,6 +1,4 @@
 /*
- * $Id: ipp_cache.c 17156 2009-11-01 20:22:05Z rmanfredi $
- *
  * Copyright (c) 2007-2009, Raphael Manfredi
  * Copyright (c) 2006, Christian Biere
  *
@@ -20,7 +18,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -28,7 +26,25 @@
  * @ingroup core
  * @file
  *
- * Caching of hosts by IP:port.
+ * Caching of hosts by IP:port to be able to make "existence" checks, i.e.
+ * determine whether a particular IP:port belongs to the cache and when
+ * it was added.
+ *
+ * This is generic code handling entries formatted as:
+ *
+ *   HOST 10.19.182.13:1033
+ *   SEEN 2010-10-27 14:25:06
+ *   END
+ *
+ * and which are added through ipp_cache_insert(), and removed through
+ * ipp_cache_remove().  Lookups are made via ipp_cache_lookup().
+ *
+ * Convenient wrappers are provided for each of the caches we manage here.
+ * For instance, tls_cache_lookup() can be used to operate on the TLS cache.
+ *
+ * Each cache is configured to hold a maximum amount of entries, along with
+ * a maximum lifetime for the data so that old-enough entries can be
+ * expired.
  *
  * @author Raphael Manfredi
  * @date 2007-2009
@@ -38,21 +54,20 @@
 
 #include "common.h"
 
-RCSID("$Id: ipp_cache.c 17156 2009-11-01 20:22:05Z rmanfredi $")
-
-#include "lib/bit_array.h"
-#include "lib/file.h"
-#include "lib/getdate.h"
-#include "lib/hashlist.h"
-#include "lib/walloc.h"
-
-#include "lib/endian.h"
-#include "lib/host_addr.h"
-#include "lib/timestamp.h"
+#include "ipp_cache.h"
 
 #include "hosts.h"
 #include "settings.h"
-#include "ipp_cache.h"
+
+#include "lib/bit_array.h"
+#include "lib/endian.h"
+#include "lib/file.h"
+#include "lib/getdate.h"
+#include "lib/hashlist.h"
+#include "lib/host_addr.h"
+#include "lib/timestamp.h"
+#include "lib/tokenizer.h"
+#include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
 
@@ -69,11 +84,11 @@ struct ipp_cache {
 	const char *name;				/**< Cache name */
 	const char *item_name;			/**< Cache item name */
 	const char *description;		/**< Cache description for comments */
-	const guint32 *max_cache_time;	/**< Amount of time data can stay cached */
-	const guint32 *max_cache_size;	/**< Max amount od items to cache */
+	const uint32 *max_cache_time;	/**< Amount of time data can stay cached */
+	const uint32 *max_cache_size;	/**< Max amount od items to cache */
 	hash_list_t *hosts;				/**< The caching structure */
 	time_t last_stored;				/**< Last time cache was persisted */
-	const guint32 *debug;			/**< Debug level for this cache */
+	const uint32 *debug;			/**< Debug level for this cache */
 };
 
 /**
@@ -89,18 +104,18 @@ struct ipp_cache_item {
  */
 static ipp_cache_t *caches[IPP_CACHE_COUNT];
 
-static guint
-ipp_cache_item_hash(gconstpointer key)
+static uint
+ipp_cache_item_hash(const void *key)
 {
 	const struct ipp_cache_item *item = key;
 	return gnet_host_hash(&item->host);
 }
 
 static int
-ipp_cache_item_eq(gconstpointer v1, gconstpointer v2)
+ipp_cache_item_eq(const void *v1, const void *v2)
 {
 	const struct ipp_cache_item *a = v1, *b = v2;
-	return gnet_host_eq(&a->host,& b->host);
+	return gnet_host_equal(&a->host,& b->host);
 }
 
 /**
@@ -110,12 +125,12 @@ static ipp_cache_t *
 ipp_cache_alloc(
 	const char *name, const char *item_name, const char *file_name,
 	const char *description,
-	const guint32 *max_cache_time, const guint32 *max_cache_size,
-	const guint32 *debug)
+	const uint32 *max_cache_time, const uint32 *max_cache_size,
+	const uint32 *debug)
 {
 	ipp_cache_t *ic;
 
-	ic = walloc(sizeof *ic);
+	WALLOC(ic);
 	ic->name = name;
 	ic->item_name = item_name;
 	ic->description = description;
@@ -133,7 +148,7 @@ ipp_cache_alloc(
 /**
  * Has a cached entry expired?
  */
-static gboolean
+static bool
 ipp_cache_item_expired(const ipp_cache_t *ic, time_t seen, time_t now)
 {
 	time_delta_t d = delta_time(now, seen);
@@ -153,13 +168,10 @@ typedef enum {
 	NUM_IPP_CACHE_TAGS
 } ipp_cache_tag_t;
 
-static const struct ipp_cache_tag {
-	ipp_cache_tag_t	tag;
-	const char *str;
-} ipp_cache_tag_map[] = {
+static const tokenizer_t ipp_cache_tags[] = {
 	/* Must be sorted alphabetically for dichotomic search */
 
-#define IPP_CACHE_TAG(x) { CAT2(IPP_CACHE_TAG_,x), #x }
+#define IPP_CACHE_TAG(x) { #x, CAT2(IPP_CACHE_TAG_,x) }
 	IPP_CACHE_TAG(END),
 	IPP_CACHE_TAG(HOST),
 	IPP_CACHE_TAG(SEEN),
@@ -171,24 +183,10 @@ static const struct ipp_cache_tag {
 /**
  * Convert a string representation of a serializing tag to its token number.
  */
-static ipp_cache_tag_t
+static inline ipp_cache_tag_t
 ipp_cache_string_to_tag(const char *s)
 {
-	STATIC_ASSERT(G_N_ELEMENTS(ipp_cache_tag_map) == (NUM_IPP_CACHE_TAGS - 1));
-
-#define GET_ITEM(i) (ipp_cache_tag_map[(i)].str)
-#define FOUND(i) G_STMT_START { \
-	return ipp_cache_tag_map[(i)].tag; \
-	/* NOTREACHED */ \
-} G_STMT_END
-
-	/* Perform a binary search to find ``s'' */
-	BINARY_SEARCH(const char *, s, G_N_ELEMENTS(ipp_cache_tag_map), strcmp,
-		GET_ITEM, FOUND);
-
-#undef FOUND
-#undef GET_ITEM
-	return IPP_CACHE_TAG_UNKNOWN;
+	return TOKENIZE(s, ipp_cache_tags);
 }
 
 /**
@@ -207,7 +205,7 @@ ipp_cache_dump(ipp_cache_t *ic, FILE *f)
 	iter = hash_list_iterator(ic->hosts);
 	while (hash_list_iter_has_next(iter)) {
 		const struct ipp_cache_item *item;
-		
+
 		item = hash_list_iter_next(iter);
 		fprintf(f, "HOST %s\nSEEN %s\nEND\n\n",
 			gnet_host_to_string(&item->host), timestamp_to_string(item->seen));
@@ -253,11 +251,11 @@ static void
 ipp_cache_remove_oldest(ipp_cache_t *ic)
 {
 	struct ipp_cache_item *item;
-	
+
 	item = hash_list_head(ic->hosts);
 	if (item) {
 		hash_list_remove(ic->hosts, item);
-		wfree(item, sizeof *item);
+		WFREE(item);
 	}
 }
 
@@ -267,7 +265,7 @@ ipp_cache_remove_oldest(ipp_cache_t *ic)
 static void
 ipp_cache_insert_intern(ipp_cache_t *ic, const struct ipp_cache_item *item)
 {
-	gconstpointer key;
+	const void *key;
 	int removed;
 	size_t max_size;
 
@@ -275,20 +273,20 @@ ipp_cache_insert_intern(ipp_cache_t *ic, const struct ipp_cache_item *item)
 
 	key = hash_list_remove(ic->hosts, item);
 	if (key) {
-		struct ipp_cache_item *item_ptr = deconstify_gpointer(key);
+		struct ipp_cache_item *item_ptr = deconstify_pointer(key);
 
 		/* We'll move the host to the end of the list */
-		if (*ic->debug) {
-			g_message("refreshing %s host %s",
+		if (*ic->debug > 3) {
+			g_debug("refreshing %s host %s",
 				ic->item_name, gnet_host_to_string(&item->host));
 		}
 		item_ptr->seen = item->seen;
 	} else {
-		if (*ic->debug) {
-			g_message("adding %s host %s",
+		if (*ic->debug > 3) {
+			g_debug("adding %s host %s",
 				ic->item_name, gnet_host_to_string(&item->host));
 		}
-		key = wcopy(item, sizeof *item);
+		key = WCOPY(item);
 	}
 	hash_list_append(ic->hosts, key);
 
@@ -302,7 +300,7 @@ ipp_cache_insert_intern(ipp_cache_t *ic, const struct ipp_cache_item *item)
 		if (++removed >= IPP_CACHE_REMOVE)
 			break;
 	}
-	
+
 	item = hash_list_head(ic->hosts);
 	if (item && ipp_cache_item_expired(ic, item->seen, tm_time())) {
 		ipp_cache_remove_oldest(ic);
@@ -325,7 +323,7 @@ get_cache(enum ipp_cache_id cid)
  * Insert host in specified cache.
  */
 void
-ipp_cache_insert(enum ipp_cache_id cid, const host_addr_t addr, guint16 port)
+ipp_cache_insert(enum ipp_cache_id cid, const host_addr_t addr, uint16 port)
 {
 	ipp_cache_t *ic = get_cache(cid);
 	struct ipp_cache_item item;
@@ -347,23 +345,23 @@ ipp_cache_insert(enum ipp_cache_id cid, const host_addr_t addr, guint16 port)
  */
 static struct ipp_cache_item *
 ipp_cache_lookup_intern(const ipp_cache_t *ic,
-	const host_addr_t addr, guint16 port)
+	const host_addr_t addr, uint16 port)
 {
 	g_assert(ic);
 
 	if (host_addr_initialized(addr) && is_host_addr(addr) && 0 != port) {
 		struct ipp_cache_item item;
-		gconstpointer key;
+		const void *key;
 
 		gnet_host_set(&item.host, addr, port);
 		if (hash_list_find(ic->hosts, &item, &key)) {
-			struct ipp_cache_item *item_ptr = deconstify_gpointer(key);
-			
+			struct ipp_cache_item *item_ptr = deconstify_pointer(key);
+
 			if (!ipp_cache_item_expired(ic, item_ptr->seen, tm_time()))
 				return item_ptr;
 
 			hash_list_remove(ic->hosts, item_ptr);
-			wfree(item_ptr, sizeof *item_ptr);
+			WFREE(item_ptr);
 		}
 	}
 	return NULL;
@@ -372,8 +370,8 @@ ipp_cache_lookup_intern(const ipp_cache_t *ic,
 /**
  * @return TRUE if addr:port is currently in the cache.
  */
-gboolean
-ipp_cache_lookup(enum ipp_cache_id cid, const host_addr_t addr, guint16 port)
+bool
+ipp_cache_lookup(enum ipp_cache_id cid, const host_addr_t addr, uint16 port)
 {
 	ipp_cache_t *ic = get_cache(cid);
 	return NULL != ipp_cache_lookup_intern(ic, addr, port);
@@ -385,7 +383,7 @@ ipp_cache_lookup(enum ipp_cache_id cid, const host_addr_t addr, guint16 port)
  */
 time_t
 ipp_cache_get_timestamp(enum ipp_cache_id cid,
-	const host_addr_t addr, guint16 port)
+	const host_addr_t addr, uint16 port)
 {
 	const struct ipp_cache_item *item;
 	ipp_cache_t *ic = get_cache(cid);
@@ -399,22 +397,22 @@ ipp_cache_get_timestamp(enum ipp_cache_id cid,
  *
  * @return whether entry was found and deleted.
  */
-static gboolean
+static bool
 ipp_cache_remove_intern(const ipp_cache_t *ic,
-	const host_addr_t addr, guint16 port)
+	const host_addr_t addr, uint16 port)
 {
 	g_assert(ic);
 
 	if (host_addr_initialized(addr) && is_host_addr(addr) && 0 != port) {
 		struct ipp_cache_item item;
-		gconstpointer key;
+		const void *key;
 
 		gnet_host_set(&item.host, addr, port);
 		if (hash_list_find(ic->hosts, &item, &key)) {
-			struct ipp_cache_item *item_ptr = deconstify_gpointer(key);
-			
+			struct ipp_cache_item *item_ptr = deconstify_pointer(key);
+
 			hash_list_remove(ic->hosts, item_ptr);
-			wfree(item_ptr, sizeof *item_ptr);
+			WFREE(item_ptr);
 			return TRUE;
 		}
 	}
@@ -426,8 +424,8 @@ ipp_cache_remove_intern(const ipp_cache_t *ic,
  *
  * @return TRUE if found and removed.
  */
-gboolean
-ipp_cache_remove(enum ipp_cache_id cid, const host_addr_t addr, guint16 port)
+bool
+ipp_cache_remove(enum ipp_cache_id cid, const host_addr_t addr, uint16 port)
 {
 	ipp_cache_t *ic = get_cache(cid);
 	return ipp_cache_remove_intern(ic, addr, port);
@@ -436,57 +434,55 @@ ipp_cache_remove(enum ipp_cache_id cid, const host_addr_t addr, guint16 port)
 /**
  * Parse persisted cache.
  */
-static void
+static void G_COLD
 ipp_cache_parse(ipp_cache_t *ic, FILE *f)
 {
 	bit_array_t tag_used[BIT_ARRAY_SIZE(NUM_IPP_CACHE_TAGS)];
 	static const struct ipp_cache_item zero_item;
 	struct ipp_cache_item item;
 	char line[1024];
-	guint line_no = 0;
-	gboolean done = FALSE;
+	uint line_no = 0;
+	bool done = FALSE;
 
 	g_return_if_fail(f);
 
 	/* Reset state */
 	done = FALSE;
 	item = zero_item;
-	bit_array_clear_range(tag_used, 0, (guint) NUM_IPP_CACHE_TAGS - 1);
+	bit_array_init(tag_used, NUM_IPP_CACHE_TAGS);
+	bit_array_clear_range(tag_used, 0, NUM_IPP_CACHE_TAGS - 1U);
 
-	while (fgets(line, sizeof line, f)) {
+	while (fgets(ARYLEN(line), f)) {
 		const char *tag_name, *value;
-		char *sp, *nl;
-		gboolean damaged;
+		char *sp;
+		bool damaged;
 		ipp_cache_tag_t tag;
 
 		line_no++;
 
 		damaged = FALSE;
-		nl = strchr(line, '\n');
-		if (!nl) {
+		if (!file_line_chomp_tail(ARYLEN(line), NULL)) {
 			/*
 			 * If the line is too long or unterminated the file is either
 			 * corrupt or was manually edited without respecting the
 			 * exact format. If we continued, we would read from the
 			 * middle of a line which could be the filename or ID.
 			 */
-			g_warning("ipp_cache_parse(\"%s\"): "
-				"line too long or missing newline in line %u",
-				ic->name, line_no);
+			g_warning("%s(\"%s\"): line %u too long or missing newline",
+				G_STRFUNC, ic->name, line_no);
 			break;
 		}
-		*nl = '\0';
 
 		/* Skip comments and empty lines */
-		if (*line == '#' || *line == '\0')
+		if (file_line_is_skipable(line))
 			continue;
 
-		sp = strchr(line, ' ');
+		sp = vstrchr(line, ' ');
 		if (sp) {
 			*sp = '\0';
 			value = &sp[1];
 		} else {
-			value = strchr(line, '\0');
+			value = vstrchr(line, '\0');
 		}
 		tag_name = line;
 
@@ -494,17 +490,16 @@ ipp_cache_parse(ipp_cache_t *ic, FILE *f)
 		g_assert(UNSIGNED(tag) < NUM_IPP_CACHE_TAGS);
 		if (IPP_CACHE_TAG_UNKNOWN != tag && !bit_array_flip(tag_used, tag)) {
 			g_warning(
-				"ipp_cache_load(\"%s\"): "
-				"duplicate tag \"%s\" in entry in line %u",
-				ic->name, tag_name, line_no);
+				"%s(\"%s\"): duplicate tag \"%s\" in entry in line %u",
+				G_STRFUNC, ic->name, tag_name, line_no);
 			break;
 		}
-		
+
 		switch (tag) {
 		case IPP_CACHE_TAG_HOST:
 			{
 				host_addr_t addr;
-				guint16 port;
+				uint16 port;
 
 				if (string_to_host_addr_port(value, NULL, &addr, &port)) {
 					gnet_host_set(&item.host, addr, port);
@@ -520,14 +515,14 @@ ipp_cache_parse(ipp_cache_t *ic, FILE *f)
 				damaged = TRUE;
 			}
 			break;
-			
+
 		case IPP_CACHE_TAG_END:
 			if (!bit_array_get(tag_used, IPP_CACHE_TAG_HOST)) {
-				g_warning("ipp_cache_load(): missing HOST tag");
+				g_warning("%s(): missing HOST tag", G_STRFUNC);
 				damaged = TRUE;
 			}
 			if (!bit_array_get(tag_used, IPP_CACHE_TAG_SEEN)) {
-				g_warning("ipp_cache_load(): missing SEEN tag");
+				g_warning("%s(): missing SEEN tag", G_STRFUNC);
 				damaged = TRUE;
 			}
 			done = TRUE;
@@ -536,16 +531,16 @@ ipp_cache_parse(ipp_cache_t *ic, FILE *f)
 		case IPP_CACHE_TAG_UNKNOWN:
 			/* Ignore */
 			break;
-			
+
 		case NUM_IPP_CACHE_TAGS:
 			g_assert_not_reached();
 			break;
 		}
 
 		if (damaged) {
-			g_warning("damaged %s cache entry in line %u: "
+			g_warning("%s(): damaged %s cache entry in line %u: "
 				"tag_name=\"%s\", value=\"%s\"",
-				ic->item_name, line_no, tag_name, value);
+				G_STRFUNC, ic->item_name, line_no, tag_name, value);
 			break;
 		}
 
@@ -554,13 +549,14 @@ ipp_cache_parse(ipp_cache_t *ic, FILE *f)
 				ipp_cache_lookup_intern(ic, gnet_host_get_addr(&item.host),
 					gnet_host_get_port(&item.host))
 			) {
-				g_warning(
-					"ignoring duplicate %s cache item around line %u (%s)",
-				   	ic->item_name, line_no, gnet_host_to_string(&item.host));
+				g_warning("%s(): ignoring duplicate %s cache item around "
+					"line %u (%s)",
+					G_STRFUNC, ic->item_name, line_no,
+					gnet_host_to_string(&item.host));
 			} else if (!ipp_cache_item_expired(ic, item.seen, tm_time())) {
 				ipp_cache_insert_intern(ic, &item);
 			}
-			
+
 			/* Reset state */
 			done = FALSE;
 			item = zero_item;
@@ -585,20 +581,20 @@ ipp_cache_clear(ipp_cache_t *ic)
 /**
  * Retrieve cache from disk file.
  */
-static void
+static void G_COLD
 ipp_cache_load(ipp_cache_t *ic)
 {
 	FILE *f;
 
 	f = file_config_open_read(ic->name, &ic->fp, 1);
 	if (f) {
-		guint n;
-		
+		uint n;
+
 		ipp_cache_clear(ic);
 		ipp_cache_parse(ic, f);
 		n = hash_list_length(ic->hosts);
-		if (*ic->debug) {
-			g_message("loaded %u items from the %s cache", n, ic->item_name);
+		if (*ic->debug > 3) {
+			g_debug("loaded %u items from the %s cache", n, ic->item_name);
 		}
 		fclose(f);
 	}
@@ -614,9 +610,9 @@ ipp_cache_free(ipp_cache_t *ic)
 
 	ipp_cache_clear(ic);
 	hash_list_free(&ic->hosts);
-	g_free(deconstify_gpointer(ic->fp.dir));
+	g_free(deconstify_pointer(ic->fp.dir));
 	ic->fp.dir = NULL;	/* Don't use G_FREE_NULL b/c of lvalue cast */
-	wfree(ic, sizeof *ic);
+	WFREE(ic);
 }
 
 /**
@@ -642,9 +638,11 @@ ipp_cache_invalidate(enum ipp_cache_id cid)
  * structures, as some callbacks may insert data in the caches.  Later on,
  * the caches can be loaded when we know the properties are all initialized.
  */
-void
+void G_COLD
 ipp_cache_init(void)
 {
+	TOKENIZE_CHECK_SORTED(ipp_cache_tags);
+
 	caches[IPP_CACHE_TLS] = ipp_cache_alloc(
 		"TLS cache", "TLS", "tls_cache", "TLS-capable hosts",
 		GNET_PROPERTY_PTR(tls_cache_max_time),
@@ -663,7 +661,7 @@ ipp_cache_init(void)
 	 * However, for stupid political reasons on the part of "G2" proponents,
 	 * the "G2" servents do not always allow sharing of their files with
 	 * Gnutella, whereas most Gnutella servents, and GTKG for sure, will fully
-	 * allow "G2" hosts to the Gnutella extensions to HTTP to download files.
+	 * allow "G2" hosts to download files.
 	 *
 	 * The purpose of this cache is to store the addresses of known G2-only
 	 * hosts so that we can prevent the propagation of their addresses in the
