@@ -1,6 +1,4 @@
 /*
- * $Id: matching.h 17226 2009-11-12 00:03:54Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -66,31 +64,45 @@
 
 #include "common.h"
 
-typedef struct _search_table search_table_t;
+typedef struct search_table search_table_t;
 
 struct query_hashvec;
-
-search_table_t *st_alloc(void);
-void st_initialize(search_table_t *);
-void st_create(search_table_t *table);
-void st_destroy(search_table_t *);
-void st_free(search_table_t **);
-gboolean st_insert_item(search_table_t *, const char *key, gpointer data);
-void st_compact(search_table_t *);
-
 struct shared_file;
 
-/**
- * Needs brief description here.
- */
-typedef void (*st_search_callback)(gpointer ctx, gpointer data);
+search_table_t *st_create(void);
+void st_free(search_table_t **);
+void st_compact(search_table_t *);
+search_table_t *st_refcnt_inc(search_table_t *st);
 
-void st_search(
+enum match_set {
+	ST_SET_PLAIN,		/**< Plain names, as they are listed */
+	ST_SET_ALIAS,		/**< Aliased names, normalized form */
+};
+
+int st_count(const search_table_t *st, enum match_set which);
+bool st_insert_item(search_table_t *, enum match_set which, const char *key,
+	const struct shared_file *sf);
+
+/**
+ * Callback for st_search().
+ *
+ * @param ctx		the search context (opaque for st_search)
+ * @param data		the matched data (shared file, opaque for st_search)
+ * @param limits	whether to apply limits (media type, size restrictions, ...)
+ *
+ * @return TRUE if the match must be accounted as a valid result.
+ */
+typedef bool (*st_search_callback)(void *ctx, const void *data, bool limits);
+
+struct search_request_info;
+
+int st_search(
 	search_table_t *table,
 	const char *search,
+	const struct search_request_info *sri,
 	st_search_callback callback,
-	gpointer ctx,
-	int max_res,
+	void *ctx,
+	uint max_res,
 	struct query_hashvec *qhv);
 
 void st_fill_qhv(const char *search_term, struct query_hashvec *qhv);
