@@ -1,6 +1,4 @@
 /*
- * $Id: move.h 16104 2009-03-10 14:58:09Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -38,6 +36,8 @@
 
 #include "common.h"
 
+struct download;
+
 /*
  * Public interface.
  */
