@@ -1,6 +1,4 @@
 /*
- * $Id: mq_tcp.h 12496 2006-11-26 09:22:49Z cbiere $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -39,13 +37,15 @@
 #include "common.h"
 
 #include "mq.h"
-#include "pmsg.h"
+#include "lib/pmsg.h"
 
 struct txdriver;
+struct gnutella_node;
 
-mqueue_t *mq_tcp_make(
-	gint maxsize, struct gnutella_node *n, struct txdriver *nd);
+void mq_tcp_putq(mqueue_t *q, pmsg_t *mb, const struct gnutella_node *from);
+mqueue_t *mq_tcp_make(int maxsize,
+	struct gnutella_node *n, struct txdriver *nd, const struct mq_uops *uops);
 
 #endif	/* _core_mq_tcp_h_ */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
