@@ -1,6 +1,4 @@
 /*
- * $Id: namesize.c 16106 2009-03-10 15:16:31Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,25 +33,25 @@
 
 #include "common.h"			/* For safe compilation under USE_DMALLOC */
 
-RCSID("$Id: namesize.c 16106 2009-03-10 15:16:31Z rmanfredi $")
-
 #include "namesize.h"
 
 #include "lib/atoms.h"
+#include "lib/hashing.h"
 #include "lib/walloc.h"
+
 #include "lib/override.h"	/* Must be the last header included */
 
 /**
  * Hash a `namesize_t' key.
  */
-guint
-namesize_hash(gconstpointer key)
+uint
+namesize_hash(const void *key)
 {
 	const namesize_t *k = key;
-	guint32 hash;
+	uint32 hash;
 
-	hash = g_str_hash(k->name);
-	hash ^= k->size;
+	hash = string_mix_hash(k->name);
+	hash ^= integer_hash_fast(k->size);
 
 	return hash;
 }
@@ -62,7 +60,7 @@ namesize_hash(gconstpointer key)
  * Compare two `namesize_t' keys.
  */
 int
-namesize_eq(gconstpointer a, gconstpointer b)
+namesize_eq(const void *a, const void *b)
 {
 	const namesize_t *ka = a, *kb = b;
 
@@ -77,7 +75,7 @@ namesize_make(const char *name, filesize_t size)
 {
 	namesize_t *ns;
 
-	ns = walloc(sizeof(*ns));
+	WALLOC(ns);
 	ns->name = atom_str_get(name);
 	ns->size = size;
 
@@ -91,7 +89,7 @@ void
 namesize_free(namesize_t *ns)
 {
 	atom_str_free_null(&ns->name);
-	wfree(ns, sizeof(*ns));
+	WFREE(ns);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
