@@ -1,6 +1,4 @@
 /*
- * $Id: qhit.c 17194 2009-11-05 21:44:06Z rmanfredi $
- *
  * Copyright (c) 2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,32 +33,40 @@
 
 #include "common.h"
 
-RCSID("$Id: qhit.c 17194 2009-11-05 21:44:06Z rmanfredi $")
-
-#include "gtk-gnutella.h"
+#include "gtk-gnutella.h"	/* For GTA_VENDOR_CODE */
 
+#include "qhit.h"
 #include "bsched.h"
 #include "dmesh.h"		/* For dmesh_fill_alternate() */
 #include "ggep.h"
 #include "ggep_type.h"
 #include "gmsg.h"
 #include "gnutella.h"
+#include "ipp_cache.h"
+#include "ipv6-ready.h"
 #include "nodes.h"
-#include "qhit.h"
+#include "search.h"		/* For QUERY_FW2FW_FILE_INDEX */
 #include "settings.h"	/* For listen_ip() */
 #include "share.h"
-#include "ipp_cache.h"
+#include "sockets.h"	/* For socket_listen_port() */
 #include "uploads.h"	/* For count_uploads */
+#include "version.h"	/* For version_get_commit() and version_is_dirty() */
 
 #include "if/gnet_property_priv.h"
 #include "if/core/main.h"			/* For main_get_build() */
 
 #include "lib/array.h"
-#include "lib/getdate.h"
 #include "lib/endian.h"
-#include "lib/misc.h"
+#include "lib/getdate.h"
+#include "lib/hashing.h"
+#include "lib/hset.h"
+#include "lib/product.h"
+#include "lib/pslist.h"
+#include "lib/random.h"
 #include "lib/sequence.h"
+#include "lib/stringify.h"
 #include "lib/tm.h"
+
 #include "lib/override.h"			/* Must be the last header included */
 
 /*
@@ -94,10 +100,11 @@ struct found_struct {
 	size_t max_size;			/**< max query hit size */
 	const struct guid *muid;	/**< the MUID to put in all query hits */
 	const struct array *token;	/**< Optional secure OOB token */
+	hset_t *hs;					/**< Records file indices and SHA1 atoms */
 	qhit_process_t process;		/**< processor once query hit is built */
-	gpointer udata;				/**< processor argument */
-	gboolean open;				/**< Set if found_open() was used */
-	gboolean use_ggep_h;        /**< whether to use GGEP "H" to send SHA-1 */
+	void *udata;				/**< processor argument */
+	unsigned flags;				/**< Set of QHIT_F_* flags */
+	unsigned open:1;			/**< Set if found_open() was used */
 };
 
 static struct found_struct *
@@ -125,10 +132,16 @@ found_add_files(size_t n)
 	found_get()->files += n;
 }
 
-static gboolean
+static bool
 found_ggep_h(void)
 {
-	return found_get()->use_ggep_h;
+	return booleanize(found_get()->flags & QHIT_F_GGEP_H);
+}
+
+static unsigned
+found_flags(void)
+{
+	return found_get()->flags;
 }
 
 static char *
@@ -142,6 +155,21 @@ found_open(void)
 	return &f->data[f->pos];
 }
 
+static host_addr_t
+found_listen_addr(void)
+{
+	host_addr_t ha = listen_addr_primary();
+
+	/* IPv6-Ready */
+
+	if ((found_flags() & QHIT_F_IPV6_ONLY) && !host_addr_is_ipv6(ha)) {
+		host_addr_t ha6 = listen_addr6();
+		return is_host_addr(ha6) ? ha6 : ha;
+	}
+
+	return ha;
+}
+
 static void
 found_close(size_t len)
 {
@@ -174,8 +202,8 @@ found_left(void)
 	return sizeof f->data - f->pos;
 }
 
-static gboolean
-found_write(gconstpointer data, size_t length)
+static bool
+found_write(const void *data, size_t length)
 {
 	struct found_struct *f = found_get();
 
@@ -199,7 +227,8 @@ found_set_header(void)
 {
 	struct found_struct *f = found_get();
 	gnutella_msg_search_results_t *msg;
-	guint32 connect_speed;		/* Connection speed, in kbits/s */
+	uint32 connect_speed;		/* Connection speed, in kbits/s */
+	uint32 ipv4;
 	size_t len;
 
 	g_assert(!f->open);
@@ -214,7 +243,7 @@ found_set_header(void)
 
 		gnutella_header_set_muid(header, f->muid);
 		gnutella_header_set_function(header, GTA_MSG_SEARCH_RESULTS);
-		/* The TTL is overriden later if sending inbound */
+		/* The TTL is overridden later if sending inbound */
 		gnutella_header_set_ttl(header, 1);
 		gnutella_header_set_hops(header, 0);
 		gnutella_header_set_size(header, len);
@@ -229,10 +258,10 @@ found_set_header(void)
 	connect_speed = GNET_PROPERTY(connection_speed);
 	if (GNET_PROPERTY(compute_connection_speed)) {
 		if (GNET_PROPERTY(max_uploads) > 0) {
-			connect_speed = bsched_avg_bps(BSCHED_BWS_OUT);
-			connect_speed = MAX(connect_speed,
-								bsched_bw_per_second(BSCHED_BWS_OUT));
-			connect_speed /= 1024 / 8;
+			uint64 bw = bsched_bw_per_second(BSCHED_BWS_OUT) / 1024;
+			connect_speed = bsched_avg_bps(BSCHED_BWS_OUT) / 1024;
+			connect_speed = MAX(connect_speed, bw);
+			connect_speed = uint32_saturate_mult(connect_speed, 8);	/* bits */
 			if (connect_speed == 0) {
 				/* No b/w limit set and no traffic yet */
 				connect_speed = 32;
@@ -244,8 +273,16 @@ found_set_header(void)
 	/* Upload speed expected per slot */
 	connect_speed /= MAX(1, GNET_PROPERTY(max_uploads));
 
+	/*
+	 * IPv6-Ready support: the QHIT message is architected with an IPv4 address.
+	 * When the address we want to send is an IPv6 one, it needs to be sent
+	 * in a GGEP "6" field and the IPv4 field be set to 127.0.0.0.
+	 */
+
+	ipv4 = ipv6_ready_advertised_ipv4(found_listen_addr());
+
 	gnutella_msg_search_results_set_host_port(msg, socket_listen_port());
-	gnutella_msg_search_results_set_host_ip(msg, host_addr_ipv4(listen_addr()));
+	gnutella_msg_search_results_set_host_ip(msg, ipv4);
 	gnutella_msg_search_results_set_host_speed(msg, connect_speed);
 }
 
@@ -277,8 +314,8 @@ found_token(void)
 }
 
 static void
-found_init(size_t max_size, const struct guid *xuid, gboolean ggep_h,
-	qhit_process_t proc, gpointer udata, const struct array *token)
+found_init(size_t max_size, const struct guid *xuid, unsigned flags,
+	qhit_process_t proc, void *udata, const struct array *token)
 {
 	struct found_struct *f = found_get();
 
@@ -286,33 +323,56 @@ found_init(size_t max_size, const struct guid *xuid, gboolean ggep_h,
 	g_assert(xuid != NULL);
 	g_assert(proc != NULL);
 	g_assert(token != NULL);
+	g_assert(NULL == f->hs);
 
 	f->max_size = max_size;
 	f->muid = xuid;
-	f->use_ggep_h = ggep_h;
+	f->flags = flags;
 	f->process = proc;
 	f->udata = udata;
 	f->open = FALSE;
 	f->token = token;
+	f->hs = hset_create(HASH_KEY_SELF, 0);
 }
 
-static time_t release_date;
+static void
+found_done(void)
+{
+	struct found_struct *f = found_get();
+
+	hset_free_null(&f->hs);
+}
+
+static bool
+found_contains(const void *key)
+{
+	struct found_struct *f = found_get();
+
+	return hset_contains(f->hs, key);
+}
+
+static void
+found_insert(const void *key)
+{
+	struct found_struct *f = found_get();
+
+	hset_insert(f->hs, key);
+}
 
 /**
  * Processor for query hits sent inbound.
  */
 static void
-qhit_send_node(gpointer data, size_t len, gpointer udata)
+qhit_send_node(void *data, size_t len, void *udata)
 {
 	gnutella_node_t *n = udata;
 	gnutella_header_t *packet_head = data;
-	guint ttl;
+	uint ttl;
 
 	if (GNET_PROPERTY(dbg) > 3) {
-		g_message("flushing query hit (%u entr%s, %u bytes sofar) to %s",
-			(guint) found_file_count(),
-			found_file_count() == 1 ? "y" : "ies",
-			(guint) found_size(),
+		g_debug("flushing query hit (%u entr%s, %u bytes sofar) to %s",
+			(uint) found_file_count(), plural_y(found_file_count()),
+			(uint) found_size(),
 			node_addr(n));
 	}
 
@@ -328,7 +388,7 @@ qhit_send_node(gpointer data, size_t len, gpointer udata)
 	 */
 
 	if (gnutella_header_get_hops(&n->header) == 0) {
-		g_warning("qhit_send_node(): hops=0, bug in route_message()?");
+		g_warning("%s(): hops=0, bug in route_message()?", G_STRFUNC);
 		/* Can't send message with TTL=0 */
 		gnutella_header_set_hops(&n->header, 1);
 	}
@@ -343,8 +403,10 @@ qhit_send_node(gpointer data, size_t len, gpointer udata)
 static void
 qhit_log_ggep_write_failure(const char *id)
 {
-	if (GNET_PROPERTY(qhit_debug))
-		g_warning("QHIT could not write GGEP \"%s\" extension", id);
+	if (GNET_PROPERTY(qhit_debug)) {
+		g_warning("QHIT could not write GGEP \"%s\" extension: %s",
+			id, ggep_errstr());
+	}
 }
 
 /**
@@ -363,8 +425,8 @@ flush_match(void)
 	 * It is compatible with BearShare's one in the "open data" section.
 	 */
 
-	memcpy(trailer, "GTKG", 4);	/* Vendor code */
-	trailer[4] = 2;					/* Open data size */
+	memcpy(trailer, GTA_VENDOR_CODE, 4);/* Vendor code */
+	trailer[4] = 2;						/* Open data size */
 	trailer[5] = 0x04 | 0x08 | 0x20;	/* Valid flags we set */
 	trailer[6] = 0x01;				/* Our flags (valid firewall bit) */
 
@@ -383,7 +445,7 @@ flush_match(void)
 	trailer_start = found_open();
 	found_close(0);	/* Nothing written */
 
-	if (!found_write(trailer, sizeof trailer)) /* Store the open trailer */
+	if (!found_write(ARYLEN(trailer))) /* Store the open trailer */
 		goto failure;
 
 	/*
@@ -397,46 +459,28 @@ flush_match(void)
 	ggep_stream_init(&gs, found_open(), QHIT_MAX_GGEP);
 
 	/*
-	 * Build the "GTKGV1" GGEP extension.
+	 * Build the "GTKGV" GGEP extension.
 	 */
 
 	{
-		guint8 major = GTA_VERSION;
-		guint8 minor = GTA_SUBVERSION;
-		const char *revp = GTA_REVCHAR;
-		guint8 revchar = (guint8) revp[0];
-		guint8 patch;
-		guint32 release;
-		guint32 date = release_date;
-		guint32 build;
-		gboolean ok;
+		char buf[GTKGV_MAX_LEN];
+		size_t len;
+		bool ok;
 
-#ifdef GTA_PATCHLEVEL
-		patch = GTA_PATCHLEVEL;
-#else
-		patch = 0;
-#endif
-
-		poke_be32(&release, date);
-		poke_be32(&build, main_get_build());
+		len = ggept_gtkgv_build(ARYLEN(buf));
 
 		ok =
-			ggep_stream_begin(&gs, GGEP_NAME(GTKGV1), 0) &&
-			ggep_stream_write(&gs, &major, 1) &&
-			ggep_stream_write(&gs, &minor, 1) &&
-			ggep_stream_write(&gs, &patch, 1) &&
-			ggep_stream_write(&gs, &revchar, 1) &&
-			ggep_stream_write(&gs, &release, 4) &&
-			ggep_stream_write(&gs, &build, 4) &&
+			ggep_stream_begin(&gs, GGEP_NAME(GTKGV), 0) &&
+			ggep_stream_write(&gs, buf, len) &&
 			ggep_stream_end(&gs);
 
 		if (!ok)
-			qhit_log_ggep_write_failure("GTKGV1");
+			qhit_log_ggep_write_failure("GTKGV");
 	}
 
 	{
 		const struct array *token = found_token();
-		
+
 		if (
 			token->data &&
 			!ggep_stream_pack(&gs, GGEP_NAME(SO), token->data, token->size, 0)
@@ -447,64 +491,15 @@ flush_match(void)
 
 	/*
 	 * Look whether we'll need a "PUSH" GGEP extension to give out
-	 * our current push proxies.  Prepare payload in `proxies'.
+	 * our current push proxies.
 	 */
 
 	if (GNET_PROPERTY(is_firewalled)) {
 		sequence_t *seq = node_push_proxies();
+		unsigned flags = found_flags();
 
-		if (!sequence_is_empty(seq)) {
-			guchar tls_bytes[(QHIT_MAX_PROXIES + 7) / 8];
-			guint tls_index, tls_length;
-			sequence_iter_t *iter;
-			gboolean ok;
-
-			ok = ggep_stream_begin(&gs, GGEP_NAME(PUSH), 0);
-			memset(tls_bytes, 0, sizeof tls_bytes);
-			tls_index = 0;
-			tls_length = 0;
-
-			/*
-			 * We iterate backwards to get the most stable of our push proxies,
-			 * namely those to which we've been connected for the longest time.
-			 */
-
-			iter = sequence_backward_iterator(seq, TRUE);
-
-			while (ok && sequence_iter_has_previous(iter)) {
-				const gnet_host_t *host = sequence_iter_previous(iter);
-				host_addr_t addr = gnet_host_get_addr(host);
-				guint16 port = gnet_host_get_port(host);
-				char proxy[6];
-
-				if (NET_TYPE_IPV4 != host_addr_net(addr))
-					continue;
-
-				poke_be32(&proxy[0], host_addr_ipv4(addr));
-				poke_le16(&proxy[4], port);
-				ok = ggep_stream_write(&gs, proxy, sizeof proxy);
-				if (tls_cache_lookup(addr, port)) {
-					tls_bytes[tls_index >> 3] |= 0x80U >> (tls_index & 7);
-					tls_length = (tls_index >> 3) + 1;
-				}
-				tls_index++;
-				if (tls_index >= QHIT_MAX_PROXIES)
-					break;
-			}
-
-			ok = ok && ggep_stream_end(&gs);
-			sequence_iterator_release(&iter);
-
-			if (!ok)
-				qhit_log_ggep_write_failure("PUSH");
-
-			if (ok && tls_length > 0) {
-				ok = ggep_stream_pack(&gs, GGEP_NAME(PUSH_TLS),
-						tls_bytes, tls_length, 0);
-				if (!ok)
-					qhit_log_ggep_write_failure("PUSH_TLS");
-			}
-		}
+		if (GGEP_OK != ggept_push_pack(&gs, seq, QHIT_MAX_PROXIES, flags))
+			qhit_log_ggep_write_failure("PUSH");
 		sequence_release(&seq);
 	}
 
@@ -518,41 +513,40 @@ flush_match(void)
 		GNET_PROPERTY(give_server_hostname) &&
 		!is_null_or_empty(GNET_PROPERTY(server_hostname))
 	) {
-		gboolean ok;
+		bool ok;
 
 		ok = ggep_stream_pack(&gs, GGEP_NAME(HNAME),
 				GNET_PROPERTY(server_hostname),
-				strlen(GNET_PROPERTY(server_hostname)),
+				vstrlen(GNET_PROPERTY(server_hostname)),
 				0);
 
 		if (!ok)
 			qhit_log_ggep_write_failure("HNAME");
 	}
 
+	/*
+	 * IPv6-Ready support: if our primary listening address is IPv6, then emit
+	 * the GGEP "6" extension.  Likewise if we are also listening on IPv6.
+	 */
+
 	{
-		const host_addr_t addr = listen_addr6();
+		host_addr_t addr;
+		host_addr_t ipv6;
 
-		if (NET_TYPE_IPV6 == host_addr_net(addr)) {
-			const guint8 *ipv6 = host_addr_ipv6(&addr);
+		addr = found_listen_addr();
 
-			if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(IPV6), ipv6, 16, 0))
-				qhit_log_ggep_write_failure("GTKG.IPV6");
+		ipv6 = host_addr_is_ipv6(addr) ? addr : listen_addr6();
+
+		if (is_host_addr(ipv6) && host_addr_is_ipv6(ipv6)) {
+			const uint8 *data = host_addr_ipv6(&ipv6);
+			if (!ggep_stream_pack(&gs, GGEP_NAME(6), data, 16, 0))
+				qhit_log_ggep_write_failure("6");
 		}
 	}
 
 	if (tls_enabled()) {
 		if (!ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0))
 			qhit_log_ggep_write_failure("TLS");
-
-		/*
-		 * FIXME TODO: Drop sending GTKG.TLS after 0.96.7 is out, only
-		 * sending the now standardized "TLS" extension which GTKG has been
-		 * understanding since 0.96.6.
-		 *		--RAM, 2009-10-24
-		 */
-
-		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0))
-			qhit_log_ggep_write_failure("GTKG.TLS");
 	}
 
 	/*
@@ -590,32 +584,113 @@ failure:
 	found_clear();
 }
 
+/**
+ * Generate a random index that is not conflicting with any of the entries
+ * already present in the query hit being constructed.
+ */
+static uint32
+qhit_random_index(void)
+{
+	unsigned i;
+
+	/*
+	 * Generate a random file index, unique to this query hit.
+	 *
+	 * This is for the sake of our own spam detector which will
+	 * frown upon duplicate file indices.
+	 */
+
+	for (i = 0; i < 100; i++) {
+		uint32 file_index = 1 + random_value(INT_MAX - 1);
+
+		if (QUERY_FW2FW_FILE_INDEX == file_index)
+			continue;
+
+		if (!found_contains(uint_to_pointer(file_index)))
+			return file_index;
+	}
+
+	g_error("%s(): no luck with random number generator", G_STRFUNC);
+}
+
 /**
  * Add file to current query hit.
  *
  * @returns TRUE if we inserted the record, FALSE if we refused it due to
- * lack of space.
+ * lack of space, or because the file is no longer shared.
  */
-static gboolean
-add_file(const struct shared_file *sf)
+static bool
+add_file(const shared_file_t *sf)
 {
-	gboolean sha1_available;
+	bool sha1_available;
 	gnet_host_t hvec[QHIT_MAX_ALT];
 	int hcnt = 0;
-	guint32 fs32, fs32_le, idx_le;
+	uint32 fs32, fs32_le, idx_le;
 	int ggep_len;
-	gboolean ok;
+	bool ok;
 	ggep_stream_t gs;
 	size_t left, needed;
-	gpointer start;
-
-	/* Cannot match partially downloaded files */
-	g_assert(!shared_file_is_partial(sf));
+	void *start;
+	bool is_partial;
+	uint32 file_index;
 
+	is_partial = shared_file_is_partial(sf);
 	needed = 8 + 2 + shared_file_name_nfc_len(sf);	/* size of hit entry */
-
 	sha1_available = sha1_hash_available(sf);
 
+	g_return_val_unless(!is_partial || sha1_available, FALSE);
+
+	/*
+	 * Make sure we never insert duplicate indices in a query hit.
+	 *
+	 * This code assumes there will never be any collision between shared
+	 * file indices and pointers to SHA1, which will always hold fortunately
+	 * in real life.
+	 */
+
+	file_index = shared_file_index(sf);
+
+	/*
+	 * A zero file index means the file is no longer known in the library.
+	 *
+	 * This can happen when there was a concurrent rescan happening after
+	 * the query hit list was done but before the query hit is actually
+	 * generated, especially if this is an OOB-delivered hit, computed a
+	 * while back.
+	 */
+
+	if G_UNLIKELY(0 == file_index) {
+		if (GNET_PROPERTY(qhit_debug)) {
+			g_warning("QHIT skipping file %s (de-indexed by library rescan)",
+				shared_file_path(sf));
+		}
+		return FALSE;		/* Cannot add file */
+	}
+
+	if (!is_partial) {
+		/*
+		 * Due to concurrent rescan, the new file index could be conflicting
+		 * with the file index of other entries already present in the query
+		 * hit.  This must not happend, as duplicate file indices are wrong
+		 * and typically indicate spam.
+		 *
+		 * It is also possible that the legitimate valid file index is in
+		 * conflict with a previously generated random file index for a
+		 * partial file.
+		 *
+		 * FIXME:
+		 * We could reserve a region in the uint32 space for these fake partial
+		 * indices to avoid that problem.
+		 */
+
+		if (found_contains(uint_to_pointer(file_index)))
+			file_index = qhit_random_index();
+	} else {
+		file_index = qhit_random_index();
+	}
+
+	found_insert(uint_to_pointer(file_index));
+
 	/*
 	 * In case we emit the SHA1 as a GGEP "H", we'll grow the buffer
 	 * larger necessary, since the extension will take at most 26 bytes,
@@ -627,10 +702,26 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (sha1_available) {
+		const sha1_t *sha1 = shared_file_sha1(sf);
+
+		/*
+		 * They can share twice or more identical files.  Make sure we only
+		 * include each SHA1 once in the query hits we return for a query:
+		 * having multiple entries would waste bandwidth anyway.
+		 */
+
+		if (found_contains(sha1)) {
+			if (GNET_PROPERTY(qhit_debug))
+				g_warning("QHIT not including SHA1 %s twice",
+					sha1_base32(sha1));
+			return TRUE;		/* Entry consumed, but not included */
+		}
+
+		found_insert(sha1);		/* SHA1 are atoms, address is unique */
+
 		needed += 9 + SHA1_BASE32_SIZE;
-		hcnt = dmesh_fill_alternate(shared_file_sha1(sf),
-					hvec, G_N_ELEMENTS(hvec));
-		needed += hcnt * 6 + 6;
+		hcnt = dmesh_fill_alternate(sha1, hvec, N_ITEMS(hvec));
+		needed += hcnt * 18 + 6;	/* Conservative, assumes IPv6 only */
 	}
 
 	/*
@@ -658,7 +749,7 @@ add_file(const struct shared_file *sf)
 
 	fs32 = shared_file_size(sf) >= (1U << 31) ? ~0U : shared_file_size(sf);
 
-	poke_le32(&idx_le, shared_file_index(sf));
+	poke_le32(&idx_le, file_index);
 	if (!found_write(&idx_le, sizeof idx_le))
 		return FALSE;
 	poke_le32(&fs32_le, fs32);
@@ -698,6 +789,38 @@ add_file(const struct shared_file *sf)
 	start = found_open();
 	ggep_stream_init(&gs, start, left);
 
+	/*
+	 * If we matched a partial file, let them know (unless the file is
+	 * being seeded, in which case it is really complete).
+	 *
+	 * For now we don't emit the available ranges (need to build the tree
+	 * of 1 KiB blocks and send numbers of the highest node in the tree
+	 * encompassing an available chunk) in PR0, PR1, PR2, PR3 or PR4 keys.
+	 *
+	 * We just emit the "PRU" key, signaling that it's a partial result
+	 * and that its data is still unverified (since we don't verify available
+	 * chunks using the TTH for now).
+	 *		--RAM, 2011-05-15
+	 */
+
+	if (is_partial && !shared_file_is_finished(sf)) {
+		time_t mtime = shared_file_modification_time(sf);
+		filesize_t available = shared_file_available(sf);
+		char buf[sizeof mtime + sizeof available];
+		uint len;
+
+		/*
+		 * Starting with 0.98.4, we emit a payload in the "PRU" key to indicate
+		 * the last modification time of the file and the amount of bytes
+		 * available on the server.		--RAM, 2012-11-03
+		 */
+
+		len = ggept_stamp_filesize_encode(mtime, available, ARYLEN(buf));
+		ok = ggep_stream_pack(&gs, GGEP_NAME(PRU), buf, len, GGEP_W_COBS);
+		if (!ok)
+			qhit_log_ggep_write_failure("PRU");
+	}
+
 	/*
 	 * Emit the SHA1 as GGEP "H" if they said they understand it. The modern
 	 * way is GGEP "H" for binary URN but only gtk-gnutella implements it.
@@ -706,7 +829,7 @@ add_file(const struct shared_file *sf)
 	if (sha1_available && found_ggep_h()) {
 		const struct sha1 * const sha1 = shared_file_sha1(sf);
 		const struct tth * const tth = shared_file_tth(sf);
-		const guint8 type = tth ? GGEP_H_BITPRINT : GGEP_H_SHA1;
+		const uint8 type = tth ? GGEP_H_BITPRINT : GGEP_H_SHA1;
 
 		ok =
 			ggep_stream_begin(&gs, GGEP_NAME(H), GGEP_W_COBS) &&
@@ -742,14 +865,12 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (fs32 == ~0U) {
-		char buf[sizeof(guint64)];
+		char buf[sizeof(uint64)];
 		int len;
 
-		len = ggept_filesize_encode(shared_file_size(sf), buf);
-
-		g_assert(len > 0 && UNSIGNED(len) <= sizeof buf);
-
+		len = ggept_filesize_encode(shared_file_size(sf), ARYLEN(buf));
 		ok = ggep_stream_pack(&gs, GGEP_NAME(LF), buf, len, GGEP_W_COBS);
+
 		if (!ok)
 			qhit_log_ggep_write_failure("LF");
 	}
@@ -760,78 +881,39 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (hcnt > 0) {
-		guchar tls_bytes[(G_N_ELEMENTS(hvec) + 7) / 8];
-		guint tls_index, tls_length;
-		int i;
+		unsigned flags = found_flags();
 
 		g_assert(hcnt <= QHIT_MAX_ALT);
-		memset(tls_bytes, 0, sizeof tls_bytes);
-		tls_index = 0;
-		tls_length = 0;
 
-		ok = ggep_stream_begin(&gs, GGEP_NAME(ALT), GGEP_W_COBS);
-
-		for (i = 0; ok && i < hcnt; i++) {
-			host_addr_t addr;
-			guint16 port;
-			char alt[6];
-
-			g_assert(start == gs.outbuf);
-			if (NET_TYPE_IPV4 != gnet_host_get_net(&hvec[i]))
-				continue;
-
-			addr = gnet_host_get_addr(&hvec[i]);
-			port = gnet_host_get_port(&hvec[i]);
-			poke_be32(&alt[0], host_addr_ipv4(addr));
-			poke_le16(&alt[4], port);
-			ok = ggep_stream_write(&gs, &alt, sizeof alt);
-
-			if (tls_cache_lookup(addr, port)) {
-				tls_bytes[tls_index >> 3] |= 0x80U >> (tls_index & 7);
-				tls_length = (tls_index >> 3) + 1;
-			}
-			tls_index++;
-		}
-
-		ok = ok && ggep_stream_end(&gs);
-
-		if (!ok)
+		if (GGEP_OK != ggept_alt_pack(&gs, hvec, hcnt, flags))
 			qhit_log_ggep_write_failure("ALT");
-
-		if (ok && tls_length > 0) {
-			ok = ggep_stream_pack(&gs, GGEP_NAME(ALT_TLS),
-					tls_bytes, tls_length, GGEP_W_COBS);
-			if (!ok)
-				qhit_log_ggep_write_failure("ALT_TLS");
-		}
 	}
 
 	{
 		const char *rp = shared_file_relative_path(sf);
-		
+
 		if (rp) {
-			ok = ggep_stream_pack(&gs, GGEP_NAME(PATH), rp, strlen(rp), 0);
+			ok = ggep_stream_pack(&gs, GGEP_NAME(PATH), rp, vstrlen(rp), 0);
 			if (!ok)
 				qhit_log_ggep_write_failure("PATH");
 		}
 	}
 
 	{
-		time_t mtime;	
+		time_t create_time;
 
-		/* FIXME: Create time != modification time */
-		mtime = shared_file_modification_time(sf);
-		if ((time_t) -1 != mtime) {
-			char buf[sizeof(guint64)];
+		create_time = shared_file_creation_time(sf);
+		if ((time_t) -1 != create_time) {
+			char buf[sizeof(uint64)];
 			int len;
 
 			/*
 			 * Suppress negative values (if time_t is signed) as this would
 			 * be interpreted as a date far in this future.
 			 */
-			mtime = MAX(0, mtime);
+			create_time = MAX(0, create_time);
 
-			len = ggept_ct_encode(mtime, buf);
+			len = ggept_ct_encode(create_time, ARYLEN(buf));
 			g_assert(UNSIGNED(len) <= sizeof buf);
 
 			ok = ggep_stream_pack(&gs, GGEP_NAME(CT), buf, len, GGEP_W_COBS);
@@ -888,12 +970,12 @@ add_file(const struct shared_file *sf)
  * have to be sent out-of-bound
  */
 static void
-found_reset(size_t max_size, const struct guid *muid, gboolean ggep_h,
-	qhit_process_t process, gpointer udata, const struct array *token)
+found_reset(size_t max_size, const struct guid *muid, unsigned flags,
+	qhit_process_t process, void *udata, const struct array *token)
 {
 	g_assert(process != NULL);
 	g_assert(max_size <= INT_MAX);
-	found_init(max_size, muid, ggep_h, process, udata, token);
+	found_init(max_size, muid, flags, process, udata, token);
 	found_clear();
 }
 
@@ -905,14 +987,17 @@ found_reset(size_t max_size, const struct guid *muid, gboolean ggep_h,
  * @param files			the list of shared_file_t entries that make up results
  * @param count			the amount of results
  * @param muid			the query's MUID
+ * @param flags			a combination of QHIT_F_* flags
  */
 void
-qhit_send_results(struct gnutella_node *n, GSList *files, int count,
-	const struct guid *muid, gboolean ggep_h)
+qhit_send_results(gnutella_node_t *n, pslist_t *files, int count,
+	const struct guid *muid, unsigned flags)
 {
-	GSList *sl;
+	pslist_t *sl;
 	int sent = 0;
 
+	g_assert(!NODE_TALKS_G2(n));
+
 	/*
 	 * We can't use n->header.muid as the query's MUID but must rely on the
 	 * parameter we're given.  Indeed, we're delivering a local hit here,
@@ -921,10 +1006,10 @@ qhit_send_results(struct gnutella_node *n, GSList *files, int count,
 	 * to forward to other nodes).
 	 */
 
-	found_reset(QHIT_SIZE_THRESHOLD, muid, ggep_h, qhit_send_node, n,
+	found_reset(QHIT_SIZE_THRESHOLD, muid, flags, qhit_send_node, n,
 		&zero_array);
 
-	for (sl = files; sl; sl = g_slist_next(sl)) {
+	PSLIST_FOREACH(files, sl) {
 		shared_file_t *sf = sl->data;
 		if (add_file(sf))
 			sent++;
@@ -934,10 +1019,12 @@ qhit_send_results(struct gnutella_node *n, GSList *files, int count,
 	if (0 != found_file_count())	/* Still some unflushed results */
 		flush_match();				/* Send last packet */
 
-	g_slist_free(files);
+	pslist_free(files);
 
 	if (GNET_PROPERTY(dbg) > 3)
-		g_message("sent %d/%d hits to %s", sent, count, node_addr(n));
+		g_debug("sent %d/%d hits to %s", sent, count, node_addr(n));
+
+	found_done();
 }
 
 /**
@@ -961,21 +1048,23 @@ qhit_send_results(struct gnutella_node *n, GSList *files, int count,
  * @param cb			the processor callback to invoke on each built hit
  * @param udata			argument to pass to callback
  * @param muid			the MUID to use on each generated hit
+ * @param flags			a combination of QHIT_F_* flags
+ * @param token			secure OOBv3 token to include in reply
  */
 void
-qhit_build_results(const GSList *files, int count, size_t max_msgsize,
-	qhit_process_t cb, gpointer udata, const struct guid *muid, gboolean ggep_h,
+qhit_build_results(const pslist_t *files, int count, size_t max_msgsize,
+	qhit_process_t cb, void *udata, const struct guid *muid, unsigned flags,
 	const struct array *token)
 {
-	const GSList *sl;
+	const pslist_t *sl;
 	int sent;
 
 	g_assert(cb != NULL);
 	g_assert(token);
 
-	found_reset(max_msgsize, muid, ggep_h, cb, udata, token);
+	found_reset(max_msgsize, muid, flags, cb, udata, token);
 
-	for (sl = files, sent = 0; sl && sent < count; sl = g_slist_next(sl)) {
+	for (sl = files, sent = 0; sl && sent < count; sl = pslist_next(sl)) {
 		const shared_file_t *sf = sl->data;
 
 		if (add_file(sf))
@@ -985,6 +1074,8 @@ qhit_build_results(const GSList *files, int count, size_t max_msgsize,
 	if (0 != found_file_count())	/* Still some unflushed results */
 		flush_match();				/* Send last packet */
 
+	found_done();
+
 	/*
 	 * Nothing to free, since everything is the property of the calling module.
 	 */
@@ -996,7 +1087,7 @@ qhit_build_results(const GSList *files, int count, size_t max_msgsize,
 void
 qhit_init(void)
 {
-	release_date = date2time(GTA_RELEASE, tm_time());
+	/* Nada */
 }
 
 /**
