@@ -1,6 +1,4 @@
 /*
- * $Id: qhit.h 16106 2009-03-10 15:16:31Z rmanfredi $
- *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -38,22 +36,39 @@
 
 #include "common.h"
 
-typedef void (*qhit_process_t)(gpointer data, size_t len, gpointer udata);
+typedef void (*qhit_process_t)(void *data, size_t len, void *udata);
+
+/**
+ * Query hit generation flags.
+ */
+#define QHIT_F_GGEP_H		(1U << 0)	/**< Host supports GGEP "H" */
+#define QHIT_F_IPV6			(1U << 1)	/**< Host accepts IPv6 addresses */
+#define QHIT_F_IPV6_ONLY	(1U << 2)	/**< Host only wants IPv6 addresses */
+
+/**
+ * Query hit generation flags for G2 queries.
+ */
+#define QHIT_F_G2_URL		(1U << 31)	/**< Wants URL (stating we share it) */
+#define QHIT_F_G2_DN		(1U << 30)	/**< Wants DN (distinguished name) */
+#define QHIT_F_G2_ALT		(1U << 29)	/**< Wants ALT (alt-locs) */
 
 /*
  * Public interface.
  */
 
 struct gnutella_node;
-struct array; 
+struct array;
+struct guid;
+struct pslist;
 
 void qhit_init(void);
 void qhit_close(void);
 
-void qhit_send_results(struct gnutella_node *n, GSList *files, int count,
-	const struct guid *muid, gboolean ggep_h);
-void qhit_build_results(const GSList *files, int count, size_t max_msgsize,
-	qhit_process_t cb, gpointer udata, const struct guid *muid, gboolean ggep_h,
+void qhit_send_results(struct gnutella_node *n, struct pslist *files, int count,
+	const struct guid *muid, unsigned flags);
+void qhit_build_results(const struct pslist *files,
+	int count, size_t max_msgsize,
+	qhit_process_t cb, void *udata, const struct guid *muid, unsigned flags,
 	const struct array *token);
 
 #endif /* _core_qhit_h_ */
