@@ -1,6 +1,4 @@
 /*
- * $Id: spam_sha1.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
  * Copyright (c) 2007, Christian Biere
  * Copyright (c) 2004, Raphael Manfredi
  * Copyright (c) 2003, Markus Goetz
@@ -21,7 +19,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -41,24 +39,24 @@
 
 #include "common.h"
 
-RCSID("$Id: spam_sha1.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
 #include "spam.h"
+
 #include "settings.h"
 
-#include "lib/atoms.h"
 #include "lib/ascii.h"
+#include "lib/atoms.h"
 #include "lib/dbmw.h"
 #include "lib/file.h"
-#include "lib/glib-missing.h"
 #include "lib/halloc.h"
 #include "lib/path.h"
 #include "lib/sorted_array.h"
+#include "lib/str.h"
 #include "lib/watcher.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 #include "if/bridge/c2ui.h"
+#include "if/core/settings.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -87,7 +85,7 @@ struct sha1_lut {
 
 static struct sha1_lut sha1_lut;
 
-static inline int
+static inline G_PURE int
 sha1_cmp_func(const void *a, const void *b)
 {
 	return sha1_cmp(a, b);
@@ -105,15 +103,15 @@ spam_lut_create(void)
 		dbmap_t *dm;
 		char *path;
 
-		path = make_pathname(settings_config_dir(), db_spambase);
-		dm = dbmap_create_sdbm(SHA1_RAW_SIZE, spam_sha1_what, path,
+		path = make_pathname(settings_gnet_db_dir(), db_spambase);
+		dm = dbmap_create_sdbm(SHA1_RAW_SIZE, NULL, spam_sha1_what, path,
 			O_CREAT | O_TRUNC | O_RDWR, S_IRUSR | S_IWUSR);
 		HFREE_NULL(path);
 
 		if (NULL == dm) {
 			if (GNET_PROPERTY(spam_debug))
-				g_warning("unable to create SDBM database for %s: %s",
-					db_spambase, g_strerror(errno));
+				g_warning("unable to create SDBM database for %s: %m",
+					db_spambase);
 			sha1_lut.tab = sorted_array_new(sizeof(struct sha1), sha1_cmp_func);
 		} else {
 			/*
@@ -151,7 +149,7 @@ static int
 sha1_collision(const void *a, const void *b)
 {
 	(void) a;
-	g_warning("spam_sha1_sync(): Removing duplicate SHA-1 %s", sha1_base32(b));
+	g_warning("spam_sha1_sync(): removing duplicate SHA-1 %s", sha1_base32(b));
 	return 1;
 }
 
@@ -171,7 +169,7 @@ spam_sha1_sync(void)
 
 		dbmap_set_cachesize(dm, SPAM_DB_RUN_CACHESIZE);
 		sha1_lut.d.dw = dbmw_create(dm, spam_sha1_what,
-			SHA1_RAW_SIZE, 0, 0,
+			0, 0,
 			NULL, NULL, NULL,
 			SPAM_DBMW_CACHESIZE, sha1_hash, sha1_eq);
 	}
@@ -189,46 +187,43 @@ spam_sha1_sync(void)
  *
  * @returns the amount of entries loaded or -1 on failure.
  */
-static gulong
+static ulong G_COLD
 spam_sha1_load(FILE *f)
 {
 	char line[1024];
-	guint line_no = 0;
-	gulong item_count = 0;
+	uint line_no = 0;
+	ulong item_count = 0;
 
 	g_assert(f);
 
 	spam_lut_create();
 	sha1_lut.state = SPAM_LOADING;
 
-	while (fgets(line, sizeof line, f)) {
+	while (fgets(ARYLEN(line), f)) {
 		const struct sha1 *sha1;
-		char *nl;
+		size_t len;
 
 		line_no++;
 
-		nl = strchr(line, '\n');
-		if (!nl) {
+		if (!file_line_chomp_tail(ARYLEN(line), &len)) {
 			/*
 			 * If the line is too long or unterminated the file is either
 			 * corrupt or was manually edited without respecting the
 			 * exact format. If we continued, we would read from the
 			 * middle of a line which could be the filename or ID.
 			 */
-			g_warning("spam_sha1_load(): "
-				"line too long or missing newline in line %u",
-				line_no);
+			g_warning("%s(): line %u too long or missing newline",
+				G_STRFUNC, line_no);
 			break;
 		}
-		*nl = '\0';
 
 		/* Skip comments and empty lines */
-		if (*line == '#' || *line == '\0')
+		if (file_line_is_skipable(line))
 			continue;
 
-		if (strlen(line) < SHA1_BASE32_SIZE) {
-			g_warning("spam_sha1_load(): SHA-1 has wrong length in line %u.",
-				line_no);
+		if (len < SHA1_BASE32_SIZE) {
+			g_warning("%s(): SHA-1 has wrong length %zu in line %u.",
+				G_STRFUNC, len, line_no);
 			continue;
 		}
 
@@ -237,15 +232,14 @@ spam_sha1_load(FILE *f)
 		 * the leading SHA-1 is separated from the trailing data.
 		 */
 		if (is_ascii_alnum(line[SHA1_BASE32_SIZE])) {
-			g_warning("spam_sha1_load(): Bad SHA-1 in line %u.",
-				line_no);
+			g_warning("%s(): bad SHA-1 in line %u.", G_STRFUNC, line_no);
 			continue;
 		}
-		
+
 		sha1 = base32_sha1(line);
 		if (NULL == sha1) {
-			g_warning("spam_sha1_load(): Could not parse SHA-1 in line %u.",
-				line_no);
+			g_warning("%s(): could not parse SHA-1 in line %u.",
+				G_STRFUNC, line_no);
 			continue;
 		}
 		spam_sha1_add(sha1);
@@ -256,17 +250,17 @@ spam_sha1_load(FILE *f)
 	sha1_lut.state = SPAM_LOADED;
 
 	if (GNET_PROPERTY(spam_debug))
-		g_message("loaded %lu SPAM SHA-1 keys", item_count);
+		g_debug("loaded %lu SPAM SHA-1 keys", item_count);
 
 	return item_count;
 }
 
 /**
- * Watcher callback, invoked when the file from which we read the spam 
+ * Watcher callback, invoked when the file from which we read the spam
  * changed.
  */
 static void
-spam_sha1_changed(const char *filename, gpointer unused_udata)
+spam_sha1_changed(const char *filename, void *unused_udata)
 {
 	FILE *f;
 
@@ -275,13 +269,13 @@ spam_sha1_changed(const char *filename, gpointer unused_udata)
 	f = file_fopen(filename, "r");
 	if (f) {
 		char buf[80];
-		gulong count;
+		ulong count;
 
 		spam_sha1_close();
 		count = spam_sha1_load(f);
 		fclose(f);
 
-		gm_snprintf(buf, sizeof(buf), "Reloaded %lu spam SHA-1 items.", count);
+		str_bprintf(ARYLEN(buf), "Reloaded %lu spam SHA-1 items.", count);
 		gcu_statusbar_message(buf);
 	}
 }
@@ -310,21 +304,17 @@ spam_sha1_retrieve_from_file(FILE *f, const char *path, const char *filename)
 static void
 spam_sha1_retrieve(void)
 {
-	static file_path_t fp[3];
-	guint num_fp = G_N_ELEMENTS(fp) - 1;
+	file_path_t fp[4];
 	FILE *f;
 	int idx;
-	
-	file_path_set(&fp[0], settings_config_dir(), spam_sha1_file);
-	file_path_set(&fp[1], PRIVLIB_EXP, spam_sha1_file);
+	unsigned length;
 
-#ifndef OFFICIAL_BUILD
-	file_path_set(&fp[2], PACKAGE_EXTRA_SOURCE_DIR, spam_sha1_file);
-	num_fp++;
-#endif	/* !OFFICIAL_BUILD */
+	length = settings_file_path_load(fp, spam_sha1_file, SFP_DFLT);
 
-	f = file_config_open_read_norename_chosen(spam_sha1_what, fp, num_fp, &idx);
-	if (f) {
+	g_assert(length <= N_ITEMS(fp));
+
+	f = file_config_open_read_norename_chosen(spam_sha1_what, fp, length, &idx);
+	if (f != NULL) {
 		spam_sha1_retrieve_from_file(f, fp[idx].dir, fp[idx].name);
 		fclose(f);
 	}
@@ -360,7 +350,7 @@ spam_sha1_close(void)
  * @param sha1 the SHA-1 to check.
  * @returns TRUE if found, and FALSE if not.
  */
-gboolean
+bool
 spam_sha1_check(const struct sha1 *sha1)
 {
 	g_return_val_if_fail(sha1, FALSE);
