@@ -1,6 +1,4 @@
 /*
- * $Id: thex_upload.h 16106 2009-03-10 15:16:31Z rmanfredi $
- *
  * Copyright (c) 2007, Christian Biere & Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -45,6 +43,7 @@
  * Flags for thex_upload_open().
  */
 enum {
+	/* Must be the same value as BH_F_CHUNKED */
 	THEX_UPLOAD_F_CHUNKED = 1 << 2		/**< Emit chunked data */
 };
 
@@ -57,7 +56,7 @@ struct wrap_io;
 size_t thex_upload_get_content_length(const struct shared_file *sf);
 
 struct special_upload *thex_upload_open(
-	gpointer owner,
+	void *owner,
 	const struct gnutella_host *host,
 	const struct shared_file *sf,
 	special_upload_writable_t writable,
