@@ -1,6 +1,4 @@
 /*
- * $Id: tsync.h 13045 2007-03-11 03:06:23Z cbiere $
- *
  * Copyright (c) 2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -45,16 +43,17 @@
  */
 
 struct gnutella_node;
+struct nid;
 
 void tsync_init(void);
 void tsync_close(void);
 
-void tsync_send(struct gnutella_node *n, const node_id_t node_id);
+void tsync_send(struct gnutella_node *n, const struct nid *node_id);
 void tsync_send_timestamp(tm_t *orig, tm_t *final);
 void tsync_got_request(struct gnutella_node *n, tm_t *got);
 void tsync_got_reply(struct gnutella_node *n,
-	tm_t *sent, tm_t *received, tm_t *replied, tm_t *got, gboolean ntp);
+	tm_t *sent, tm_t *received, tm_t *replied, tm_t *got, bool ntp);
 
 #endif /* _core_tsync_h_ */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
