@@ -1,6 +1,4 @@
 /*
- * $Id: upload_stats.h 17029 2009-10-04 15:41:06Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -36,7 +34,7 @@
 #ifndef _core_upload_stats_h_
 #define _core_upload_stats_h_
 
-#include "common.h" 
+#include "common.h"
 
 struct shared_file;
 
@@ -47,7 +45,7 @@ void upload_stats_file_begin(const struct shared_file *sf);
 void upload_stats_file_complete(const struct shared_file *sf, filesize_t done);
 void upload_stats_file_requested(const struct shared_file *sf);
 void upload_stats_flush_if_dirty(void);
-void upload_stats_load_history(const char *ul_history_file_name);
+void upload_stats_load_history(void);
 void upload_stats_enforce_local_filename(const struct shared_file *sf);
 
 #endif /* _core_upload_stats_h_ */
