@@ -1,6 +1,4 @@
 /*
- * $Id: version.c 17247 2010-02-14 16:14:14Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,27 +33,31 @@
 
 #include "common.h"
 
-RCSID("$Id: version.c 17247 2010-02-14 16:14:14Z rmanfredi $")
-
 #include "gtk-gnutella.h"
 
 #include "version.h"
 #include "token.h"
 #include "settings.h"
+#include "tls_common.h"
+
+#include "if/core/main.h"		/* For gtk_version_string() */
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
-#include "if/core/main.h"
-
-#include "lib/atoms.h"
 #include "lib/ascii.h"
+#include "lib/base16.h"
 #include "lib/getdate.h"
-#include "lib/glib-missing.h"
+#include "lib/log.h"
+#include "lib/misc.h"
+#include "lib/omalloc.h"
 #include "lib/parse.h"
-#include "lib/tm.h"
+#include "lib/product.h"
+#include "lib/str.h"			/* For str_bprintf()  and str_bcatf() */
 #include "lib/timestamp.h"
+#include "lib/tm.h"
 #include "lib/utf8.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 #define SECS_PER_DAY	86400
@@ -64,9 +66,10 @@ const char *version_string = NULL;
 const char *version_short_string = NULL;
 
 static version_t our_version;
+static version_ext_t our_ext_version;
 static version_t last_rel_version;
 static version_t last_dev_version;
-static guint8 version_code;
+static uint8 version_code;
 
 /**
  * Get version string.
@@ -80,24 +83,108 @@ version_get_string(void)
 /**
  * Get version code (year/month coded in one byte).
  */
-guint8
+uint8
 version_get_code(void)
 {
 	return version_code;
 }
 
+/**
+ * Is our version indicating dirtyness?
+ */
+bool
+version_is_dirty(void)
+{
+	return our_ext_version.dirty;
+}
+
+/**
+ * Get commit string and commit length (amount of valid nybbles).
+ */
+const struct sha1 *
+version_get_commit(uint8 *len)
+{
+	if (len != NULL)
+		*len = our_ext_version.commit_len;
+
+	return &our_ext_version.commit;
+}
+
 /**
  * Dump original version string and decompiled form to stdout.
  */
 static void
 version_dump(const char *str, const version_t *ver, const char *cmptag)
 {
-	g_message("VERSION%s \"%s\": "
+	g_debug("VERSION%s \"%s\": "
 		"major=%u minor=%u patch=%u tag=%c taglevel=%u build=%u",
 		cmptag, str, ver->major, ver->minor, ver->patchlevel,
 		ver->tag ? ver->tag : ' ', ver->taglevel, ver->build);
 }
 
+/**
+ * @return a user-friendly description of the extended version.
+ * NB: returns pointer to static data.
+ */
+const char *
+version_ext_str(const version_ext_t *vext, bool full)
+{
+	static char str[120];
+	const version_t *ver = &vext->version;
+	int rw;
+	bool has_extra = FALSE;
+	bool need_closing = FALSE;
+
+	rw = str_bprintf(ARYLEN(str), "%u.%u", ver->major, ver->minor);
+
+	if (ver->patchlevel)
+		rw += str_bprintf(ARYPOSLEN(str, rw), ".%u", ver->patchlevel);
+
+	if (ver->tag) {
+		rw += str_bprintf(ARYPOSLEN(str, rw), "%c", ver->tag);
+		if (ver->taglevel)
+			rw += str_bprintf(ARYPOSLEN(str, rw), "%u", ver->taglevel);
+	}
+
+	if (ver->build)
+		rw += str_bprintf(ARYPOSLEN(str, rw), "-%u", ver->build);
+
+	if (vext->commit_len != 0) {
+		static char digest[SHA1_BASE16_SIZE + 1];
+		size_t offset = MIN(vext->commit_len, SHA1_BASE16_SIZE);
+
+		sha1_to_base16_buf(&vext->commit, ARYLEN(digest));
+		digest[offset] = '\0';
+		rw += str_bprintf(ARYPOSLEN(str, rw), "-g%s", digest);
+	}
+
+	if (vext->dirty)
+		rw += str_bprintf(ARYPOSLEN(str, rw), "-dirty");
+
+	if (ver->timestamp || (full && vext->osname != NULL)) {
+		rw += str_bprintf(ARYPOSLEN(str, rw), " (");
+		need_closing = TRUE;
+	}
+
+	if (ver->timestamp) {
+		struct tm *tmp = localtime(&ver->timestamp);
+		rw += str_bprintf(ARYPOSLEN(str, rw), "%d-%02d-%02d",
+			tmp->tm_year + 1900, tmp->tm_mon + 1, tmp->tm_mday);
+		has_extra = TRUE;
+	}
+
+	if (full && vext->osname != NULL) {
+		if (has_extra)
+			rw += str_bprintf(ARYPOSLEN(str, rw), "; ");
+		rw += str_bprintf(ARYPOSLEN(str, rw), "%s", vext->osname);
+	}
+
+	if (need_closing)
+		rw += str_bprintf(ARYPOSLEN(str, rw), ")");
+
+	return str;
+}
+
 /**
  * @return a user-friendly description of the version.
  * NB: returns pointer to static data.
@@ -108,23 +195,23 @@ version_str(const version_t *ver)
 	static char str[80];
 	int rw;
 
-	rw = gm_snprintf(str, sizeof(str), "%u.%u", ver->major, ver->minor);
+	rw = str_bprintf(ARYLEN(str), "%u.%u", ver->major, ver->minor);
 
 	if (ver->patchlevel)
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, ".%u", ver->patchlevel);
+		rw += str_bprintf(ARYPOSLEN(str, rw), ".%u", ver->patchlevel);
 
 	if (ver->tag) {
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%c", ver->tag);
+		rw += str_bprintf(ARYPOSLEN(str, rw), "%c", ver->tag);
 		if (ver->taglevel)
-			rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%u", ver->taglevel);
+			rw += str_bprintf(ARYPOSLEN(str, rw), "%u", ver->taglevel);
 	}
 
 	if (ver->build)
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "-%u", ver->build);
+		rw += str_bprintf(ARYPOSLEN(str, rw), "-%u", ver->build);
 
 	if (ver->timestamp) {
 		struct tm *tmp = localtime(&ver->timestamp);
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, " (%d-%02d-%02d)",
+		rw += str_bprintf(ARYPOSLEN(str, rw), " (%d-%02d-%02d)",
 			tmp->tm_year + 1900, tmp->tm_mon + 1, tmp->tm_mday);
 	}
 
@@ -153,24 +240,22 @@ version_stamp(const char *str, version_t *ver)
 	 * being used starting 2004-03-02.
 	 */
 
-	p = strchr(str, '(');
+	p = vstrchr(str, '(');
 	if (p) {
 		const char *end;
 
 		p++;
-		end = strchr(p, ';');
+		end = vstrchr(p, ';');
 		if (end == NULL)
-			end = strchr(p, ')');		/* Only date present: short version */
+			end = vstrchr(p, ')');		/* Only date present: short version */
 		if (end) {
-			size_t size = end - p + 1;
-
 			/*
 			 * Using date2time() will allow us to possibly change the date
 			 * format in the future, without impacting the ability of older
 			 * servents to parse it.
 			 */
 
-			g_strlcpy(stamp, p, MIN(size, sizeof(stamp)));
+			clamp_strncpy(ARYLEN(stamp), p, end - p);
 			ver->timestamp = date2time(stamp, tm_time());
 
 			if (ver->timestamp == -1) {
@@ -182,16 +267,102 @@ version_stamp(const char *str, version_t *ver)
 	}
 }
 
+/**
+ * Parse extended build information (git's commit string + dirtyness)
+ * from a version string, starting at the first byte after the build number.
+ *
+ * A typical extended build information would look like:
+ *
+ *		-g5c02b-dirty
+ *
+ * The hexadecimal string after "-g" points to the leading git commit ID.
+ * The "-dirty" indication is present when the binary was built with
+ * uncommitted changes.
+ *
+ * @returns TRUE if we parsed the extended build information properly, FALSE
+ * if we were not facing a proper extension.
+ */
+static bool
+version_ext_parse(const char *str, version_ext_t *vext)
+{
+	const char *v;
+	const char *e;
+	size_t commit_len;
+	char commit[SHA1_BASE16_SIZE + 1];
+
+	v = is_strprefix(str, "-g");
+	if (NULL == v) {
+		/*
+		 * There may not be any git tag, but there may be a dirty indication.
+		 */
+
+		e = is_strprefix(str, "dirty");
+		if (e != NULL) {
+			vext->dirty = TRUE;
+			return TRUE;
+		} else if (' ' == *str || '\0' == *str) {
+			return TRUE;
+		}
+		return FALSE;
+	}
+
+	/*
+	 * Compute the length of the hexadecimal string, up to the next '-' or
+	 * the next ' ' or the end of the string.
+	 */
+
+	e = vstrchr(v, ' ');
+	if (e != NULL) {
+		const char *d;
+		d = vstrchr(v, '-');
+		if (NULL == d || d > e) {
+			commit_len = e - v;		/* '-' before ' ' or no '-' at all */
+		} else {
+			commit_len = d - v;		/* '-' after ' ' */
+			e = d;
+		}
+	} else {
+		e = vstrchr(v, '-');
+		if (e != NULL) {
+			commit_len = e - v;
+		} else {
+			commit_len = vstrlen(v);
+		}
+	}
+
+	if (commit_len > SHA1_BASE16_SIZE)
+		return FALSE;
+
+	vext->commit_len = commit_len;
+
+	ZERO(&commit);
+	memcpy(commit, v, commit_len);
+
+	ZERO(&vext->commit);
+	(void) base16_decode(VARLEN(vext->commit), commit, SHA1_BASE16_SIZE);
+
+	if (e != NULL && is_strprefix(e, "-dirty")) {
+		vext->dirty = TRUE;
+	} else {
+		vext->dirty = FALSE;
+	}
+
+	return TRUE;
+}
 
 /**
  * Parse gtk-gnutella's version number in User-Agent/Server string `str'
  * and extract relevant information into `ver'.
  *
+ * @param str	the version string to be parsed
+ * @param ver	the structure filled with parsed information
+ * @param end	filled with a pointer to the first unparsed character
+ *
  * @returns TRUE if we parsed a gtk-gnutella version correctly, FALSE if we
  * were not facing a gtk-gnutella version, or if we did not recognize it.
  */
-static gboolean
-version_parse(const char *str, version_t *ver)
+static bool
+version_parse(const char *str, version_t *ver, const char **end)
 {
 	const char *v;
 	int error;
@@ -227,6 +398,27 @@ version_parse(const char *str, version_t *ver)
 	 *    gtk-gnutella/0.90b2-3404 (04/04/2002; X11; FreeBSD 4.6-STABLE i386)
 	 *
 	 * where the '-3404' introduces the build number.
+	 *
+	 * Since switching to git (2011-09-11), the SVN build is no longer present
+	 * but rather the output of "git describe" is used to show any difference
+	 * with the latest release tag.
+	 *
+	 * A release would be described as:
+	 *
+	 *    gtk-gnutella/0.97.1 (2011-09-11; GTK1; Linux i686)
+	 *
+	 * but any change on top of that would be seen as:
+	 *
+	 *    gtk-gnutella/0.97.1-24-g844b7 (2011-09-11; GTK1; Linux i686)
+	 *
+	 * to indicate that 24 commits were made after the release, and the commit
+	 * ID is 844b7... (only enough hexadecimal digits are output to make the
+	 * string unique at the time it was generated.
+	 *
+	 * If furthermore a build is done from a dirty git repository, the string
+	 * "-dirty" is appended after the commit ID:
+	 *
+	 *    gtk-gnutella/0.97.1-24-g844b7-dirty (2011-09-11; GTK1; Linux i686)
 	 */
 
 	if (NULL == (v = is_strprefix(str, GTA_PRODUCT_NAME "/")))
@@ -273,17 +465,24 @@ version_parse(const char *str, version_t *ver)
 	}
 
 	/*
-	 * Parse optional "-build".
+	 * Parse optional "-build" (legacy SVN) or "-change-bxxxxxxxx" (git
+	 * version scheme) to be able to sort identical versions with distinct
+	 * changes applied to them.
 	 */
 
 	if ('-' == *v) {
 		v++;
-		ver->build = parse_uint32(v, &v, 10, &error);
-		if (error)
-			return FALSE;
+		if (!is_strprefix(v, "dirty")) {
+			ver->build = parse_uint32(v, &v, 10, &error);
+			if (error)
+				return FALSE;
+		}
 	} else
 		ver->build = 0;
 
+	if (end != NULL)
+		*end = v;
+
 	if (GNET_PROPERTY(version_debug) > 1)
 		version_dump(str, ver, "#");
 
@@ -295,7 +494,7 @@ version_parse(const char *str, version_t *ver)
  * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
 static int
-version_tagcmp(guchar a, guchar b)
+version_tagcmp(uchar a, uchar b)
 {
 	if (a == b)
 		return 0;
@@ -358,10 +557,10 @@ version_build_cmp(const version_t *a, const version_t *b)
  *
  * @returns OK if we were able to parse correctly.
  */
-gboolean
+bool
 version_fill(const char *version, version_t *vs)
 {
-	if (!version_parse(version, vs))
+	if (!version_parse(version, vs, NULL))
 		return FALSE;
 
 	version_stamp(version, vs);			/* Optional, set to 0 if missing */
@@ -373,28 +572,28 @@ version_fill(const char *version, version_t *vs)
  * Invoked when a newer version is found.
  */
 static void
-version_new_found(const char *text, gboolean stable)
+version_new_found(const char *text, bool stable)
 {
     static char last_stable[256] = "";
     static char last_dev[256] = "";
 	char s[1024];
 
-    if (stable)
-        utf8_strlcpy(last_stable, text, sizeof last_stable);
-    else
-        utf8_strlcpy(last_dev, text, sizeof last_dev);
+	if (stable)
+		utf8_strlcpy(last_stable, text, sizeof last_stable);
+	else
+		utf8_strlcpy(last_dev, text, sizeof last_dev);
 
 	if ('\0' != last_stable[0] && '\0' != last_dev[0]) {
-		gm_snprintf(s, sizeof s,
-			_(" - Newer versions available: release %s / from SVN %s"),
+		str_bprintf(ARYLEN(s),
+			_(" - Newer versions available: release %s / from git %s"),
 			last_stable, last_dev);
 	} else if ('\0' != last_stable[0]) {
-		gm_snprintf(s, sizeof s,
+		str_bprintf(ARYLEN(s),
 			_(" - Newer version available: release %s"),
 			last_stable);
 	} else if ('\0' != last_dev[0]) {
-		gm_snprintf(s, sizeof s,
-			_(" - Newer version available: from SVN %s"),
+		str_bprintf(ARYLEN(s),
+			_(" - Newer version available: from git %s"),
 			last_dev);
 	}
 
@@ -410,17 +609,31 @@ version_new_found(const char *text, gboolean stable)
  * @returns TRUE if we properly checked the version, FALSE if we got something
  * looking as gtk-gnutella but which failed the token-based sanity checks.
  */
-gboolean
+bool
 version_check(const char *str, const char *token, const host_addr_t addr)
 {
 	version_t their_version;
+	version_ext_t their_version_ext;
 	version_t *target_version;
 	int cmp;
 	const char *version;
+	const char *end;
+	bool extended;
 
-	if (!version_parse(str, &their_version))
+	if (!version_parse(str, &their_version, &end))
 		return TRUE;			/* Not gtk-gnutella, or unparseable */
 
+	/*
+	 * Check for extended version information (git commit ID, dirty status).
+	 */
+
+	ZERO(&their_version_ext);
+	extended = version_ext_parse(end, &their_version_ext);
+	if (!extended) {
+		/* Structure could have been partially filled */
+		ZERO(&their_version_ext);
+	}
+
 	/*
 	 * Is their version a development one, or a release?
 	 */
@@ -442,9 +655,10 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 	 */
 
 	version_stamp(str, &their_version);
+	their_version_ext.version = their_version;		/* Struct copy */
 
 	if (GNET_PROPERTY(version_debug) > 3)
-		g_message("VERSION time=%d", (int) their_version.timestamp);
+		g_debug("VERSION time=%d", (int) their_version.timestamp);
 
 	/*
 	 * If version claims something older than TOKEN_START_DATE, then
@@ -456,12 +670,12 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 
 		if (token == NULL) {
             if (GNET_PROPERTY(version_debug)) {
-                g_message("got GTKG vendor string \"%s\" without token!", str);
+                g_debug("got GTKG vendor string \"%s\" without token!", str);
             }
 			return FALSE;	/* Can't be correct */
 		}
 
-		error = tok_version_valid(str, token, strlen(token), addr);
+		error = tok_version_valid(str, token, vstrlen(token));
 
 		/*
 		 * Unfortunately, if our token has expired, we can no longer
@@ -479,7 +693,7 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 
 		if (error != TOK_OK) {
             if (GNET_PROPERTY(version_debug)) {
-                g_message("vendor string \"%s\" [%s] has wrong token "
+                g_debug("vendor string \"%s\" [%s] has wrong token "
                     "\"%s\": %s ", str, host_addr_to_string(addr), token,
                     tok_strerror(error));
             }
@@ -509,11 +723,11 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 		target_version == &last_rel_version
 	) {
 		if (GNET_PROPERTY(version_debug) > 3)
-			g_message("VERSION is a SVN update of a release");
+			g_debug("VERSION is a SVN update of a release");
 
 		if (version_build_cmp(&last_dev_version, &their_version) > 0) {
 			if (GNET_PROPERTY(version_debug) > 3)
-				g_message("VERSION is less recent than latest dev we know");
+				g_debug("VERSION is less recent than latest dev we know");
 			return TRUE;
 		}
 		target_version = &last_dev_version;
@@ -544,7 +758,7 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 	 */
 
 	if (GNET_PROPERTY(version_debug) > 1)
-		g_message("more recent %s VERSION \"%s\"",
+		g_debug("more recent %s VERSION \"%s\"",
 			target_version == &last_dev_version ? "dev" : "rel", str);
 
 	*target_version = their_version;		/* struct copy */
@@ -556,7 +770,7 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 	 * updates to them: they're probably not interested.
 	 */
 
-	version =  version_str(&their_version);
+	version =  version_ext_str(&their_version_ext, FALSE);	/* No OS name */
 
 	g_message("more recent %s version of gtk-gnutella: %s",
 		target_version == &last_dev_version ? "development" : "released",
@@ -571,23 +785,21 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 }
 
 /**
- * Generates the version string. This function does not require any
- * initialization, thus may be called very early e.g., for showing
- * version information if the executable was invoked with --version
- * as argument.
+ * Generates the version string.
+ *
+ * @param hide	if TRUE, hides build information
  *
  * @return A pointer to a static buffer holding the version string.
  */
-const char *
-version_build_string(void)
+static const char * G_COLD
+version_build_internal(bool hide)
 {
-	static gboolean initialized;
+	static bool initialized;
 	static char buf[128];
+	static const char *sysname = "Unknown";
+	static const char *machine;
 
 	if (!initialized) {
-		const char *sysname = "Unknown";
-		const char *machine = NULL;
-
 		initialized = TRUE;
 
 #ifdef HAS_UNAME
@@ -598,40 +810,77 @@ version_build_string(void)
 				sysname = un.sysname;
 				machine = un.machine;
 			} else {
-				g_warning("uname() failed: %s", g_strerror(errno));
+				s_carp("uname() failed: %m");
 			}
 		}
 #endif /* HAS_UNAME */
-
-		gm_snprintf(buf, sizeof buf,
-			GTA_PRODUCT_NAME "/%s-%u (%s; %s; %s%s%s)",
-			GTA_VERSION_NUMBER, main_get_build(),
-			GTA_RELEASE, gtk_gnutella_interface(),
-			sysname,
-			machine && machine[0] ? " " : "",
-			machine ? machine : "");
 	}
+
+	str_bprintf(ARYLEN(buf),
+		"%s/%s%s (%s; %s; %s%s%s)",
+		product_name(), product_version(),
+		hide ? "" : product_build_full(),
+	   	product_date(), product_interface(),
+		sysname,
+		machine && machine[0] ? " " : "",
+		machine ? machine : "");
+
+	return buf;
+}
+
+/**
+ * Generates the version string. This function does not require any
+ * initialization, thus may be called very early e.g., for showing
+ * version information if the executable was invoked with --version
+ * as argument.
+ *
+ * Regardless of whether they want to hide the build information,
+ * this always returns the true fully qualified version.  For instance,
+ * the crash log must always contain the real version, not the one we
+ * wish to otherwise advertize with no build information!
+ *
+ * @return A pointer to a static buffer holding the version string.
+ */
+const char * G_COLD
+version_build_string(void)
+{
+	static char buf[128];
+
+	if G_UNLIKELY('\0' == buf[0]) {
+		const char *v = version_build_internal(FALSE);
+		clamp_strcpy(ARYLEN(buf), v);
+	}
+
 	return buf;
 }
 
 /**
  * Initialize version string.
+ *
+ * @param hide	if TRUE, hide build information in version string
  */
-void
-version_init(void)
+void G_COLD
+version_init(bool hide)
 {
 	time_t now;
 
-	version_string = atom_str_get(version_build_string());
+	version_string = ostrdup_readonly(version_build_internal(hide));
 	now = tm_time();
 
 	{
-		gboolean ok;
-		ok = version_parse(version_string, &our_version);
+		bool ok;
+		const char *end;
+
+		ok = version_parse(version_string, &our_version, &end);
+		g_assert(ok);
+		if (!hide)
+			ok = version_ext_parse(end, &our_ext_version);
 		g_assert(ok);
 	}
 
-	g_message("%s", version_string);
+	g_info("%s", version_string);
+	if (hide)
+		g_info("really running %s", version_build_string());
 
 	version_stamp(version_string, &our_version);
 	g_assert(our_version.timestamp != 0);
@@ -639,15 +888,17 @@ version_init(void)
 	{
 		char buf[128];
 
-		gm_snprintf(buf, sizeof(buf),
-			GTA_PRODUCT_NAME "/%s-%u (%s)",
-			GTA_VERSION_NUMBER, main_get_build(), GTA_RELEASE);
+		str_bprintf(ARYLEN(buf),
+			"%s/%s%s (%s)",
+			product_name(), product_version(),
+			hide ? "" : product_build_full(), product_date());
 
-		version_short_string = atom_str_get(buf);
+		version_short_string = ostrdup_readonly(buf);
 	}
 
-	last_rel_version = our_version;		/* struct copy */
-	last_dev_version = our_version;		/* struct copy */
+	last_rel_version = our_version;			/* struct copy */
+	last_dev_version = our_version;			/* struct copy */
+	our_ext_version.version = our_version;	/* struct copy */
 
 	/*
 	 * The version code is a one-byte encoding of the year/month, since
@@ -671,7 +922,7 @@ version_init(void)
 		tok_is_ancient(now) ||
 		delta_time(now, our_version.timestamp) > VERSION_ANCIENT_WARN
 	) {
-		*deconstify_gboolean(&GNET_PROPERTY(ancient_version)) = TRUE;
+		*deconstify_bool(&GNET_PROPERTY(ancient_version)) = TRUE;
 	}
 }
 
@@ -681,7 +932,7 @@ version_init(void)
  *
  * If the version being ran is not a stable one, warn after 60 days, otherwise
  * warn after a year.  If we're not "expired" yet but are approaching the
- * deadline, start to remind them. 
+ * deadline, start to remind them.
  */
 void
 version_ancient_warn(void)
@@ -704,9 +955,9 @@ version_ancient_warn(void)
 	elapsed = delta_time(now, our_version.timestamp);
 
 	if (elapsed > VERSION_ANCIENT_WARN || tok_is_ancient(now)) {
-		static gboolean warned = FALSE;
+		static bool warned = FALSE;
 		if (GNET_PROPERTY(version_debug)) {
-			g_message("VERSION our_version = %s (elapsed = %ld, token %s)",
+			g_debug("VERSION our_version = %s (elapsed = %ld, token %s)",
 				timestamp_to_string(our_version.timestamp),
 				(long) elapsed, tok_is_ancient(now) ? "ancient" : "ok");
 		}
@@ -756,17 +1007,66 @@ version_ancient_warn(void)
 	}
 }
 
+/**
+ * Dump version to specified logging agent.
+ */
+void G_COLD
+version_string_dump_log(logagent_t *la, bool full)
+{
+	static char buf[80];
+
+	log_info(la, "%s", version_build_string());
+
+	if (!full)
+		return;
+
+#ifndef OFFICIAL_BUILD
+	log_info(la, "(unofficial build, accessing \"%s\")", PACKAGE_SOURCE_DIR);
+#endif
+
+	/*
+	 * Flag only when compiled with --disable-malloc since this is not the
+	 * default setup.  It is important to know that fact easily from the
+	 * command line.
+	 */
+#ifndef USE_MY_MALLOC
+	log_info(la, "(libraries are using the system's malloc() implementation)");
+#endif
+
+	str_bprintf(ARYLEN(buf), "GLib %u.%u.%u",
+			glib_major_version, glib_minor_version, glib_micro_version);
+	if (
+			GLIB_MAJOR_VERSION != glib_major_version ||
+			GLIB_MINOR_VERSION != glib_minor_version ||
+			GLIB_MICRO_VERSION != glib_micro_version
+	   ) {
+		str_bcatf(ARYLEN(buf), " (compiled against %u.%u.%u)",
+				GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+	}
+	log_info(la, "%s", buf);
+
+	if (gtk_version_string() != NULL)
+		log_info(la, "%s", gtk_version_string());
+
+	if (tls_version_string() != NULL)
+		log_info(la, "%s", tls_version_string());
+}
+
+/**
+ * Dump version to stdout.
+ */
+void G_COLD
+version_string_dump(void)
+{
+	version_string_dump_log(log_agent_stdout_get(), TRUE);
+}
+
 /**
  * Free version string.
  */
-void
+void G_COLD
 version_close(void)
 {
-	atom_str_free(version_string);
-	version_string = NULL;
-	atom_str_free(version_short_string);
-	version_short_string = NULL;
-
 	if (version_cmp(&our_version, &last_rel_version) < 0)
 		g_warning("upgrade recommended: most recent released version seen: %s",
 			version_str(&last_rel_version));
