@@ -1,6 +1,4 @@
 /*
- * $Id: roots.c 17383 2010-03-18 08:44:53Z rmanfredi $
- *
  * Copyright (c) 2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -60,23 +58,24 @@
 
 #include "common.h"
 
-RCSID("$Id: roots.c 17383 2010-03-18 08:44:53Z rmanfredi $")
-
 #include "roots.h"
 #include "keys.h"
 #include "kuid.h"
 #include "knode.h"
-#include "storage.h"
 
 #include "core/gnet_stats.h"
+#include "core/settings.h"
 
 #include "if/gnet_property_priv.h"
 #include "if/dht/kademlia.h"
+#include "if/dht/dht.h"			/* For dht_is_active() */
 
 #include "lib/atoms.h"
 #include "lib/cq.h"
-#include "lib/dbmap.h"
+#include "lib/crash.h"
 #include "lib/dbmw.h"
+#include "lib/dbstore.h"
+#include "lib/hset.h"
 #include "lib/map.h"
 #include "lib/patricia.h"
 #include "lib/stringify.h"
@@ -84,6 +83,7 @@ RCSID("$Id: roots.c 17383 2010-03-18 08:44:53Z rmanfredi $")
 #include "lib/unsigned.h"
 #include "lib/vendors.h"
 #include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 #define ROOTS_CALLOUT		5000		/**< Heartbeat every 5 seconds */
@@ -148,18 +148,17 @@ rootinfo_check(const struct rootinfo *ri)
  * The structure is serialized first, not written as-is.
  */
 struct rootdata {
-	guint64 dbkeys[KDA_K];	/**< SDBM keys pointing to contact information */
+	uint64 dbkeys[KDA_K];	/**< SDBM keys pointing to contact information */
 	time_t last_update;		/**< When we last updated the key set */
-	guint8 count;			/**< Amount of dbkeys contained */
-	guint8 version;			/**< Structure version */
+	uint8 count;			/**< Amount of dbkeys contained */
 };
 
 /**
  * Internal counter used to assign DB keys to the contacts we're storing.
  */
-static guint64 contactid = 1;		/* 0 is not a valid key (used as marker) */
+static uint64 contactid = 1;		/* 0 is not a valid key (used as marker) */
 
-#define CONTACT_STRUCT_VERSION	0
+#define CONTACT_STRUCT_VERSION	1
 
 /**
  * Contact information.
@@ -168,11 +167,11 @@ static guint64 contactid = 1;		/* 0 is not a valid key (used as marker) */
 struct contact {
 	kuid_t *id;				/**< KUID of the node (atom) */
 	vendor_code_t vcode;	/**< Vendor code */
+	time_t first_seen;		/**< First seen time */
 	host_addr_t addr;		/**< IP of the node */
-	guint16 port;			/**< Port of the node */
-	guint8 major;			/**< Major version */
-	guint8 minor;			/**< Minor version */
-	guint8 version;			/**< Structure version */
+	uint16 port;			/**< Port of the node */
+	uint8 major;			/**< Major version */
+	uint8 minor;			/**< Minor version */
 };
 
 static unsigned targets_managed;	/**< Amount of targets held in database */
@@ -186,7 +185,7 @@ allocate_rootinfo(const kuid_t *kuid)
 {
 	struct rootinfo *ri;
 
-	ri = walloc0(sizeof *ri);
+	WALLOC0(ri);
 	ri->magic = ROOTINFO_MAGIC;
 	ri->kuid = kuid_get_atom(kuid);
 
@@ -201,9 +200,9 @@ free_rootinfo(struct rootinfo *ri)
 {
 	rootinfo_check(ri);
 
-	cq_cancel(roots_cq, &ri->expire_ev);
+	cq_cancel(&ri->expire_ev);
 	kuid_atom_free_null(&ri->kuid);
-	wfree(ri, sizeof *ri);
+	WFREE(ri);
 }
 
 /**
@@ -218,10 +217,12 @@ get_rootdata(const kuid_t *id)
 
 	if (NULL == rd) {
 		if (dbmw_has_ioerr(db_rootdata)) {
-			g_warning("DBMW \"%s\" I/O error, bad things could happen...",
+			s_warning_once_per(LOG_PERIOD_MINUTE,
+				"DBMW \"%s\" I/O error, bad things could happen...",
 				dbmw_name(db_rootdata));
 		} else {
-			g_warning("key %s exists but was not found in DBMW \"%s\"",
+			s_warning_once_per(LOG_PERIOD_SECOND,
+				"key %s exists but was not found in DBMW \"%s\"",
 				kuid_to_hex_string(id), dbmw_name(db_rootdata));
 		}
 	}
@@ -233,7 +234,7 @@ get_rootdata(const kuid_t *id)
  * Get contact from database.
  */
 static struct contact *
-get_contact(guint64 dbkey)
+get_contact(uint64 dbkey, bool shout)
 {
 	struct contact *c;
 
@@ -241,10 +242,12 @@ get_contact(guint64 dbkey)
 
 	if (NULL == c) {
 		if (dbmw_has_ioerr(db_contact)) {
-			g_warning("DBMW \"%s\" I/O error, bad things could happen...",
+			s_warning_once_per(LOG_PERIOD_MINUTE,
+				"DBMW \"%s\" I/O error, bad things could happen...",
 				dbmw_name(db_contact));
-		} else {
-			g_warning("key %s exists but was not found in DBMW \"%s\"",
+		} else if (shout) {
+			s_warning_once_per(LOG_PERIOD_SECOND,
+				"key %s exists but was not found in DBMW \"%s\"",
 				uint64_to_string(dbkey), dbmw_name(db_contact));
 		}
 	}
@@ -256,17 +259,17 @@ get_contact(guint64 dbkey)
  * Delete contact from database.
  */
 static void
-delete_contact(guint64 dbkey)
+delete_contact(uint64 dbkey)
 {
 	g_assert(uint_is_positive(contacts_managed));
 
 	contacts_managed--;
-	gnet_stats_count_general(GNR_DHT_CACHED_ROOTS_HELD, -1);
+	gnet_stats_dec_general(GNR_DHT_CACHED_ROOTS_HELD);
 
 	dbmw_delete(db_contact, &dbkey);
 
 	if (GNET_PROPERTY(dht_roots_debug) > 2)
-		g_message("DHT contact DB-key %s reclaimed", uint64_to_string(dbkey));
+		g_debug("DHT contact DB-key %s reclaimed", uint64_to_string(dbkey));
 }
 
 /**
@@ -289,7 +292,7 @@ delete_rootdata(const kuid_t *id)
 	dbmw_delete(db_rootdata, id);
 
 	if (GNET_PROPERTY(dht_roots_debug) > 2)
-		g_message("DHT ROOTS k-closest nodes from %s reclaimed",
+		g_debug("DHT ROOTS k-closest nodes from %s reclaimed",
 			kuid_to_hex_string(id));
 }
 
@@ -297,9 +300,9 @@ delete_rootdata(const kuid_t *id)
  * Map iterator callback to reclaim a DB-key value.
  */
 static void
-reclaim_dbkey(gpointer u_key, gpointer val, gpointer u_data)
+reclaim_dbkey(void *u_key, void *val, void *u_data)
 {
-	guint64 *dbkey = val;
+	uint64 *dbkey = val;
 
 	(void) u_key;
 	(void) u_data;
@@ -311,20 +314,19 @@ reclaim_dbkey(gpointer u_key, gpointer val, gpointer u_data)
  * Callout queue callback to expire target.
  */
 static void
-roots_expire(cqueue_t *unused_cq, gpointer obj)
+roots_expire(cqueue_t *cq, void *obj)
 {
 	struct rootinfo *ri = obj;
-	(void) unused_cq;
 
 	rootinfo_check(ri);
 	g_assert(uint_is_positive(targets_managed));
 
-	ri->expire_ev = NULL;		/* Event triggered */
+	cq_zero(cq, &ri->expire_ev);		/* Event triggered */
 	delete_rootdata(ri->kuid);
 	patricia_remove(roots, ri->kuid);
 	free_rootinfo(ri);
 	targets_managed--;
-	gnet_stats_count_general(GNR_DHT_CACHED_KUID_TARGETS_HELD, -1);
+	gnet_stats_dec_general(GNR_DHT_CACHED_KUID_TARGETS_HELD);
 }
 
 /**
@@ -341,13 +343,13 @@ roots_record(patricia_t *nodes, const kuid_t *kuid)
 	struct rootdata new_rd;
 	struct {
 		kuid_t id;
-		guint64 dbkey;
+		uint64 dbkey;
 	} previous[KDA_K];
 	map_t *existing;
 	patricia_iter_t *iter;
 	unsigned i;
-	unsigned new = 0, reused = 0;		/* For logging */
-	gboolean existed = FALSE;			/* For logging */
+	unsigned new = 0, reused = 0;	/* For logging */
+	bool existed = FALSE;			/* For logging */
 
 	g_assert(nodes != NULL);
 	g_assert(kuid != NULL);
@@ -355,10 +357,10 @@ roots_record(patricia_t *nodes, const kuid_t *kuid)
 	/*
 	 * If KUID is within our k-ball, there's no need to cache the roots, as
 	 * we routinely refresh our k-bucket and have normally a perfect knowledge
-	 * of our KDA_K neighbours.
+	 * of our KDA_K neighbours (if we are an active node).
 	 */
 
-	if (keys_within_kball(kuid))
+	if (dht_is_active() && keys_within_kball(kuid))
 		return;
 
 	ri = patricia_lookup(roots, kuid);
@@ -369,7 +371,7 @@ roots_record(patricia_t *nodes, const kuid_t *kuid)
 		rd = &new_rd;
 		new_rd.count = 0;
 		targets_managed++;
-		gnet_stats_count_general(GNR_DHT_CACHED_KUID_TARGETS_HELD, +1);
+		gnet_stats_inc_general(GNR_DHT_CACHED_KUID_TARGETS_HELD);
 	} else {
 		rd = get_rootdata(kuid);
 		if (NULL == rd) {
@@ -393,15 +395,26 @@ roots_record(patricia_t *nodes, const kuid_t *kuid)
 	 * are no longer present among the k-closest roots will be deleted.
 	 */
 
-	STATIC_ASSERT(G_N_ELEMENTS(rd->dbkeys) == G_N_ELEMENTS(previous));
+	STATIC_ASSERT(N_ITEMS(rd->dbkeys) == N_ITEMS(previous));
 
 	existing = map_create_patricia(KUID_RAW_BITSIZE);
 
 	for (i = 0; i < rd->count; i++) {
-		struct contact *c = get_contact(rd->dbkeys[i]);
+		struct contact *c = get_contact(rd->dbkeys[i], FALSE);
+
+		/*
+		 * It can happen that ``rd'' contains DB keys that no longer exist
+		 * in the contact database.  At startup time, we clear orphaned keys,
+		 * but we do nothing against "ghost" DB keys referenced by entries
+		 * within db_rootdata.  These are harmless anyway and will disappear
+		 * as we expire entries or update them after a lookup.
+		 *
+		 * That's why the above get_contact() call silences "key not found"
+		 * type of warnings.
+		 */
 
 		if (NULL == c)
-			continue;		/* I/O error */
+			continue;		/* I/O error or stale info in ``rd'' */
 
 		previous[i].id = *c->id;		/* Struct copy */
 		previous[i].dbkey = rd->dbkeys[i];
@@ -415,29 +428,34 @@ roots_record(patricia_t *nodes, const kuid_t *kuid)
 	iter = patricia_metric_iterator_lazy(nodes, kuid, TRUE);
 	i = 0;
 
-	while (patricia_iter_has_next(iter) && i < G_N_ELEMENTS(rd->dbkeys)) {
+	while (patricia_iter_has_next(iter) && i < N_ITEMS(rd->dbkeys)) {
 		knode_t *kn = patricia_iter_next_value(iter);
-		guint64 *dbkey_ptr;
+		uint64 *dbkey_ptr;
 
 		/*
 		 * If entry existed in the previous set, we reuse the old contact.
+		 *
+		 * Note that presence in the ``existing'' map means that we were
+		 * able to read the contact from the database earlier, so this time
+		 * we issue a verbose get_contact() call to warn if we fail to
+		 * retrieve the contact from the database again.
 		 */
 
 		dbkey_ptr = map_lookup(existing, kn->id);
 
 		if (NULL != dbkey_ptr) {
-			struct contact *c = get_contact(*dbkey_ptr);
+			struct contact *c = get_contact(*dbkey_ptr, TRUE);
 
 			if (NULL == c)
-				continue;		/* I/O error */
+				continue;		/* I/O error, most probably */
 
 			/* Update contact addr:port information, if stale */
-			if (c->port != kn->port || !host_addr_equal(c->addr, kn->addr)) {
+			if (c->port != kn->port || !host_addr_equiv(c->addr, kn->addr)) {
 				c->port = kn->port;
 				c->addr = kn->addr;
-				dbmw_write(db_contact, dbkey_ptr, c, sizeof *c);
-				gnet_stats_count_general(
-					GNR_DHT_CACHED_ROOTS_CONTACT_REFRESHED, 1);
+				c->first_seen = tm_time();	/* New node address */
+				dbmw_write(db_contact, dbkey_ptr, PTRLEN(c));
+				gnet_stats_inc_general(GNR_DHT_CACHED_ROOTS_CONTACT_REFRESHED);
 			}
 
 			map_remove(existing, kn->id);	/* We reused it */
@@ -445,7 +463,7 @@ roots_record(patricia_t *nodes, const kuid_t *kuid)
 			reused++;
 		} else {
 			struct contact nc;
-			guint64 dbkey;
+			uint64 dbkey;
 
 			nc.id = kuid_get_atom(kn->id);	/* Freed through free_contact() */
 			nc.vcode = kn->vcode;	/* Struct copy */
@@ -453,12 +471,13 @@ roots_record(patricia_t *nodes, const kuid_t *kuid)
 			nc.port = kn->port;
 			nc.major = kn->major;
 			nc.minor = kn->minor;
+			nc.first_seen = kn->first_seen;
 			dbkey = contactid++;
 			contacts_managed++;
-			gnet_stats_count_general(GNR_DHT_CACHED_ROOTS_HELD, +1);
+			gnet_stats_inc_general(GNR_DHT_CACHED_ROOTS_HELD);
+
+			dbmw_write(db_contact, &dbkey, VARLEN(nc));
 
-			dbmw_write(db_contact, &dbkey, &nc, sizeof nc);
-				
 			rd->dbkeys[i++] = dbkey;
 			new++;
 		}
@@ -479,19 +498,18 @@ roots_record(patricia_t *nodes, const kuid_t *kuid)
 	 */
 
 	rd->last_update = tm_time();
-	dbmw_write(db_rootdata, kuid, rd, sizeof *rd);
+	dbmw_write(db_rootdata, kuid, PTRLEN(rd));
 
 	if (ri->expire_ev) {
-		cq_resched(roots_cq, ri->expire_ev, ROOTKEY_LIFETIME);
+		cq_resched(ri->expire_ev, ROOTKEY_LIFETIME);
 	} else {
 		ri->expire_ev = cq_insert(roots_cq, ROOTKEY_LIFETIME, roots_expire, ri);
 	}
 
 	if (GNET_PROPERTY(dht_roots_debug) > 1) {
-		g_message("DHT ROOTS cached %u/%u k-closest node%s to %s target %s "
+		g_debug("DHT ROOTS cached %u/%u k-closest node%s to %s target %s "
 			"(new=%u, reused=%u, elapsed=%s)",
-			rd->count, (unsigned) patricia_count(nodes),
-			1 == rd->count ? "" : "s",
+			rd->count, (unsigned) patricia_count(nodes), plural(rd->count),
 			existed ? "existing" : "new",
 			kuid_to_hex_string(kuid), new, reused,
 			existed ?
@@ -526,7 +544,7 @@ roots_fill_vector(struct rootdata *rd,
 	g_assert(NULL == furthest || id != NULL);
 
 	for (i = 0; i < rd->count && i < kcnt; i++) {
-		struct contact *c = get_contact(rd->dbkeys[i]);
+		struct contact *c = get_contact(rd->dbkeys[i], FALSE);
 		knode_t *kn;
 
 		if (NULL == c)
@@ -546,6 +564,7 @@ roots_fill_vector(struct rootdata *rd,
 		kn = knode_new(c->id, 0,
 			c->addr, c->port, c->vcode, c->major, c->minor);
 		kn->flags |= KNODE_F_CACHED;
+		kn->first_seen = c->first_seen;
 		kvec[j++] = kn;
 	}
 
@@ -570,6 +589,7 @@ roots_fill_closest(const kuid_t *id,
 {
 	struct rootinfo *ri;
 	int filled = 0;
+	bool approximate = TRUE;
 
 	g_assert(id != NULL);
 	g_assert(kcnt > 0);
@@ -578,11 +598,12 @@ roots_fill_closest(const kuid_t *id,
 
 	/*
 	 * Do not count a cache miss if the lookup was for a key within
-	 * our k-ball, since we do not cache roots for these keys.
+	 * our k-ball, since we do not cache roots for these keys (when we
+	 * are an active node).
 	 */
 
-	if (keys_within_kball(id)) {
-		gnet_stats_count_general(GNR_DHT_CACHED_ROOTS_KBALL_LOOKUPS, 1);
+	if (dht_is_active() && keys_within_kball(id)) {
+		gnet_stats_inc_general(GNR_DHT_CACHED_ROOTS_KBALL_LOOKUPS);
 		return 0;
 	}
 
@@ -600,17 +621,28 @@ roots_fill_closest(const kuid_t *id,
 		 */
 
 		filled = roots_fill_vector(rd, kvec, kcnt, known, NULL, NULL);
-		gnet_stats_count_general(GNR_DHT_CACHED_ROOTS_EXACT_HITS, 1);
+		gnet_stats_inc_general(GNR_DHT_CACHED_ROOTS_EXACT_HITS);
 
 		if (GNET_PROPERTY(dht_roots_debug) > 1) {
-			g_message("DHT ROOTS exact match for %s (%s), filled %d new node%s",
+			g_debug("DHT ROOTS exact match for %s (%s), filled %d new node%s",
 				kuid_to_hex_string(id),
 				compact_time(delta_time(tm_time(), ri->last_update)),
-				filled, 1 == filled ? "" : "s");
+				PLURAL(filled));
 		}
-	} else {
+
+		/*
+		 * If we had less than KDA_K nodes, then we probably never conducted
+		 * a node search for this ID.  Rather this is the result of a value
+		 * lookup.
+		 */
+
+		approximate = rd->count < KDA_K;
+	}
+
+	if (approximate) {
 		knode_t *furthest = patricia_furthest(known, id);
-		struct rootinfo *cri = patricia_closest(roots, id);
+		struct rootinfo *cri = NULL;
+		patricia_t *aknown = known;
 
 		/*
 		 * We don't have an exact target match but maybe we known about
@@ -619,6 +651,81 @@ roots_fill_closest(const kuid_t *id,
 		 * far, it may be worth it.
 		 */
 
+		if (NULL == ri) {
+			cri = patricia_closest(roots, id);
+		} else {
+			patricia_iter_t *iter;
+
+			/*
+			 * Since "ri" is not NULL, we got an exact match with a lookup
+			 * on "id", so necessarily the closest node will again be the
+			 * same set of nodes.
+			 *
+			 * If we come here, it's because we had less than KDA_K nodes
+			 * to fill in from the closest ID so look for IDs further away
+			 * but which are nonetheless closer that the furthest ID they
+			 * know about...
+			 */
+
+			iter = patricia_metric_iterator(roots, id, TRUE);
+			(void) patricia_iter_next_value(iter);	/* Skip exact match */
+
+			while (patricia_iter_has_next(iter)) {
+				struct rootdata *rd;
+
+				cri = patricia_iter_next_value(iter);
+				if (
+					furthest != NULL &&
+					kuid_cmp3(id, cri->kuid, furthest->id) >= 0
+				) {
+					cri = NULL;
+					break;
+				}
+
+				rd = get_rootdata(cri->kuid);
+				if (NULL == rd) {
+					cri = NULL;		/* I/O error or corrupted database */
+					break;
+				}
+				if (rd->count >= KDA_K)
+					break;
+			}
+
+			patricia_iterator_release(&iter);
+
+			if (NULL == cri) {
+				if (GNET_PROPERTY(dht_roots_debug) > 1) {
+					g_debug("DHT ROOTS no better approximate match for %s",
+						kuid_to_hex_string(id));
+				}
+			} else {
+				int i;
+
+				if (GNET_PROPERTY(dht_roots_debug) > 1) {
+					g_debug("DHT ROOTS trying to add from approximate match");
+				}
+
+				/*
+				 * Since we partially added some nodes in the vector,
+				 * we need to recompute the set of known KUID so that
+				 * we do not attempt to insert duplicates again in the
+				 * approximate matching step.
+				 */
+
+				aknown = patricia_create(KUID_RAW_BITSIZE);
+				iter = patricia_tree_iterator(known, TRUE);
+				while (patricia_iter_has_next(iter)) {
+					knode_t *kn = patricia_iter_next_value(iter);
+					patricia_insert(aknown, kn->id, kn);
+				}
+				patricia_iterator_release(&iter);
+				for (i = 0; i < filled; i++) {
+					knode_t *kn = kvec[i];
+					patricia_insert(aknown, kn->id, kn);
+				}
+			}
+		}
+
 		if (
 			cri != NULL &&
 			(
@@ -627,38 +734,46 @@ roots_fill_closest(const kuid_t *id,
 			)
 		) {
 			struct rootdata *rd = get_rootdata(cri->kuid);
+			int added;
 
-			if (NULL == rd)
-				return 0;		/* I/O error or corrupted database */
+			if (NULL == rd)				/* I/O error or corrupted database */
+				goto approximate_done;	/* May have filled some nodes above */
 
-			filled = roots_fill_vector(rd, kvec, kcnt, known, furthest, id);
+			added = roots_fill_vector(rd, &kvec[filled], kcnt - filled, aknown,
+				furthest, id);
+			filled += added;
 
-			if (filled > 0) {
-				gnet_stats_count_general(
-					GNR_DHT_CACHED_ROOTS_APPROXIMATE_HITS, 1);
-			} else {
-				gnet_stats_count_general(GNR_DHT_CACHED_ROOTS_MISSES, 1);
+			if (added > 0) {
+				gnet_stats_inc_general(GNR_DHT_CACHED_ROOTS_APPROXIMATE_HITS);
+			} else if (NULL == ri) {
+				gnet_stats_inc_general(GNR_DHT_CACHED_ROOTS_MISSES);
 			}
 
 			if (GNET_PROPERTY(dht_roots_debug) > 1) {
-				g_message("DHT ROOTS approximate match of %s with %s (%s), "
+				g_debug("DHT ROOTS approximate match of %s with %s (%s), "
 					"filled %d new node%s",
 					kuid_to_hex_string(id),
 					kuid_to_hex_string2(cri->kuid),
 					compact_time(delta_time(tm_time(), cri->last_update)),
-					filled, 1 == filled ? "" : "s");
+					PLURAL(added));
 			}
-		} else {
-			gnet_stats_count_general(GNR_DHT_CACHED_ROOTS_MISSES, 1);
+		} else if (NULL == ri) {
+			gnet_stats_inc_general(GNR_DHT_CACHED_ROOTS_MISSES);
 
 			if (GNET_PROPERTY(dht_roots_debug) > 1) {
-				g_message("DHT ROOTS no suitable cached entry for %s, "
+				g_debug("DHT ROOTS no suitable cached entry for %s, "
 					"closest was %s",
 					kuid_to_hex_string(id),
 					cri != NULL ?
 						kuid_to_hex_string2(cri->kuid) : "<none>");
 			}
 		}
+
+	approximate_done:
+
+		if (aknown != known) {
+			patricia_destroy(aknown);
+		}
 	}
 
 	return filled;
@@ -668,7 +783,7 @@ roots_fill_closest(const kuid_t *id,
  * Serialization routine for rootdata.
  */
 static void
-serialize_rootdata(pmsg_t *mb, gconstpointer data)
+serialize_rootdata(pmsg_t *mb, const void *data)
 {
 	const struct rootdata *rd = data;
 	unsigned i;
@@ -695,42 +810,30 @@ serialize_rootdata(pmsg_t *mb, gconstpointer data)
  * Deserialization routine for rootdata.
  */
 static void
-deserialize_rootdata(bstr_t *bs, gpointer valptr, size_t len)
+deserialize_rootdata(bstr_t *bs, void *valptr, size_t len)
 {
 	struct rootdata *rd = valptr;
 	unsigned i;
+	uint8 version;
 
 	g_assert(sizeof *rd == len);
 
 	bstr_read_u8(bs, &rd->count);
 	bstr_read_time(bs, &rd->last_update);
-	g_assert(rd->count <= G_N_ELEMENTS(rd->dbkeys));
+	g_assert(rd->count <= N_ITEMS(rd->dbkeys));
 
 	for (i = 0; i < rd->count; i++) {
 		bstr_read_be64(bs, &rd->dbkeys[i]);
 	}
 
-	/*
-	 * Temporary, until 0.96.7 is out: we cannot blindly read the version
-	 * since it was lacking in previous experimental versions.  Therefore
-	 * only do it if we have unread data.
-	 *
-	 * The test will be removed in versions after 0.96.7, when we can be
-	 * certain that the new data format was serialized.
-	 *		--RAM, 2009-10-18
-	 */
-
-	if (bstr_unread_size(bs))
-		bstr_read_u8(bs, &rd->version);
-	else
-		rd->version = 0;
+	bstr_read_u8(bs, &version);
 }
 
 /**
  * Serialization routine for contacts.
  */
 static void
-serialize_contact(pmsg_t *mb, gconstpointer data)
+serialize_contact(pmsg_t *mb, const void *data)
 {
 	const struct contact *c = data;
 
@@ -751,16 +854,18 @@ serialize_contact(pmsg_t *mb, gconstpointer data)
 	 */
 
 	pmsg_write_u8(mb, CONTACT_STRUCT_VERSION);
+	pmsg_write_time(mb, c->first_seen);	/* Added at version 1 */
 }
 
 /**
  * Deserialization routine for contacts.
  */
 static void
-deserialize_contact(bstr_t *bs, gpointer valptr, size_t len)
+deserialize_contact(bstr_t *bs, void *valptr, size_t len)
 {
 	struct contact *c = valptr;
 	kuid_t id;
+	uint8 version;
 
 	g_assert(sizeof *c == len);
 
@@ -772,21 +877,16 @@ deserialize_contact(bstr_t *bs, gpointer valptr, size_t len)
 	bstr_read_be16(bs, &c->port);
 	bstr_read_u8(bs, &c->major);
 	bstr_read_u8(bs, &c->minor);
+	bstr_read_u8(bs, &version);
 
 	/*
-	 * Temporary, until 0.96.7 is out: we cannot blindly read the version
-	 * since it was lacking in previous experimental versions.  Therefore
-	 * only do it if we have unread data.
-	 *
-	 * The test will be removed in versions after 0.96.7, when we can be
-	 * certain that the new data format was serialized.
-	 *		--RAM, 2009-10-18
+	 * "first_seen" was added at version 1.
 	 */
 
-	if (bstr_unread_size(bs))
-		bstr_read_u8(bs, &c->version);
+	if (version < 1)
+		c->first_seen = tm_time();
 	else
-		c->version = 0;
+		bstr_read_time(bs, &c->first_seen);
 
 	/*
 	 * Only create the KUID atom if there was no error in the deserialization
@@ -804,7 +904,7 @@ deserialize_contact(bstr_t *bs, gpointer valptr, size_t len)
  * the structure itself.
  */
 static void
-free_contact(gpointer valptr, size_t len)
+free_contact(void *valptr, size_t len)
 {
 	struct contact *c = valptr;
 
@@ -817,16 +917,15 @@ free_contact(gpointer valptr, size_t len)
  * Context for recreate_ri() and remove_orphan().
  */
 struct recreate_context {
-	GHashTable *dbkeys;		/* Seen DB keys (atoms) */
-	guint orphans;			/* Orphan keys found */
+	hset_t *dbkeys;			/* Seen DB keys (atoms) */
+	uint orphans;			/* Orphan keys found */
 };
 
 static void
-free_dbkey_kv(gpointer key, gpointer u_value, gpointer u_data)
+free_dbkey_kv(const void *key, void *u_data)
 {
-	guint64 *dbkey = key;
+	const uint64 *dbkey = key;
 
-	(void) u_value;
 	(void) u_data;
 
 	atom_uint64_free(dbkey);
@@ -836,8 +935,8 @@ free_dbkey_kv(gpointer key, gpointer u_value, gpointer u_data)
  * DBMW foreach iterator to recreate keyinfo if not too ancient.
  * @return TRUE if entry is too ancient and key must be deleted.
  */
-static gboolean
-recreate_ri(gpointer key, gpointer value, size_t u_len, gpointer data)
+static bool
+recreate_ri(void *key, void *value, size_t u_len, void *data)
 {
 	struct recreate_context *ctx = data;
 	const struct rootdata *rd = value;
@@ -855,7 +954,7 @@ recreate_ri(gpointer key, gpointer value, size_t u_len, gpointer data)
 	d = delta_time(tm_time(), rd->last_update);
 
 	if (GNET_PROPERTY(dht_roots_debug) > 4)
-		g_message("DHT ROOTS retrieved target %s (%s)",
+		g_debug("DHT ROOTS retrieved target %s (%s)",
 			kuid_to_hex_string(id), compact_time(d));
 
 	if (d >= ROOTKEY_LIFETIME / 1000) {
@@ -873,14 +972,14 @@ recreate_ri(gpointer key, gpointer value, size_t u_len, gpointer data)
 	 */
 
 	for (i = 0; i < rd->count; i++) {
-		guint64 dbkey = rd->dbkeys[i];
+		uint64 dbkey = rd->dbkeys[i];
 
 		if (dbkey >= contactid)
 			contactid = dbkey + 1;
 
-		if (!g_hash_table_lookup(ctx->dbkeys, &dbkey)) {
-			const guint64 *dbatom = atom_uint64_get(&dbkey);
-			gm_hash_table_insert_const(ctx->dbkeys, dbatom, uint_to_pointer(1));
+		if (!hset_contains(ctx->dbkeys, &dbkey)) {
+			const uint64 *dbatom = atom_uint64_get(&dbkey);
+			hset_insert(ctx->dbkeys, dbatom);
 		}
 	}
 
@@ -895,14 +994,14 @@ recreate_ri(gpointer key, gpointer value, size_t u_len, gpointer data)
 	 */
 
 	targets_managed++;
-	gnet_stats_count_general(GNR_DHT_CACHED_KUID_TARGETS_HELD, +1);
+	gnet_stats_inc_general(GNR_DHT_CACHED_KUID_TARGETS_HELD);
 
 	contacts_managed += rd->count;
 	gnet_stats_count_general(GNR_DHT_CACHED_ROOTS_HELD, rd->count);
 
 	if (GNET_PROPERTY(dht_roots_debug) > 3)
-		g_message("DHT ROOTS retrieved %u closest node%s from %s kept (for %s)",
-			rd->count, 1 == rd->count ? "" : "s",
+		g_debug("DHT ROOTS retrieved %u closest node%s from %s kept (for %s)",
+			PLURAL(rd->count),
 			kuid_to_hex_string(id), compact_time(ROOTKEY_LIFETIME / 1000 - d));
 
 	return FALSE;
@@ -910,18 +1009,18 @@ recreate_ri(gpointer key, gpointer value, size_t u_len, gpointer data)
 
 /**
  * DBMW foreach iterator to remove orphan DB keys.
- * @return TRUE if entry is orphaned must be deleted.
+ * @return TRUE if entry is an orphan and must be deleted.
  */
-static gboolean
-remove_orphan(gpointer key, gpointer u_value, size_t u_len, gpointer data)
+static bool
+remove_orphan(void *key, void *u_value, size_t u_len, void *data)
 {
 	struct recreate_context *ctx = data;
-	guint64 *dbkey = key;
+	uint64 *dbkey = key;
 
 	(void) u_value;
 	(void) u_len;
 
-	if (!g_hash_table_lookup(ctx->dbkeys, dbkey)) {
+	if (!hset_contains(ctx->dbkeys, dbkey)) {
 		ctx->orphans++;
 		return TRUE;
 	}
@@ -932,13 +1031,13 @@ remove_orphan(gpointer key, gpointer u_value, size_t u_len, gpointer data)
 /**
  * Periodic DB synchronization.
  */
-static gboolean
-roots_sync(gpointer unused_obj)
+static bool
+roots_sync(void *unused_obj)
 {
 	(void) unused_obj;
 
-	storage_sync(db_rootdata);
-	storage_sync(db_contact);
+	dbstore_sync(db_rootdata);
+	dbstore_sync(db_contact);
 
 	return TRUE;
 }
@@ -950,63 +1049,96 @@ static void
 roots_init_rootinfo(void)
 {
 	struct recreate_context ctx;
+	size_t count;
 
 	if (GNET_PROPERTY(dht_roots_debug)) {
-		size_t count = dbmw_count(db_rootdata);
-		g_message("DHT ROOTS scanning %u retrieved target KUID%s",
-			(unsigned) count, 1 == count ? "" : "s");
+		count = dbmw_count(db_rootdata);
+		g_debug("DHT ROOTS scanning %u retrieved target KUID%s",
+			(unsigned) PLURAL(count));
 	}
 
-	ctx.dbkeys = g_hash_table_new(uint64_mem_hash, uint64_mem_eq);
+	ctx.dbkeys = hset_create(HASH_KEY_FIXED, sizeof(uint64));
 	ctx.orphans = 0;
 
 	dbmw_foreach_remove(db_rootdata, recreate_ri, &ctx);
 	dbmw_foreach_remove(db_contact, remove_orphan, &ctx);
 
-	g_hash_table_foreach(ctx.dbkeys, free_dbkey_kv, NULL);
-	g_hash_table_destroy(ctx.dbkeys);
+	hset_foreach(ctx.dbkeys, free_dbkey_kv, NULL);
+	hset_free_null(&ctx.dbkeys);
+
+	count = dbmw_count(db_rootdata);
 
 	if (GNET_PROPERTY(dht_roots_debug)) {
-		size_t count = dbmw_count(db_rootdata);
-		g_message("DHT ROOTS kept %u target KUID%s: targets=%u, contacts=%u",
-			(unsigned) count, 1 == count ? "" : "s",
+		g_debug("DHT ROOTS kept %u target KUID%s: targets=%u, contacts=%u",
+			(unsigned) PLURAL(count),
 			targets_managed, contacts_managed);
-		g_message("DHT ROOTS stripped %u orphan contact DB-key%s",
-			ctx.orphans, 1 == ctx.orphans ? "" : "s");
-		g_message("DHT ROOTS first allocated contact DB-key will be %s",
+		g_debug("DHT ROOTS stripped %u orphan contact DB-key%s",
+			PLURAL(ctx.orphans));
+	}
+
+	/*
+	 * If we retained no entries, issue a dbmw_clear() to restore underlying
+	 * SDBM files to their smallest possible value.  This is necessary because
+	 * the database is persistent and it can grow very large on disk, but still
+	 * holding only a few values per page.  Being able to get a fresh start
+	 * occasionally is a plus.
+	 */
+
+	if (0 == count) {
+		contactid = 1;
+		targets_managed = contacts_managed = 0;
+	}
+
+
+	if (!crash_was_restarted()) {
+		if (GNET_PROPERTY(dht_roots_debug)) {
+			if (0 == count)
+				g_debug("DHT ROOTS clearing database");
+			else
+				g_debug("DHT ROOTS shrinking database files");
+		}
+
+		dbstore_compact(db_rootdata);
+		dbstore_compact(db_contact);
+	}
+
+	if (GNET_PROPERTY(dht_roots_debug)) {
+		g_debug("DHT ROOTS first allocated contact DB-key will be %s",
 			uint64_to_string(contactid));
 	}
-
-	if (!dbmw_shrink(db_rootdata)) {
-		if (GNET_PROPERTY(dht_roots_debug))
-			g_warning("DHT ROOTS unable to shrink %s", db_rootdata_what);
-	}
-
-	if (!dbmw_shrink(db_contact)) {
-		if (GNET_PROPERTY(dht_roots_debug))
-			g_warning("DHT ROOTS unable to shrink %s", db_contact_what);
-	}
 }
 
 /**
  * Initialize root node caching.
  */
-void
+void G_COLD
 roots_init(void)
 {
-	roots_cq = cq_submake("roots", callout_queue, ROOTS_CALLOUT);
+	dbstore_kv_t root_kv = { KUID_RAW_SIZE, NULL, sizeof(struct rootdata), 0 };
+	dbstore_kv_t contact_kv = { sizeof(uint64), NULL, sizeof(struct contact),
+		sizeof(struct contact) + KUID_RAW_SIZE };
+	dbstore_packing_t root_packing =
+		{ serialize_rootdata, deserialize_rootdata, NULL };
+	dbstore_packing_t contact_packing =
+		{ serialize_contact, deserialize_contact, free_contact };
+
+	g_assert(NULL == roots_cq);
+	g_assert(NULL == roots);
+	g_assert(NULL == db_rootdata);
+	g_assert(NULL == db_contact);
+
+	roots_cq = cq_main_submake("roots", ROOTS_CALLOUT);
 	roots = patricia_create(KUID_RAW_BITSIZE);
 
-	db_rootdata = storage_open(db_rootdata_what, db_rootdata_base,
-		KUID_RAW_SIZE, sizeof(struct rootdata), 0,
-		serialize_rootdata, deserialize_rootdata, NULL,
-		ROOTKEYS_DB_CACHE_SIZE, sha1_hash, sha1_eq);
+	db_rootdata = dbstore_open(db_rootdata_what, settings_dht_db_dir(),
+		db_rootdata_base, root_kv, root_packing,
+		ROOTKEYS_DB_CACHE_SIZE, kuid_hash, kuid_eq,
+		GNET_PROPERTY(dht_storage_in_memory));
 
-	db_contact = storage_open(db_contact_what, db_contact_base,
-		sizeof(guint64), sizeof(struct contact),
-			sizeof(struct contact) + KUID_RAW_SIZE,
-		serialize_contact, deserialize_contact, free_contact,
-		CONTACT_DB_CACHE_SIZE, uint64_mem_hash, uint64_mem_eq);
+	db_contact = dbstore_open(db_contact_what, settings_dht_db_dir(),
+		db_contact_base, contact_kv, contact_packing,
+		CONTACT_DB_CACHE_SIZE, uint64_mem_hash, uint64_mem_eq,
+		GNET_PROPERTY(dht_storage_in_memory));
 
 	dbmw_set_map_cache(db_contact, CONTACT_MAP_CACHE_SIZE);
 
@@ -1018,7 +1150,7 @@ roots_init(void)
  * Map iterator to free the items held in `roots'.
  */
 static void
-roots_free_kv(gpointer u_key, gpointer val, gpointer u_x)
+roots_free_kv(void *u_key, void *val, void *u_x)
 {
 	struct rootinfo *ri = val;
 
@@ -1031,7 +1163,7 @@ roots_free_kv(gpointer u_key, gpointer val, gpointer u_x)
 /**
  * Close root node caching.
  */
-void
+void G_COLD
 roots_close(void)
 {
 	if (roots) {
@@ -1043,14 +1175,14 @@ roots_close(void)
 	}
 
 	if (GNET_PROPERTY(dht_roots_debug)) {
-		g_message("DHT ROOTS shutdown with targets=%u, contacts=%u",
+		g_debug("DHT ROOTS shutdown with targets=%u, contacts=%u",
 			targets_managed, contacts_managed);
 	}
 
-	storage_close(db_rootdata, db_rootdata_base);
+	dbstore_close(db_rootdata, settings_dht_db_dir(), db_rootdata_base);
 	db_rootdata = NULL;
 
-	storage_close(db_contact, db_contact_base);
+	dbstore_close(db_contact, settings_dht_db_dir(), db_contact_base);
 	db_contact = NULL;
 
 	cq_free_null(&roots_cq);
