@@ -1,6 +1,4 @@
 /*
- * $Id: ulq.h 17215 2009-11-09 21:14:29Z rmanfredi $
- *
  * Copyright (c) 2008, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -44,7 +42,7 @@
  */
 
 void ulq_init(void);
-void ulq_close(gboolean exiting);
+void ulq_close(bool exiting);
 
 #endif /* _dht_ulq_h_ */
 
