@@ -1,6 +1,4 @@
 /*
- * $Id: c2ui.c 16909 2009-06-05 23:49:38Z cbiere $
- *
  * Copyright (c) 2004, Emile Roberts
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -51,6 +49,7 @@
 #define	gui_update_files_scanned()
 #define	gui_allow_rescan_dir(flag) ((void) flag)
 #define search_gui_new_search(query, flags, x) (((query) && (flags)) ? 0 : 0)
+#define search_gui_store_searches()
 
 #define upload_stats_gui_add(s) ((void) s)
 #define upload_stats_gui_update(s) ((void) s)
@@ -62,13 +61,10 @@
 #define statusbar_gui_warning(sec, fmt, message) g_message((fmt), (message))
 #define statusbar_gui_message(sec, fmt, message) g_message((fmt), (message))
 
-#define bitzi_gui_update(bitzi_data) ((void) bitzi_data)
-
 #endif /* USE_TOPLESS */
 
 #if defined(USE_GTK1) || defined(USE_GTK2)
-extern gboolean running_topless;
-#include "if/ui/gtk/bitzi.h"
+extern bool running_topless;
 #include "if/ui/gtk/downloads.h"
 #include "if/ui/gtk/misc.h"
 #include "if/ui/gtk/search.h"
@@ -107,19 +103,19 @@ gcu_gui_update_files_scanned(void)
 }
 
 void
-guc_allow_rescan_dir(gboolean flag)
+guc_allow_rescan_dir(bool flag)
 {
 	if (!running_topless) {
 		gui_allow_rescan_dir(flag);
 	}
 }
 
-gint
+int
 gcu_gtk_main_flush(void)
 {
 #if defined(USE_GTK1) || defined(USE_GTK2)
 	if (!running_topless) {
-		extern gint gtk_main_flush();
+		extern int gtk_main_flush();
 		/* Don't include any GTK-related header */
 		return gtk_main_flush();
 	}
@@ -128,8 +124,8 @@ gcu_gtk_main_flush(void)
 }
 
 /**	search interface functions (CORE -> UI) */
-gboolean
-gcu_search_gui_new_search(const gchar *query, guint32 flags)
+bool
+gcu_search_gui_new_search(const gchar *query, uint32 flags)
 {
 	if (!running_topless) {
 		return search_gui_new_search(query, flags, NULL);
@@ -138,6 +134,11 @@ gcu_search_gui_new_search(const gchar *query, guint32 flags)
 	}
 }
 
+void
+gcu_search_gui_store_searches(void)
+{
+	search_gui_store_searches();
+}
 
 /*	upload interface functions (CORE -> UI)*/
 void
@@ -188,21 +189,6 @@ gcu_upload_stats_gui_thaw(void)
 	}
 }
 
-/**
- * Bitzi result notification (Core->UI)
- *
- * If the IPC split goes ahead then the data will need to be copied
- * across. For the time being we just pass the pointer.
- */
-
-void
-gcu_bitzi_result(bitzi_data_t *bitzi_data)
-{
-	if (!running_topless) {
-    	bitzi_gui_update(bitzi_data);
-	}
-}
-
 /*	statusbar interface functions (CORE -> UI) */
 void
 gcu_statusbar_warning(const gchar *message)
