@@ -1,6 +1,4 @@
 /*
- * $Id: main.h 16110 2009-03-10 15:27:43Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -28,14 +26,35 @@
 
 #include "common.h"
 
-guint32 main_get_build(void);
-
 #ifdef CORE_SOURCES
 
+/**
+ * Mode for gtk_gnutella_request_shutdown().
+ */
+
+enum shutdown_mode {
+	GTKG_SHUTDOWN_NORMAL = 1,
+	GTKG_SHUTDOWN_ASSERT,
+	GTKG_SHUTDOWN_ERROR,
+	GTKG_SHUTDOWN_MEMORY,
+	GTKG_SHUTDOWN_SIGNAL
+};
+
+/**
+ * Shutdown options.
+ */
+
+#define GTKG_SHUTDOWN_OFAST		(1U << 0) /**< BYE sent to supporting nodes */
+#define GTKG_SHUTDOWN_ORESTART	(1U << 1) /**< Restart gtk-gnutella */
+#define GTKG_SHUTDOWN_OCRASH	(1U << 2) /**< Crash restart */
+#define GTKG_SHUTDOWN_OEXTEND	(1U << 3) /**< Extend session at next launch */
+
 void gtk_gnutella_exit(int n);
-void gtk_gnutella_request_shutdown(void);
-gboolean debugging(guint t);
-const char *gtk_gnutella_interface(void);
+void gtk_gnutella_request_shutdown(enum shutdown_mode mode, unsigned flags);
+bool debugging(uint t);
+
+char *main_command_line(void);
+const char *gtk_version_string(void);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_main_h_ */
