@@ -1,6 +1,4 @@
 /*
- * $Id: dht.h 17324 2010-03-01 17:36:14Z rmanfredi $
- *
  * Copyright (c) 2008, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -31,13 +29,6 @@
 
 #include "if/gnet_property_priv.h"
 
-typedef enum {
-	DHT_MODE_INACTIVE = 0x0,		/**< DHT capable, but not in DHT */
-	DHT_MODE_ACTIVE = 0x1,			/**< Active DHT node */
-	DHT_MODE_PASSIVE = 0x2,			/**< Passive DHT node */
-	DHT_MODE_PASSIVE_LEAF = 0x3		/**< Passive leaf DHT node */
-} dht_mode_t;
-
 /*
  * Public interface.
  */
@@ -45,32 +36,37 @@ typedef enum {
 struct gnutella_node;
 
 void dht_init(void);
-void dht_close(gboolean exiting);
-void dht_initialize(gboolean post_init);
+void dht_close(bool exiting);
+void dht_initialize(bool post_init);
 void dht_reset_kuid(void);
-gboolean dht_seeded(void);
-gboolean dht_bootstrapped(void);
 void dht_ipp_extract(
-	const struct gnutella_node *n, const char *payload, int paylen);
+	const struct gnutella_node *n,
+	const char *payload, int paylen, enum net_type nt);
 int dht_fill_random(gnet_host_t *hvec, int hcnt);
 
 void dht_route_store_if_dirty(void);
-void dht_bootstrap_if_needed(host_addr_t addr, guint16 port);
+void dht_bootstrap_if_needed(host_addr_t addr, uint16 port);
 void dht_attempt_bootstrap(void);
 void dht_update_size_estimate(void);
 
-const char *dht_mode_to_string(dht_mode_t mode);
+bool dht_is_active(void);
 
 /**
  * Is the DHT enabled?
  */
-static inline gboolean
+static inline bool
 dht_enabled(void)
 {
 	return GNET_PROPERTY(enable_udp) && GNET_PROPERTY(enable_dht) &&
 		GNET_PROPERTY(listen_port) != 0;
 }
 
+/*
+ * Debugging interface.
+ */
+
+void tcache_debugging_changed(void);
+
 #endif /* _if_dht_dht_h */
 
 /* vi: set ts=4 sw=4 cindent: */
