@@ -1,6 +1,4 @@
 /*
- * $Id: knode.h 17363 2010-03-06 11:13:03Z rmanfredi $
- *
  * Copyright (c) 2008, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -31,6 +29,7 @@
 #include "kuid.h"
 #include "lib/vendors.h"
 #include "lib/host_addr.h"
+#include "lib/unsigned.h"		/* For uint8_saturate_add() */
 
 struct kbucket;
 
@@ -53,17 +52,19 @@ typedef struct knode {
 	knode_magic_t magic;
 	int refcnt;					/**< Reference count */
 	kuid_t *id;					/**< KUID of the node (atom) */
+	time_t first_seen;			/**< First time we heard about that node */
 	time_t last_seen;			/**< Last seen message from that node */
 	time_t last_sent;			/**< Last sent RPC to that node */
 	vendor_code_t vcode;		/**< Vendor code (vcode.u32 == 0 if unknown) */
-	guint32 rtt;				/**< Round-trip time in milliseconds */
-	guint32 flags;				/**< Operating flags */
+	uint32 rtt;					/**< Round-trip time in milliseconds */
+	uint32 flags;				/**< Operating flags */
 	host_addr_t addr;			/**< IP of the node */
 	knode_status_t status;		/**< Node status (good, stale, pending) */
-	guint16 port;				/**< Port of the node */
-	guchar rpc_timeouts;		/**< Amount of consecutive RPC timeouts */
-	guint8 major;				/**< Major version */
-	guint8 minor;				/**< Minor version */
+	uint16 port;				/**< Port of the node */
+	uint8 rpc_pending;			/**< Amount of pending RPCs (may saturate) */
+	uint8 rpc_timeouts;			/**< Amount of consecutive RPC timeouts */
+	uint8 major;				/**< Major version */
+	uint8 minor;				/**< Minor version */
 } knode_t;
 
 /**
@@ -77,16 +78,18 @@ typedef struct knode {
 #define KNODE_F_SHUTDOWNING	(1 << 4)	/**< Host said it was shutdowning */
 #define KNODE_F_PCONTACT	(1 << 5)	/**< Patched contact address */
 #define KNODE_F_CACHED		(1 << 6)	/**< Node comes from root cache */
+#define KNODE_F_RPC			(1 << 7)	/**< Performed successful RPC */
 
 knode_t *get_our_knode(void);
 
 void knode_free(knode_t *kn);
-void knode_patricia_free(gpointer, size_t, gpointer, gpointer);
-void knode_map_free(gpointer, gpointer, gpointer);
+void knode_patricia_free(void *, size_t, void *, void *);
+void knode_map_free(void *, void *, void *);
 
-unsigned int knode_hash(gconstpointer key);
-int knode_eq(gconstpointer a, gconstpointer b);
-int knode_seen_cmp(gconstpointer a, gconstpointer b);
+unsigned int knode_hash(const void *key);
+int knode_eq(const void *a, const void *b);
+int knode_seen_cmp(const void *a, const void *b);
+int knode_dead_probability_cmp(const void *a, const void *b);
 const char * knode_status_to_string(knode_status_t status);
 const char *knode_to_string(const knode_t *kn);
 const char *knode_to_string2(const knode_t *kn);
@@ -117,7 +120,7 @@ knode_refcnt(const knode_t *kn)
 static inline knode_t *
 knode_refcnt_inc(const knode_t *kn)
 {
-	knode_t *knm = deconstify_gpointer(kn);
+	knode_t *knm = deconstify_pointer(kn);
 
 	knode_check(kn);
 
@@ -133,7 +136,7 @@ knode_refcnt_inc(const knode_t *kn)
 static inline knode_t *
 knode_refcnt_dec(const knode_t *kn)
 {
-	knode_t *knm = deconstify_gpointer(kn);
+	knode_t *knm = deconstify_pointer(kn);
 
 	knode_check(kn);
 	g_assert(kn->refcnt > 1);
@@ -147,8 +150,8 @@ knode_refcnt_dec(const knode_t *kn)
  *
  * @param no_routing_table		if TRUE, do not count the routing table
  */
-static inline gboolean
-knode_is_shared(const knode_t *kn, gboolean no_routing_table)
+static inline bool
+knode_is_shared(const knode_t *kn, bool no_routing_table)
 {
 	int refcnt;
 
@@ -161,6 +164,52 @@ knode_is_shared(const knode_t *kn, gboolean no_routing_table)
 	return refcnt > 1;
 }
 
+/**
+ * Add one more RPC pending for this node.
+ */
+static inline void
+knode_rpc_inc(knode_t *kn)
+{
+	/*
+	 * We don't care if that counter saturates because it is used solely
+	 * to optimize the sending of "alive" PING RPCs from the routing table:
+	 * when at least one pending RPC is registered, we can avoid sending
+	 * such an "alive" PING since any timeout on the pending RPCs would make
+	 * the node become stale anyway, and the purpose of "alive" PINGs is
+	 * precisely to be able to detect such stale nodes.
+	 *
+	 * Because the counter can saturate, it also can indicate 0 when in fact
+	 * we do have pending RPCs still, but then the optimization won't kick-in
+	 * and there's no harm done.
+	 */
+
+	kn->rpc_pending = uint8_saturate_add(kn->rpc_pending, 1);
+}
+
+/**
+ * Remove one pending RPC for this node.
+ */
+static inline void
+knode_rpc_dec(knode_t *kn)
+{
+	/*
+	 * Because counter can saturate, decrease it only if not zero.
+	 * See knode_rpc_inc() for why saturation is not a problem.
+	 */
+
+	if (G_LIKELY(kn->rpc_pending != 0))
+		kn->rpc_pending--;
+}
+
+/**
+ * Are there any RPC pending for this node?
+ */
+static inline bool
+knode_rpc_pending(knode_t *kn)
+{
+	return booleanize(kn->rpc_pending);
+}
+
 #endif /* _if_dht_knode_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
