@@ -1,6 +1,4 @@
 /*
- * $Id: stable.h 17106 2009-10-20 22:41:40Z rmanfredi $
- *
  * Copyright (c) 2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -44,7 +42,7 @@
  */
 
 double stable_store_presence(time_delta_t d,
-	const lookup_rs_t *rs, const guint16 *status);
+	const lookup_rs_t *rs, const uint16 *status);
 
 #endif	/* _if_dht_stable_h_ */
 
