@@ -1,6 +1,4 @@
 /*
- * $Id: ascii.h 16992 2009-09-25 19:07:30Z rmanfredi $
- *
  * Copyright (c) 2001-2008, Christian Biere & Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -41,6 +39,20 @@
 #include "common.h"
 #include "casts.h"
 
+#define A_UPPER		(1 << 0)
+#define A_LOWER		(1 << 1)
+#define A_BLANK		(1 << 2)
+#define A_CTRL		(1 << 3)
+#define A_DIGIT		(1 << 4)
+#define A_HEXA		(1 << 5)
+#define A_SPACE		(1 << 6)
+#define A_GRAPH		(1 << 7)
+#define A_PRINT		(1 << 8)
+#define A_PUNCT		(1 << 9)
+#define A_IDENT		(1 << 10)	/* Identifier: alphanumeric + "_" */
+
+extern const uint16 ascii_ctype[];
+
 int ascii_strcasecmp(const char *s1, const char *s2);
 int ascii_strncasecmp(const char *s1, const char *s2, size_t len);
 
@@ -50,10 +62,10 @@ int ascii_strncasecmp(const char *s1, const char *s2, size_t len);
  * @param c the character to convert.
  * @return 0..15 for valid hexadecimal ASCII characters, -1 otherwise.
  */
-static inline int
-hex2int_inline(guchar c)
+static inline ALWAYS_INLINE int
+hex2int_inline(uchar c)
 {
-	extern const gint8 *hex2int_tab;
+	extern const int8 *hex2int_tab;
 	return hex2int_tab[c];
 }
 
@@ -63,10 +75,10 @@ hex2int_inline(guchar c)
  * @param c the character to convert.
  * @return 0..9 for valid decimal ASCII characters, -1 otherwise.
  */
-static inline int
-dec2int_inline(guchar c)
+static inline ALWAYS_INLINE int
+dec2int_inline(uchar c)
 {
-	extern const gint8 *dec2int_tab;
+	extern const int8 *dec2int_tab;
 	return dec2int_tab[c];
 }
 
@@ -76,103 +88,114 @@ dec2int_inline(guchar c)
  * @param c the character to convert.
  * @return 0..9 for valid alphanumeric ASCII characters, -1 otherwise.
  */
-static inline int
-alnum2int_inline(guchar c)
+static inline ALWAYS_INLINE int
+alnum2int_inline(uchar c)
 {
-	extern const gint8 *alnum2int_tab;
+	extern const int8 *alnum2int_tab;
 	return alnum2int_tab[c];
 }
 
 /**
  * ctype-like functions that allow only ASCII characters whereas the locale
- * would allow others. The parameter doesn't have to be casted to (unsigned
- * char) because these functions return false for everything out of [0..127].
+ * would allow others. The parameter doesn't have to be cast to (unsigned
+ * char) because these functions return FALSE for everything out of [0..127].
  *
  * GLib 2.x has similar macros/functions but defines only a subset.
  */
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_blank(int c)
 {
-	return c == 32 || c == 9;	/* space, tab */
+	return ((uint) c < 0x80) & (0 != (A_BLANK & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_cntrl(int c)
 {
-	return (c >= 0 && c <= 31) || c == 127;
+	return ((uint) c < 0x80) & (0 != (A_CTRL & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_digit(int c)
 {
-	return c >= 48 && c <= 57;	/* 0-9 */
+	return ((uint) c < 0x80) & (0 != (A_DIGIT & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_xdigit(int c)
 {
-	return -1 != hex2int_inline(c) && !(c & ~0x7f);
+	return ((uint) c < 0x80) & (0 != (A_HEXA & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_upper(int c)
 {
-	return c >= 65 && c <= 90;		/* A-Z */
+	/* Using && here helps ascii_tolower() be faster */
+	return ((uint) c < 0x80) && (0 != (A_UPPER & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_lower(int c)
 {
-	return c >= 97 && c <= 122;		/* a-z */
+	/* Using && here helps ascii_toupper() be faster */
+	return ((uint) c < 0x80) && (0 != (A_LOWER & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_alpha(int c)
 {
-	return is_ascii_upper(c) || is_ascii_lower(c);	/* A-Z, a-z */
+	return ((uint) c < 0x80) &
+		(0 != ((A_UPPER | A_LOWER) & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_alnum(int c)
 {
-	return -1 != alnum2int_inline(c) && !(c & ~0x7f); /* A-Z, a-z, 0-9 */
+	return ((uint) c < 0x80) &
+		(0 != ((A_DIGIT | A_UPPER | A_LOWER) & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
+is_ascii_ident(int c)
+{
+	/* Part of an identifier, i,e, one of [A-Za-z0-9_] */
+	return ((uint) c < 0x80) & (0 != (A_IDENT & ascii_ctype[(uint8) c]));
+}
+
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_space(int c)
 {
-	return c == 32 || (c >= 9 && c <= 13);
+	return ((uint) c < 0x80) & (0 != (A_SPACE & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_graph(int c)
 {
-	return c >= 33 && c <= 126;
+	return ((uint) c < 0x80) & (0 != (A_GRAPH & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_print(int c)
 {
-	return is_ascii_graph(c) || c == 32;
+	return ((uint) c < 0x80) & (0 != (A_PRINT & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT gboolean
+static inline G_CONST WARN_UNUSED_RESULT bool
 is_ascii_punct(int c)
 {
-	return c >= 33 && c <= 126 && !is_ascii_alnum(c);
+	return ((uint) c < 0x80) & (0 != (A_PUNCT & ascii_ctype[(uint8) c]));
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT int
+static inline G_CONST WARN_UNUSED_RESULT int
 ascii_toupper(int c)
 {
-	return is_ascii_lower(c) ? c - 32 : c;
+	return c - (is_ascii_lower(c) << 5);
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT int
+static inline G_CONST WARN_UNUSED_RESULT int
 ascii_tolower(int c)
 {
-	return is_ascii_upper(c) ? c + 32 : c;
+	return c + (is_ascii_upper(c) << 5);
 }
 
 /**
@@ -180,7 +203,7 @@ ascii_tolower(int c)
  *
  * @return a pointer to the first non-space character starting from s.
  */
-static inline WARN_UNUSED_RESULT char *
+static inline G_PURE WARN_UNUSED_RESULT char *
 skip_ascii_spaces(const char *s)
 {
 	while (is_ascii_space(*s))
@@ -194,7 +217,7 @@ skip_ascii_spaces(const char *s)
  *
  * @return a pointer to the first space or NUL character starting from s.
  */
-static inline WARN_UNUSED_RESULT char *
+static inline G_PURE WARN_UNUSED_RESULT char *
 skip_ascii_non_spaces(const char *s)
 {
 	while ('\0' != *s && !is_ascii_space(*s))
@@ -210,7 +233,7 @@ skip_ascii_non_spaces(const char *s)
  * @return a pointer to the first non-alphanumerical or NUL character
  * starting from s.
  */
-static inline WARN_UNUSED_RESULT char *
+static inline G_PURE WARN_UNUSED_RESULT char *
 skip_ascii_alnum(const char *s)
 {
 	while (is_ascii_alnum(*s))
@@ -224,7 +247,7 @@ skip_ascii_alnum(const char *s)
  *
  * @return A pointer to the first non-blank character starting from s.
  */
-static inline WARN_UNUSED_RESULT char *
+static inline G_PURE WARN_UNUSED_RESULT char *
 skip_ascii_blanks(const char *s)
 {
 	while (is_ascii_blank(*s))
@@ -236,11 +259,11 @@ skip_ascii_blanks(const char *s)
 void ascii_strlower(char *dst, const char *src);
 int ascii_strcasecmp_delimit(const char *a, const char *b,
 		const char *delimit);
-char *ascii_strcasestr(const char *haystack, const char *needle);
 int ascii_strcmp_delimit(const char *a, const char *b, const char *delimit);
+size_t ascii_chomp_trailing_spaces(char *str, size_t len);
 
-guint ascii_strcase_hash(gconstpointer key);
-int ascii_strcase_eq(gconstpointer a, gconstpointer b);
+uint ascii_strcase_hash(const void *key);
+int ascii_strcase_eq(const void *a, const void *b);
 
 #endif /* _ascii_h_ */
 
