@@ -1,6 +1,4 @@
 /*
- * $Id: base32.h 13644 2007-05-13 19:29:02Z rmanfredi $
- *
  * Copyright (c) 2006, Christian Biere
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -66,7 +64,7 @@
 #ifndef BASE32_HEADER_FILE
 #define BASE32_HEADER_FILE
 
-size_t base32_decode(char *dst, size_t size, const void *data, size_t len);
+size_t base32_decode(void *dst, size_t size, const char *data, size_t len);
 size_t base32_encode(char *dst, size_t size, const void *data, size_t len);
 
 #endif /* BASE32_HEADER_FILE */
