@@ -1,6 +1,4 @@
 /*
- * $Id: cobs.c 16108 2009-03-10 15:19:52Z rmanfredi $
- *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -39,8 +37,6 @@
 
 #include "common.h"
 
-RCSID("$Id: cobs.c 16108 2009-03-10 15:19:52Z rmanfredi $")
-
 #include "cobs.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
@@ -54,14 +50,14 @@ RCSID("$Id: cobs.c 16108 2009-03-10 15:19:52Z rmanfredi $")
  * NB: the output is a linear buffer, not a vector.
  */
 char *
-cobs_encodev(struct iovec *iov, int iovcnt, size_t *retlen)
+cobs_encodev(iovec_t *iov, int iovcnt, size_t *retlen)
 {
 	size_t maxsize, len;
 	char *out;
 	char *o;						/* Iterates over output */
 	char *cp;						/* Where we'll write the code length */
-	guchar code, last_code = 0;
-	struct iovec *xiov;
+	uchar code, last_code = 0;
+	iovec_t *xiov;
 	int i;
 
 	g_assert(iov);
@@ -93,8 +89,8 @@ cobs_encodev(struct iovec *iov, int iovcnt, size_t *retlen)
 } while (0)
 
 	for (i = iovcnt, xiov = iov; i--; xiov++) {
-		const char *p = xiov->iov_base;		/* Iterates over buffer */
-		const char *end = &p[xiov->iov_len];	/* First byte off buffer */
+		const char *p = iovec_base(xiov);		/* Iterates over buffer */
+		const char *end = &p[iovec_len(xiov)];	/* First byte off buffer */
 
 		while (p != end) {
 			const char c = *p++;
@@ -136,10 +132,9 @@ cobs_encodev(struct iovec *iov, int iovcnt, size_t *retlen)
 char *
 cobs_encode(char *buf, size_t len, size_t *retlen)
 {
-	struct iovec iov;
+	iovec_t iov;
 
-	iov.iov_base = buf;
-	iov.iov_len = len;
+	iovec_set(&iov, buf, len);
 	*retlen = len;
 
 	return cobs_encodev(&iov, 1, retlen);
@@ -152,7 +147,7 @@ cobs_encode(char *buf, size_t len, size_t *retlen)
  * @returns whether the input was valid COBS encoding.
  * The length of the decoded buffer is returned in `retlen'.
  */
-gboolean
+bool
 cobs_decode_into(const char *buf, size_t len, char *out,
 	size_t outlen, size_t *retlen)
 {
@@ -160,7 +155,7 @@ cobs_decode_into(const char *buf, size_t len, char *out,
 	const char *oend = &out[outlen];	/* First byte off buffer */
 	const char *p;
 	char *o;
-	guchar last_code = 0;
+	uchar last_code = 0;
 
 	g_assert(NULL != buf);
 	g_assert(len > 0);
@@ -173,7 +168,7 @@ cobs_decode_into(const char *buf, size_t len, char *out,
 	 */
 
 	for (p = buf, o = out; p != end && o != oend; /* empty */) {
-		guchar code;
+		uchar code;
 
 		last_code = code = *p++;
 
@@ -217,7 +212,7 @@ cobs_decode_into(const char *buf, size_t len, char *out,
 /**
  * Check whether supplied buffer forms a valid COBS encoding.
  */
-gboolean
+bool
 cobs_is_valid(const char *buf, size_t len)
 {
 	const char *p, *end = &buf[len];		/* First byte off buffer */
@@ -226,7 +221,7 @@ cobs_is_valid(const char *buf, size_t len)
 	g_assert(len > 0);
 
 	for (p = buf; p != end; /* empty */) {
-		guchar code;
+		uchar code;
 
 		if (0 == (code = *p++))
 			return FALSE;			/* There cannot be any NUL */
@@ -251,7 +246,7 @@ cobs_is_valid(const char *buf, size_t len)
  * encoding.  The length of the decoded buffer is in `retlen'.
  */
 char *
-cobs_decode(char *buf, size_t len, size_t *retlen, gboolean inplace)
+cobs_decode(char *buf, size_t len, size_t *retlen, bool inplace)
 {
 	char *out;
 
@@ -286,7 +281,7 @@ cobs_decode(char *buf, size_t len, size_t *retlen, gboolean inplace)
  * @param len		length of supplied buffer
  */
 void
-cobs_stream_init(cobs_stream_t *cs, gpointer data, size_t len)
+cobs_stream_init(cobs_stream_t *cs, void *data, size_t len)
 {
 	g_assert(len != 0);
 	g_assert(data != NULL);
@@ -311,8 +306,8 @@ cobs_stream_init(cobs_stream_t *cs, gpointer data, size_t len)
  *
  * @return TRUE if OK, FALSE if we cannot put the data any more.
  */
-gboolean
-cobs_stream_write(cobs_stream_t *cs, gpointer data, size_t len)
+bool
+cobs_stream_write(cobs_stream_t *cs, void *data, size_t len)
 {
 	char *p = data;
 	const char *end = &p[len];
@@ -360,7 +355,7 @@ cobs_stream_write(cobs_stream_t *cs, gpointer data, size_t len)
  * @return the final length of the stream on success, 0 on error.
  */
 size_t
-cobs_stream_close(cobs_stream_t *cs, gboolean *saw_nul)
+cobs_stream_close(cobs_stream_t *cs, bool *saw_nul)
 {
 	g_assert(cobs_stream_is_valid(cs));
 
@@ -398,13 +393,14 @@ closed:
 void
 cobs_stream_invalidate(cobs_stream_t *cs)
 {
-	memset(cs, 0, sizeof *cs);
+	ZERO(cs);
 }
 
 /**
  * Check whether the stream descriptor is valid, for assertions.
  */
-gboolean cobs_stream_is_valid(cobs_stream_t *cs)
+bool
+cobs_stream_is_valid(cobs_stream_t *cs)
 {
 	if (NULL == cs)
 		return FALSE;
