@@ -1,6 +1,4 @@
 /*
- * $Id: header.c 17350 2010-03-03 20:48:57Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,24 +33,30 @@
 
 #include "common.h"
 
-RCSID("$Id: header.c 17350 2010-03-03 20:48:57Z rmanfredi $")
-
 #include "header.h"
-#include "atoms.h"
+
 #include "ascii.h"
-#include "glib-missing.h"
-#include "halloc.h"
-#include "unsigned.h"
-#include "misc.h"
-#include "walloc.h"
+#include "atoms.h"
+#include "buf.h"
 #include "getline.h"		/* For MAX_LINE_SIZE */
+#include "halloc.h"
+#include "hstrfn.h"
+#include "htable.h"
+#include "log.h"			/* For log_file_printable() */
+#include "misc.h"
 #include "slist.h"
+#include "str.h"
+#include "stringify.h"
+#include "unsigned.h"
+#include "walloc.h"
 
 #include "override.h"		/* Must be the last header included */
 
+enum header_magic { HEADER_MAGIC = 0x71b8484fU };
+
 /*
  * The `headers' field is a hash table indexed by field name (case-insensitive).
- * Each value (GString) holds a private copy of the string making that header,
+ * Each value (str_t *) holds a private copy of the string making that header,
  * with all continuations removed (leading spaces collapsed into one), and
  * indentical fields concatenated using ", " separators, per RFC2616.
  *
@@ -62,13 +66,25 @@ RCSID("$Id: header.c 17350 2010-03-03 20:48:57Z rmanfredi $")
  */
 
 struct header {
-	GHashTable *headers;		/**< Indexed by name */
+	enum header_magic magic;
+	htable_t *headers;			/**< Indexed by name (case-insensitively) */
 	slist_t *fields;			/**< Ordered list of header_field_t */
 	int flags;					/**< Various operating flags */
 	int size;					/**< Total header size, in bytes */
 	int num_lines;				/**< Total header lines seen */
+	int refcnt;					/**< Reference count on the structure */
 };
 
+static inline void
+header_check(const header_t * const h)
+{
+	g_assert(h != NULL);
+	g_assert(HEADER_MAGIC == h->magic);
+	g_assert(h->refcnt > 0);
+}
+
+enum header_field { HEADER_FIELD_MAGIC = 0x6e29aad7U };
+
 /**
  * A header field.
  *
@@ -89,10 +105,18 @@ struct header {
  */
 
 typedef struct {
-	char *name;				/**< Field name */
+	enum header_field magic;
+	char *name;					/**< Field name */
 	slist_t *lines;				/**< List of lines making this header */
 } header_field_t;
 
+static inline void
+header_field_check(const header_field_t * const hf)
+{
+	g_assert(hf != NULL);
+	g_assert(HEADER_FIELD_MAGIC == hf->magic);
+}
+
 /***
  *** Operating flags
  ***/
@@ -122,9 +146,9 @@ static const char *error_str[] = {
  * @return human-readable error string corresponding to error code `errnum'.
  */
 const char *
-header_strerror(guint errnum)
+header_strerror(uint errnum)
 {
-	if (errnum >= G_N_ELEMENTS(error_str))
+	if (errnum >= N_ITEMS(error_str))
 		return "Invalid error code";
 
 	return error_str[errnum];
@@ -149,16 +173,16 @@ hfield_make(const char *name)
 {
 	header_field_t *h;
 
-	h = walloc0(sizeof *h);
+	WALLOC0(h);
+	h->magic = HEADER_FIELD_MAGIC;
 	h->name = h_strdup(name);
 
 	return h;
 }
 
 static void
-hfield_free_item(gpointer p, gpointer unused_data)
+hfield_free_item(void *p)
 {
-	(void) unused_data;
 	HFREE_NULL(p);
 }
 
@@ -168,12 +192,12 @@ hfield_free_item(gpointer p, gpointer unused_data)
 static void
 hfield_free(header_field_t *h)
 {
-	if (h->lines) {
-		slist_foreach(h->lines, hfield_free_item, NULL);
-		slist_free(&h->lines);
-	}
+	header_field_check(h);
+
+	slist_free_all(&h->lines, hfield_free_item);
 	HFREE_NULL(h->name);
-	wfree(h, sizeof *h);
+	h->magic = 0;
+	WFREE(h);
 }
 
 /**
@@ -183,6 +207,8 @@ hfield_free(header_field_t *h)
 static void
 hfield_append(header_field_t *h, const char *text)
 {
+	header_field_check(h);
+
 	if (!h->lines) {
 		h->lines = slist_new();
 	}
@@ -196,8 +222,9 @@ static void
 hfield_dump(const header_field_t *h, FILE *out)
 {
 	slist_iter_t *iter;
-	gboolean first;
+	bool first;
 
+	header_field_check(h);
 	g_assert(h->lines);
 
 	fprintf(out, "%s: ", h->name);
@@ -207,10 +234,11 @@ hfield_dump(const header_field_t *h, FILE *out)
 	for (/* NOTHING */; slist_iter_has_item(iter); slist_iter_next(iter)) {
 		const char *s;
 
-		if (!first) {
+		if (first)
 			first = FALSE;
+		else
 			fputs("    ", out);			/* Continuation line */
-		}
+
 		s = slist_iter_current(iter);
 		if (is_printable_iso8859_string(s)) {
 			fputs(s, out);
@@ -218,9 +246,9 @@ hfield_dump(const header_field_t *h, FILE *out)
 			char buf[80];
 			const char *p = s;
 			int c;
-			size_t len = strlen(s);
-			gm_snprintf(buf, sizeof buf, "<%u non-printable byte%s>",
-				(unsigned) len, 1 == len ? "" : "s");
+			size_t len = vstrlen(s);
+			str_bprintf(ARYLEN(buf), "<%u non-printable byte%s>",
+				(unsigned) PLURAL(len));
 			fputs(buf, out);
 			while ((c = *p++)) {
 				if (is_ascii_print(c) || is_ascii_space(c))
@@ -231,18 +259,22 @@ hfield_dump(const header_field_t *h, FILE *out)
 		}
 		fputc('\n', out);
 	}
-	slist_iter_free(&iter);	
+	slist_iter_free(&iter);
 }
 
 /***
  *** header object
  ***/
 
-static GHashTable *
+static htable_t *
 header_get_table(header_t *o)
 {
-	if (!o->headers)
-		o->headers = g_hash_table_new(ascii_strcase_hash, ascii_strcase_eq);
+	header_check(o);
+
+	if (NULL == o->headers) {
+		o->headers = htable_create_any(ascii_strcase_hash,
+			NULL, ascii_strcase_eq);
+	}
 
 	return o->headers;
 }
@@ -255,40 +287,71 @@ header_make(void)
 {
 	header_t *o;
 
-	o = walloc0(sizeof *o);
+	WALLOC0(o);
+	o->magic = HEADER_MAGIC;
+	o->refcnt = 1;
 	return o;
 }
 
 /**
  * Frees the key/values from the headers hash.
  */
-static gboolean
-free_header_data(gpointer key, gpointer value, gpointer unused_udata)
+static bool
+free_header_data(const void *key, void *value, void *unused_udata)
 {
+	void *k;
+
 	(void) unused_udata;
 
-	HFREE_NULL(key);			/* XXX if shared, don't do that */
-	g_string_free(value, TRUE);
+	k = deconstify_pointer(key);
+	hfree(k);					/* XXX if shared, don't do that */
+	str_destroy(value);
 	return TRUE;
 }
 
+/**
+ * Take an extra reference on the header object.
+ * @return the header object.
+ */
+header_t *
+header_refcnt_inc(header_t *o)
+{
+	header_check(o);
+
+	o->refcnt++;
+	return o;
+}
+
 /**
  * Destroy header object.
  */
 void
 header_free(header_t *o)
 {
-	g_assert(o);
+	header_check(o);
+
+	if (o->refcnt > 1) {
+		o->refcnt--;
+		return;
+	}
 
 	header_reset(o);
-	wfree(o, sizeof *o);
+	o->magic = 0;
+	WFREE(o);
 }
 
-static void
-header_reset_item(gpointer p, gpointer unused_data)
+/**
+ * Destroy header object and nullify pointer holding it.
+ */
+void
+header_free_null(header_t **o_ptr)
 {
-	(void) unused_data;
-	hfield_free(p);
+	header_t *o = *o_ptr;
+
+	if (o != NULL) {
+		header_free(o);
+		*o_ptr = NULL;
+	}
 }
 
 /**
@@ -297,20 +360,14 @@ header_reset_item(gpointer p, gpointer unused_data)
 void
 header_reset(header_t *o)
 {
-	static const header_t zero_header;
+	header_check(o);
 
-	g_assert(o);
-
-	if (o->headers) {
-		g_hash_table_foreach_remove(o->headers, free_header_data, NULL);
-		g_hash_table_destroy(o->headers);
-		o->headers = NULL;
+	if (o->headers != NULL) {
+		htable_foreach_remove(o->headers, free_header_data, NULL);
+		htable_free_null(&o->headers);
 	}
-	if (o->fields) {
-		slist_foreach(o->fields, header_reset_item, NULL);
-		slist_free(&o->fields);
-	}
-	*o = zero_header;
+	slist_free_all(&o->fields, cast_to_free_fn(hfield_free));
+	o->flags = o->size = o->num_lines = 0;
 }
 
 /**
@@ -321,14 +378,16 @@ header_reset(header_t *o)
 char *
 header_get(const header_t *o, const char *field)
 {
-	GString *v;
+	str_t *v;
+
+	header_check(o);
 
 	if (o->headers) {
-		v = g_hash_table_lookup(o->headers, deconstify_gchar(field));
+		v = htable_lookup(o->headers, deconstify_char(field));
 	} else {
 		v = NULL;
 	}
-	return v ? v->str : NULL;
+	return str_2c(v);
 }
 
 /**
@@ -342,17 +401,19 @@ header_get(const header_t *o, const char *field)
 char *
 header_get_extended(const header_t *o, const char *field, size_t *len_ptr)
 {
-	GString *v;
+	str_t *v;
+
+	header_check(o);
 
 	if (o->headers) {
-		v = g_hash_table_lookup(o->headers, deconstify_gchar(field));
+		v = htable_lookup(o->headers, deconstify_char(field));
 	} else {
 		v = NULL;
 	}
 	if (v && len_ptr != NULL) {
-		*len_ptr = v->len;
+		*len_ptr = str_len(v);
 	}
-	return v ? v->str : NULL;
+	return str_2c(v);
 }
 
 /**
@@ -362,19 +423,21 @@ header_get_extended(const header_t *o, const char *field, size_t *len_ptr)
 static void
 add_header(header_t *o, const char *field, const char *text)
 {
-	GHashTable *ht;
-	GString *v;
+	htable_t *ht;
+	str_t *v;
+
+	header_check(o);
 
 	ht = header_get_table(o);
-	v = g_hash_table_lookup(ht, field);
+	v = htable_lookup(ht, field);
 	if (v) {
 		/*
 		 * Header already exists, according to RFC2616 we need to append
 		 * the value, comma-separated.
 		 */
 
-		g_string_append(v, ", ");
-		g_string_append(v, text);
+		STR_CAT(v, ", ");
+		str_cat(v, text);
 
 	} else {
 		char *key;
@@ -384,8 +447,8 @@ add_header(header_t *o, const char *field, const char *text)
 		 */
 
 		key = h_strdup(field);
-		v = g_string_new(text);
-		g_hash_table_insert(ht, key, v);
+		v = str_new_from(text);
+		htable_insert(ht, key, v);
 	}
 }
 
@@ -396,13 +459,15 @@ add_header(header_t *o, const char *field, const char *text)
 static void
 add_continuation(header_t *o, const char *field, const char *text)
 {
-	GString *v;
+	str_t *v;
 
+	header_check(o);
 	g_assert(o->headers);
-	v = g_hash_table_lookup(o->headers, field);
-	g_assert(v);
-	g_string_append_c(v, ' ');
-	g_string_append(v, text);
+
+	v = htable_lookup(o->headers, field);
+	g_assert(v != NULL);
+	str_putc(v, ' ');
+	str_cat(v, text);
 }
 
 /**
@@ -416,9 +481,10 @@ header_append(header_t *o, const char *text, int len)
 {
 	char buf[MAX_LINE_SIZE];
 	const char *p = text;
-	guchar c;
+	uchar c;
 	header_field_t *hf;
 
+	header_check(o);
 	g_assert(len >= 0);
 
 	if (o->flags & HEAD_F_EOH)
@@ -506,7 +572,7 @@ header_append(header_t *o, const char *text, int len)
 
 	} else {
 		char *b;
-		gboolean seen_space = FALSE;
+		bool seen_space = FALSE;
 
 		/*
 		 * It's a new header line.
@@ -530,7 +596,7 @@ header_append(header_t *o, const char *text, int len)
 				continue;
 			}
 			if (
-				seen_space || (c != '-' &&
+				seen_space || (c != '-' && c != '.' &&
 					(!isascii(c) || is_ascii_cntrl(c) || is_ascii_punct(c)))
 			) {
 				o->flags |= HEAD_F_SKIP;
@@ -586,7 +652,7 @@ header_append(header_t *o, const char *text, int len)
 }
 
 static void
-header_dump_item(gpointer p, gpointer user_data)
+header_dump_item(void *p, void *user_data)
 {
 	hfield_dump(p, user_data);
 }
@@ -598,6 +664,11 @@ header_dump_item(gpointer p, gpointer user_data)
 void
 header_dump(FILE *out, const header_t *o, const char *trailer)
 {
+	header_check(o);
+
+	if (!log_file_printable(out))
+		return;
+
 	if (o->fields) {
 		slist_foreach(o->fields, header_dump_item, out);
 	}
@@ -622,7 +693,7 @@ struct header_fmt {
 	enum header_fmt_magic magic;
 	size_t max_size;		/**< Maximum line length, including "\r\n" + NUL */
 	size_t maxlen;			/**< Maximum line length before continuation */
-	GString *header;		/**< Header being built */
+	str_t *header;			/**< Header being built */
 	const char *sep;		/**< Optional separator */
 	size_t seplen;			/**< Length of separator string */
 	size_t stripped_seplen;	/**< Length of separator without trailing space */
@@ -691,20 +762,20 @@ header_fmt_make(const char *field, const char *separator,
 
 	g_assert(size_is_non_negative(len_hint));
 
-	hf = walloc(sizeof(*hf));
+	WALLOC(hf);
 	hf->magic = HEADER_FMT_MAGIC;
-	hf->header = g_string_sized_new(len_hint ? len_hint : HEADER_FMT_DFLT_LEN);
+	hf->header = str_new(len_hint ? len_hint : HEADER_FMT_DFLT_LEN);
 	hf->maxlen = HEADER_FMT_LINE_LEN;
 	hf->data_emitted = FALSE;
 	hf->frozen = FALSE;
 	hf->max_size = max_size;
 	hf->sep = atom_str_get(separator ? separator : "");
-	hf->seplen = strlen(hf->sep);
+	hf->seplen = vstrlen(hf->sep);
 	hf->stripped_seplen = stripped_strlen(hf->sep, hf->seplen);
-	g_string_append(hf->header, field);
-	g_string_append(hf->header, ": ");
+	str_cat(hf->header, field);
+	STR_CAT(hf->header, ": ");
 
-	hf->current_len = hf->header->len;
+	hf->current_len = str_len(hf->header);
 
 	/*
 	 * If right from the start the header would be larger than the configured
@@ -712,9 +783,9 @@ header_fmt_make(const char *field, const char *separator,
 	 * will be "", the empty string.
 	 */
 
-	if (hf->header->len + sizeof("\r\n") > hf->max_size) {
+	if (str_len(hf->header) + sizeof("\r\n") > hf->max_size) {
 		hf->empty = TRUE;
-		g_string_truncate(hf->header, 0);
+		str_setlen(hf->header, 0);
 	} else {
 		hf->empty = FALSE;
 	}
@@ -747,10 +818,10 @@ header_fmt_free(header_fmt_t **hf_ptr)
 	if (hf) {
 		header_fmt_check(hf);
 
-		g_string_free(hf->header, TRUE);
+		str_destroy_null(&hf->header);
 		atom_str_free_null(&hf->sep);
 		hf->magic = 0;
-		wfree(hf, sizeof *hf);
+		WFREE(hf);
 		*hf_ptr = NULL;
 	}
 }
@@ -760,7 +831,7 @@ header_fmt_free(header_fmt_t **hf_ptr)
  * within the maximum header size requirement in case a continuation
  * is emitted, and using the configured separator.
  */
-gboolean
+bool
 header_fmt_value_fits(const header_fmt_t *hf, size_t len)
 {
 	size_t final_len;
@@ -779,7 +850,7 @@ header_fmt_value_fits(const header_fmt_t *hf, size_t len)
 	 * followed by "\r\n\t" (3 chars).
 	 */
 
-	final_len = size_saturate_add(hf->header->len, len);
+	final_len = size_saturate_add(str_len(hf->header), len);
 
 	n = size_saturate_add(hf->current_len, size_saturate_add(len, hf->seplen));
 	if (n <= hf->maxlen) {
@@ -807,13 +878,13 @@ header_fmt_value_fits(const header_fmt_t *hf, size_t len)
  * resulted in the header being larger than the configured max size (the
  * header line is left in the state it was in upon entry, in that case).
  */
-static gboolean
+static bool
 header_fmt_append_full(header_fmt_t *hf, const char *str,
 	const char *separator, size_t slen, size_t sslen)
 {
 	size_t len, curlen;
 	gsize gslen;
-	gboolean success;
+	bool success;
 
 	header_fmt_check(hf);
 	g_assert(size_is_non_negative(slen));
@@ -822,48 +893,50 @@ header_fmt_append_full(header_fmt_t *hf, const char *str,
 	if (hf->empty)
 		return FALSE;
 
-	gslen = hf->header->len;
-	len = strlen(str);
+	gslen = str_len(hf->header);
+	len = vstrlen(str);
 	curlen = hf->current_len;
 	g_assert(size_is_non_negative(curlen));
 
 	g_assert(len <= INT_MAX);	/* Legacy bug */
 
 	if (
-		size_saturate_add(curlen, size_saturate_add(len, slen)) > UNSIGNED(hf->maxlen)
+		size_saturate_add(curlen, size_saturate_add(len, slen)) >
+			UNSIGNED(hf->maxlen)
 	) {
 		/*
 		 * Emit sperator, if any and data was already emitted.
 		 */
 
 		if (separator != NULL && hf->data_emitted) {
-			sslen = (size_t)-1 != sslen ? sslen : stripped_strlen(separator, slen);
-			g_string_append_len(hf->header, separator, sslen);
+			sslen = (size_t)-1 != sslen ? sslen :
+				stripped_strlen(separator, slen);
+			str_cat_len(hf->header, separator, sslen);
 		}
 
-		g_string_append(hf->header, "\r\n\t");	/* Includes continuation */
+		STR_CAT(hf->header, "\r\n\t");			/* Includes continuation */
 		curlen = 1;								/* One tab */
 	} else if (hf->data_emitted) {
-		g_string_append(hf->header, separator);
+		str_cat(hf->header, separator);
 		curlen += slen;
 	}
 
-	g_string_append(hf->header, str);
+	str_cat(hf->header, str);
 
 	/*
 	 * Check for overflows, undoing string changes if needed.
 	 */
 
-	if (hf->header->len + sizeof("\r\n") > hf->max_size) {
+	if (str_len(hf->header) + sizeof("\r\n") > hf->max_size) {
 		success = FALSE;
-		g_string_truncate(hf->header, gslen);	/* Undo! */
+		str_setlen(hf->header, gslen);			/* Undo! */
 	} else {
 		success = TRUE;
 		hf->data_emitted = TRUE;
 		hf->current_len = curlen + len;
 	}
 
-	g_assert(hf->header->len + sizeof("\r\n") <= hf->max_size);
+	g_assert(str_len(hf->header) + sizeof("\r\n") <= hf->max_size);
 
 	return success;
 }
@@ -881,7 +954,7 @@ header_fmt_append_full(header_fmt_t *hf, const char *str,
  * @return TRUE if we were able to append the data whilst remaining under
  * the configured maximum length.
  */
-gboolean
+bool
 header_fmt_append(header_fmt_t *hf, const char *str, const char *separator)
 {
 	size_t seplen;
@@ -889,7 +962,7 @@ header_fmt_append(header_fmt_t *hf, const char *str, const char *separator)
 	header_fmt_check(hf);
 	g_assert(!hf->frozen);
 
-	seplen = (separator == NULL) ? 0 : strlen(separator);
+	seplen = (separator == NULL) ? 0 : vstrlen(separator);
 
 	return header_fmt_append_full(hf, str, separator, seplen, (size_t)-1);
 }
@@ -906,7 +979,7 @@ header_fmt_append(header_fmt_t *hf, const char *str, const char *separator)
  * @return TRUE if we were able to append the data whilst remaining under
  * the configured maximum length.
  */
-gboolean
+bool
 header_fmt_append_value(header_fmt_t *hf, const char *str)
 {
 	header_fmt_check(hf);
@@ -924,7 +997,7 @@ header_fmt_length(const header_fmt_t *hf)
 {
 	header_fmt_check(hf);
 
-	return hf->header->len;
+	return str_len(hf->header);
 }
 
 /**
@@ -938,10 +1011,10 @@ header_fmt_end(header_fmt_t *hf)
 	g_assert(!hf->frozen);
 
 	if (!hf->empty)
-		g_string_append(hf->header, "\r\n");
+		STR_CAT(hf->header, "\r\n");
 	hf->frozen = TRUE;
 
-	g_assert(UNSIGNED(hf->header->len) < hf->max_size);
+	g_assert(str_len(hf->header) < hf->max_size);
 }
 
 /**
@@ -952,7 +1025,7 @@ header_fmt_string(const header_fmt_t *hf)
 {
 	header_fmt_check(hf);
 
-	return hf->header->str;		/* Guaranteed to be always NUL-terminated */
+	return str_2c(hf->header);	/* Guaranteed to be always NUL-terminated */
 }
 
 /**
@@ -964,16 +1037,18 @@ header_fmt_string(const header_fmt_t *hf)
 const char *
 header_fmt_to_string(const header_fmt_t *hf)
 {
-	static char line[HEADER_FMT_MAX_SIZE + 1];
+	buf_t *b = buf_private(G_STRFUNC, HEADER_FMT_MAX_SIZE + 1);
+	char *p = buf_data(b);
+	size_t n = buf_size(b);
 
 	header_fmt_check(hf);
 
-	if (UNSIGNED(hf->header->len) >= sizeof line) {
-		g_warning("trying to format too long an HTTP line (%lu bytes)",
-			(unsigned long) hf->header->len);
+	if (str_len(hf->header) >= n) {
+		g_warning("trying to format too long an HTTP line (%zu bytes)",
+			str_len(hf->header));
 	}
-	clamp_strncpy(line, sizeof line, hf->header->str, hf->header->len);
-	return line;
+	clamp_strncpy(p, n, str_2c(hf->header), str_len(hf->header));
+	return p;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
