@@ -1,6 +1,4 @@
 /*
- * $Id: misc.h 17213 2009-11-09 17:13:18Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -40,7 +38,6 @@
  * - Size string conversions
  * - SHA1<->base32 string conversion
  * - Tests
- * - Random numbers
  * - Stuff...
  *
  * @author Raphael Manfredi
@@ -53,7 +50,8 @@
 #include "common.h"
 
 #include "fs_free_space.h"
-#include "tm.h"
+#include "pslist.h"
+#include "sha1.h"
 #include "vmm.h"
 
 #define SIZE_FIELD_MAX		64	/**< Max size of sprintf-ed size quantity */
@@ -65,6 +63,12 @@ typedef struct short_string {
 	char str[SIZE_FIELD_MAX];
 } short_string_t;
 
+static inline G_CONST ALWAYS_INLINE int
+is_dir_separator(int c)
+{
+	return '/' == c || G_DIR_SEPARATOR == c;
+}
+
 /**
  * Converts an integer to a single hexadecimal ASCII digit. The are no checks,
  * this is just a convenience function.
@@ -72,11 +76,11 @@ typedef struct short_string {
  * @param x An integer between 0 and 15.
  * @return The ASCII character corresponding to the hex digit [0-9a-f].
  */
-static inline guchar
-hex_digit(guchar x)
+static inline G_PURE ALWAYS_INLINE uchar
+hex_digit(uchar x)
 {
 	extern const char hex_alphabet_lower[];
-	return hex_alphabet_lower[x & 0xf]; 
+	return hex_alphabet_lower[x & 0xf];
 }
 
 #if !GLIB_CHECK_VERSION(2,4,0)
@@ -85,7 +89,7 @@ g_strip_context(const char *id, const char *val)
 {
 	const char *s;
 
-	s = id != val ? NULL : strchr(id, '|');
+	s = id != val ? NULL : vstrchr(id, '|');
 	return s ? ++s : val;
 }
 #endif /* GLib < 2.4.0 */
@@ -93,7 +97,7 @@ g_strip_context(const char *id, const char *val)
 static inline WARN_UNUSED_RESULT char *
 skip_dir_separators(const char *s)
 {
-	while ('/' == s[0] || G_DIR_SEPARATOR == s[0])
+	while (is_dir_separator(s[0]))
 		s++;
 
 	return deconstify_gchar(s);
@@ -113,32 +117,35 @@ const char *local_hostname(void);
 /*
  * Size string conversions
  */
-const char *short_size(guint64 size, gboolean metric);
-const char *short_html_size(guint64 size, gboolean metric);
-const char *short_kb_size(guint64 size, gboolean metric);
-const char *short_kb_size2(guint64 size, gboolean metric);
-const char *short_rate(guint64 rate, gboolean metric);
-const char *short_byte_size(guint64 size, gboolean metric);
-const char *short_byte_size2(guint64 size, gboolean metric);
-const char *compact_size(guint64 size, gboolean metric);
-const char *compact_size2(guint64 size, gboolean metric);
-const char *compact_rate(guint64 rate, gboolean metric);
-const char *compact_kb_size(guint32 size, gboolean metric);
-const char *nice_size(guint64 size, gboolean metric);
-char *short_value(char *buf, size_t size, guint64 v, gboolean metric);
-char *compact_value(char *buf, size_t size, guint64 v, gboolean metric);
+const char *short_frequency(uint64 freq);
+const char *short_size(uint64 size, bool metric);
+const char *short_size2(uint64 size, bool metric);
+const char *short_size3(uint64 size, bool metric);
+const char *short_html_size(uint64 size, bool metric);
+const char *short_kb_size(uint64 size, bool metric);
+const char *short_kb_size2(uint64 size, bool metric);
+const char *short_rate(uint64 rate, bool metric);
+const char *short_byte_size(uint64 size, bool metric);
+const char *short_byte_size2(uint64 size, bool metric);
+const char *compact_size(uint64 size, bool metric);
+const char *compact_size2(uint64 size, bool metric);
+const char *compact_rate(uint64 rate, bool metric);
+const char *compact_kb_size(uint32 size, bool metric);
+const char *nice_size(uint64 size, bool metric);
+char *long_value(char *buf, size_t size, uint64 v, bool metric);
+char *short_value(char *buf, size_t size, uint64 v, bool metric);
+char *compact_value(char *buf, size_t size, uint64 v, bool metric);
 
-size_t short_byte_size_to_buf(guint64 size, gboolean metric, char *, size_t);
-size_t short_kb_size_to_buf(guint64 size, gboolean metric, char *, size_t);
+size_t short_byte_size_to_buf(uint64 size, bool metric, char *, size_t);
+size_t short_kb_size_to_buf(uint64 size, bool metric, char *, size_t);
+size_t short_size_to_string_buf(uint64 size, bool metric, char *, size_t);
 
-short_string_t short_rate_get_string(guint64 rate, gboolean metric);
+short_string_t short_rate_get_string(uint64 rate, bool metric);
+short_string_t long_value_get_string(uint64 value, bool metric);
 
 /*
  * SHA1<->base32 string conversion
  */
-typedef struct sha1 {
-	char data[SHA1_RAW_SIZE];
-} sha1_t;
 
 #define SHA1_URN_LENGTH	(CONST_STRLEN("urn:sha1:") + SHA1_BASE32_SIZE)
 
@@ -147,9 +154,11 @@ const char *sha1_to_urn_string(const struct sha1 *);
 size_t sha1_to_urn_string_buf(const struct sha1 *, char *dst, size_t size);
 char *sha1_to_base32_buf(const struct sha1 *, char *dst, size_t size);
 const char *sha1_base32(const struct sha1 *);
+char *sha1_to_base16_buf(const struct sha1 *, char *dst, size_t size);
+const char *sha1_base16(const struct sha1 *);
 const struct sha1 *base32_sha1(const char *base32);
 
-static inline int
+static inline G_PURE int
 sha1_cmp(const struct sha1 *a, const struct sha1 *b)
 {
 	return memcmp(a, b, SHA1_RAW_SIZE);
@@ -179,7 +188,7 @@ const char *bitprint_to_urn_string(const struct sha1 *, const struct tth *);
 struct guid;
 
 const char *guid_hex_str(const struct guid *);
-gboolean hex_to_guid(const char *, struct guid *);
+bool hex_to_guid(const char *, struct guid *);
 size_t guid_to_string_buf(const struct guid *, char *, size_t);
 const char *guid_to_string(const struct guid *);
 
@@ -187,7 +196,7 @@ const char *guid_to_string(const struct guid *);
  * GUID<->base32 string conversion
  */
 const char *guid_base32_str(const struct guid *);
-const struct guid *base32_to_guid(const char *);
+const struct guid *base32_to_guid(const char *, struct guid *);
 
 /*
  * Generic binary to hexadecimal conversion.
@@ -197,13 +206,13 @@ size_t bin_to_hex_buf(const void *data, size_t len, char *dst, size_t size);
 /*
  * Tests
  */
-gboolean is_directory(const char *pathname);
-gboolean is_regular(const char *pathname);
-gboolean is_symlink(const char *pathname);
+bool is_directory(const char *pathname);
+bool is_regular(const char *pathname);
+bool is_symlink(const char *pathname);
 int is_same_file(const char *, const char *);
 
 /**
- * Tries to extrace the file mode from a struct dirent. Not all systems
+ * Tries to extract the file mode from a struct dirent. Not all systems
  * support this, in which case zero is returned. Types other than regular
  * files, directories and symlinks are ignored and gain a value of zero
  * as well.
@@ -231,15 +240,30 @@ dir_entry_mode(const struct dirent *dir_entry)
 	return 0;
 }
 
-/*
- * Random numbers
- */
+#ifndef MINGW32
+/* There is a MINGW32 version defined for Windows in lib/mingw32.c */
+static inline const char *
+dir_entry_filename(const struct dirent *dir_entry)
+{
+	g_assert(dir_entry != NULL);
 
-void random_init(void);
-guint32 random_value(guint32 max) WARN_UNUSED_RESULT;
-guint32 random_u32(void) WARN_UNUSED_RESULT;
-void random_bytes(void *dst, size_t size);
-void guid_random_fill(struct guid *);
+	return dir_entry->d_name;
+}
+
+/* There is a MINGW32 version defined for Windows in lib/mingw32.c */
+static inline size_t
+dir_entry_namelen(const struct dirent *dir_entry)
+{
+	g_assert(dir_entry != NULL);
+
+#ifdef HAS_DIRENT_D_NAMLEN
+	if G_LIKELY(dir_entry->d_namlen != 0)
+		return dir_entry->d_namlen;
+#endif	/* HAS_DIRENT_D_NAMLEN */
+
+	return vstrlen(dir_entry->d_name);
+}
+#endif	/* MINGW32 */
 
 /*
  * Stuff
@@ -248,19 +272,25 @@ void guid_random_fill(struct guid *);
 void misc_init(void);
 void misc_close(void);
 
-size_t str_chomp(char *str, size_t len);
-int hex2int(guchar c);
-gboolean is_printable(const char *buf, int len);
-void dump_hex(FILE *, const char *, gconstpointer, int);
+size_t strchomp(char *str, size_t len);
+int hex2int(uchar c);
+bool is_printable(const char *buf, int len);
+void dump_hex(FILE *, const char *, const void *, int);
+void dump_hex_vec(FILE *out, const char *title,
+	const iovec_t *iov, size_t iovcnt);
 void dump_string(FILE *out, const char *str, size_t len, const char *trailer);
-gboolean is_printable_iso8859_string(const char *s);
+bool is_printable_iso8859_string(const char *s);
 void locale_strlower(char *, const char *);
-int highest_bit_set(guint32 n) G_GNUC_CONST;
 size_t common_leading_bits(
-	gconstpointer k1, size_t k1bits, gconstpointer k2, size_t k2bits);
+	const void *k1, size_t k1bits, const void *k2, size_t k2bits)
+	G_PURE;
 float force_range(float value, float min, float max);
-char *short_filename(char *fullname);
+const char *short_filename(const char *fullname);
 char *data_hex_str(const char *data, size_t len);
+char *xml_indent(const char *text, size_t *lenp);
+char *xml_indent_buf(const void *buf, size_t len, size_t *lenp);
+pslist_t *dirlist_parse(const char *dirs);
+char *dirlist_to_string(const pslist_t *pl_dirs);
 
 #if defined(S_IROTH) && defined(S_IXOTH)
 /* 0755 */
@@ -272,90 +302,44 @@ char *data_hex_str(const char *data, size_t len);
 
 int create_directory(const char *dir, mode_t mode);
 
-int parse_major_minor(const char *src, char const **endptr,
-	guint *major, guint *minor);
 char *is_strprefix(const char *s, const char *prefix) WARN_UNUSED_RESULT;
 char *is_strcaseprefix(const char *s, const char *prefix) WARN_UNUSED_RESULT;
-gboolean is_strsuffix(const char *str, size_t len, const char *suffix);
+char *is_strsuffix(const char *str, size_t len, const char *suffix);
+char *is_strcasesuffix(const char *str, size_t len, const char *suffix);
+char *is_bufprefix(const char *str, size_t len, const char *prefix);
+char *is_bufcaseprefix(const char *str, size_t len, const char *prefix);
 size_t html_escape(const char *src, char *dst, size_t dst_size);
-guint32 html_decode_entity(const char *src, const char **endptr);
-int canonize_path(char *dst, const char *path);
-
-typedef void (*signal_handler_t)(int signo);
-signal_handler_t set_signal(int signo, signal_handler_t handler);
-
+uint32 html_decode_entity(const char *src, const char **endptr);
+const char *symbolic_errno(int errnum);
+const char * english_strerror(int errnum);
 void normalize_dir_separators(char *);
 size_t memcmp_diff(const void *a, const void *b, size_t n);
-guint32 cpu_noise(void);
+int bitcmp(const void *s1, const void *s2, size_t n);
 
-static inline guint
-pointer_hash_func(const void *p)
-{
-	unsigned long v = pointer_to_ulong(p);
-	return (((guint64) 0x4F1BBCDCUL * v) >> 32) ^ v;
-}
+size_t clamp_strlen(const char *src, size_t src_size);
 
 /**
- * An strcpy() that returns the length of the copied string.
+ * Returns the length of the string plus one, i.o.w.
+ * the required buffer size in bytes.
  */
 static inline size_t
-strcpy_len(char *dest, const char *src)
+strsize(const char *src)
 {
-	const char *p = src;
-	char *q = dest;
-	int c;
-
-	g_assert(dest != NULL);
-
-	if (NULL == src)
-		return 0;
-	
-	while ((c = *p++))
-		*q++ = c;
-
-	*q = '\0';
-
-	return q - dest;
-}
-
-/**
- * Determines the length of a NUL-terminated string looking only at the first
- * "src_size" bytes. If src[0..size] contains no NUL byte, "src_size" is
- * returned. Otherwise, the returned value is identical to strlen(str). Thus,
- * it is safe to pass a possibly non-terminated buffer.
- *
- * @param src An initialized buffer.
- * @param src_size The size of src in number of bytes. IF AND ONLY IF,
- *        src is NUL-terminated, src_size may exceed the actual buffer length.
- * @return The number of bytes in "src" before the first found NUL or src_size
- *		   if there is no NUL.
- */
-static inline size_t
-clamp_strlen(const char *src, size_t src_size)
-{
-	const char *p;
-
-	/* @NOTE: memchr() is intentionally NOT used because 'src_size' is allowed
-	 *        to exceed the size of the memory object 'src'.
-	 */
-	for (p = src; src_size-- > 0 && '\0' != *p; p++)
-		continue;
-
-	return p - src;
+	return vstrlen(src) + 1;
 }
 
 /**
  * Copies at most MIN(dst_size, src_len) bytes from "src" to "dst".
  *
  * @param dst the destination buffer.
- * @param dst_size the size of dst in number of bytes. 
- * @param src the source buffer. 
+ * @param dst_size the size of dst in number of bytes.
+ * @param src the source buffer.
  * @param src_len the length of src in number of bytes.
  *
  * @return The number of copied bytes.
  */
 static inline size_t
-clamp_memcpy(char *dst, size_t dst_size, const char *src, size_t src_len)
+clamp_memcpy(void *dst, size_t dst_size, const void *src, size_t src_len)
 {
 	size_t n;
 
@@ -364,6 +348,40 @@ clamp_memcpy(char *dst, size_t dst_size, const char *src, size_t src_len)
 	return n;
 }
 
+/**
+ * Sets MIN(dst_size, src_len) bytes starting at dst to 'c'.
+ *
+ * @param dst the destination buffer.
+ * @param dst_size the size of dst in number of bytes.
+ * @param c the value to set each byte to.
+ * @param n the number of bytes to set.
+ *
+ * @return The number of set bytes.
+ */
+static inline size_t
+clamp_memset(void *dst, size_t dst_size, char c, size_t n)
+{
+	n = MIN(dst_size, n);
+	memset(dst, c, n);
+	return n;
+}
+
+/**
+ * Compare at most MIN(a_len, b_len) bytes between "a" and "b".
+ *
+ * @param a		the first buffer.
+ * @param a_len the length of the first buffer, in bytes
+ * @param b		the second buffer.
+ * @param b_len the length of the second buffer, in bytes
+ *
+ * @return the result of the memcmp() operation (-1, 0, +1).
+ */
+static inline int
+clamp_memcmp(const void *a, size_t a_len, const void *b, size_t b_len)
+{
+	return memcmp(a, b, MIN(a_len, b_len));
+}
+
 /**
  * Copies at most MIN(dst_size - 1, src_len) characters from the buffer "src"
  * to the buffer "dst", ensuring the resulting string in "dst" is
@@ -374,7 +392,7 @@ clamp_memcpy(char *dst, size_t dst_size, const char *src, size_t src_len)
  * @NOTE: The 'dst' buffer is NOT padded with NUL-bytes.
  *
  * @param dst the destination buffer.
- * @param dst_size the size of dst in number of bytes. 
+ * @param dst_size the size of dst in number of bytes.
  * @param src a NUL-terminated string or at an initialized buffer of least
  *        "src_len" bytes.
  * @param src_len the length of src in number of bytes to copy at maximum. May
@@ -407,7 +425,7 @@ clamp_strncpy(char *dst, size_t dst_size, const char *src, size_t src_len)
  * @NOTE: The 'dst' buffer is NOT padded with NUL-bytes.
  *
  * @param dst the destination buffer.
- * @param dst_size the size of dst in number of bytes. 
+ * @param dst_size the size of dst in number of bytes.
  * @param src a NUL-terminated string.
  *
  * @return The length of the resulting string in number of bytes.
@@ -418,6 +436,29 @@ clamp_strcpy(char *dst, size_t dst_size, const char *src)
 	return clamp_strncpy(dst, dst_size, src, (size_t) -1);
 }
 
+/**
+ * Appends at most "dst_size - 1" characters from the NUL-terminated string
+ * "src" to the buffer "dst", ensuring the resulting string in "dst" is
+ * NUL-terminated and truncating it if necessary.
+ *
+ * @NOTE: The 'dst' buffer is NOT padded with NUL-bytes.
+ *
+ * @param dst the destination buffer. Must be initialized.
+ * @param dst_size the size of dst in number of bytes.
+ * @param src a NUL-terminated string.
+ *
+ * @return The length of the resulting string in number of bytes.
+ */
+static inline size_t
+clamp_strcat(char *dst, size_t dst_size, const char *src)
+{
+	size_t dst_len;
+
+	dst_len = clamp_strlen(dst, dst_size);
+	dst += dst_len;
+	dst_size -= dst_len;
+	return dst_len + clamp_strcpy(dst, dst_size, src);
+}
 
 static inline const char *
 NULL_STRING(const char *s)
@@ -434,30 +475,30 @@ EMPTY_STRING(const char *s)
 /**
  * Is string NULL or empty?
  */
-static inline gboolean
+static inline bool
 is_null_or_empty(const char *s)
 {
 	return NULL == s || '\0' == *s;
 }
 
 /**
- * Swap endianness of a guint32.
+ * Swap endianness of a uint32.
  *
- * @param i the guint32 to swap
+ * @param i the uint32 to swap
  *
  * @returns the value of i after swapping its byte order.
  */
-static inline G_GNUC_CONST guint32
-swap_guint32(guint32 i)
+static inline G_CONST uint32
+swap_uint32(uint32 i)
 {
-	guint32 a;
-	guint32 b;
+	uint32 a;
+	uint32 b;
                                   /* i -> ABCD */
 	a = (i & 0x00ff00ff) << 8;    /* a -> B0D0 */
 	b = (i & 0xff00ff00) >> 8;    /* b -> 0A0C */
 	i = a | b;                    /* i -> BADC */
 	i = (i << 16) | (i >> 16);    /* i -> DCBA */
-    
+
 	return i;
 }
 
@@ -469,15 +510,15 @@ swap_guint32(guint32 i)
  * @param netmask an IPv4 netmask in host byte order.
  * @return The CIDR prefix length (0..32).
  */
-static inline G_GNUC_CONST WARN_UNUSED_RESULT guint8
-netmask_to_cidr(guint32 netmask)
-#ifdef HAVE_BUILTIN_POPCOUNT
+static inline G_CONST WARN_UNUSED_RESULT uint8
+netmask_to_cidr(uint32 netmask)
+#ifdef HAS_BUILTIN_POPCOUNT
 {
-	__builtin_popcount(netmask);
+	return __builtin_popcount(netmask);
 }
-#else	/* HAVE_BUILTIN_POPCOUNT */
+#else	/* !HAS_BUILTIN_POPCOUNT */
 {
-	guint8 bits = 32;
+	uint8 bits = 32;
 
 	while (0 == (netmask & 0x1)) {
 		netmask >>= 1;
@@ -485,7 +526,7 @@ netmask_to_cidr(guint32 netmask)
 	}
 	return bits;
 }
-#endif /* HAVE_BUILTIN_POPCOUNT */
+#endif /* HAS_BUILTIN_POPCOUNT */
 
 /**
  * Converts the CIDR prefix length to a IPv4 netmask in host byte order.
@@ -494,10 +535,10 @@ netmask_to_cidr(guint32 netmask)
  * @param bits A value between 1..32.
  * @return The equivalent netmask in host byte order.
  */
-static inline G_GNUC_CONST WARN_UNUSED_RESULT guint32
-cidr_to_netmask(guint bits)
+static inline ALWAYS_INLINE G_CONST WARN_UNUSED_RESULT uint32
+cidr_to_netmask(uint bits)
 {
-	return (guint32)-1 << (32 - bits);
+	return (uint32)-1 << (32 - bits);
 }
 
 /**
@@ -522,13 +563,13 @@ round_size_fast(size_t align, size_t n)
 	return (n + mask) & ~mask;
 }
 
+void guid_random_fill(struct guid *);
+
 /*
  * Syscall wrappers for errno == 0 bug. --RAM, 27/10/2003
  */
 
-struct stat;
-
-static inline gboolean
+static inline bool
 is_temporary_error(int error)
 {
   switch (error) {
@@ -542,22 +583,22 @@ is_temporary_error(int error)
   return FALSE;
 }
 
-/* Wrapper around lseek() to handle filesize -> off_t conversion. */
+/* Wrapper around lseek() to handle filesize -> fileoffset_t conversion. */
 int seek_to_filepos(int fd, filesize_t pos);
 filesize_t get_random_file_offset(const filesize_t size);
 
-guint filesize_per_100(filesize_t size, filesize_t part);
-guint filesize_per_1000(filesize_t size, filesize_t part);
-guint filesize_per_10000(filesize_t size, filesize_t part);
+uint filesize_per_100(filesize_t size, filesize_t part);
+uint filesize_per_1000(filesize_t size, filesize_t part);
+uint filesize_per_10000(filesize_t size, filesize_t part);
 
 /*
  * CIDR split of IP range.
  */
 
-typedef void (*cidr_split_t)(guint32 ip, guint bits, gpointer udata);
+typedef void (*cidr_split_t)(uint32 ip, uint bits, void *udata);
 
 void ip_range_split(
-	guint32 lower_ip, guint32 upper_ip, cidr_split_t cb, gpointer udata);
+	uint32 lower_ip, uint32 upper_ip, cidr_split_t cb, void *udata);
 
 /**
  * Perform a binary search over an array.
@@ -607,7 +648,7 @@ G_STMT_START { \
 #define BINARY_ARRAY_SORTED(bs_array, bs_type, bs_field, bs_cmp, bs_field2str) \
 G_STMT_START { \
 	size_t bs_index; \
-	size_t bs_size = G_N_ELEMENTS(bs_array); \
+	size_t bs_size = N_ITEMS(bs_array); \
 \
 	for (bs_index = 1; bs_index < bs_size; bs_index++) { \
 		const bs_type *prev = &bs_array[bs_index - 1]; \
@@ -626,8 +667,8 @@ G_STMT_START { \
  * @param x An integer between 0 and 9.
  * @return The ASCII character corresponding to the decimal digit [0-9].
  */
-static inline guchar
-dec_digit(guchar x)
+static inline uchar
+dec_digit(uchar x)
 {
 	static const char dec_alphabet[] = "0123456789";
 	return dec_alphabet[x % 10];
@@ -648,14 +689,14 @@ dec_digit(guchar x)
  * @return The resulting length of string not counting the termating NUL.
  *         Note that NULs that might have been copied from "src" are
  *         included in this count. Thus strlen(dst) would return a lower
- *         value in this case. 
+ *         value in this case.
  */
 static inline size_t
 reverse_strlcpy(char * const dst, size_t size,
 	const char *src, size_t src_len)
 {
 	char *p = dst;
-	
+
 	if (size-- > 0) {
 		const char *q = &src[src_len], *end = &dst[MIN(src_len, size)];
 
@@ -668,6 +709,52 @@ reverse_strlcpy(char * const dst, size_t size,
 	return p - dst;
 }
 
+/**
+ * Encodes a variable-length integer. This encoding is equivalent to
+ * little-endian encoding whereas trailing zeros are discarded.
+ *
+ * @param v		the value to encode.
+ * @param data  must point to a sufficiently large buffer. At maximum
+ *				8 bytes are required.
+ *
+ * @return the length in bytes of the encoded variable-length integer.
+ */
+static inline int
+vlint_encode(uint64 v, char *data)
+{
+	char *p;
+
+	for (p = data; v != 0; v >>= 8)	{
+		*p++ = v & 0xff;
+	}
+
+	return p - data;
+}
+
+/**
+ * Decodes a variable-length integer. This encoding is equivalent to
+ * little-endian encoding whereas trailing zeros are discarded.
+ *
+ * @param data	the payload to decode.
+ * @param len	the length of data in bytes.
+ *
+ * @return The decoded value.
+ */
+static inline uint64
+vlint_decode(const char *data, size_t len)
+{
+	uint64 v;
+	uint i;
+
+	v = 0;
+	if (len <= 8) {
+		for (i = 0; i < len; i++) {
+			v |= (((uint64) data[i]) & 0xff) << (i * 8);
+		}
+	}
+	return v;
+}
+
 #endif /* _misc_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
