@@ -1,6 +1,4 @@
 /*
- * $Id: stats.h 16188 2009-03-17 17:42:15Z rmanfredi $
- *
  * Copyright (c) 2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -42,13 +40,14 @@
  * One dimension statistics.
  */
 
-struct statx;
+struct statistics;
 
-typedef struct statx statx_t;
+typedef struct statistics statx_t;
 
 statx_t *statx_make(void);
-statx_t * statx_make_nodata(void);
+statx_t *statx_make_nodata(void);
 void statx_free(statx_t *);
+void statx_free_null(statx_t **);
 void statx_clear(statx_t *);
 void statx_add(statx_t *, double);
 void statx_remove(statx_t *, double);
@@ -57,8 +56,14 @@ int statx_n(const statx_t *);
 double statx_avg(const statx_t *);
 double statx_sdev(const statx_t *);
 double statx_var(const statx_t *);
+double statx_stderr(const statx_t *);
 double *statx_data(const statx_t *);
 
+void statx_foreach(const statx_t *sx, double_data_fn_t cb, void *udata);
+size_t statx_foreach_remove(statx_t *sx, double_data_rm_fn_t cb, void *udata);
+
+size_t statx_remove_outliers(statx_t *sx, double range);
+
 #endif /* _stats_h_ */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
