@@ -1,6 +1,4 @@
 /*
- * $Id: tiger.c 16103 2009-03-10 14:53:39Z rmanfredi $
- *
  * Copyright (c) 2003, Jeroen Asselman
  *
  *----------------------------------------------------------------------
@@ -19,12 +17,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
 /* (PD) 2001 The Bitzi Corporation
- * Please see file COPYING or http://bitzi.com/publicdomain 
+ * Please see file COPYING or http://bitzi.com/publicdomain
  * for more info.
  *
  * Created and released into the public domain by Eli Biham
@@ -48,8 +46,6 @@
 
 #include "common.h"
 
-RCSID("$Id: tiger.c 16103 2009-03-10 14:53:39Z rmanfredi $")
-
 #include "endian.h"
 #include "misc.h"
 #include "base32.h"
@@ -69,7 +65,7 @@ RCSID("$Id: tiger.c 16103 2009-03-10 14:53:39Z rmanfredi $")
 
 #include "tiger_sboxes.h"
 
-#define U64_FROM_2xU32(hi, lo) (((guint64) (hi) << 32) | (lo))
+#define U64_FROM_2xU32(hi, lo) (((uint64) (hi) << 32) | (lo))
 
 #define t1 (tiger_sboxes)
 #define t2 (&tiger_sboxes[256])
@@ -139,9 +135,9 @@ RCSID("$Id: tiger.c 16103 2009-03-10 14:53:39Z rmanfredi $")
 
 #define tiger_compress_macro(str, state) \
 { \
-  guint64 a, b, c, tmpa; \
-  guint64 aa, bb, cc; \
-  guint64 x[8]; \
+  uint64 a, b, c, tmpa; \
+  uint64 aa, bb, cc; \
+  uint64 x[8]; \
   int pass_no, i; \
 \
   a = state[0]; \
@@ -158,27 +154,27 @@ RCSID("$Id: tiger.c 16103 2009-03-10 14:53:39Z rmanfredi $")
 }
 
 /* The compress function is a function. Requires smaller cache?    */
-static inline void
-tiger_compress(const guint64 *data, guint64 state[3])
+static void G_HOT
+tiger_compress(const uint64 *data, uint64 state[3])
 {
   tiger_compress_macro(data, state);
 }
 
 void
-tiger(gconstpointer data, guint64 length, char hash[24])
+tiger(const void *data, uint64 length, char hash[24])
 {
-  guint64 i, j, res[3];
-  const guint8 *data_u8 = data;
+  uint64 i, j, res[3];
+  const uint8 *data_u8 = data;
   union {
-    guint64 u64[8];
-    guint8 u8[64];
+    uint64 u64[8];
+    uint8 u8[64];
   } temp;
 
   res[0] = U64_FROM_2xU32(0x01234567UL, 0x89ABCDEFUL);
   res[1] = U64_FROM_2xU32(0xFEDCBA98UL, 0x76543210UL);
   res[2] = U64_FROM_2xU32(0xF096A5B4UL, 0xC3B2E187UL);
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if IS_BIG_ENDIAN
   for (i = length; i >= 64; i -= 64) {
     for (j = 0; j < 64; j++) {
       temp.u8[j ^ 7] = data_u8[j];
@@ -186,8 +182,8 @@ tiger(gconstpointer data, guint64 length, char hash[24])
     tiger_compress(temp.u64, res);
     data_u8 += 64;
   }
-#else	/* !BIG ENDIAN */
-  if ((gulong) data & 7) {
+#else	/* !IS_BIG_ENDIAN */
+  if ((ulong) data & 7) {
     for (i = length; i >= 64; i -= 64) {
       memcpy(temp.u64, data_u8, 64);
       tiger_compress(temp.u64, res);
@@ -195,13 +191,13 @@ tiger(gconstpointer data, guint64 length, char hash[24])
     }
   } else {
     for (i = length; i >= 64; i -= 64) {
-      tiger_compress((gconstpointer) data_u8, res);
+      tiger_compress((void *) data_u8, res);
       data_u8 += 64;
     }
   }
-#endif	/* BIG ENDIAN */
+#endif	/* IS_BIG_ENDIAN */
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if IS_BIG_ENDIAN
   for (j = 0; j < i; j++) {
     temp.u8[j ^ 7] = data_u8[j];
   }
@@ -220,7 +216,7 @@ tiger(gconstpointer data, guint64 length, char hash[24])
   for (; j & 7; j++) {
     temp.u8[j] = 0;
   }
-#endif
+#endif	/* IS_BIG_ENDIAN */
 
   if (j > 56) {
     for (; j < 64; j++) {
@@ -246,7 +242,7 @@ tiger(gconstpointer data, guint64 length, char hash[24])
  * Runs some test cases to check whether the implementation of the tiger
  * hash algorithm is alright.
  */
-void
+void G_COLD
 tiger_check(void)
 {
 	static const char zeros[1025];
@@ -263,17 +259,17 @@ tiger_check(void)
 		{ "D6UXHPOSAGHITCD4VVRHJQ4PCKIWY2WEHPJOUWY", zeros, 1024 },
 		{ "CMKDYROZKSC6VTM4I7LSMMHPAE4UG3FXPXZGGKY", zeros, sizeof zeros },
 	};
-	guint i;
+	uint i;
 
-	for (i = 0; i < G_N_ELEMENTS(tests); i++) {
+	for (i = 0; i < N_ITEMS(tests); i++) {
 		char hash[24];
 		char buf[40];
-		gboolean ok;
+		bool ok;
 
-		memset(buf, 0, sizeof buf);	
+		ZERO(&buf);
 		tiger(tests[i].s, tests[i].len, hash);
-		base32_encode(buf, sizeof buf, hash, sizeof hash);
-		buf[G_N_ELEMENTS(buf) - 1] = '\0';
+		base32_encode(ARYLEN(buf), ARYLEN(hash));
+		buf[N_ITEMS(buf) - 1] = '\0';
 
 		ok = 0 == strcmp(tests[i].r, buf);
 		if (!ok) {
