@@ -1,6 +1,4 @@
 /*
- * $Id: tiger_sboxes.h 12500 2006-11-26 10:53:21Z cbiere $
- *
  * Copyright (c) 2005, Christian Biere
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,9 +33,9 @@
 
 #include "common.h"
 
-#define D(x, y) ((guint64) (x) << 32 | (y))
+#define D(x, y) ((uint64) (x) << 32 | (y))
 
-static const guint64 tiger_sboxes[4 * 256] = {
+static const uint64 tiger_sboxes[4 * 256] = {
 	D(0x02AAB17CUL, 0xF7E90C5EUL)  /*    0 */,
 	D(0xAC424B03UL, 0xE243A8ECUL)  /*    1 */,
 	D(0x72CD5BE3UL, 0x0DD5FCD3UL)  /*    2 */,
@@ -1065,3 +1063,5 @@ static const guint64 tiger_sboxes[4 * 256] = {
 };
 
 #undef D
+
+/* vi: set ts=4 sw=4 cindent: */
