@@ -1,6 +1,4 @@
 /*
- * $Id: url.c 17240 2010-02-13 12:26:50Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,16 +33,21 @@
 
 #include "common.h"
 
-RCSID("$Id: url.c 17240 2010-02-13 12:26:50Z rmanfredi $")
-
+#include "url.h"
 #include "ascii.h"
+#include "debug.h"
 #include "glib-missing.h"
 #include "halloc.h"
 #include "host_addr.h"
+#include "hstrfn.h"
+#include "htable.h"
+#include "mempcpy.h"
+#include "misc.h"			/* For is_strprefix() */
 #include "parse.h"
 #include "path.h"
-#include "url.h"
+#include "str.h"
 #include "walloc.h"
+
 #include "override.h"		/* Must be the last header included */
 
 #define ESCAPE_CHAR		'%'
@@ -59,7 +62,7 @@ RCSID("$Id: url.c 17240 2010-02-13 12:26:50Z rmanfredi $")
  * - Bit 2 encodes the set for fixing an incomplete escaping.
  * - Bit 3 encodes the set for use in a shell.
  */
-static const guint8 is_transparent[96] = {
+static const uint8 is_transparent[96] = {
 /*  0   1   2   3   4   5   6   7 */	/* 01234567 -            */
     0x0,0x0,0x0,0x0,0x7,0x0,0x4,0x0,	/*  !"#$%&' -	 32..39  */
 	0x7,0x7,0x7,0x0,0xf,0xf,0xf,0xf,	/* ()*+,-./ -	 40..47	 */
@@ -82,13 +85,22 @@ enum escape_mask {
 	SHELL_MASK	= (1 << 3)
 };
 
-static inline gboolean
+static inline bool
 is_transparent_char(const int c, const enum escape_mask m)
 {
-	return c >= 32 && c < 128 && (is_transparent[c - 32] & (guint8) m);
+	return c >= 32 && c < 128 && (is_transparent[c - 32] & (uint8) m);
 }
 
 static const char hex_alphabet[] = "0123456789ABCDEF";
+static const char http_prefix[] = "http://";
+
+/**
+ * Parsed URL parameters (from query string).
+ */
+struct url_params {
+	htable_t *params;		/**< parameter => value (halloc'ed) */
+	size_t count;			/**< Amount of parameters */
+};
 
 /**
  * Escape undesirable characters using %xx, where xx is an hex code.
@@ -100,12 +112,12 @@ static const char hex_alphabet[] = "0123456789ABCDEF";
  * which must be freed via hfree().
  */
 static char *
-url_escape_mask(const char *url, guint8 mask)
+url_escape_mask(const char *url, uint8 mask)
 {
 	const char *p;
 	char *q;
 	int need_escape = 0;
-	guchar c;
+	uchar c;
 	char *new;
 
 	for (p = url, c = *p++; c; c = *p++)
@@ -140,11 +152,11 @@ url_escape_mask(const char *url, guint8 mask)
  * NUL), or -1 if the buffer was too small.
  */
 static int
-url_escape_mask_into(const char *url, char *target, int len, guint8 mask)
+url_escape_mask_into(const char *url, char *target, int len, uint8 mask)
 {
 	const char *p = url;
 	char *q;
-	guchar c;
+	uchar c;
 	char *end = target + len;
 
 	for (q = target, c = *p++; c && q < end; c = *p++) {
@@ -212,10 +224,10 @@ char *
 url_from_absolute_path(const char *path)
 {
 	char *escaped, *url;
-	
+
 	g_return_val_if_fail(is_absolute_path(path), NULL);
 	escaped = url_escape_mask(path, SHELL_MASK);
-	url = g_strconcat("file://", escaped, (void *) 0);
+	url = g_strconcat("file://", escaped, NULL_PTR);
 	if (escaped != path) {
 		HFREE_NULL(escaped);
 	}
@@ -237,33 +249,35 @@ url_escape_into(const char *url, char *target, int len)
 
 /**
  * Don't touch '?', '&', '=', ':', '[', ']', %HH.
+ *
+ * @return newly allocated string through halloc().
  */
 char *
 url_fix_escape(const char *url)
 {
 	const char *p;
-	GString *gs;
-	guchar c;
+	str_t *s;
+	uchar c;
 
-	gs = g_string_new(NULL);
+	s = str_new(0);
 
 	for (p = url; '\0' != (c = *p); p++) {
 		if (
 			is_transparent_char(c, FIX_MASK) ||
 			('%' == c && is_ascii_xdigit(p[1]) && is_ascii_xdigit(p[2]))
 		) {
-			gs = g_string_append_c(gs, c);
+			str_putc(s, c);
 		} else {
 			char buf[3];
 
 			buf[0] = ESCAPE_CHAR;
 			buf[1] = hex_alphabet[c >> 4];
 			buf[2] = hex_alphabet[c & 0xf];
-			gs = g_string_append_len(gs, buf, sizeof buf);
+			str_cat_len(s, ARYLEN(buf));
 		}
 	}
 
-	return gm_string_finalize(gs);
+	return str_s2c_null(&s);
 }
 
 /**
@@ -285,7 +299,7 @@ url_escape_cntrl(const char *url)
 	if (need_escape > 0) {
 		char *escaped, *q;
 		size_t size;
-		guchar c;
+		uchar c;
 
 		size = p - url + 1 + need_escape * 2;
 		escaped = halloc(size);
@@ -316,19 +330,19 @@ url_escape_cntrl(const char *url)
  * @return NULL if the argument isn't valid encoded.
  */
 char *
-url_unescape(char *url, gboolean inplace)
+url_unescape(char *url, bool inplace)
 {
 	char *p;
 	char *q;
 	int need_unescape = 0;
-	guint unescaped_memory = 0;
-	guchar c;
+	uint unescaped_memory = 0;
+	uchar c;
 	char *new;
 
 	for (p = url; (c = *p) != '\0'; c = *p++)
 		if (c == ESCAPE_CHAR) {
-			guchar h = *(++p);
-			guchar l = *(++p);
+			uchar h = *(++p);
+			uchar l = *(++p);
 
 			if (
 				(h == '0' && l == '0') ||	/* Forbid %00 */
@@ -343,7 +357,7 @@ url_unescape(char *url, gboolean inplace)
 		return url;
 
 	/*
-	 * The "+ 1" in the g_malloc() call below is for the impossible case where
+	 * The "+ 1" in the halloc() call below is for the impossible case where
 	 * the string would finish on a truncated escape sequence.  In that
 	 * case, we would not have enough room for the final trailing NUL.
 	 */
@@ -359,6 +373,9 @@ url_unescape(char *url, gboolean inplace)
 		if (c != ESCAPE_CHAR)
 			*q++ = c;
 		else {
+			if G_UNLIKELY(0 == hex2int_inline('a'))
+				misc_init();	/* Auto-initialization of hex2int_inline() */
+
 			if ((c = *p++)) {
 				int v = hex2int_inline(c) << 4;
 				if ((c = *p++))
@@ -397,10 +414,10 @@ url_params_parse(char *query)
 	char *start;
 	char *name = NULL;
 	char *value = NULL;
-	gboolean in_value = FALSE;
+	bool in_value = FALSE;
 
-	up = walloc(sizeof *up);
-	up->params = g_hash_table_new(g_str_hash, g_str_equal);
+	WALLOC(up);
+	up->params = htable_create(HASH_KEY_STRING, 0);
 	up->count = 0;
 
 	for (q = start = query; /* empty */; q++) {
@@ -418,7 +435,7 @@ url_params_parse(char *query)
 				if (value == start)				/* No unescaping took place */
 					value = h_strdup(start);
 				*q = c;
-				g_hash_table_insert(up->params, name, value);
+				htable_insert(up->params, name, value);
 				up->count++;
 				in_value = FALSE;
 				name = NULL;
@@ -450,19 +467,22 @@ url_params_parse(char *query)
  * The value returned has already been URL-unescaped.
  */
 const char *
-url_params_get(url_params_t *up, const char *name)
+url_params_get(const url_params_t *up, const char *name)
 {
 	g_assert(up != NULL);
 	g_assert(up->params != NULL);
 
-	return g_hash_table_lookup(up->params, name);
+	return htable_lookup(up->params, name);
 }
 
 static void
-free_params_kv(gpointer key, gpointer value, gpointer unused_udata)
+free_params_kv(const void *key, void *value, void *unused_udata)
 {
+	void *k = deconstify_pointer(key);
+
 	(void) unused_udata;
-	HFREE_NULL(key);
+
+	HFREE_NULL(k);
 	HFREE_NULL(value);
 }
 
@@ -474,13 +494,22 @@ url_params_free(url_params_t *up)
 {
 	g_assert(up != NULL);
 
-	g_hash_table_foreach(up->params, free_params_kv, NULL);
-	g_hash_table_destroy(up->params);
+	htable_foreach(up->params, free_params_kv, NULL);
+	htable_free_null(&up->params);
 
-	wfree(up, sizeof *up);
+	WFREE(up);
 }
 
-static gboolean
+/**
+ * How many URL parameters are present?
+ */
+size_t
+url_params_count(const url_params_t *up)
+{
+	return up->count;
+}
+
+static bool
 url_safe_char(char c, url_policy_t p)
 {
 	if (!isascii(c) || is_ascii_cntrl(c))
@@ -490,7 +519,7 @@ url_safe_char(char c, url_policy_t p)
     	if (
 			!is_ascii_lower(c) &&
 			!is_ascii_digit(c) &&
-			NULL == strchr("/._-~", c)
+			NULL == vstrchr("/._-~", c)
 		) {
       		/* Unsafe character in GWC URL; rejected */
       		return FALSE;
@@ -500,13 +529,72 @@ url_safe_char(char c, url_policy_t p)
 	return TRUE;
 }
 
+/**
+ * Creates the canonical representation of a path in place, delimited by '/'.
+ *
+ * @param path a NUL-terminated string representing the input path.
+ * @return zero on sucess, non-zero on failure.
+ */
+int
+url_canonize_path(char *path)
+{
+	const char *p;
+	char c, *q, *ep, *dst = path;
+
+	g_assert(path);
+
+	/* Scan path */
+	for (p = path, q = dst; '\0' != (c = *p); q++, p++) {
+
+		/* Handle relative paths i.e., /. and /.. */
+		if ('/' != c) {
+			*q = c;
+			continue;
+		}
+
+		/* Special handling for '/' follows */
+
+		do {
+			*q = '/';
+
+			while ('/' == p[1]) {
+				p++;
+			}
+
+			if (0 == strcmp(p, "/.")) {
+				/* Ignoring trailing "/." in URI */
+				p++;
+			} else if (0 == strcmp(p, "/..")) {
+				return -1;
+			} else if (NULL != (ep = is_strprefix(p, "/./"))) {
+				/* Ignoring unnecessary "/./" in URI */
+				p = ep - 1;
+			} else if (NULL != (ep = is_strprefix(p, "/../"))) {
+				p = ep - 1;
+
+				/* Ascending one component in URI */
+				do {
+					if (q == dst)
+						return -1; /* beyond root */
+				} while ('/' != *--q);
+			} else {
+				break;
+			}
+		} while ('/' == p[0] && ('/' == p[1] || '.' == p[1]));
+	}
+
+	*q = '\0';
+
+	return 0;
+}
+
 /**
  * @attention
- * NB: May modify ``url'' in all cased; pass a copy if necessary!
+ * NB: May modify ``url'' in all cases; pass a copy if necessary!
  *
  * @returns NULL if ``url'' isn't a valid resp. allowed URL. Otherwise,
  * it returns either a pointer to the original URL or a newly allocated
- * string holding a modified copy.
+ * string holding a modified copy.  Use hfree() to free the allocated string.
  *
  * The URL is validated according to the specified policy. Unnecessary
  * parts such as ":80" (for the port) and "/./" are removed: the hostname
@@ -517,7 +605,6 @@ url_safe_char(char c, url_policy_t p)
 char *
 url_normalize(char *url, url_policy_t pol)
 {
-	static const char http_prefix[] = "http://";
 	const char *p, *uri, *endptr, *tld = NULL, *warn = NULL;
 	char c, *q;
 	host_addr_t addr;
@@ -594,7 +681,7 @@ url_normalize(char *url, url_policy_t pol)
 
 	p = q;
 	if (':' == *q) {
-		guint32 port;
+		uint32 port;
 		int error;
 
 		q++; /* Skip ':' */
@@ -618,7 +705,7 @@ url_normalize(char *url, url_policy_t pol)
 			80 != port &&
 			443 != port
 		) {
-			warn = "Ports below 1024 other than 80 and 443 are disallowed";
+			warn = "ports below 1024 other than 80 and 443 are disallowed";
 			goto bad;
 		}
 
@@ -650,13 +737,13 @@ url_normalize(char *url, url_policy_t pol)
 	if (q != uri) {
 		size_t len;
 
-		len = strlen(uri);
+		len = vstrlen(uri);
 		memmove(q, uri, len);
 		q[len] = '\0';
 	}
 	uri = q;
 
-	if (0 != canonize_path(q, q)) {
+	if (0 != url_canonize_path(q)) {
 		warn = "Could not canonize URI";
 		goto bad;
 	}
@@ -672,9 +759,9 @@ url_normalize(char *url, url_policy_t pol)
 			D(".txt"),
 #undef D
 		};
-		guint i;
+		uint i;
 
-		for (i = 0; i < G_N_ELEMENTS(static_types); i++)
+		for (i = 0; i < N_ITEMS(static_types); i++)
     		if (
 				0 == ascii_strcasecmp(q - static_types[i].len,
 						static_types[i].ext)
@@ -687,20 +774,116 @@ url_normalize(char *url, url_policy_t pol)
 	/* Add a trailing slash; if the URI is empty (to prevent dupes) */
 	if ('\0' == uri[0]) {
 		ssize_t len = q - url;
-		char *s;
+		char *s, *w;
 
 		g_assert(len > 0);
-		s = g_malloc(len + sizeof "/");
-		memcpy(s, url, len);
-		s[len] = '/';
-		s[len + 1] = '\0';
+		s = halloc(len + sizeof "/");
+		w = mempcpy(s, url, len);
+		*w++ = '/';
+		*w = '\0';
 		url = s;
 	}
 
 	return url;
 
 bad:
+	if (common_dbg)
+		g_warning("%s() rejecting URL \"%s\": %s", G_STRFUNC, url, warn);
+
 	return NULL;
 }
 
+/**
+ * Is URL absolute?
+ */
+bool
+url_is_absolute(const char *url)
+{
+	return is_strprefix(url, http_prefix) != NULL;
+}
+
+/**
+ * Interpret possible relative URL in the context of a base (absolute) one.
+ *
+ * If the relative URL is actually absolute, return it.  Otherwise returns
+ * a newly allocated string that must tbe freed with hfree().
+ *
+ * @param base		the base URL
+ * @param relative	the supposedly relative URL
+ *
+ * @return the relative URL if absolute, otherwise a newly allocated string.
+ */
+char *
+url_absolute_within(const char *base, const char *relative)
+{
+	char *dbase = NULL;
+
+	g_assert(url_is_absolute(base));
+
+	if (url_is_absolute(relative))
+		return deconstify_char(relative);		/* Was already absolute */
+
+	if (is_strprefix(relative, "//"))
+		/* Was missing the scheme */
+		return h_strconcat("http:", relative, NULL_PTR);
+
+	if (is_strprefix(relative, "#")) {
+		char *p;
+
+		if (NULL == (p = vstrchr(base, '#'))) {
+			return h_strconcat(base, relative, NULL_PTR);
+		} else {
+			/* Replace the fragment */
+			dbase = h_strdup(base);
+			p = dbase + (p - base);
+			g_assert(*p == '#');
+			*p = '\0';
+			goto concatenate;
+		}
+	}
+
+	if (is_strprefix(relative, "/")) {
+		char *p;
+
+		dbase = h_strdup(base);
+		p = is_strprefix(dbase, http_prefix);
+		g_assert(p != NULL);		/* base was absolute */
+		p = vstrchr(p, '/');
+
+		if (NULL == p) {
+			/* base was "http://host" with no trailing path */
+			hfree(dbase);
+			return h_strconcat(base, relative, NULL_PTR);
+		} else {
+			/* replace relative path in the base */
+			*p = '\0';
+			goto concatenate;
+		}
+	} else {
+		char *p;
+
+		/*
+		 * Find the last '/' in the base and append the relative URL after it.
+		 */
+
+		dbase = h_strdup(base);
+		p = vstrrchr(dbase, '/');
+		g_assert(p != NULL);		/* base was absolute */
+		*(++p) = '\0';				/* truncate string after last '/' */
+
+		/* FALL THROUGH */
+	}
+
+concatenate:
+	{
+		char *result;
+
+		g_assert(dbase != NULL);
+
+		result = h_strconcat(dbase, relative, NULL_PTR);
+		hfree(dbase);
+		return result;
+	}
+}
+
 /* vi: set ts=4 sw=4 cindent: */
