@@ -1,6 +1,4 @@
 /*
- * $Id: watcher.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
  * Copyright (c) 2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -38,15 +36,16 @@
 
 #include "common.h"
 
-RCSID("$Id: watcher.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
 #include "watcher.h"
+
 #include "atoms.h"
 #include "cq.h"
-#include "glib-missing.h"
 #include "halloc.h"
+#include "hikset.h"
+#include "once.h"
 #include "path.h"
 #include "walloc.h"
+
 #include "override.h"		/* Must be the last header included */
 
 #define MONITOR_PERIOD_MS	(30*1000)	/**< 30 seconds */
@@ -58,10 +57,10 @@ struct monitored {
 	const char *filename;	/**< Filename to monitor */
 	time_t mtime;			/**< Last known modified time */
 	watcher_cb_t cb;		/**< Callback to invoke on change */
-	gpointer udata;			/**< User supplied data to hand-out to callback */
+	void *udata;			/**< User supplied data to hand-out to callback */
 };
 
-static GHashTable *monitored;	/**< filename -> struct monitored */
+static hikset_t *monitored;	/**< filename -> struct monitored */
 
 /**
  * Compute the modified time of the file on disk.
@@ -69,7 +68,7 @@ static GHashTable *monitored;	/**< filename -> struct monitored */
 static time_t
 watcher_mtime(const char *filename)
 {
-	struct stat buf;
+	filestat_t buf;
 
 	if (-1 == stat(filename, &buf))
 		return 0;
@@ -81,12 +80,11 @@ watcher_mtime(const char *filename)
  * Check each registered file for change -- hash table iterator callback.
  */
 static void
-watcher_check_mtime(gpointer unused_key, gpointer value, gpointer unused_udata)
+watcher_check_mtime(void *value, void *unused_udata)
 {
 	struct monitored *m = value;
 	time_t new_mtime;
 
-	(void) unused_key;
 	(void) unused_udata;
 
 	new_mtime = watcher_mtime(m->filename);
@@ -101,12 +99,15 @@ watcher_check_mtime(gpointer unused_key, gpointer value, gpointer unused_udata)
  * Callout queue periodic event to perform periodic monitoring of the
  * registered files.
  */
-static gboolean
-watcher_timer(gpointer unused_udata)
+static bool
+watcher_timer(void *unused_udata)
 {
 	(void) unused_udata;
 
-	g_hash_table_foreach(monitored, watcher_check_mtime, NULL);
+	if G_UNLIKELY(NULL == monitored)
+		return FALSE;	/* Stop calling, layer disabled */
+
+	hikset_foreach(monitored, watcher_check_mtime, NULL);
 
 	return TRUE;		/* Keep calling */
 }
@@ -122,20 +123,25 @@ watcher_timer(gpointer unused_udata)
  * @param udata extra data to pass to the callback, along with filename
  */
 void
-watcher_register(const char *filename, watcher_cb_t cb, gpointer udata)
+watcher_register(const char *filename, watcher_cb_t cb, void *udata)
 {
 	struct monitored *m;
 
-	m = walloc0(sizeof(*m));
+	g_assert(filename != NULL);
+	g_assert(cb != NULL);
+
+	watcher_init();		/* Auto-initialization */
+
+	WALLOC0(m);
 	m->filename = atom_str_get(filename);
 	m->cb = cb;
 	m->udata = udata;
 	m->mtime = watcher_mtime(filename);
 
-	if (g_hash_table_lookup(monitored, filename) != NULL)
+	if (hikset_contains(monitored, filename))
 		watcher_unregister(filename);
 
-	gm_hash_table_insert_const(monitored, m->filename, m);
+	hikset_insert_key(monitored, &m->filename);
 }
 
 /**
@@ -143,10 +149,13 @@ watcher_register(const char *filename, watcher_cb_t cb, gpointer udata)
  * given instead of a complete filename.
  */
 void
-watcher_register_path(const file_path_t *fp, watcher_cb_t cb, gpointer udata)
+watcher_register_path(const file_path_t *fp, watcher_cb_t cb, void *udata)
 {
 	char *path;
 
+	g_assert(fp != NULL);
+	g_assert(cb != NULL);
+
 	path = make_pathname(fp->dir, fp->name);
 	watcher_register(path, cb, udata);
 	HFREE_NULL(path);
@@ -159,7 +168,7 @@ static void
 watcher_free(struct monitored *m)
 {
 	atom_str_free(m->filename);
-	wfree(m, sizeof(*m));
+	WFREE(m);
 }
 
 /**
@@ -170,11 +179,14 @@ watcher_unregister(const char *filename)
 {
 	struct monitored *m;
 
-	m = g_hash_table_lookup(monitored, filename);
+	g_return_unless(monitored != NULL);
+	g_assert(filename != NULL);
+
+	m = hikset_lookup(monitored, filename);
 
 	g_assert(m != NULL);
 
-	g_hash_table_remove(monitored, m->filename);
+	hikset_remove(monitored, m->filename);
 	watcher_free(m);
 }
 
@@ -187,30 +199,43 @@ watcher_unregister_path(const file_path_t *fp)
 {
 	char *path;
 
+	g_assert(fp != NULL);
+
 	path = make_pathname(fp->dir, fp->name);
 	watcher_unregister(path);
 	HFREE_NULL(path);
 }
 
+/**
+ * Configure the watcher layer, once.
+ */
+static void
+watcher_init_once(void)
+{
+	monitored = hikset_create(
+		offsetof(struct monitored, filename), HASH_KEY_STRING, 0);
+	cq_periodic_main_add(MONITOR_PERIOD_MS, watcher_timer, NULL);
+}
+
 /**
  * Initialization.
  */
 void
 watcher_init(void)
 {
-	monitored = g_hash_table_new(g_str_hash, g_str_equal);
-	cq_periodic_add(callout_queue, MONITOR_PERIOD_MS, watcher_timer, NULL);
+	static once_flag_t watcher_inited;
+
+	ONCE_FLAG_RUN(watcher_inited, watcher_init_once);
 }
 
 /**
  * Free monitored structure -- hash table iterator callback.
  */
 static void
-free_monitored_kv(gpointer unused_key, gpointer value, gpointer unused_udata)
+free_monitored_kv(void *value, void *unused_udata)
 {
 	struct monitored *m = value;
 
-	(void) unused_key;
 	(void) unused_udata;
 	watcher_free(m);
 }
@@ -221,8 +246,8 @@ free_monitored_kv(gpointer unused_key, gpointer value, gpointer unused_udata)
 void
 watcher_close(void)
 {
-	g_hash_table_foreach(monitored, free_monitored_kv, NULL);
-	g_hash_table_destroy(monitored);
+	hikset_foreach(monitored, free_monitored_kv, NULL);
+	hikset_free_null(&monitored);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
