@@ -1,6 +1,4 @@
 /*
- * $Id: set.c 17171 2009-11-03 20:08:11Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,12 +33,9 @@
 
 #include "common.h"
 
-RCSID("$Id: set.c 17171 2009-11-03 20:08:11Z rmanfredi $")
-
 #include "cmd.h"
 
 #include "if/gnet_property.h"
-#include "if/gnet_property_priv.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -58,7 +53,7 @@ shell_exec_set(struct gnutella_shell *sh, int argc, const char *argv[])
 	g_assert(argv);
 	g_assert(argc > 0);
 
-	parsed = shell_options_parse(sh, argv, options, G_N_ELEMENTS(options));
+	parsed = shell_options_parse(sh, argv, options, N_ITEMS(options));
 	if (parsed < 0)
 		return REPLY_ERROR;
 
@@ -81,18 +76,21 @@ shell_exec_set(struct gnutella_shell *sh, int argc, const char *argv[])
 		goto error;
 	}
 
+	if (gnet_prop_is_internal(prop)) {
+		shell_set_msg(sh, _("Property cannot be changed"));
+		goto error;
+	}
+
 	if (verbose) {
-		shell_write(sh, "100-Previous value was ");
-		shell_write(sh, gnet_prop_to_string(prop));
-		shell_write(sh, "\n");
+		shell_write_linef(sh, REPLY_READY, _("Previous value was %s"),
+			gnet_prop_to_typed_string(prop));
 	}
 
 	gnet_prop_set_from_string(prop,	argv[1]);
 
 	if (verbose) {
-		shell_write(sh, "100-New value is ");
-		shell_write(sh, gnet_prop_to_string(prop));
-		shell_write(sh, "\n");
+		shell_write_linef(sh, REPLY_READY, _("New value is %s"),
+			gnet_prop_to_typed_string(prop));
 	}
 
 	shell_set_msg(sh, _("Value found and set"));
