@@ -1,7 +1,6 @@
 /*
- * $Id: shell.c 17234 2010-02-08 23:18:32Z rmanfredi $
- *
- * Copyright (c) 2002-2003, Richard Eckart
+ * Copyright (c) 2002-2003 Richard Eckart
+ * Copyright (c) 2009,2013 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -19,7 +18,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -32,13 +31,11 @@
  * @author Richard Eckart
  * @date 2002-2003
  * @author Raphael Manfredi
- * @date 2009
+ * @date 2009, 2013
  */
 
 #include "common.h"
 
-RCSID("$Id: shell.c 17234 2010-02-08 23:18:32Z rmanfredi $")
-
 #include "shell.h"
 #include "cmd.h"
 
@@ -49,39 +46,64 @@ RCSID("$Id: shell.c 17234 2010-02-08 23:18:32Z rmanfredi $")
 #include "if/gnet_property_priv.h"
 
 #include "lib/ascii.h"
+#include "lib/atomic.h"
+#include "lib/constants.h"
+#include "lib/elist.h"
+#include "lib/entropy.h"
 #include "lib/file.h"
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/halloc.h"
+#include "lib/hashing.h"
+#include "lib/hstrfn.h"
+#include "lib/htable.h"
 #include "lib/inputevt.h"
-#include "lib/halloc.h"
+#include "lib/iovec.h"
 #include "lib/pmsg.h"
+#include "lib/pslist.h"
+#include "lib/random.h"
 #include "lib/sha1.h"
 #include "lib/slist.h"
+#include "lib/spinlock.h"
+#include "lib/str.h"
 #include "lib/stringify.h"
+#include "lib/teq.h"
+#include "lib/thread.h"
 #include "lib/tm.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
+#include "lib/xmalloc.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
-#define SHELL_MAX_ARGS	1024	/**< Maximum number of arguments in command */
-
-static GSList *sl_shells;
+#define SHELL_MAX_ARGS		1024	/**< Max number of arguments in command */
+#define SHELL_STACK_SIZE	THREAD_STACK_MIN
 
 enum shell_magic {
 	SHELL_MAGIC = 0x33f3e711U
 };
 
+struct shell_pending {
+	enum shell_reply code;	/**< Reply code */
+	char *msg;				/**< Line of text (no trailing new-line) */
+};
+
 struct gnutella_shell {
-	enum shell_magic	magic;
+	enum shell_magic magic;
+	struct shell_pending pending;
 	struct gnutella_socket *socket;
+	inputevt_cond_t cond;	/** Current I/O event condition */
 	slist_t *output;
 	char *msg;   			/**< Additional information to reply code */
 	time_t last_update; 	/**< Last update (needed for timeout) */
-	guint64 line_count;		/**< Number of input lines after HELO */
-	gboolean shutdown;  	/**< In shutdown mode? */
-	gboolean interactive;	/**< Interactive mode? */
+	uint64 line_count;		/**< Number of input lines after HELO */
+	link_t lnk;				/**< Embedded list pointer */
+	spinlock_t lock;		/**< Thread-safe lock */
+	uint shutdown:1;  		/**< In shutdown mode? */
+	uint exiting:1;  		/**< Should shell exit? */
+	uint interactive:1;		/**< Interactive mode? */
+	uint async:1;			/**< In the middle of asynchronous processing */
+	uint eof:1;				/**< Got EOF on the input side */
 };
 
 void
@@ -92,11 +114,21 @@ shell_check(const struct gnutella_shell * const sh)
 	socket_check(sh->socket);
 }
 
-static inline gboolean
+#define SHELL_LOCK(sh)		spinlock(&sh->lock)
+#define SHELL_UNLOCK(sh)	spinunlock(&sh->lock)
+
+static elist_t shells = ELIST_INIT(offsetof(struct gnutella_shell, lnk));
+static htable_t *shell_cmds;
+
+static bool shell_interpret_line(struct gnutella_shell *sh, const char *line);
+static void shell_interpret_data(struct gnutella_shell *sh);
+static bool shell_handle_event(struct gnutella_shell *, inputevt_cond_t);
+
+static inline bool
 shell_has_pending_output(struct gnutella_shell *sh)
 {
 	shell_check(sh);
-	return sh->output && slist_length(sh->output) > 0;
+	return sh->output != NULL && slist_length(sh->output) > 0;
 }
 
 /**
@@ -105,18 +137,19 @@ shell_has_pending_output(struct gnutella_shell *sh)
 static struct gnutella_shell *
 shell_new(struct gnutella_socket *s)
 {
-	static const struct gnutella_shell zero_shell;
 	struct gnutella_shell *sh;
 
 	socket_check(s);
 
-	sh = walloc(sizeof *sh);
-	*sh = zero_shell;
+	WALLOC0(sh);
 	sh->magic = SHELL_MAGIC;
 	sh->socket = s;
 	sh->output = slist_new();
+	sh->cond = INPUT_EVENT_NONE;
+	slist_thread_safe(sh->output);
+	spinlock_init(&sh->lock);
 
-	sl_shells = g_slist_prepend(sl_shells, sh);
+	elist_append(&shells, sh);
 
 	return sh;
 }
@@ -127,14 +160,58 @@ shell_new(struct gnutella_socket *s)
 static void
 shell_free(struct gnutella_shell *sh)
 {
-	g_assert(sh);
-	g_assert(SHELL_MAGIC == sh->magic);
+	g_assert(sh != NULL);
 	g_assert(NULL == sh->socket); /* must have called shell_destroy before */
 	g_assert(NULL == sh->output); /* must have called shell_destroy before */
+
+	spinlock_destroy(&sh->lock);
 	HFREE_NULL(sh->msg);
+	HFREE_NULL(sh->pending.msg);
 
 	sh->magic = 0;
-	wfree(sh, sizeof *sh);
+	WFREE(sh);
+}
+
+/**
+ * Called whenever some event occurs on a shell socket.
+ */
+static void
+shell_handle_data(void *data, int unused_source, inputevt_cond_t cond)
+{
+	(void) unused_source;
+
+	(void) shell_handle_event(data, cond);
+}
+
+/**
+ * Change I/O event condition on shell, if needed.
+ */
+static void
+shell_evt_set(struct gnutella_shell *sh, inputevt_cond_t cond)
+{
+	shell_check(sh);
+
+	if (G_LIKELY(sh->cond == cond))
+		return;
+
+	SHELL_LOCK(sh);
+
+	if (G_LIKELY(sh->cond == cond)) {
+		SHELL_UNLOCK(sh);
+		return;
+	}
+
+	socket_evt_clear(sh->socket);
+
+	if (INPUT_EVENT_NONE != cond)
+		socket_evt_set(sh->socket, cond, shell_handle_data, sh);
+
+	sh->cond = cond;
+
+	SHELL_UNLOCK(sh);
+
+	if (GNET_PROPERTY(shell_debug) > 1)
+		s_debug("%s(%p): %s", G_STRFUNC, sh, inputevt_cond_to_string(cond));
 }
 
 static void
@@ -153,11 +230,10 @@ shell_destroy(struct gnutella_shell *sh)
 {
 	shell_check(sh);
 
-	if (GNET_PROPERTY(dbg) > 0) {
-		g_message("shell_destroy");
-	}
-	sl_shells = g_slist_remove(sl_shells, sh);
-	socket_evt_clear(sh->socket);
+	if (GNET_PROPERTY(shell_debug))
+		s_debug("%s(%p) async=%s", G_STRFUNC, sh, bool_to_string(sh->async));
+
+	elist_remove(&shells, sh);
 	shell_discard_output(sh);
 	socket_free_null(&sh->socket);
 	shell_free(sh);
@@ -172,6 +248,19 @@ shell_set_msg(struct gnutella_shell *sh, const char *text)
 	sh->msg = h_strdup(text);
 }
 
+void
+shell_set_formatted(struct gnutella_shell *sh, const char *fmt, ...)
+{
+	va_list args;
+
+	shell_check(sh);
+
+	HFREE_NULL(sh->msg);
+	va_start(args, fmt);
+	sh->msg = str_vcmsg(fmt, args);
+	va_end(args);
+}
+
 static void
 shell_write_msg(struct gnutella_shell *sh)
 {
@@ -188,20 +277,20 @@ static void
 shell_write_welcome(struct gnutella_shell *sh)
 {
 	shell_check(sh);
-	
+
 	shell_write(sh, "100 Welcome to ");
 	shell_write(sh, version_short_string);
 	shell_write(sh, "\n");
 }
 
-guint64
+uint64
 shell_line_count(struct gnutella_shell *sh)
 {
 	shell_check(sh);
 	return sh->line_count;
 }
 
-gboolean
+bool
 shell_toggle_interactive(struct gnutella_shell *sh)
 {
 	shell_check(sh);
@@ -212,6 +301,47 @@ shell_toggle_interactive(struct gnutella_shell *sh)
 	return sh->interactive;
 }
 
+static void
+shell_shutdown(struct gnutella_shell *sh)
+{
+	shell_check(sh);
+
+	if (GNET_PROPERTY(shell_debug) > 1)
+		s_debug("%s(%p) async=%s", G_STRFUNC, sh, bool_to_string(sh->async));
+
+	sh->shutdown = TRUE;
+	atomic_mb();
+	shell_evt_set(sh, INPUT_EVENT_NONE);
+}
+
+static void
+shell_eof(struct gnutella_shell *sh)
+{
+	inputevt_cond_t ev = INPUT_EVENT_NONE;
+
+	shell_check(sh);
+
+	sh->eof = TRUE;
+	atomic_mb();
+
+	if (shell_has_pending_output(sh))
+		ev = INPUT_EVENT_WX;
+
+	shell_evt_set(sh, ev);
+}
+
+/**
+ * Request that shell exits once all pending output has been flushed.
+ */
+void
+shell_exit(struct gnutella_shell *sh)
+{
+	shell_check(sh);
+
+	sh->exiting = TRUE;
+	atomic_mb();
+}
+
 /**
  * Grabs next token, dynamically allocated, which must be gfree()'ed by caller.
  *
@@ -226,38 +356,38 @@ shell_toggle_interactive(struct gnutella_shell *sh)
  * @param start		the beginning of the string to parse
  * @param endptr	written with the address of the first char following token
  *
- * @return next token, or a NULL pointer on error.
+ * @return next token allocated through halloc(), or a NULL pointer on error.
  */
 static char *
 shell_next_token(struct gnutella_shell *sh,
 	const char *start, const char **endptr)
 {
-	GString *token;
-	gboolean escape = FALSE;
-	gboolean quote  = FALSE;
-	gboolean squote  = FALSE;
+	str_t *token;
+	bool escape = FALSE;
+	bool quote  = FALSE;
+	bool squote  = FALSE;
 	const char *s = start;
 	char c;
 
 	shell_check(sh);
 	g_assert(s != NULL);
 
-	token = g_string_sized_new(40);
+	token = str_new(40);
 
 	for (c = *s; '\0' != c; c = *++s) {
 		if (escape || '\\' == c) {
 			escape = !escape;
 			if (!escape)
-				g_string_append_c(token, c);
+				str_putc(token, c);
 		} else if ('"' == c) {
 			if (squote) {
-				g_string_append_c(token, c);
+				str_putc(token, c);
 				continue;		/* Grab a quote inside 'string' */
 			}
 			quote = !quote;
 		} else if ('\'' == c) {
 			if (quote) {
-				g_string_append_c(token, c);
+				str_putc(token, c);
 				continue;		/* Grab a single quote inside "string" */
 			}
 			squote = !squote;
@@ -266,9 +396,9 @@ shell_next_token(struct gnutella_shell *sh,
 				break;
 			if (';' == c)
 				break;
-			g_string_append_c(token, c);
+			str_putc(token, c);
 		} else {
-			g_string_append_c(token, c);
+			str_putc(token, c);
 		}
 	}
 
@@ -287,10 +417,10 @@ shell_next_token(struct gnutella_shell *sh,
 	}
 
 	*endptr = s;
-	return gm_string_finalize(token);
+	return str_s2c_null(&token);
 
 error:
-	g_string_free(token, TRUE);
+	str_destroy(token);
 	return NULL;
 }
 
@@ -310,9 +440,8 @@ shell_options_parse(struct gnutella_shell *sh,
 
 	ret = options_parse(argv, ovec, ovcnt);
 	if (ret < 0) {
-		shell_write(sh, "400-Syntax error: ");
-		shell_write(sh, options_parse_last_error());
-		shell_write(sh, "\n");
+		shell_write_linef(sh, REPLY_ERROR, "Syntax error: %s",
+			options_parse_last_error());
 		shell_set_msg(sh, _("Invalid command syntax"));
 	}
 	return ret;
@@ -329,7 +458,7 @@ shell_options_parse(struct gnutella_shell *sh,
  * @return TRUE if OK, FALSE on error with an error message recorded
  * in the shell structure (in which case no token was allocated).
  */
-static gboolean
+static bool
 shell_get_token(struct gnutella_shell *sh,
 	const char *line, const char **endptr, const char **token_ptr)
 {
@@ -363,10 +492,10 @@ static void
 shell_free_argv(const char ***argv_ptr)
 {
 	if (*argv_ptr) {
-		char **argv = deconstify_gpointer(*argv_ptr);
+		char **argv = deconstify_pointer(*argv_ptr);
 
 		while (NULL != argv[0]) {
-			G_FREE_NULL(argv[0]);
+			HFREE_NULL(argv[0]);
 			argv++;
 		}
 		HFREE_NULL(*argv_ptr);
@@ -388,7 +517,7 @@ shell_free_argv(const char ***argv_ptr)
  * @return TRUE if OK, FALSE on error with an error message recorded in the
  * shell structure.
  */
-static gboolean 
+static bool
 shell_parse_command(struct gnutella_shell *sh,
 	const char *line, const char **endptr,
 	int *argc_ptr, const char ***argv_ptr)
@@ -396,7 +525,7 @@ shell_parse_command(struct gnutella_shell *sh,
 	const char **argv = NULL;
 	unsigned argc = 0;
 	size_t n = 0;
-	gboolean ok = TRUE;
+	bool ok = TRUE;
 	const char *start;
 
 	shell_check(sh);
@@ -410,19 +539,18 @@ shell_parse_command(struct gnutella_shell *sh,
 	for (start = line; /* empty */; /* empty */) {
 		if (argc >= n) {
 			n = 2 * MAX(16, n);
-			argv = hrealloc(argv, n * sizeof argv[0]);
+			HREALLOC_ARRAY(argv, n);
 		}
 		if (argc > SHELL_MAX_ARGS) {
 			argv[SHELL_MAX_ARGS] = NULL;
-			shell_free_argv(&argv);
 			shell_set_msg(sh, _("too many arguments in command"));
-			argc = 0;
-			ok = FALSE;
-			break;
+			goto error;
 		}
 		if (!shell_get_token(sh, start, endptr, &argv[argc])) {
-			ok = FALSE;
-			break;
+			argv[argc] = NULL;
+			shell_set_msg(sh,
+				str_smsg(_("un-parseable argument #%u in command"), argc));
+			goto error;
 		}
 		if (NULL == argv[argc])
 			break;
@@ -430,7 +558,7 @@ shell_parse_command(struct gnutella_shell *sh,
 		argc++;
 		if (';' == *start) {
 			if (argc >= n)
-				argv = hrealloc(argv, (argc + 1) * sizeof argv[0]);
+				HREALLOC_ARRAY(argv, argc + 1);
 			argv[argc] = NULL;
 			*endptr = ++start;
 			break;
@@ -439,33 +567,249 @@ shell_parse_command(struct gnutella_shell *sh,
 
 	g_assert(NULL == argv || NULL == argv[argc]);
 
+	if (GNET_PROPERTY(shell_debug)) {
+		if (NULL == argv) {
+			s_debug("%s(%p): empty command", G_STRFUNC, sh);
+		} else {
+			char *cmd = h_strjoinv(" ", (char **) argv);
+			s_debug("%s(%p): got \"%s\"", G_STRFUNC, sh, cmd);
+			HFREE_NULL(cmd);
+		}
+	}
+
+
 	*argv_ptr = argv;
 	*argc_ptr = argc;
 	return ok;
+
+error:
+	shell_free_argv(&argv);
+	argc = 0;
+	return FALSE;
 }
 
 /**
+ * Fetch handler for the command.
+ *
+ * @param cmd		command name (lower-cased)
+ * @param threaded	if non-NULL, set with whether processing can be threaded
+ *
  * @return command handler based on command name (case-insensitive).
  */
 static shell_cmd_handler_t
-shell_cmd_get_handler(const char *cmd)
+shell_cmd_get_handler(const char *cmd, bool *threaded)
 {
-	static const struct {
+	static const struct shellcmd {
 		const char * const cmd;
 		shell_cmd_handler_t handler;
+		bool threadable;
 	} commands[] = {
-#define SHELL_CMD(x)	{ #x, shell_exec_ ## x },
+#define SHELL_CMD(x,t)	{ #x, shell_exec_ ## x, booleanize(t) },
 #include "cmd.inc"
-#undef	SHELL_CMD 
+#undef	SHELL_CMD
 	};
+	struct shellcmd *scd;
 	size_t i;
 
 	g_return_val_if_fail(cmd, NULL);
 
-	for (i = 0; i < G_N_ELEMENTS(commands); i++) {
-		if (ascii_strcasecmp(commands[i].cmd, cmd) == 0)
-			return commands[i].handler;
+	/*
+	 * Build table for quick lookups.
+	 */
+
+	if G_UNLIKELY(NULL == shell_cmds) {
+		shell_cmds = htable_create(HASH_KEY_STRING, 0);
+
+		for (i = 0; i < N_ITEMS(commands); i++) {
+			char *name = xstrdup(commands[i].cmd);
+
+			ascii_strlower(name, name);		/* In-place conversion */
+			htable_insert(shell_cmds, constant_str(name),
+				deconstify_pointer(&commands[i]));
+			xfree(name);
+		}
 	}
+
+	scd = htable_lookup(shell_cmds, cmd);
+	if (scd != NULL) {
+		if (threaded != NULL)
+			*threaded = scd->threadable;
+		return scd->handler;
+	}
+
+	return NULL;
+}
+
+/**
+ * Flush any pending line.
+ */
+static void
+shell_pending_flush(struct gnutella_shell *sh, bool last)
+{
+	shell_check(sh);
+	g_return_if_fail(sh->output);
+
+	if (sh->pending.msg != NULL) {
+		char buf[5];
+
+		str_bprintf(ARYLEN(buf), "%03d%c", sh->pending.code, last ? ' ' : '-');
+		shell_write(sh, buf);
+		shell_write(sh, sh->pending.msg);
+		shell_write(sh, "\n");
+		HFREE_NULL(sh->pending.msg);
+	}
+}
+
+static void
+shell_post_handle(struct gnutella_shell *sh, enum shell_reply reply_code)
+{
+	shell_check(sh);
+
+	shell_pending_flush(sh, !sh->interactive);
+	if (NULL == sh->msg) {
+		switch (reply_code) {
+		case REPLY_ERROR:
+			shell_set_msg(sh, _("Malformed command"));
+			break;
+		case REPLY_READY:
+			shell_set_msg(sh, _("OK"));
+			break;
+		case REPLY_NONE:
+		case REPLY_BYE:
+			break;
+		case REPLY_ASYNC:
+			g_assert_not_reached();
+		}
+	}
+}
+
+static void
+shell_handler_done(struct gnutella_shell *sh, enum shell_reply reply_code)
+{
+	if (
+		REPLY_NONE != reply_code &&
+		!(!sh->interactive && REPLY_READY == reply_code)
+	) {
+		/*
+		 * On error, when running non-interactively, remind them
+		 * about the command that failed first.
+		 */
+
+		if (REPLY_ERROR == reply_code && !sh->interactive) {
+			shell_write(sh, uint32_to_string(reply_code));
+			shell_write(sh, "-Error for: \"");
+			shell_write(sh, getline_str(sh->socket->getline));
+			shell_write(sh, "\"\n");
+		}
+
+		shell_write(sh, uint32_to_string(reply_code));
+		shell_write_msg(sh);
+		shell_write(sh, "\n");
+	}
+}
+
+/**
+ * Context for asynchronous shell command execution.
+ */
+struct shell_async_args {
+	struct gnutella_shell *sh;
+	shell_cmd_handler_t handler;
+	const char *next;				/* First character unparsed in line */
+	const char **argv;
+	int argc;
+	enum shell_reply reply_code;
+};
+
+/**
+ * Resume execution of the shell command line after asynchronous processing.
+ */
+static void
+shell_resume_processing(void *p)
+{
+	struct shell_async_args *args = p;
+	struct gnutella_shell *sh = args->sh;
+	const char *line = args->next;
+	bool ok;
+
+	if (GNET_PROPERTY(shell_debug) > 1) {
+		s_debug("%s(%p): resuming in main after %s()%s",
+			G_STRFUNC, sh, stacktrace_function_name(args->handler),
+			sh->shutdown ? " [\"shutdown\" on]" : "");
+	}
+
+	WFREE(args);
+	sh->last_update = tm_time();
+	sh->async = FALSE;
+
+	ok = shell_handle_event(sh, INPUT_EVENT_NONE);	/* Ensure we can read/write */
+
+	if (!ok) {
+		if (GNET_PROPERTY(shell_debug) > 1) {
+			s_debug("%s(%p): shell was destroyed", G_STRFUNC, sh);
+		}
+		return;
+	}
+
+	/*
+	 * Once the line is fully interpreted and we're no longer running any
+	 * asynchronous command, return to shell_interpret_data() to parse any
+	 * remaining (already read) data in the socket buffer.
+	 */
+
+	if (!shell_interpret_line(sh, line))
+		shell_interpret_data(sh);
+}
+
+/**
+ * Asynchronous shell command handler.
+ *
+ * @note
+ * This is the thread entry point for asynchronous shell command processing.
+ */
+static void *
+shell_async_handler(void *p)
+{
+	struct shell_async_args *args = p;
+	struct gnutella_shell *sh = args->sh;
+
+	shell_check(sh);
+
+	thread_set_name("async shell handler");
+
+	if (GNET_PROPERTY(shell_debug) > 1) {
+		s_debug("%s(%p): invoking %s() asynchronously for \"%s\"",
+			G_STRFUNC, sh, stacktrace_function_name(args->handler),
+			args->argv[0]);
+	}
+
+	args->reply_code = args->handler(sh, args->argc, args->argv);
+
+	shell_post_handle(sh, args->reply_code);
+	shell_handler_done(sh, args->reply_code);
+
+	if (GNET_PROPERTY(shell_debug) > 1) {
+		s_debug("%s(%p): done with %s() for \"%s\"",
+			G_STRFUNC, sh, stacktrace_function_name(args->handler),
+			args->argv[0]);
+	}
+
+	shell_free_argv(&args->argv);
+	shell_post_handle(sh, args->reply_code);
+
+	/*
+	 * Transfer control back to the main thread to resume processing of
+	 * the shell command line.
+	 *
+	 * Use a "safe" event to avoid interrupting processing in the main
+	 * thread that could be re-entered by the processing we're about to resume.
+	 */
+
+	if (!sh->shutdown) {
+		teq_safe_post(THREAD_MAIN_ID, shell_resume_processing, args);
+	} else {
+		sh->async = FALSE;		/* Signal that async processing is done */
+	}
+
 	return NULL;
 }
 
@@ -481,6 +825,16 @@ shell_exec(struct gnutella_shell *sh, const char *line, const char **endptr)
 	const char *start = line;
 
 	shell_check(sh);
+	g_assert(thread_is_main());
+	g_assert(!sh->async);		/* One async request at a time */
+	g_assert(endptr != NULL);
+
+	/*
+	 * Collect randomness each time a command is executed, regardless
+	 * of whether we end-up actually running it.
+	 */
+
+	entropy_harvest_time();
 
 	if (!shell_parse_command(sh, start, endptr, &argc, &argv)) {
 		shell_write(sh, "400-Syntax error:");
@@ -496,26 +850,72 @@ shell_exec(struct gnutella_shell *sh, const char *line, const char **endptr)
 		reply_code = REPLY_NONE;
 	} else {
 		shell_cmd_handler_t handler;
+		char *cmd = deconstify_char(argv[0]);
+		bool threadable;
 
-		handler = shell_cmd_get_handler(argv[0]);
-		if (handler) {
-			reply_code = (*handler)(sh, argc, argv);
-			if (NULL == sh->msg) {
-				switch (reply_code) {
-				case REPLY_ERROR:
-					shell_set_msg(sh, _("Malformed command"));
-					break;
-				case REPLY_READY:
-					shell_set_msg(sh, _("OK"));
-					break;
-				case REPLY_NONE:
-				case REPLY_BYE:
-					break;
+		ascii_strlower(cmd, cmd);	/* In-place conversion */
+		handler = shell_cmd_get_handler(cmd, &threadable);
+
+		if (handler != NULL) {
+			HFREE_NULL(sh->pending.msg);
+
+			if (threadable) {
+				struct shell_async_args *args;
+
+				/*
+				 * When the shell command is threadable, it will be run
+				 * asynchronously.  We simply return REPLY_ASYNC to the
+				 * caller to let it know that continuation following this
+				 * shell command will happen in the shell_resume_processing()
+				 * routine.
+				 */
+
+				WALLOC(args);
+				args->sh = sh;
+				args->next = *endptr;
+				args->argc = argc;
+				args->argv = argv;
+				args->handler = handler;
+
+				if (GNET_PROPERTY(shell_debug) > 1) {
+					s_debug("%s(%p): can call %s() asynchronously for \"%s\"",
+						G_STRFUNC, sh, stacktrace_function_name(handler),
+						argv[0]);
 				}
+
+				/*
+				 * Forbid any more reading on the socket until we're done
+				 * processing the previous command asynchronously.
+				 */
+
+				sh->async = TRUE;
+				shell_evt_set(sh, INPUT_EVENT_NONE);
+
+				if (
+					-1 == thread_create(shell_async_handler, args,
+						THREAD_F_NO_POOL | THREAD_F_DETACH |
+							THREAD_F_NO_CANCEL | THREAD_F_WARN,
+						SHELL_STACK_SIZE)
+				) {
+					bool ok;
+
+					WFREE(args);
+					sh->async = FALSE;
+					ok = shell_handle_event(sh, INPUT_EVENT_NONE);
+					g_assert(ok);
+					goto synchronous;
+				}
+
+				return REPLY_ASYNC;
 			}
+
+		synchronous:
+
+			reply_code = (*handler)(sh, argc, argv);
+			shell_post_handle(sh, reply_code);
 		} else {
 			char buf[80];
-			gm_snprintf(buf, sizeof buf, _("Unknown command: \"%s\""), argv[0]);
+			str_bprintf(ARYLEN(buf), _("Unknown command: \"%s\""), argv[0]);
 			shell_set_msg(sh, buf);
 			reply_code = REPLY_ERROR;
 		}
@@ -535,7 +935,7 @@ shell_write_data(struct gnutella_shell *sh)
 {
 	struct gnutella_socket *s;
 	ssize_t written;
-	struct iovec *iov;
+	iovec_t *iov;
 	int iov_cnt;
 
 	shell_check(sh);
@@ -545,31 +945,165 @@ shell_write_data(struct gnutella_shell *sh)
 	s = sh->socket;
 	socket_check(s);
 
-	sh->last_update = tm_time();
+	while (shell_has_pending_output(sh)) {
+		sh->last_update = tm_time();
 
-	iov = pmsg_slist_to_iovec(sh->output, &iov_cnt, NULL);
-	written = s->wio.writev(&s->wio, iov, iov_cnt);
+		slist_lock(sh->output);
+		iov = pmsg_slist_to_iovec(sh->output, &iov_cnt, NULL);
+		slist_unlock(sh->output);
 
-	switch (written) {
-	case (ssize_t) -1:
-		if (is_temporary_error(errno))
-			goto done;
+		g_assert(iov != NULL);
 
-		/* FALL THRU */
-	case 0:
-		shell_discard_output(sh);
-		if (!sh->shutdown) {
-			shell_shutdown(sh);
+		written = s->wio.writev(&s->wio, iov, iov_cnt);
+
+		if (GNET_PROPERTY(shell_debug) > 2) {
+			s_debug("%s(%p): wrote %zd byte%s (out of %zu)",
+				G_STRFUNC, sh, PLURAL(written),
+				iov_calculate_size(iov, iov_cnt));
 		}
-		break;
 
-	default:
-		pmsg_slist_discard(sh->output, written);
+		HFREE_NULL(iov);
+
+		switch (written) {
+		case (ssize_t) -1:
+			if (is_temporary_error(errno)) {
+				if (GNET_PROPERTY(shell_debug) > 2)
+					s_debug("%s(%p): temporary write error: %m", G_STRFUNC, sh);
+				return;
+			}
+			s_warning("%s(%p): writev() failed: %m", G_STRFUNC, sh);
+			/* FALL THROUGH */
+		case 0:
+			shell_discard_output(sh);
+			shell_shutdown(sh);
+			return;
+		default:
+			slist_lock(sh->output);
+			pmsg_slist_discard(sh->output, written);
+			slist_unlock(sh->output);
+			break;
+		}
+	}
+}
+
+/**
+ * Interpret commands on the line (separated by ';').
+ *
+ * @return TRUE if we are executing an asynchronous command and need to pause
+ * processing until the command is finished.
+ */
+static bool
+shell_interpret_line(struct gnutella_shell *sh, const char *line)
+{
+	shell_check(sh);
+	g_assert(line != NULL);
+	g_assert(!sh->async);		/* Done with possible previous async command */
+
+	shell_set_msg(sh, NULL);
+
+	while (line != NULL && *line != '\0') {
+		enum shell_reply reply_code;
+		const char *endptr = NULL;
+
+		reply_code = shell_exec(sh, line, &endptr);
+
+		/*
+		 * If the shell command runs asynchronously, then we need to wait for
+		 * the remainder of the command.  The shell_resume_processing() routine
+		 * will be called when it is completed.
+		 */
+
+		if (REPLY_ASYNC == reply_code)
+			return TRUE;
+
+		shell_handler_done(sh, reply_code);
+
+		line = endptr;
+		shell_set_msg(sh, NULL);
+
+		if (sh->exiting)
+			break;
+	}
+
+	getline_reset(sh->socket->getline);
+
+	return FALSE;
+}
+
+/**
+ * Interpret all the data held in the shell's socket read buffer, which
+ * may contain several lines of text.
+ */
+static void
+shell_interpret_data(struct gnutella_shell *sh)
+{
+	struct gnutella_socket *s;
+
+	shell_check(sh);
+	g_assert(sh->socket->getline);
+	g_assert(!sh->shutdown);
+	g_assert(!sh->async);
+
+	sh->last_update = tm_time();		/* Continuing to process */
+	s = sh->socket;
+
+	if (GNET_PROPERTY(shell_debug) > 1) {
+		s_debug("%s(%p): %zu byte%s in socket buffer",
+			G_STRFUNC, sh, PLURAL(s->pos));
+	}
+
+	while (s->pos > 0) {
+		size_t parsed;
+		const char *line;
+
+		switch (getline_read(s->getline, s->buf, s->pos, &parsed)) {
+		case READ_OVERFLOW:
+			s_warning("%s(%p): line is too long (from shell at %s)\n",
+				G_STRFUNC, sh, host_addr_port_to_string(s->addr, s->port));
+			shell_write(sh, "400 Line is too long.\n");
+			shell_shutdown(sh);
+			return;
+		case READ_DONE:
+			if (s->pos != parsed)
+				memmove(s->buf, &s->buf[parsed], s->pos - parsed);
+			s->pos -= parsed;
+			break;
+		case READ_MORE:
+			g_assert(parsed == s->pos);
+			s->pos = 0;
+			goto done;
+		}
+
+		/*
+		 * We come here everytime we get a full line.
+		 *
+		 * When command returns REPLY_READY, meaning it was executed
+		 * properly, do not show them any feedback if not running
+		 * interactively.
+		 */
+
+		sh->line_count++;
+		line = getline_str(s->getline);
+
+		if (shell_interpret_line(sh, line))
+			return;
+
+		if (sh->shutdown || sh->exiting || sh->eof)
+			return;
 	}
 
 done:
-	HFREE_NULL(iov);
-	return;
+	if (GNET_PROPERTY(shell_debug) > 1) {
+		s_debug("%s(%p): finished normally (setting INPUT_EVENT_RWX)",
+			G_STRFUNC, sh);
+	}
+
+	/*
+	 * The lock is not necessary here, as we are supposed to be running the
+	 * shell in a single thread at this point, with no concurrent shell thread.
+	 */
+
+	shell_evt_set(sh, INPUT_EVENT_RWX);
 }
 
 /**
@@ -585,11 +1119,15 @@ shell_read_data(struct gnutella_shell *sh)
 	g_assert(sh->socket->getline);
 	g_assert(!sh->shutdown);
 
+	if G_UNLIKELY(sh->async)
+		return;					/* In the middle of an async command */
+
 	sh->last_update = tm_time();
 	s = sh->socket;
 
 	if (s->buf_size - s->pos < 1) {
-		g_warning("Remote shell: Read more than buffer size.\n");
+		s_critical("%s(%p) read more than buffer size (%zu bytes)",
+			G_STRFUNC, sh, s->buf_size);
 	} else {
 		size_t size = s->buf_size - s->pos - 1;
 		ssize_t ret;
@@ -598,135 +1136,103 @@ shell_read_data(struct gnutella_shell *sh)
 		if (0 == ret) {
 			if (0 == s->pos) {
 				if (GNET_PROPERTY(shell_debug)) {
-					g_message("shell connection closed: EOF");
+					s_debug("%s(%p): shell connection closed: EOF",
+						G_STRFUNC, sh);
 				}
-				shell_shutdown(sh);
-				goto finish;
+				shell_eof(sh);
+				return;
 			}
 		} else if ((ssize_t) -1 == ret) {
 			if (!is_temporary_error(errno)) {
-				g_warning("Receiving data failed: %s\n", g_strerror(errno));
+				s_warning("%s(%p): receiving data failed: %m", G_STRFUNC, sh);
 				shell_shutdown(sh);
-				goto finish;
+				return;
 			}
 		} else {
 			s->pos += ret;
 		}
 	}
 
-	while (s->pos > 0) {
-		size_t parsed;
-		const char *line;
-
-		switch (getline_read(s->getline, s->buf, s->pos, &parsed)) {
-		case READ_OVERFLOW:
-			g_warning("Line is too long (from shell at %s)\n",
-				host_addr_port_to_string(s->addr, s->port));
-			shell_write(sh, "400 Line is too long.\n");
-			shell_shutdown(sh);
-			goto finish;
-		case READ_DONE:
-			if (s->pos != parsed)
-				memmove(s->buf, &s->buf[parsed], s->pos - parsed);
-			s->pos -= parsed;
-			break;
-		case READ_MORE:
-			g_assert(parsed == s->pos);
-			s->pos = 0;
-			goto finish;
-		}
-
-		/*
-		 * We come here everytime we get a full line.
-		 *
-		 * When command returns REPLY_READY, meaning it was executed
-		 * properly, do not show them any feedback if not running
-		 * interactively.
-		 */
-
-		sh->line_count++;
-		line = getline_str(s->getline);
-
-		while (line != NULL && *line != '\0') {
-			enum shell_reply reply_code;
-			const char *endptr = NULL;
-
-			reply_code = shell_exec(sh, line, &endptr);
-			if (
-				REPLY_NONE != reply_code &&
-				!(!sh->interactive && REPLY_READY == reply_code)
-			) {
-				/*
-				 * On error, when running non-interactively, remind them
-				 * about the command that failed first.
-				 */
-
-				if (REPLY_ERROR == reply_code && !sh->interactive) {
-					shell_write(sh, uint32_to_string(reply_code));
-					shell_write(sh, "-Error for: \"");
-					shell_write(sh, getline_str(s->getline));
-					shell_write(sh, "\"\n");
-				}
-
-				shell_write(sh, uint32_to_string(reply_code));
-				shell_write_msg(sh);
-				shell_write(sh, "\n");
-			}
-
-			line = endptr;
-			shell_set_msg(sh, NULL);
-		}
-
-		getline_reset(s->getline);
-		if (sh->shutdown)
-			goto finish;
-	}
-
-finish:
-	return;
+	shell_interpret_data(sh);
 }
 
-static void shell_handle_event(struct gnutella_shell *, inputevt_cond_t);
-
 /**
- * Called whenever some event occurs on a shell socket.
+ * Handle event condition on the shell's socket.
+ *
+ * @return TRUE if we can continue, FALSE if the shell was destroyed
+ * and the `sh' object is therefore invalid!
  */
-static void
-shell_handle_data(void *data, int unused_source, inputevt_cond_t cond)
-{
-	(void) unused_source;
-	shell_handle_event(data, cond);
-}
-
-static void
+static bool
 shell_handle_event(struct gnutella_shell *sh, inputevt_cond_t cond)
 {
+	inputevt_cond_t ev = INPUT_EVENT_NONE;
+
 	shell_check(sh);
 
+	if (GNET_PROPERTY(shell_debug) > 3) {
+		s_debug("%s(%p): got %c%c%c events, shutdown=%s, eof=%s",
+			G_STRFUNC, sh,
+			(cond & INPUT_EVENT_R) ? 'r' : '-',
+			(cond & INPUT_EVENT_W) ? 'w' : '-',
+			(cond & INPUT_EVENT_EXCEPTION) ? 'x' : '-',
+			bool_to_string(sh->shutdown),
+			bool_to_string(sh->eof));
+	}
+
 	if (cond & INPUT_EVENT_EXCEPTION) {
-		g_warning ("shell connection closed: exception");
-		goto destroy;
+		s_warning ("%s(%p): got %c%c exception", G_STRFUNC, sh,
+			(cond & INPUT_EVENT_R) ? 'r' : '-',
+			(cond & INPUT_EVENT_W) ? 'w' : '-');
+		if (cond & INPUT_EVENT_R)
+			shell_eof(sh);
+		if (cond & INPUT_EVENT_W)
+			shell_shutdown(sh);
 	}
 
-	if ((cond & INPUT_EVENT_W) && shell_has_pending_output(sh)) {
+	if ((cond & INPUT_EVENT_W) && shell_has_pending_output(sh))
 		shell_write_data(sh);
-	}
 
-	if ((cond & INPUT_EVENT_R) && !sh->shutdown) {
+	if ((cond & INPUT_EVENT_R) && !(sh->shutdown || sh->eof)) {
+		g_assert(!sh->async);
 		shell_read_data(sh);
 	}
 
 	if (!shell_has_pending_output(sh)) {
-		if (sh->shutdown)
+		if (sh->shutdown || sh->exiting)
 			goto destroy;
 
-		socket_evt_clear(sh->socket);
-		socket_evt_set(sh->socket, INPUT_EVENT_RX, shell_handle_data, sh);
+		if (!sh->async) {
+			if (sh->eof)
+				goto destroy;
+			ev = INPUT_EVENT_RX;
+		}
+
+		shell_evt_set(sh, ev);
 	}
-	return;
+
+	if (!sh->shutdown)
+		return TRUE;
+
+	/* FALL THROUGH */
 
 destroy:
-	shell_destroy(sh);
+	if (sh->async)
+		shell_shutdown(sh);
+	else
+		shell_destroy(sh);
+
+	return FALSE;		/* Cannot continue processing */
+}
+
+static void
+shell_evt_writable(struct gnutella_shell *sh)
+{
+	shell_check(sh);
+
+	if G_UNLIKELY(sh->shutdown)
+		return;
+
+	shell_evt_set(sh, INPUT_EVENT_WX);
 }
 
 void
@@ -738,25 +1244,101 @@ shell_write(struct gnutella_shell *sh, const char *text)
 	g_return_if_fail(sh->output);
 	g_return_if_fail(text);
 
-	len = strlen(text);
+	/*
+	 * If running asynchronously, the main thread could have decided to
+	 * shutdown the shell.   In which case we must no longer issue any
+	 * output.
+	 */
+
+	if (sh->shutdown)
+		return;
+
+	len = vstrlen(text);
 	g_return_if_fail(len < (size_t) -1);
 
 	if (len > 0) {
-		if (!shell_has_pending_output(sh)) {
-			socket_evt_clear(sh->socket);
-			socket_evt_set(sh->socket, INPUT_EVENT_WX, shell_handle_data, sh);
-		}
+		bool install_write_event;
+
+		slist_lock(sh->output);
+		install_write_event = !shell_has_pending_output(sh);
 		pmsg_slist_append(sh->output, text, len);
+		slist_unlock(sh->output);
+
+		if (install_write_event)
+			shell_evt_writable(sh);
 	}
 }
 
+static void
+shell_pending_add(struct gnutella_shell *sh, int code, const char *text)
+{
+	shell_pending_flush(sh, FALSE);
+	sh->pending.msg = h_strdup(text);
+	sh->pending.code = code;
+}
+
+/**
+ * Writes single line of text, appending final trailing "\n".
+ */
 void
-shell_shutdown(struct gnutella_shell *sh)
+shell_write_line(struct gnutella_shell *sh, int code, const char *text)
 {
 	shell_check(sh);
-	g_assert(!sh->shutdown);
+	g_return_if_fail(text);
 
-	sh->shutdown = TRUE;
+	shell_pending_add(sh, code, text);
+}
+
+/**
+ * Writes multiple lines of text, appending final trailing "\n" if needed.
+ */
+void
+shell_write_lines(struct gnutella_shell *sh, int code, const char *text)
+{
+	str_t *str;
+	int c;
+	const char *p = text;
+
+	shell_check(sh);
+	g_return_if_fail(text);
+
+	str = str_new(0);
+
+	while ((c = *p++)) {
+		if ('\n' == c) {
+			shell_pending_add(sh, code, str_2c(str));
+			str_reset(str);
+		} else {
+			str_putc(str, c);
+		}
+	}
+
+	if (0 != str_len(str))
+		shell_pending_add(sh, code, str_2c(str));
+
+	str_destroy_null(&str);
+}
+
+/**
+ * Writes single formatted line of text, appending final trailing "\n".
+ */
+void
+shell_write_linef(struct gnutella_shell *sh, int code, const char *fmt, ...)
+{
+	va_list args;
+	char *s;
+
+	shell_check(sh);
+	g_return_if_fail(sh->output);
+	g_return_if_fail(fmt);
+
+	va_start(args, fmt);
+	s = str_vcmsg(fmt, args);
+	va_end(args);
+
+	shell_pending_flush(sh, FALSE);
+	sh->pending.msg = s;
+	sh->pending.code = code;
 }
 
 #ifdef USE_REMOTE_CTRL
@@ -779,20 +1361,17 @@ shell_dump_cookie(const struct sha1 *cookie)
 	}
 }
 
-static const struct sha1 * 
+static const struct sha1 *
 shell_auth_cookie(void)
 {
 	static struct sha1 cookie;
-	static gboolean initialized;
+	static bool initialized;
 
 	if (!initialized) {
-		SHA1Context ctx;
-		guint32 noise[64];
+		uint32 noise[64];
 
-		random_bytes(noise, sizeof noise);
-		SHA1Reset(&ctx);
-		SHA1Input(&ctx, &noise, sizeof noise);
-		SHA1Result(&ctx, &cookie);
+		random_key_bytes(ARYLEN(noise));
+		SHA1_COMPUTE(noise, &cookie);
 		shell_dump_cookie(&cookie);
 		initialized = TRUE;
 	}
@@ -805,12 +1384,12 @@ shell_auth_cookie(void)
  *
  * @return TRUE if the connection is allowed.
  */
-static gboolean
+static bool
 shell_auth(struct gnutella_shell *sh, const char *str)
 {
 	const struct sha1 *cookie;
 	const char *tok_helo = NULL, *tok_cookie = NULL;
-	gboolean ok = FALSE;
+	bool ok = FALSE;
 	const char *endptr;
 
 	if (!shell_get_token(sh, str, &endptr, &tok_helo))
@@ -819,33 +1398,33 @@ shell_auth(struct gnutella_shell *sh, const char *str)
 		goto done;
 
 	if (GNET_PROPERTY(shell_debug)) {
-		g_message("auth: [%s] [<cookie not displayed>]", tok_helo);
+		s_debug("%s: [%s] [<cookie not displayed>]", G_STRFUNC, tok_helo);
 	}
 
 	cookie = shell_auth_cookie();
 	if (
 		tok_helo && 0 == strcmp("HELO", tok_helo) &&
-		tok_cookie && SHA1_BASE32_SIZE == strlen(tok_cookie) &&
+		tok_cookie && SHA1_BASE32_SIZE == vstrlen(tok_cookie) &&
 		0 == memcmp_diff(sha1_base32(cookie), tok_cookie, SHA1_BASE32_SIZE)
 	) {
 		ok = TRUE;
 	} else {
-		cpu_noise();
+		random_cpu_noise();
 	}
 
 done:
 	if (tok_helo != NULL)
-		g_free(deconstify_gpointer(tok_helo));
+		g_free(deconstify_pointer(tok_helo));
 	if (tok_cookie != NULL)
-		g_free(deconstify_gpointer(tok_cookie));
+		g_free(deconstify_pointer(tok_cookie));
 
 	return ok;
 }
 
-static gboolean
+static bool
 shell_grant_remote_shell(struct gnutella_shell *sh)
 {
-	gboolean granted = FALSE;
+	bool granted = FALSE;
 
 	shell_check(sh);
 
@@ -855,12 +1434,12 @@ shell_grant_remote_shell(struct gnutella_shell *sh)
 			sh->interactive = TRUE;
 			shell_write_welcome(sh);
 		} else {
-			g_warning("invalid credentials");
+			s_warning("invalid credentials");
 			shell_write(sh, "400 Invalid credentials\n");
 		}
 		getline_reset(sh->socket->getline); /* clear AUTH command from buffer */
 	} else {
-		g_warning("remote shell control interface disabled");
+		s_warning("remote shell control interface disabled");
 		shell_write(sh, "401 Disabled\n");
 	}
 	return granted;
@@ -868,22 +1447,22 @@ shell_grant_remote_shell(struct gnutella_shell *sh)
 
 #else /* !USE_REMOTE_CTRL */
 
-static const struct sha1 * 
+static const struct sha1 *
 shell_auth_cookie(void)
 {
 	return NULL;
 }
 
-static gboolean
+static bool
 shell_grant_remote_shell(const struct gnutella_shell *sh)
 {
 	shell_check(sh);
-	g_warning("remote shell control interface disabled");
+	s_warning("remote shell control interface disabled");
 	return FALSE;
 }
 #endif /* USE_REMOTE_CTRL */
 
-static gboolean
+static bool
 shell_grant_local_shell(struct gnutella_shell *sh)
 {
 	shell_check(sh);
@@ -892,7 +1471,7 @@ shell_grant_local_shell(struct gnutella_shell *sh)
 		getline_reset(sh->socket->getline); /* remove HELO command */
 		return TRUE;
 	} else {
-		g_warning("local shell control interface disabled");
+		s_warning("local shell control interface disabled");
 		shell_write(sh, "401 Disabled\n");
 		return FALSE;
 	}
@@ -905,15 +1484,15 @@ void
 shell_add(struct gnutella_socket *s)
 {
 	struct gnutella_shell *sh;
-	gboolean granted = FALSE;
+	bool granted = FALSE;
 
 	socket_check(s);
 	g_assert(0 == s->gdk_tag);
 	g_assert(s->getline);
 
 	if (GNET_PROPERTY(shell_debug)) {
-		g_message("Incoming shell connection from %s",
-			host_addr_port_to_string(s->addr, s->port));
+		s_debug("%s: incoming shell connection from %s",
+			G_STRFUNC, host_addr_port_to_string(s->addr, s->port));
 	}
 
 	s->type = SOCK_TYPE_SHELL;
@@ -932,33 +1511,35 @@ shell_add(struct gnutella_socket *s)
 	}
 
 	/* We don't read anymore on shutdown, but be paranoid just in case. */
-	shell_handle_event(sh, sh->shutdown ? INPUT_EVENT_W : INPUT_EVENT_RW);
+	(void) shell_handle_event(sh, sh->shutdown ? INPUT_EVENT_W : INPUT_EVENT_RW);
 }
 
 void
 shell_timer(time_t now)
 {
 	time_delta_t timeout = GNET_PROPERTY(remote_shell_timeout);
+	struct gnutella_shell *sh;
 
 	if (timeout > 0) {
-		GSList *sl, *to_remove = NULL;
-
-		for (sl = sl_shells; sl != NULL; sl = g_slist_next(sl)) {
-			struct gnutella_shell *sh = sl->data;
+		pslist_t *to_remove = NULL, *sl;
 
+		ELIST_FOREACH_DATA(&shells, sh) {
 			shell_check(sh);
 			if (
 				0 == (SOCK_F_LOCAL & sh->socket->flags) &&
-				delta_time(now, sh->last_update) > timeout
+				delta_time(now, sh->last_update) > timeout &&
+				!sh->async	/* No timeout if command done by other thread */
 			) {
-				to_remove = g_slist_prepend(to_remove, sh);
+				to_remove = pslist_prepend(to_remove, sh);
+			} else if (sh->shutdown && !sh->async) {
+				to_remove = pslist_prepend(to_remove, sh);
 			}
 		}
-		for (sl = to_remove; sl != NULL; sl = g_slist_next(sl)) {
-			struct gnutella_shell *sh = sl->data;
+		PSLIST_FOREACH(to_remove, sl) {
+			sh = sl->data;
 			shell_destroy(sh);
 		}
-		g_slist_free(to_remove);
+		pslist_free_null(&to_remove);
 	}
 }
 
@@ -971,10 +1552,12 @@ shell_init(void)
 void
 shell_close(void)
 {
-	while (sl_shells) {
-		struct gnutella_shell *sh = sl_shells->data;
+	struct gnutella_shell *sh;
+
+	while (NULL != (sh = elist_head(&shells))) {
 		shell_destroy(sh);
 	}
+	htable_free_null(&shell_cmds);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
