@@ -1,6 +1,4 @@
 /*
- * $Id: uploads.c 17171 2009-11-03 20:08:11Z rmanfredi $
- *
  * Copyright (c) 2002-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,18 +33,16 @@
 
 #include "common.h"
 
-RCSID("$Id: uploads.c 17171 2009-11-03 20:08:11Z rmanfredi $")
-
 #include "cmd.h"
 
 #include "core/uploads.h"
 
-#include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
-#include "lib/glib-missing.h"
 #include "lib/iso3166.h"
 #include "lib/misc.h"
+#include "lib/pslist.h"
+#include "lib/str.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -59,7 +55,7 @@ print_upload_info(struct gnutella_shell *sh,
 	g_return_if_fail(sh);
 	g_return_if_fail(info);
 
-	gm_snprintf(buf, sizeof buf, "%-3.3s %-16.40s %s %s@%s %s%s%s",
+	str_bprintf(ARYLEN(buf), "%-3.3s %-16.40s %s %s@%s %s%s%s",
 		info->encrypted ? "(E)" : "",
 		host_addr_to_string(info->addr),
 		iso3166_country_cc(info->country),
@@ -81,8 +77,8 @@ print_upload_info(struct gnutella_shell *sh,
 enum shell_reply
 shell_exec_uploads(struct gnutella_shell *sh, int argc, const char *argv[])
 {
-	const GSList *sl;
-	GSList *sl_info;
+	const pslist_t *sl;
+	pslist_t *sl_info;
 
 	shell_check(sh);
 	g_assert(argv);
@@ -93,7 +89,7 @@ shell_exec_uploads(struct gnutella_shell *sh, int argc, const char *argv[])
 	shell_write(sh, "100~ \n");
 
 	sl_info = upload_get_info_list();
-	for (sl = sl_info; sl; sl = g_slist_next(sl)) {
+	PSLIST_FOREACH(sl_info, sl) {
 		print_upload_info(sh, sl->data);
 	}
 	upload_free_info_list(&sl_info);
@@ -114,7 +110,7 @@ shell_help_uploads(int argc, const char *argv[])
 {
 	g_assert(argv);
 	g_assert(argc > 0);
-	
+
 	return NULL;
 }
 
