@@ -1,6 +1,4 @@
 /*
- * $Id: clipboard.c 16492 2009-04-11 22:48:08Z cbiere $
- *
  * Copyright (c) 2007, Christian Biere
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,8 +33,6 @@
 
 #include "gui.h"
 
-RCSID("$Id: clipboard.c 16492 2009-04-11 22:48:08Z cbiere $")
-
 #include "clipboard.h"
 
 #include "lib/override.h"		/* Must be the last header included */
@@ -47,14 +43,14 @@ void
 clipboard_set_text(GtkWidget *unused_owner, const char *text)
 {
 	(void) unused_owner;
-	
+
 	gtk_clipboard_clear(gtk_clipboard_get(GDK_SELECTION_PRIMARY));
 	gtk_clipboard_clear(gtk_clipboard_get(GDK_SELECTION_CLIPBOARD));
 
 	if (text) {
 		size_t length;
 
-		length = strlen(text);
+		length = vstrlen(text);
 		if (length < UNSIGNED(INT_MAX)) {
 			gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
 				text, length);
@@ -63,7 +59,7 @@ clipboard_set_text(GtkWidget *unused_owner, const char *text)
 		}
 	}
 }
-	
+
 #else	/* Gtk+ 1.2 */
 
 #define CLIPBOARD_FETCH_TEXT ((const void *) -1)
@@ -113,9 +109,13 @@ on_clipboard_selection_get(GtkWidget *unused_widget,
 	(void) unused_eventtime;
 
 	text = clipboard_text_get();
-	length = strlen(text);
-	if (length >= UNSIGNED(INT_MAX)) {
-		text = NULL;
+	if (NULL != text) {
+		length = vstrlen(text);
+		if (length >= UNSIGNED(INT_MAX)) {
+			text = NULL;
+			length = 0;
+		}
+	} else {
 		length = 0;
 	}
     gtk_selection_data_set(data, GDK_SELECTION_TYPE_STRING,
