@@ -1,6 +1,4 @@
 /*
- * $Id: filter_cb.c 14553 2007-08-18 23:31:39Z cbiere $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,14 +33,13 @@
 
 #include "gui.h"
 
-RCSID("$Id: filter_cb.c 14553 2007-08-18 23:31:39Z cbiere $")
-
 #include "filter_core.h"
 #include "filter_cb.h"
 #include "filter.h"
 
 #include "if/gui_property_priv.h"
 
+#include "lib/glib-missing.h"
 #include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -303,30 +300,30 @@ void
 on_button_filter_reset_all_clicked(GtkButton *unused_button,
 	gpointer unused_udata)
 {
-    gint row;
-    GtkCTree *ctree;
-    GtkCList *clist;
+	gint row;
+	GtkCTree *ctree;
+	GtkCList *clist;
 
 	(void) unused_button;
 	(void) unused_udata;
 
-    if (gui_filter_dialog() == NULL)
-        return;
+	if (gui_filter_dialog() == NULL)
+		return;
 
-    ctree = GTK_CTREE(gui_filter_dialog_lookup("ctree_filter_filters"));
-    clist = GTK_CLIST(ctree);
+	ctree = GTK_CTREE(gui_filter_dialog_lookup("ctree_filter_filters"));
+	clist = GTK_CLIST(ctree);
 
-    for (row = 0; row < clist->rows; row++) {
-        filter_t *filter;
-        GtkCTreeNode *node;
+	for (row = 0; row < clist->rows; row++) {
+		filter_t *filter;
+		GtkCTreeNode *node;
 
-        node = gtk_ctree_node_nth(ctree, row);
-        filter = gtk_ctree_node_get_row_data(ctree, node);
-        if (filter == NULL)
-            continue;
+		node = gtk_ctree_node_nth(ctree, row);
+		filter = gtk_ctree_node_get_row_data(ctree, node);
+		if (filter == NULL)
+			continue;
 
-        filter_reset_stats(filter);
-    }
+		filter_reset_stats(filter);
+	}
 }
 
 void
@@ -661,7 +658,7 @@ on_treeview_filter_rules_button_press_event(GtkWidget *widget,
 		GdkEventButton *event, gpointer unused_udata)
 {
     gboolean sensitive;
-	
+
 	(void) unused_udata;
 
     if (event->button != 3)
@@ -678,10 +675,10 @@ on_treeview_filter_rules_button_press_event(GtkWidget *widget,
         clear_clipboard();
         return TRUE;
     }
-	
+
 	{
 		GtkTreePath *path;
-		
+
 		gtk_tree_view_get_cursor(GTK_TREE_VIEW(widget), &path, NULL);
     	sensitive = path != NULL && work_filter != NULL;
 		gtk_tree_path_free(path);
@@ -733,16 +730,16 @@ filter_reset_all_helper(GtkTreeModel *model, GtkTreePath *unused_path,
 	GtkTreeIter *iter, gpointer unused_udata)
 {
 	gpointer p;
-	
+
 	(void) unused_path;
 	(void) unused_udata;
-	
+
    	gtk_tree_model_get(model, iter, 0, &p, (-1));
 	if (p) {
 		filter_t *filter = p;
         filter_reset_stats(filter);
 	}
-	
+
 	return FALSE; /* continue traversal */
 }
 
@@ -753,8 +750,8 @@ on_button_filter_reset_all_clicked(GtkButton *unused_button,
 	(void) unused_button;
 	(void) unused_udata;
 
-    if (gui_filter_dialog() == NULL)
-        return;
+	if (gui_filter_dialog() == NULL)
+		return;
 
 	gtk_tree_model_foreach(gtk_tree_view_get_model(GTK_TREE_VIEW(
 		gui_filter_dialog_lookup("treeview_filter_filters"))),
@@ -862,7 +859,7 @@ on_button_filter_add_rule_state_clicked(GtkButton *unused_button,
 
 	gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(
 		GTK_TREE_VIEW(gui_filter_dialog_lookup("treeview_filter_rules"))));
-	
+
 	filter_gui_edit_state_rule(NULL);
 }
 
@@ -939,7 +936,7 @@ filter_clear_helper(GtkTreeModel *model, GtkTreePath *unused_path,
 {
 	GSList **sl_ptr = slist_ptr;
 	gpointer p;
-	
+
 	(void) unused_path;
 
 	gtk_tree_model_get(model, iter, 0, &p, (-1));
@@ -947,12 +944,12 @@ filter_clear_helper(GtkTreeModel *model, GtkTreePath *unused_path,
 
 	return FALSE; /* continue traversal */
 }
-	
+
 void
 on_button_filter_clear_clicked(GtkButton *unused_button, gpointer unused_udata)
 {
 	GSList *sl, *sl_rules = NULL;
-	
+
 	(void) unused_button;
 	(void) unused_udata;
 
@@ -962,11 +959,10 @@ on_button_filter_clear_clicked(GtkButton *unused_button, gpointer unused_udata)
 		filter_clear_helper,
 		&sl_rules);
 
-	for (sl = sl_rules; sl != NULL; sl = g_slist_next(sl))	
+	for (sl = sl_rules; sl != NULL; sl = g_slist_next(sl))
 		filter_remove_rule_from_session(work_filter, sl->data);
-	g_slist_free(sl_rules);
-	sl_rules = NULL;
-	
+	gm_slist_free_null(&sl_rules);
+
     gtk_notebook_set_current_page(
         GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_buttons);
@@ -1015,7 +1011,7 @@ on_button_filter_reset_rule_clicked(GtkButton *unused_button,
 
 	(void) unused_button;
 	(void) unused_udata;
-	
+
     tv = GTK_TREE_VIEW(gui_filter_dialog_lookup("treeview_filter_rules"));
 	gtk_tree_view_get_cursor(tv, &path, NULL);
     if (path) {
@@ -1041,16 +1037,16 @@ filter_reset_all_rules_helper(GtkTreeModel *model, GtkTreePath *unused_path,
 	GtkTreeIter *iter, gpointer unused_udata)
 {
 	gpointer p;
-	
+
 	(void) unused_path;
 	(void) unused_udata;
-	
+
    	gtk_tree_model_get(model, iter, 0, &p, (-1));
 	if (p) {
         rule_t *rule = p;
         filter_rule_reset_stats(rule);
 	}
-	
+
 	return FALSE; /* continue traversal */
 }
 
