@@ -1,6 +1,4 @@
 /*
- * $Id: fileinfo.c 16053 2009-03-03 16:56:12Z rmanfredi $
- *
  * Copyright (c) 2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -35,8 +33,6 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: fileinfo.c 16053 2009-03-03 16:56:12Z rmanfredi $")
-
 #include "gtk/columns.h"
 #include "gtk/downloads_common.h"
 #include "gtk/drag.h"
@@ -45,6 +41,7 @@ RCSID("$Id: fileinfo.c 16053 2009-03-03 16:56:12Z rmanfredi $")
 #include "if/gui_property.h"
 
 #include "lib/cq.h"
+#include "lib/htable.h"
 #include "lib/utf8.h"
 
 #include "lib/override.h"		/* Must be the last header included */
@@ -60,9 +57,9 @@ static GtkCList *clist_download_sources;
 static GtkCList *clist_download_aliases;
 static GtkCList *clist_download_details;
 
-static GHashTable *file_rows;		/* row -> struct fileinfo_data */
-static GHashTable *source_rows;		/* row -> struct download */
-static GHashTable *fi_sources;		/* struct download -> row */
+static htable_t *file_rows;		/* row -> struct fileinfo_data */
+static htable_t *source_rows;	/* row -> struct download */
+static htable_t *fi_sources;	/* struct download -> row */
 
 static cevent_t *cursor_ev;
 
@@ -85,7 +82,7 @@ get_fileinfo_data(int row)
 {
 	struct fileinfo_data *file;
 
-	file = g_hash_table_lookup(file_rows, int_to_pointer(row));
+	file = htable_lookup(file_rows, int_to_pointer(row));
 	g_assert(file);
 	g_assert(row == fileinfo_data_get_row(file));
 
@@ -112,12 +109,11 @@ fi_gui_get_file_at_cursor(void)
 }
 
 static void
-cursor_expire(cqueue_t *unused_cq, gpointer unused_udata)
+cursor_expire(cqueue_t *cq, gpointer unused_udata)
 {
-	(void) unused_cq;
 	(void) unused_udata;
 
-	cursor_ev = NULL;
+	cq_zero(cq, &cursor_ev);
 	fi_gui_files_cursor_update();
 }
 
@@ -125,10 +121,9 @@ static void
 cursor_update(void)
 {
 	if (cursor_ev) {
-		cq_resched(callout_queue, cursor_ev, ROW_SELECT_TIMEOUT);
+		cq_resched(cursor_ev, ROW_SELECT_TIMEOUT);
 	} else {
-		cursor_ev = cq_insert(callout_queue, ROW_SELECT_TIMEOUT,
-						cursor_expire, NULL);
+		cursor_ev = cq_main_insert(ROW_SELECT_TIMEOUT, cursor_expire, NULL);
 	}
 }
 
@@ -166,7 +161,7 @@ on_clist_download_files_row_moved(int dst, void *user_data)
 
 	if (src != dst) {
 		fileinfo_data_set_row(file, dst);
-		g_hash_table_insert(file_rows, int_to_pointer(dst), file);
+		htable_insert(file_rows, int_to_pointer(dst), file);
 	}
 }
 
@@ -175,7 +170,7 @@ download_details_get_text(GtkWidget *widget)
 {
 	GtkCList *clist;
 	int row;
-	
+
 	clist = GTK_CLIST(widget);
 	row = clist_get_cursor_row(clist);
 	return clist_copy_text(clist, row, 1);
@@ -186,7 +181,7 @@ download_aliases_get_text(GtkWidget *widget)
 {
 	GtkCList *clist;
 	int row;
-	
+
 	clist = GTK_CLIST(widget);
 	row = clist_get_cursor_row(clist);
 	return clist_copy_text(clist, row, 0);
@@ -204,7 +199,7 @@ on_clist_download_files_row_removed(void *data)
 	struct fileinfo_data *file = data;
 	int row = fileinfo_data_get_row(file);
 
-	g_hash_table_remove(file_rows, int_to_pointer(row));
+	htable_remove(file_rows, int_to_pointer(row));
 	fi_gui_file_invalidate(file);
 	clist_sync_rows(clist_download_files, on_clist_download_files_row_moved);
 }
@@ -223,12 +218,12 @@ fi_gui_file_show(struct fileinfo_data *file)
 	if (row < 0) {
 		const char *titles[c_fi_num];
 
-		for (i = 0; i < G_N_ELEMENTS(titles); i++) {
+		for (i = 0; i < N_ITEMS(titles); i++) {
 			titles[i] = "";
 		}
 		row = gtk_clist_append(clist_download_files, (char **) &titles);
 		fileinfo_data_set_row(file, row);
-		g_hash_table_insert(file_rows, int_to_pointer(row), file);
+		htable_insert(file_rows, int_to_pointer(row), file);
 	}
 	gtk_clist_set_row_data_full(clist, row, file,
 		on_clist_download_files_row_removed);
@@ -256,8 +251,8 @@ get_source(int row)
 {
 	struct download *d;
 
-	d = g_hash_table_lookup(source_rows, int_to_pointer(row));
-	g_assert(pointer_to_int(g_hash_table_lookup(fi_sources, d)) == row);
+	d = htable_lookup(source_rows, int_to_pointer(row));
+	g_assert(pointer_to_int(htable_lookup(fi_sources, d)) == row);
 
 #ifdef FILEINFO_C_ROW_CACHE_REGRESSION
 	{
@@ -295,6 +290,8 @@ void
 fi_gui_clear_sources(void)
 {
    	gtk_clist_clear(clist_download_sources);
+	htable_clear(fi_sources);
+	htable_clear(source_rows);
 }
 
 
@@ -328,48 +325,96 @@ on_clist_download_sources_row_moved(int dst, void *user_data)
 	int src;
 
 	download_check(d);
-	src = pointer_to_int(g_hash_table_lookup(fi_sources, d));
+	src = pointer_to_int(htable_lookup(fi_sources, d));
 	if (src != dst) {
-		g_hash_table_insert(fi_sources, d, int_to_pointer(dst));
-		g_hash_table_insert(source_rows, int_to_pointer(dst), d);
+		htable_insert(fi_sources, d, int_to_pointer(dst));
+		htable_insert(source_rows, int_to_pointer(dst), d);
 	}
 }
 
 static void
 on_clist_download_sources_row_removed(void *data)
 {
+	int count;
+
 	download_check(data);
-	g_hash_table_remove(fi_sources, data);
+	g_return_unless(htable_contains(fi_sources, data));
+	g_soft_assert_log(htable_count(fi_sources) == htable_count(source_rows),
+		"fi_sources count: %zu, source_rows count: %zu",
+		htable_count(fi_sources), htable_count(source_rows));
+
+	count = htable_count(fi_sources);		/* Old row count */
+	htable_remove(fi_sources, data);
+
 	clist_sync_rows(clist_download_sources,
 		on_clist_download_sources_row_moved);
+
+	htable_remove(source_rows, int_to_pointer(count - 1));	/* Last row gone */
+
+	g_soft_assert_log(htable_count(fi_sources) == htable_count(source_rows),
+		"fi_sources count: %zu, source_rows count: %zu",
+		htable_count(fi_sources), htable_count(source_rows));
+}
+
+/**
+ * Called before invoking fi_gui_source_show() repeatedly.
+ *
+ * This allows freezing the download clist to avoid a costly redrawing of
+ * the list after each addition, preventing O(n^2) GUI updates.
+ */
+void
+fi_gui_source_massive_update(bool starting)
+{
+	GtkCList *clist = clist_download_sources;
+
+	g_return_if_fail(clist);
+
+	/*
+	 * Freeze / thaw operations are counted by the clist, so it is perfectly
+	 * fine to freeze here even though fi_gui_source_show() will also perform
+	 * some freeze/thaw operations.
+	 */
+
+	if (starting) {
+		gtk_clist_freeze(clist);
+	} else {
+		gtk_clist_thaw(clist);
+	}
 }
 
 void
 fi_gui_source_show(struct download *key)
 {
-	const char *titles[c_fi_sources];
+	const char *titles[c_src_num];
 	GtkCList *clist;
 	unsigned i;
 	int row;
 
 	clist = clist_download_sources;
 	g_return_if_fail(clist);
-	g_return_if_fail(
-		!g_hash_table_lookup_extended(fi_sources, key, NULL, NULL));
+	g_return_if_fail(!htable_contains(fi_sources, key));
 
-	for (i = 0; i < G_N_ELEMENTS(titles); i++) {
+	for (i = 0; i < N_ITEMS(titles); i++) {
 		titles[i] = "";
 	}
+
+	gtk_clist_freeze(clist);
 	row = gtk_clist_append(clist, (char **) titles);
 	g_return_if_fail(row >= 0);
 
-	g_hash_table_insert(fi_sources, key, int_to_pointer(row));
-	g_hash_table_insert(source_rows, int_to_pointer(row), key);
+	htable_insert(fi_sources, key, int_to_pointer(row));
+	htable_insert(source_rows, int_to_pointer(row), key);
+
+	g_soft_assert_log(htable_count(fi_sources) == htable_count(source_rows),
+		"fi_sources count: %zu, source_rows count: %zu",
+		htable_count(fi_sources), htable_count(source_rows));
+
 	gtk_clist_set_row_data_full(clist, row, key,
 		on_clist_download_sources_row_removed);
-	for (i = 0; i < c_fi_sources; i++) {
+	for (i = 0; i < N_ITEMS(titles); i++) {
 		render_sources(key, row, i);
 	}
+	gtk_clist_thaw(clist);
 }
 
 void
@@ -379,7 +424,7 @@ fi_gui_source_hide(struct download *key)
 
 	g_return_if_fail(clist_download_sources);
 
-	if (g_hash_table_lookup_extended(fi_sources, key, NULL, &value)) {
+	if (htable_lookup_extended(fi_sources, key, NULL, &value)) {
 		int row = pointer_to_int(value);
 
 		gtk_clist_remove(clist_download_sources, row);
@@ -457,10 +502,10 @@ fi_gui_source_update(struct download *d)
 
 	download_check(d);
 
-	if (g_hash_table_lookup_extended(fi_sources, d, NULL, &value)) {
+	if (htable_lookup_extended(fi_sources, d, NULL, &value)) {
 		int i, row = pointer_to_int(value);
 
-		for (i = 0; i < c_fi_sources; i++) {
+		for (i = 0; i < c_src_num; i++) {
 			render_sources(d, row, i);
 		}
 	}
@@ -599,9 +644,9 @@ fi_gui_files_widget_destroy(void)
 void
 fi_gui_init(void)
 {
-	file_rows = g_hash_table_new(NULL, NULL);
-	source_rows = g_hash_table_new(NULL, NULL);
-	fi_sources = g_hash_table_new(NULL, NULL);
+	file_rows = htable_create(HASH_KEY_SELF, 0);
+	source_rows = htable_create(HASH_KEY_SELF, 0);
+	fi_sources = htable_create(HASH_KEY_SELF, 0);
 
 	clist_download_aliases = GTK_CLIST(
 		gui_main_window_lookup("clist_download_aliases"));
@@ -619,13 +664,13 @@ fi_gui_init(void)
 			"key-press-event", on_details_key_press_event, NULL);
 
 		clipboard_attach(GTK_WIDGET(clist));
-		drag_attach(GTK_WIDGET(clist), download_details_get_text);
+		drag_attach_text(GTK_WIDGET(clist), download_details_get_text);
 	}
 
 	{
 		GtkCList *clist = clist_download_aliases;
 
-		drag_attach(GTK_WIDGET(clist), download_aliases_get_text);
+		drag_attach_text(GTK_WIDGET(clist), download_aliases_get_text);
 	}
 
 	{
@@ -642,7 +687,7 @@ fi_gui_init(void)
 			title = gtk_label_get_text(label);
 			gtk_clist_set_column_name(clist, i, EMPTY_STRING(title));
 		}
-		widget_add_popup_menu(GTK_WIDGET(clist), fi_gui_sources_get_popup_menu);	
+		widget_add_popup_menu(GTK_WIDGET(clist), fi_gui_sources_get_popup_menu);
 	}
 	fi_gui_common_init();
 }
@@ -650,7 +695,7 @@ fi_gui_init(void)
 void
 fi_gui_shutdown(void)
 {
-	cq_cancel(callout_queue, &cursor_ev);
+	cq_cancel(&cursor_ev);
 
 	clist_save_visibility(clist_download_files, PROP_FILE_INFO_COL_VISIBLE);
 	clist_save_widths(clist_download_files, PROP_FILE_INFO_COL_WIDTHS);
@@ -671,13 +716,9 @@ fi_gui_shutdown(void)
 		clist_download_sources = NULL;
 	}
 
-	g_hash_table_destroy(fi_sources);
-	fi_sources = NULL;
-
-	g_hash_table_destroy(file_rows);
-	file_rows = NULL;
-	g_hash_table_destroy(source_rows);
-	source_rows = NULL;
+	htable_free_null(&fi_sources);
+	htable_free_null(&file_rows);
+	htable_free_null(&source_rows);
 }
 
 void
@@ -697,7 +738,7 @@ fi_gui_file_select(struct fileinfo_data *file)
 
 	g_return_if_fail(file);
 	g_return_if_fail(clist);
-	
+
    	row = fileinfo_data_get_row(file);
 	g_return_if_fail(row >= 0);
 
@@ -712,7 +753,7 @@ fi_gui_files_foreach(fi_gui_files_foreach_cb func, void *user_data)
 
 	g_return_if_fail(func);
 	g_return_if_fail(clist);
-	
+
 	gtk_clist_freeze(clist);
     for (row = 0; row < clist->rows; row++) {
 		(*func)(get_fileinfo_data(row), user_data);
