@@ -1,6 +1,4 @@
 /*
- * $Id: monitor.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -36,8 +34,6 @@
 
 #include "common.h"
 
-RCSID("$Id: monitor.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
 #include "gtk/gui.h"
 #include "gtk/monitor.h"
 
@@ -45,7 +41,8 @@ RCSID("$Id: monitor.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
 
-#include "lib/glib-missing.h"
+#include "lib/str.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 /***
@@ -91,7 +88,7 @@ monitor_gui_append_to_monitor(query_type_t type, const gchar *item,
     	if (type == QUERY_SHA1) {
         	/* If the query is empty and we have a SHA1 extension,
         	 * we print a urn:sha1-query instead. */
-        	gm_snprintf(tmpstr, sizeof(tmpstr), "urn:sha1:%s", item);
+        	str_bprintf(tmpstr, sizeof(tmpstr), "urn:sha1:%s", item);
     	} else {
         	g_strlcpy(tmpstr, item, sizeof(tmpstr));
     	}
