@@ -1,6 +1,4 @@
 /*
- * $Id: search.c 16990 2009-09-25 15:23:43Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
  * GUI filtering functions.
@@ -21,7 +19,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -37,11 +35,8 @@
  */
 #include "gtk/gui.h"
 
-RCSID("$Id: search.c 16990 2009-09-25 15:23:43Z rmanfredi $")
-
 #include "search_cb.h"
 
-#include "gtk/bitzi.h"
 #include "gtk/columns.h"
 #include "gtk/drag.h"
 #include "gtk/misc.h"
@@ -58,6 +53,10 @@ RCSID("$Id: search.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 #include "lib/atoms.h"
 #include "lib/base32.h"
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/hashing.h"
+#include "lib/hset.h"
+#include "lib/htable.h"
 #include "lib/iso3166.h"
 #include "lib/mime_type.h"
 #include "lib/stringify.h"
@@ -105,12 +104,11 @@ clist_search(void)
  *	Emile 02/15/2004
  */
 static inline void
-add_parent_with_sha1(search_t *search, const gchar *sha1, GtkCTreeNode *data)
+add_parent_with_sha1(search_t *search, const char *sha1, GtkCTreeNode *data)
 {
-	gm_hash_table_insert_const(search->parents, sha1, data);
+	htable_insert(search->parents, sha1, data);
 }
 
-
 /**
  *	Removes the tree node matching the given sha1 from the hash table.
  *  The atom used for the key is then freed
@@ -118,19 +116,19 @@ add_parent_with_sha1(search_t *search, const gchar *sha1, GtkCTreeNode *data)
 static inline void
 remove_parent_with_sha1(search_t *search, const struct sha1 *sha1)
 {
-	gconstpointer key;
-	gpointer orig_key;
+	const void *key;
+	const void *orig_key;
 
 	key = atom_sha1_get(sha1);
 
-	if (g_hash_table_lookup_extended(search->parents, key, &orig_key, NULL)) {
+	if (htable_lookup_extended(search->parents, key, &orig_key, NULL)) {
 		/* Must first free memory used by the original key */
 		atom_sha1_free(orig_key);
 
 		/* Then remove the key */
-		g_hash_table_remove(search->parents, key);
+		htable_remove(search->parents, key);
 	} else
-		g_warning("remove_parent_with_sha1: can't find sha1 in hash table!");
+		g_warning("%s(): can't find sha1 in hash table!", G_STRFUNC);
 
 	atom_sha1_free(key);
 }
@@ -143,11 +141,11 @@ remove_parent_with_sha1(search_t *search, const struct sha1 *sha1)
 GtkCTreeNode *
 find_parent_with_sha1(search_t *search, gconstpointer key)
 {
-	return g_hash_table_lookup(search->parents, key);
+	return htable_lookup(search->parents, key);
 }
 
-gboolean
-search_gui_free_parent(gpointer key, gpointer unused_value, gpointer unused_x)
+static bool
+search_gui_free_parent(const void *key, void *unused_value, void *unused_x)
 {
 	(void) unused_value;
 	(void) unused_x;
@@ -165,11 +163,10 @@ search_gui_free_gui_record(gpointer gui_rc)
  * Decrement refcount of hash table key entry.
  */
 static gboolean
-dec_records_refcount(gpointer key, gpointer unused_value, gpointer unused_x)
+dec_records_refcount(const void *key, void *unused_x)
 {
-	(void) unused_value;
 	(void) unused_x;
-	search_gui_unref_record(key);
+	search_gui_unref_record(deconstify_pointer(key));
 	return TRUE;
 }
 
@@ -211,8 +208,8 @@ search_gui_clear_search(search_t *sch)
 	g_assert(sch->dups);
 
 	search_gui_clear_ctree(GTK_CTREE(sch->tree));
-	g_hash_table_foreach_remove(sch->dups, dec_records_refcount, NULL);
-	g_hash_table_foreach_remove(sch->parents, search_gui_free_parent, NULL);
+	hset_foreach_remove(sch->dups, dec_records_refcount, NULL);
+	htable_foreach_remove(sch->parents, search_gui_free_parent, NULL);
 }
 
 static gint search_gui_cursor_x, search_gui_cursor_y;
@@ -231,7 +228,7 @@ search_gui_set_cursor_position(gint x, gint y)
 }
 
 
-char * 
+char *
 search_gui_get_local_file_url(GtkWidget *widget)
 {
 	GtkCTreeNode *node;
@@ -248,7 +245,7 @@ search_gui_get_local_file_url(GtkWidget *widget)
 
 	if (row < 0)
 		return NULL;
-	
+
 	node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
 	grc = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
 	record = grc->shared_record;
@@ -257,7 +254,7 @@ search_gui_get_local_file_url(GtkWidget *widget)
 
 	if (!record->tag)
 		return NULL;
-	
+
 	return url_from_absolute_path(record->tag);
 }
 
@@ -268,7 +265,7 @@ search_gui_init_tree(search_t *search)
     int row;
 
 	g_assert(NULL == search->parents);
-	search->parents = g_hash_table_new(pointer_hash_func, NULL);
+	search->parents = htable_create(HASH_KEY_SELF, 0);
 
     titles[c_sl_name] = lazy_utf8_to_ui_string(search_gui_query(search));
     titles[c_sl_hit] = "0";
@@ -276,7 +273,7 @@ search_gui_init_tree(search_t *search)
     row = gtk_clist_append(GTK_CLIST(clist_search()), (gchar **) titles);
     gtk_clist_set_row_data(GTK_CLIST(clist_search()), row, search);
 
-	search_gui_sort_column(search, search->sort_col);
+	search_gui_sort_column(search, search->sorting.s_column);
 }
 
 
@@ -323,7 +320,7 @@ search_gui_compare_records(gint sort_col,
             result = strcmp(EMPTY_STRING(r1->ext), EMPTY_STRING(r2->ext));
             break;
 
-		
+
         case c_sr_mime:
 			{
 				enum mime_type mt1, mt2;
@@ -346,7 +343,7 @@ search_gui_compare_records(gint sort_col,
 			if (r1->size == r2->size)
             	result = search_gui_cmp_sha1s(r1->sha1, r2->sha1);
 			else
-				result = (r1->size > r2->size) ? +1 : -1;
+				result = CMP(r1->size, r2->size);
             break;
 
         case c_sr_vendor:
@@ -371,9 +368,6 @@ search_gui_compare_records(gint sort_col,
             result = CMP(rs1->country, rs2->country);
             break;
 
-        case c_sr_meta:
-			break;				/* XXX Can't sort, metadata not in record! */
-			
         case c_sr_route:
 			result = host_addr_cmp(rs1->last_hop, rs2->last_hop);
 			break;
@@ -421,7 +415,7 @@ search_gui_compare_records(gint sort_col,
         case c_sr_sha1:
            	result = search_gui_cmp_sha1s(r1->sha1, r2->sha1);
 			break;
-			
+
         case c_sr_ctime:
            	result = delta_time(r1->create_time, r2->create_time);
 			break;
@@ -950,7 +944,7 @@ search_gui_sort_column(search_t *search, gint column)
     }
 
     /* create new arrow */
-    switch (search->sort_order) {
+    switch (search->sorting.s_order) {
     case SORT_ASC:
         search->arrow = create_pixmap(gui_main_window(), "arrow_down.xpm");
 		ascending = TRUE;
@@ -966,7 +960,7 @@ search_gui_sort_column(search_t *search, gint column)
     }
 
     /* display arrow if necessary and set sorting parameters*/
-    if (search->sort_order != SORT_NONE) {
+    if (search->sorting.s_order != SORT_NONE) {
         cw = gtk_clist_get_column_widget(GTK_CLIST(search->tree), column);
         if (cw != NULL) {
             gtk_box_pack_start(GTK_BOX(cw), search->arrow,
@@ -982,7 +976,6 @@ search_gui_sort_column(search_t *search, gint column)
     }
 }
 
-
 /**
  *	Adds the record to gth GtkCTree for this search.
  *	This is where the search grouping (parenting) is done
@@ -1030,7 +1023,7 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 				break;
 	 		case c_sr_vendor:
 				if (!(ST_LOCAL & rs->status))
-					text = vendor_get_name(rs->vendor);
+					text = vendor_code_get_name(rs->vendor);
 				break;
 	 		case c_sr_info:
 				text = rc->info;
@@ -1044,7 +1037,9 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 				break;
 			case c_sr_protocol:
 				if (!((ST_LOCAL | ST_BROWSE) & rs->status))
-					text = ST_UDP & rs->status ? "UDP" : "TCP";
+					text = ST_UDP & rs->status ?
+						(ST_SR_UDP & rs->status ? "UDP (semi-reliable)" : "UDP")
+						: "TCP";
 				break;
 			case c_sr_hops:
 				if (!((ST_LOCAL | ST_BROWSE) & rs->status)) {
@@ -1091,7 +1086,6 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 					text = timestamp_to_string(rc->create_time);
 				}
 				break;
-			case c_sr_meta:
 	 		case c_sr_size:
 	 		case c_sr_count:
 				break;
@@ -1107,15 +1101,18 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 
 	/* Add the search result to the ctree */
 
-	/* Record memory is freed automatically by function set later on using
+	/*
+	 * Record memory is freed automatically by function set later on using
 	 * gtk_ctree_node_set_row_data_full
 	 */
-	gui_rc = walloc(sizeof *gui_rc);
+
+	WALLOC(gui_rc);
 	gui_rc->shared_record = rc;
 
 	if (NULL != rc->sha1) {
 
-		/* We use the sch->parents hash table to store pointers to all the
+		/*
+		 * We use the sch->parents hash table to store pointers to all the
 		 * parent tree nodes referenced by their atomized sha1.
 		 */
 		parent = find_parent_with_sha1(sch, rc->sha1);
@@ -1126,7 +1123,7 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 			/* A parent exists with that sha1, add as child to that parent */
 			node = gtk_ctree_insert_node(ctree, parent, NULL,
 						(gchar **) titles, /* override const */
-						G_N_ELEMENTS(titles), NULL, NULL, NULL, NULL, 0, 0);
+						N_ITEMS(titles), NULL, NULL, NULL, NULL, 0, 0);
 
 			/* Update the "#" column of the parent, +1 for parent */
 			count = gtk_ctree_count_node_children(ctree, parent);
@@ -1149,7 +1146,7 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 			/* Add node as a parent */
 			node = gtk_ctree_insert_node(ctree, parent = NULL, NULL,
 						(gchar **) titles, /* override const */
-						G_N_ELEMENTS(titles), NULL, NULL, NULL, NULL, 0, 0);
+						N_ITEMS(titles), NULL, NULL, NULL, NULL, 0, 0);
 			add_parent_with_sha1(sch, key, node);
 
 			/* Update count in the records (use for column sorting) */
@@ -1162,7 +1159,7 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 
 		node = gtk_ctree_insert_node(ctree, parent = NULL, NULL,
 					(gchar **) titles, /* override */
-					G_N_ELEMENTS(titles), NULL, NULL, NULL, NULL, 0, 0);
+					N_ITEMS(titles), NULL, NULL, NULL, NULL, 0, 0);
 		/* Update count in the records (use for column sorting) */
 		gui_rc->num_children = 0;
 		is_parent = TRUE;
@@ -1187,7 +1184,7 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 		 * moving the parent will move the children too so we just pretend that
 		 * the parent node was actually the node that was added, not the child.
 		 */
-		if (!is_parent && (c_sr_count == sch->sort_col)) {
+		if (!is_parent && (c_sr_count == sch->sorting.s_column)) {
 			is_parent = TRUE;
 			parent_row = GTK_CTREE_ROW(node);
 			auto_node = parent_row->parent;
@@ -1219,13 +1216,19 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 				if (grc1 == grc2)
 					continue;
 
-				if (SORT_ASC == sch->sort_order) {
- 	            	if (search_gui_compare_records(sch->sort_col, grc1, grc2) < 0){
+				if (SORT_ASC == sch->sorting.s_order) {
+					if (
+						search_gui_compare_records(
+							sch->sorting.s_column, grc1, grc2) < 0
+					) {
 						sibling = cur_node;
 						break;
 					}
 				} else { /* SORT_DESC */
-					if (search_gui_compare_records(sch->sort_col, grc1, grc2) > 0){
+					if (
+						search_gui_compare_records(
+							sch->sorting.s_column, grc1, grc2) > 0
+					){
 						sibling = cur_node;
 						break;
 					}
@@ -1246,13 +1249,19 @@ search_gui_add_record(search_t *sch, record_t *rc, enum gui_color color)
 
 				grc2 = gtk_ctree_node_get_row_data(ctree, cur_node);
 
-				if (SORT_ASC == sch->sort_order) {
- 	            	if (search_gui_compare_records(sch->sort_col, grc1, grc2) < 0){
+				if (SORT_ASC == sch->sorting.s_order) {
+					if (
+						search_gui_compare_records(
+							sch->sorting.s_column, grc1, grc2) < 0
+					){
 						sibling = cur_node;
 						break;
 					}
 				} else { /* SORT_DESC */
-					if (search_gui_compare_records(sch->sort_col, grc1, grc2) > 0){
+					if (
+						search_gui_compare_records(
+							sch->sorting.s_column, grc1, grc2) > 0
+					){
 						sibling = cur_node;
 						break;
 					}
@@ -1397,8 +1406,8 @@ search_gui_remove_result(struct search *search, GtkCTreeNode *node)
 	 * One is a gui reference, the other is the dups hash table reference
 	 * (Note that GTK2 hashtables allow us to define an auto remove function,
 	 *  not so with GTK1, so we have to do it ourselves)
-	*/
-	g_hash_table_remove(search->dups, rc);
+	 */
+	hset_remove(search->dups, rc);
 	search_gui_unref_record(rc);
 	search_gui_unref_record(rc);
 }
@@ -1427,7 +1436,7 @@ search_resort_required(struct search *search, GtkCTreeNode *node)
 	 *     -- Richard, 17/04/2004
 	 */
 
-	if (c_sr_count != search->sort_col)
+	if (c_sr_count != search->sorting.s_column)
 		return FALSE;
 
 	row = GTK_CTREE_ROW(node);
@@ -1451,6 +1460,7 @@ search_resort_required(struct search *search, GtkCTreeNode *node)
 	}
 	return FALSE;
 #endif
+	(void) row;		/* #if 0 above */
 
 	return TRUE;
 }
@@ -1464,7 +1474,7 @@ selection_of_ctree(GtkCTree *ctree)
 {
 	GSList *nodes = NULL;
 	GList *iter;
-	
+
 	iter = GTK_CLIST(ctree)->selection;
 	for (/* NOTHING */; NULL != iter; iter = g_list_next(iter)) {
 		GtkCTreeNode *node;
@@ -1508,7 +1518,7 @@ discard_selection_of_ctree(struct search *search)
 
 	ctree = GTK_CTREE(search->tree);
 	gtk_clist_freeze(GTK_CLIST(ctree));
-	
+
 	gtk_signal_handler_block_by_func(GTK_OBJECT(ctree),
 		GTK_SIGNAL_FUNC(on_ctree_search_results_select_row), NULL);
 
@@ -1523,8 +1533,7 @@ discard_selection_of_ctree(struct search *search)
 		g_assert(node);
 		gtk_ctree_select(ctree, node);
 	}
-	g_slist_free(selection);
-	selection = NULL;
+	gm_slist_free_null(&selection);
 
 	while (GTK_CLIST(ctree)->selection) {
 		GtkCTreeNode *node;
@@ -1543,7 +1552,7 @@ discard_selection_of_ctree(struct search *search)
 	}
 
 	if (resort) {
-		search_gui_sort_column(search, search->sort_col);
+		search_gui_sort_column(search, search->sorting.s_column);
 	}
 
 	gtk_signal_handler_unblock_by_func(GTK_OBJECT(ctree),
@@ -1587,7 +1596,7 @@ download_selection_of_ctree(struct search *search)
 		grc = gtk_ctree_node_get_row_data(ctree, node);
 		g_assert(grc);
 
-		search_gui_download(grc->shared_record);
+		search_gui_download(grc->shared_record, search->search_handle);
 		n++;
 
         if (!remove_downloaded) {
@@ -1596,8 +1605,7 @@ download_selection_of_ctree(struct search *search)
 				gui_color_get(GUI_COLOR_DOWNLOADING));
         }
 	}
-	g_slist_free(selection);
-	selection = NULL;
+	gm_slist_free_null(&selection);
 
 	if (remove_downloaded) {
 		discard_selection_of_ctree(search);
@@ -1683,12 +1691,12 @@ on_search_list_row_move_event(GtkCList *clist,
  *** Public functions
  ***/
 
-void
+void G_COLD
 search_gui_init(void)
 {
 	{
 		GtkCList *clist = clist_search();
-		
+
 		gtk_clist_set_reorderable(clist, TRUE);
 		gtk_clist_set_selection_mode(clist, GTK_SELECTION_EXTENDED);
 		widget_add_popup_menu(GTK_WIDGET(clist),
@@ -1699,22 +1707,32 @@ search_gui_init(void)
 			"button-release-event", on_search_list_button_release_event, NULL);
 		gui_signal_connect(clist,
 			"key-release-event", on_search_list_key_release_event, NULL);
+		clist_restore_widths(clist, PROP_SEARCH_LIST_COL_WIDTHS);
 	}
 
 	{
 		GtkCList *clist;
-		
+
 		clist = GTK_CLIST(gui_main_window_lookup("clist_search_details"));
 		gtk_clist_set_selection_mode(clist, GTK_SELECTION_SINGLE);
 		gui_signal_connect(clist, "key-press-event",
 			on_search_details_key_press_event, NULL);
 		clipboard_attach(GTK_WIDGET(clist));
-		drag_attach(GTK_WIDGET(clist), search_gui_details_get_text);
+		drag_attach_text(GTK_WIDGET(clist), search_gui_details_get_text);
 	}
 
 	search_gui_common_init();
 }
 
+void G_COLD
+search_gui_shutdown(void)
+{
+	GtkCList *clist = clist_search();
+
+	clist_save_widths(clist, PROP_SEARCH_LIST_COL_WIDTHS);
+	search_gui_common_shutdown();
+}
+
 /**
  * Remove the search from the gui and update all widgets accordingly.
  */
@@ -1806,7 +1824,7 @@ search_gui_create_tree(void)
 	for (i = 0; i < c_sr_num; i++) {
 		GtkWidget *label, *hbox;
 		const gchar *title;
-	
+
 		title = search_gui_column_title(i);
 		label = gtk_label_new(title);
     	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
@@ -1819,7 +1837,7 @@ search_gui_create_tree(void)
 
 	clist_restore_visibility(ctree, PROP_SEARCH_RESULTS_COL_VISIBLE);
 	clist_restore_widths(ctree, PROP_SEARCH_RESULTS_COL_WIDTHS);
-	
+
 	gui_signal_connect(ctree,
 		"tree_select_row", on_ctree_search_results_select_row, NULL);
 	gui_signal_connect(ctree,
@@ -1892,94 +1910,29 @@ search_gui_collapse_all(struct search *search)
 	}
 }
 
-void
+bool
 search_gui_start_massive_update(struct search *search)
 {
-	g_return_if_fail(search);
+	g_return_val_if_fail(search, FALSE);
+
+	if (search->frozen)
+		return FALSE;
+
 	gtk_clist_freeze(GTK_CLIST(search->tree));
+	search->frozen = TRUE;
+	return TRUE;
 }
 
 void
 search_gui_end_massive_update(struct search *search)
 {
 	g_return_if_fail(search);
+	g_return_if_fail(search->frozen);
+
+	search->frozen = FALSE;
 	gtk_clist_thaw(GTK_CLIST(search->tree));
 }
 
-/**
- * Update the search displays with the correct meta-data
- *
- */
-void
-search_gui_metadata_update(const bitzi_data_t *data)
-{
-    const GList *iter;
-    gchar *text;
-
-	text = bitzi_gui_get_metadata(data);
-
-	/*
-	 * Fill in the columns in each search that contains a reference
-	 */
-
-	iter = search_gui_get_searches();
-	for (/* NOTHING */; NULL != iter; iter = g_list_next(iter)) {
-		search_t *search = iter->data;
-		GtkCTree *ctree = GTK_CTREE(search->tree);
-    	GtkCTreeNode *parent;
-
-		parent = find_parent_with_sha1(search, data->sha1);
-		if (parent) {
-			gui_record_t *grc;
-			record_t *record;
-
-			gtk_ctree_node_set_text(ctree, parent,
-					c_sr_meta, text ? text : _("Not in database"));
-			grc = gtk_ctree_node_get_row_data(ctree, parent);
-			record = grc->shared_record;
-			if (search_gui_item_is_inspected(record)) {
-				search_gui_set_bitzi_metadata(record);
-			}
-		}
-	}
-
-	G_FREE_NULL(text);
-}
-
-/**
- * Update the search displays with the correct meta-data.
- * (called from search_cb.c)
- */
-void
-search_gui_queue_bitzi_by_sha1(const record_t *rec)
-{
-	const GList *iter;
-	GtkCTreeNode *parent;
-
-	g_assert(rec != NULL);
-
-	if (!rec->sha1)
-		return;
-
-	/*
-	 * Add some feedback that a search has been kicked off.
-	 */
-
-	iter = search_gui_get_searches();
-	for (/* NOTHING */; NULL != iter; iter = g_list_next(iter)) {
-		search_t *search = iter->data;
-		GtkCTree *ctree = GTK_CTREE(search->tree);
-
-		parent = find_parent_with_sha1(search, rec->sha1);
-		if (parent)
-			gtk_ctree_node_set_text(ctree, parent, c_sr_meta,
-					_("Query queued..."));
-	}
-
-	/* and then send the query... */
-	guc_query_bitzi_by_sha1(rec->sha1, rec->size, TRUE);
-}
-
 record_t *
 search_gui_record_get_parent(search_t *search, record_t *record)
 {
@@ -1990,7 +1943,7 @@ search_gui_record_get_parent(search_t *search, record_t *record)
 	if (record->sha1) {
 		GtkCTreeNode *parent;
 		gui_record_t *grc;
-	
+
 		parent = find_parent_with_sha1(search, record->sha1);
 		if (parent) {
 			grc = gtk_ctree_node_get_row_data(GTK_CTREE(search->tree), parent);
@@ -2040,7 +1993,7 @@ search_gui_get_selected_searches(void)
 	for (/* NOTHING */; selection; selection = g_list_next(selection)) {
     	search_t *search;
 		gint row;
-		
+
 		row = GPOINTER_TO_UINT(selection->data);
 		if (row < 0)
 			break;
@@ -2067,12 +2020,12 @@ search_gui_search_list_clicked(void)
 	int row;
 
 	row = clist_get_cursor_row(clist_search());
-	if (row >= 0) { 
+	if (row >= 0) {
 		search_t *search;
 
 		search = gtk_clist_get_row_data(clist_search(), row);
 		g_return_if_fail(search);
-		
+
 		search_gui_set_current_search(search);
 	}
 }
