@@ -1,6 +1,4 @@
 /*
- * $Id: fileinfo.c 16053 2009-03-03 16:56:12Z rmanfredi $
- *
  * Copyright (c) 2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -27,7 +25,7 @@
  * @ingroup gtk
  * @file
  *
- * Displaying of file information in the GUI.
+ * Displaying of file information in the GUI ("Downloads" pane).
  *
  * @author Richard Eckart
  * @date 2003
@@ -35,21 +33,23 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: fileinfo.c 16053 2009-03-03 16:56:12Z rmanfredi $")
-
 #include "gtk/columns.h"
 #include "gtk/downloads_common.h"
 #include "gtk/drag.h"
 #include "gtk/misc.h"
 
+#include "column_sort.h"
+
 #include "if/gui_property.h"
 
-#include "lib/walloc.h"
+#include "lib/entropy.h"
+#include "lib/htable.h"
 #include "lib/utf8.h"
+#include "lib/walloc.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
-static GHashTable *fi_sources;
+static htable_t *fi_sources;
 
 static GtkTreeView *treeview_download_aliases;
 static GtkTreeView *treeview_download_details;
@@ -60,18 +60,46 @@ static GtkListStore *store_aliases;
 static GtkListStore *store_files;
 static GtkListStore *store_sources;
 
+enum downloads_info_nb_page {
+	DOWNLOADS_INFO_NB_PAGE_DETAILS,
+	DOWNLOADS_INFO_NB_PAGE_ALIASES,
+	DOWNLOADS_INFO_NB_PAGE_SOURCES,
+	DOWNLOADS_INFO_NB_PAGE_TOOLS,
+	DOWNLOADS_INFO_NB_PAGE_SETTINGS,
+
+	NUM_DOWNLOADS_INFO_NB_PAGES
+};
+
+static enum downloads_info_nb_page notebook_downloads_info_current_page;
+
 #if GTK_CHECK_VERSION(2,6,0)
-static GtkSortType files_sort_order;
-static int files_sort_column;
+static GtkSortType files_sort_type;
+static struct sorting_context files_sort;
 static int files_sort_depth;
 #endif	/* Gtk+ => 2.6.0 */
 
+static void
+on_notebook_downloads_info_switch_page(GtkNotebook *unused_notebook,
+	GtkNotebookPage *unused_page, int page_num, void *unused_udata)
+{
+	(void) unused_notebook;
+	(void) unused_page;
+	(void) unused_udata;
+
+	g_return_if_fail(UNSIGNED(page_num) < NUM_DOWNLOADS_INFO_NB_PAGES);
+
+	entropy_harvest_single(VARLEN(page_num));
+
+	notebook_downloads_info_current_page = page_num;
+	gui_prop_set_guint32_val(PROP_DOWNLOADS_INFO_NOTEBOOK_TAB, page_num);
+}
+
 static void
 fi_gui_files_sort_reset(void)
 {
 #if GTK_CHECK_VERSION(2,6,0)
-	files_sort_column = GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID;
-	files_sort_order = GTK_SORT_ASCENDING;
+	files_sort.s_column = GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID;
+	files_sort_type = GTK_SORT_ASCENDING;
 	files_sort_depth = 0;
 #endif	/* Gtk+ => 2.6.0 */
 }
@@ -87,8 +115,8 @@ fi_gui_files_sort_save(void)
 
 		sortable = GTK_TREE_SORTABLE(store_files);
 		if (gtk_tree_sortable_get_sort_column_id(sortable, &column, &order)) {
-			files_sort_column = column;
-			files_sort_order = order;
+			files_sort.s_column = column;
+			files_sort_type = order;
 			gtk_tree_sortable_set_sort_column_id(sortable,
 					GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, order);
 		}
@@ -104,9 +132,9 @@ fi_gui_files_sort_restore(void)
 	files_sort_depth--;
 
 	if (0 == files_sort_depth) {
-		if (GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID != files_sort_column) {
+		if (GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID != files_sort.s_column) {
 			gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store_files),
-					files_sort_column, files_sort_order);
+					files_sort.s_column, files_sort_type);
 		}
 	}
 #endif	/* Gtk+ => 2.6.0 */
@@ -130,7 +158,7 @@ fi_gui_file_invalidate(struct fileinfo_data *file)
 	GtkTreeIter *iter = fileinfo_data_get_iter(file);
 	if (iter) {
 		fileinfo_data_set_iter(file, NULL);
-		WFREE_NULL(iter, sizeof *iter);
+		WFREE_TYPE_NULL(iter);
 	}
 }
 
@@ -144,7 +172,7 @@ fi_gui_file_show(struct fileinfo_data *file)
 
 	iter = fileinfo_data_get_iter(file);
 	if (!iter) {
-		iter = walloc(sizeof *iter);
+		WALLOC(iter);
 		fileinfo_data_set_iter(file, iter);
 		list_store_append_pointer(store_files, iter, 0, file);
 	} else {
@@ -206,10 +234,10 @@ render_files(GtkTreeViewColumn *column, GtkCellRenderer *cell,
 	idx = pointer_to_uint(udata);
 	if (c_fi_progress == idx) {
 		unsigned value = fi_gui_file_get_progress(file);
-		g_object_set(cell, "value", value, (void *) 0);
+		g_object_set(cell, "value", value, NULL_PTR);
 	} else {
 		const char *text = fi_gui_file_column_text(file, idx);
-		g_object_set(cell, "text", text, (void *) 0);
+		g_object_set(cell, "text", text, NULL_PTR);
 	}
 }
 
@@ -231,10 +259,10 @@ render_sources(GtkTreeViewColumn *column, GtkCellRenderer *cell,
 	idx = pointer_to_uint(udata);
 	if (c_src_progress == idx) {
 		unsigned value = fi_gui_source_get_progress(d);
-		g_object_set(cell, "value", value, (void *) 0);
+		g_object_set(cell, "value", value, NULL_PTR);
 	} else {
 		const char *text = fi_gui_source_column_text(d, idx);
-		g_object_set(cell, "text", text, (void *) 0);
+		g_object_set(cell, "text", text, NULL_PTR);
 	}
 }
 
@@ -242,7 +270,7 @@ static GtkCellRenderer *
 create_text_cell_renderer(gfloat xalign)
 {
 	GtkCellRenderer *renderer;
-	
+
 	renderer = gtk_cell_renderer_text_new();
 	gtk_cell_renderer_text_set_fixed_height_from_font(
 		GTK_CELL_RENDERER_TEXT(renderer), 1);
@@ -250,13 +278,13 @@ create_text_cell_renderer(gfloat xalign)
 		"mode",		GTK_CELL_RENDERER_MODE_INERT,
 		"xalign",	xalign,
 		"ypad",		(unsigned) GUI_CELL_RENDERER_YPAD,
-		(void *) 0);
+		NULL_PTR);
 
 	return renderer;
 }
 
-static gboolean
-fi_sources_remove(void *unused_key, void *value, void *unused_udata)
+static bool
+fi_sources_remove(const void *unused_key, void *value, void *unused_udata)
 {
 	GtkTreeIter *iter;
 
@@ -265,7 +293,7 @@ fi_sources_remove(void *unused_key, void *value, void *unused_udata)
 	(void) unused_udata;
 
 	iter = value;
-	wfree(iter, sizeof *iter);
+	WFREE(iter);
 	return TRUE; /* Remove the handle from the hashtable */
 }
 
@@ -279,7 +307,7 @@ void
 fi_gui_clear_sources(void)
 {
     gtk_list_store_clear(store_sources);
-	g_hash_table_foreach_remove(fi_sources, fi_sources_remove, NULL);
+	htable_foreach_remove(fi_sources, fi_sources_remove, NULL);
 }
 
 void
@@ -300,16 +328,23 @@ fi_gui_show_aliases(const char * const *aliases)
 	}
 }
 
+void
+fi_gui_source_massive_update(bool starting)
+{
+	(void) starting;
+	/* Nothing to do */
+}
+
 void
 fi_gui_source_show(struct download *d)
 {
 	GtkTreeIter *iter;
 
 	g_return_if_fail(store_sources);
-	g_return_if_fail(NULL == g_hash_table_lookup(fi_sources, d));
+	g_return_if_fail(!htable_contains(fi_sources, d));
 
-	iter = walloc(sizeof *iter);
-	g_hash_table_insert(fi_sources, d, iter);
+	WALLOC(iter);
+	htable_insert(fi_sources, d, iter);
 
 	list_store_append_pointer(store_sources, iter, 0, d);
 }
@@ -451,7 +486,7 @@ fi_gui_source_update(struct download *d)
 
 	download_check(d);
 
-	iter = g_hash_table_lookup(fi_sources, d);
+	iter = htable_lookup(fi_sources, d);
 	if (iter) {
 		tree_model_iter_changed(GTK_TREE_MODEL(store_sources), iter);
 	}
@@ -476,8 +511,8 @@ create_column(int column_id, const char *title, gfloat xalign,
 		renderer = create_text_cell_renderer(xalign);
 	}
 
-	column = gtk_tree_view_column_new_with_attributes(title,
-				renderer, (void *) 0);
+	column =
+		gtk_tree_view_column_new_with_attributes(title, renderer, NULL_PTR);
 	gtk_tree_view_column_set_cell_data_func(column, renderer,
 		cell_data_func, uint_to_pointer(column_id), NULL);
 	return column;
@@ -492,7 +527,7 @@ configure_column(GtkTreeViewColumn *column)
 		"reorderable", FALSE,
 		"resizable", TRUE,
 		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-		(void *) 0);
+		NULL_PTR);
 }
 
 static GtkTreeViewColumn *
@@ -505,6 +540,9 @@ add_column(GtkTreeView *tv, int column_id, const char *title, gfloat xalign,
 	configure_column(column);
 	gtk_tree_view_column_set_sort_column_id(column, column_id);
     gtk_tree_view_append_column(tv, column);
+
+	gui_column_map(column, tv);	/* Capture resize events */
+
 	return column;
 }
 
@@ -575,23 +613,23 @@ fi_gui_details_treeview_init(void)
 	g_return_if_fail(tv);
 	treeview_download_details = tv;
 
-	model = GTK_TREE_MODEL(gtk_list_store_new(G_N_ELEMENTS(tab),
+	model = GTK_TREE_MODEL(gtk_list_store_new(N_ITEMS(tab),
 				G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING));
 
 	gtk_tree_view_set_model(tv, model);
 	g_object_unref(model);
 
-	for (i = 0; i < G_N_ELEMENTS(tab); i++) {
+	for (i = 0; i < N_ITEMS(tab); i++) {
     	GtkTreeViewColumn *column;
 		GtkCellRenderer *renderer;
-		
+
 		renderer = create_text_cell_renderer(tab[i].xalign);
 		g_object_set(G_OBJECT(renderer),
 			"editable", tab[i].editable,
-			(void *) 0);
+			NULL_PTR);
 		gui_signal_connect(renderer, "edited", on_cell_edited, tv);
 		column = gtk_tree_view_column_new_with_attributes(tab[i].title,
-					renderer, "text", i, (void *) 0);
+					renderer, "text", i, NULL_PTR);
 		g_object_set(column,
 			"visible",	i > 0 ? TRUE : FALSE,
 			"min-width", 1,
@@ -599,11 +637,11 @@ fi_gui_details_treeview_init(void)
 			"sizing", 1 == i
 						? GTK_TREE_VIEW_COLUMN_AUTOSIZE
 						: GTK_TREE_VIEW_COLUMN_FIXED,
-			(void *) 0);
+			NULL_PTR);
     	gtk_tree_view_append_column(tv, column);
 	}
 
-	drag_attach(GTK_WIDGET(tv), download_details_get_text);
+	drag_attach_text(GTK_WIDGET(tv), download_details_get_text);
 }
 
 static void
@@ -624,6 +662,17 @@ store_files_init(void)
 	}
 }
 
+/**
+ * Enforce a tri-state sorting.
+ */
+static void
+on_fileinfo_treeview_column_clicked(GtkTreeViewColumn *column, void *udata)
+{
+	(void) udata;
+
+	 column_sort_tristate(column, &files_sort);
+}
+
 static void
 treeview_download_files_init(void)
 {
@@ -635,12 +684,19 @@ treeview_download_files_init(void)
 	tv = GTK_TREE_VIEW(gtk_tree_view_new());
 	treeview_download_files = tv;
 
+	gui_parent_widths_saveto(tv, PROP_FILE_INFO_COL_WIDTHS);
+
 	for (i = 0; i < c_fi_num; i++) {
-		add_column(tv, i,
+		GtkTreeViewColumn *column;
+
+		column = add_column(tv, i,
 			fi_gui_files_column_title(i),
 			fi_gui_files_column_justify_right(i) ? 1.0 : 0.0,
 			c_fi_progress == i ? gtk_cell_renderer_progress_new() : NULL,
 			render_files);
+
+		column_sort_tristate_register(column,
+			on_fileinfo_treeview_column_clicked, NULL);
 	}
 
 	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(tv),
@@ -697,10 +753,9 @@ void
 fi_gui_files_widget_destroy(void)
 {
 	if (treeview_download_files) {
+		gui_parent_forget(treeview_download_files);
 		tree_view_save_visibility(treeview_download_files,
 			PROP_FILE_INFO_COL_VISIBLE);
-		tree_view_save_widths(treeview_download_files,
-			PROP_FILE_INFO_COL_WIDTHS);
 		gtk_widget_destroy(GTK_WIDGET(treeview_download_files));
 		treeview_download_files = NULL;
 	}
@@ -709,8 +764,8 @@ fi_gui_files_widget_destroy(void)
 void
 fi_gui_init(void)
 {
-	fi_sources = g_hash_table_new(NULL, NULL);
-	
+	fi_sources = htable_create(HASH_KEY_SELF, 0);
+
 	{
 		GtkTreeViewColumn *column;
 		GtkTreeView *tv;
@@ -724,13 +779,13 @@ fi_gui_init(void)
 		column = gtk_tree_view_column_new_with_attributes(_("Aliases"),
 					create_text_cell_renderer(0.0),
 					"text", 0,
-					(void *) 0);
+					NULL_PTR);
 		configure_column(column);
 		gtk_tree_view_column_set_sort_column_id(column, 0);
     	gtk_tree_view_append_column(tv, column);
 
 		tree_view_set_fixed_height_mode(tv, TRUE);
-		drag_attach(GTK_WIDGET(tv), fi_gui_get_alias);
+		drag_attach_text(GTK_WIDGET(tv), fi_gui_get_alias);
 	}
 
 	{
@@ -748,21 +803,23 @@ fi_gui_init(void)
 		GtkTreeView *tv;
 		unsigned i;
 
-		STATIC_ASSERT(c_src_num == G_N_ELEMENTS(tab));
-		
+		STATIC_ASSERT(c_src_num == N_ITEMS(tab));
+
 		tv = GTK_TREE_VIEW(gui_main_window_lookup("treeview_download_sources"));
 		treeview_download_sources = tv;
 
 		store_sources = gtk_list_store_new(1, G_TYPE_POINTER);
 		gtk_tree_view_set_model(tv, GTK_TREE_MODEL(store_sources));
 
-		for (i = 0; i < G_N_ELEMENTS(tab); i++) {
+		gui_parent_widths_saveto(tv, PROP_SOURCES_COL_WIDTHS);
+
+		for (i = 0; i < N_ITEMS(tab); i++) {
 			GtkCellRenderer *renderer;
 
 			renderer = tab[i].id == c_src_progress
 						? gtk_cell_renderer_progress_new()
 						: NULL;
-    		add_column(tv, tab[i].id, _(tab[i].title), 0.0,
+			(void) add_column(tv, tab[i].id, _(tab[i].title), 0.0,
 				renderer, render_sources);
 		}
 
@@ -772,11 +829,28 @@ fi_gui_init(void)
 		tree_view_restore_widths(tv, PROP_SOURCES_COL_WIDTHS);
 		tree_view_set_fixed_height_mode(tv, TRUE);
 
-		widget_add_popup_menu(GTK_WIDGET(tv), fi_gui_sources_get_popup_menu);	
+		widget_add_popup_menu(GTK_WIDGET(tv), fi_gui_sources_get_popup_menu);
 	}
 
 	fi_gui_details_treeview_init();
 	fi_gui_common_init();
+
+	{
+		uint32 page;
+		GtkNotebook *notebook_downloads_info =
+			GTK_NOTEBOOK(gui_main_window_lookup("notebook_downloads_info"));
+
+		gui_prop_get_guint32_val(PROP_DOWNLOADS_INFO_NOTEBOOK_TAB, &page);
+
+		if (page >= NUM_DOWNLOADS_INFO_NB_PAGES)
+			page = DOWNLOADS_INFO_NB_PAGE_DETAILS;
+
+		notebook_downloads_info_current_page = page;
+		gtk_notebook_set_current_page(notebook_downloads_info, page);
+
+		gui_signal_connect(notebook_downloads_info,
+			"switch-page", on_notebook_downloads_info_switch_page, NULL);
+	}
 }
 
 void
@@ -784,10 +858,6 @@ fi_gui_shutdown(void)
 {
 	tree_view_save_visibility(treeview_download_files,
 		PROP_FILE_INFO_COL_VISIBLE);
-	tree_view_save_widths(treeview_download_files,
-		PROP_FILE_INFO_COL_WIDTHS);
-	tree_view_save_widths(treeview_download_sources,
-		PROP_SOURCES_COL_WIDTHS);
 
 	fi_gui_common_shutdown();
 
@@ -816,8 +886,7 @@ fi_gui_shutdown(void)
 		store_sources = NULL;
 	}
 
-	g_hash_table_destroy(fi_sources);
-	fi_sources = NULL;
+	htable_free_null(&fi_sources);
 }
 
 void
@@ -825,13 +894,13 @@ fi_gui_source_hide(struct download *d)
 {
 	GtkTreeIter *iter;
 
-	iter = g_hash_table_lookup(fi_sources, d);
+	iter = htable_lookup(fi_sources, d);
 	if (iter) {
 		if (store_sources) {
 			gtk_list_store_remove(store_sources, iter);
 		}
-		g_hash_table_remove(fi_sources, d);
-		wfree(iter, sizeof *iter);
+		htable_remove(fi_sources, d);
+		WFREE(iter);
 	}
 }
 
