@@ -1,6 +1,4 @@
 /*
- * $Id: hcache.c 14389 2007-08-09 22:10:53Z cbiere $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,14 +17,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
 #include "gtk/gui.h"
 
-RCSID("$Id: hcache.c 14389 2007-08-09 22:10:53Z cbiere $")
-
 #include "gtk/hcache.h"
 #include "gtk/columns.h"
 #include "gtk/misc.h"
@@ -65,18 +61,20 @@ add_column(GtkTreeView *treeview,
 	g_object_set(renderer,
 		"xalign", xalign,
 		"ypad", GUI_CELL_RENDERER_YPAD,
-		(void *) 0);
+		NULL_PTR);
 	column = gtk_tree_view_column_new_with_attributes(label, renderer,
 		"text", column_id,
-		(void *) 0);
+		NULL_PTR);
 	g_object_set(column,
 		"fixed-width", 100,
 		"min-width", 1,
 		"reorderable", TRUE,
 		"resizable", TRUE,
 		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-		(void *) 0);
+		NULL_PTR);
 	gtk_tree_view_append_column(treeview, column);
+
+	gui_column_map(column, treeview);	/* Capture resize events */
 }
 
 /***
@@ -89,7 +87,7 @@ hcache_gui_init(void)
     GtkTreeModel *model;
     gint n;
 
-	STATIC_ASSERT(G_N_ELEMENTS(hcache_col_labels) ==
+	STATIC_ASSERT(N_ITEMS(hcache_col_labels) ==
 		HCACHE_STATS_VISIBLE_COLUMNS);
 
     treeview_hcache = GTK_TREE_VIEW(gui_main_window_lookup("treeview_hcache"));
@@ -112,7 +110,9 @@ hcache_gui_init(void)
             (-1));
 	}
 
-	for (n = 0; (guint) n < G_N_ELEMENTS(hcache_col_labels); n++) {
+	gui_parent_widths_saveto(treeview_hcache, PROP_HCACHE_COL_WIDTHS);
+
+	for (n = 0; (guint) n < N_ITEMS(hcache_col_labels); n++) {
 		add_column(treeview_hcache, n, (gfloat) (n != 0),
 			_(hcache_col_labels[n].text));
 	}
@@ -127,7 +127,7 @@ hcache_gui_init(void)
 void
 hcache_gui_shutdown(void)
 {
-    tree_view_save_widths(treeview_hcache, PROP_HCACHE_COL_WIDTHS);
+	/* Nothing to do now */
 }
 
 void
