@@ -1,6 +1,4 @@
 /*
- * $Id: uploads.c 15242 2008-04-22 23:10:56Z cbiere $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -27,7 +25,7 @@
  * @ingroup gtk
  * @file
  *
- * Needs short description here.
+ * Management of the GTK2 "Uploads" pane.
  *
  * @author Richard Eckart
  * @date 2001-2003
@@ -41,19 +39,22 @@
 #include "gtk/misc.h"
 #include "gtk/settings.h"
 
+#include "column_sort.h"
+
 #include "if/gui_property.h"
 #include "if/bridge/ui2c.h"
 
 #include "lib/atoms.h"
+#include "lib/cstr.h"
 #include "lib/host_addr.h"
-#include "lib/glib-missing.h"
+#include "lib/htable.h"
 #include "lib/iso3166.h"
 #include "lib/misc.h"
+#include "lib/str.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
-#include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: uploads.c 15242 2008-04-22 23:10:56Z cbiere $")
+#include "lib/override.h"		/* Must be the last header included */
 
 static gboolean uploads_remove_lock;
 static gboolean uploads_shutting_down;
@@ -63,12 +64,16 @@ static GtkListStore *store_uploads;
 static GtkWidget *button_uploads_clear_completed;
 
 /** hash table for fast handle -> GtkTreeIter mapping */
-static GHashTable *upload_handles;
+static htable_t *upload_handles;
 /** list of all *removed* uploads; contains the handles */
 static GSList *sl_removed_uploads;
 
+#if GTK_CHECK_VERSION(2,6,0)
+static struct sorting_context uploads_sort;
+#endif	/* GTK+ >= 2.6.0 */
+
 static void uploads_gui_update_upload_info(const gnet_upload_info_t *u);
-static void uploads_gui_add_upload(gnet_upload_info_t *u);
+static void uploads_gui_add_upload(const gnet_upload_info_t *u);
 
 static const char * const column_titles[UPLOADS_GUI_VISIBLE_COLUMNS] = {
 	N_("Filename"),
@@ -99,10 +104,10 @@ static inline upload_row_data_t *
 find_upload(gnet_upload_t u)
 {
 	upload_row_data_t *rd;
-	gpointer key;
-	gboolean found;
+	void *key;
+	bool found;
 
-	found = g_hash_table_lookup_extended(upload_handles, GUINT_TO_POINTER(u),
+	found = htable_lookup_extended(upload_handles, uint_to_pointer(u),
 				NULL, &key);
 	g_assert(found);
 	rd = key;
@@ -155,7 +160,7 @@ upload_removed(gnet_upload_t uh, const gchar *reason)
 	if (reason != NULL)
 		gtk_list_store_set(store_uploads, &rd->iter, c_ul_status, reason, (-1));
 	sl_removed_uploads = g_slist_prepend(sl_removed_uploads, rd);
-	g_hash_table_remove(upload_handles, GUINT_TO_POINTER(uh));
+	htable_remove(upload_handles, uint_to_pointer(uh));
 	/* NB: rd MUST NOT be freed yet because it contains the GtkTreeIter! */
 }
 
@@ -243,11 +248,12 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 	rd = find_upload(u->upload_handle);
 	g_assert(NULL != rd);
 
-	rd->last_update  = tm_time();
+	rd->last_update = tm_time();
+	rd->send_date = u->start_date;	/* Updated by "core" when header sent back */
 
 	if (
-		!host_addr_equal(u->addr, rd->addr) ||
-		!host_addr_equal(u->gnet_addr, rd->gnet_addr) ||
+		!host_addr_equiv(u->addr, rd->addr) ||
+		!host_addr_equiv(u->gnet_addr, rd->gnet_addr) ||
 		u->gnet_port != rd->gnet_port
 	) {
 		rd->addr = u->addr;
@@ -259,25 +265,31 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 	}
 
 	if (u->range_start != rd->range_start || u->range_end != rd->range_end) {
-		static gchar str[256];
+		static char str[256];
 
 		rd->range_start  = u->range_start;
 		rd->range_end  = u->range_end;
 
 		if (u->range_start == 0 && u->range_end == 0)
-			g_strlcpy(str, "...", sizeof str);
+			cstr_lcpy(ARYLEN(str), "...");
 		else {
-			range_len = gm_snprintf(str, sizeof str, "%s%s",
+			char pct[20];
+
+			if (u->partial && u->file_size != 0) {
+				str_bprintf(ARYLEN(pct), " (%.2f%%)",
+					u->available * 100.0 / u->file_size);
+			} else {
+				pct[0] = '\0';
+			}
+			range_len = str_bprintf(ARYLEN(str), "%s%s%s",
 				short_size(u->range_end - u->range_start + 1,
 					show_metric_units()),
-				u->partial ? _(" (partial)") : "");
+				u->shrunk_chunk ? "-" : "", pct);
 
 			if ((guint) range_len < sizeof str) {
 				if (u->range_start)
-					range_len += gm_snprintf(&str[range_len],
-									sizeof str - range_len,
-									" @ %s", short_size(u->range_start,
-												show_metric_units()));
+					range_len += str_bprintf(ARYPOSLEN(str, range_len),
+						" @ %s", short_size(u->range_start, show_metric_units()));
 				g_assert((guint) range_len < sizeof str);
 			}
 		}
@@ -333,22 +345,21 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 	}
 }
 
-
-
 /**
  * Adds the given upload to the gui.
  */
 void
-uploads_gui_add_upload(gnet_upload_info_t *u)
+uploads_gui_add_upload(const gnet_upload_info_t *u)
 {
 	gint range_len, progress;
 	const gchar *titles[UPLOADS_GUI_VISIBLE_COLUMNS];
-    upload_row_data_t *rd = walloc(sizeof *rd);
+    upload_row_data_t *rd;
 	gnet_upload_status_t status;
 	static gchar size_tmp[256];
 
-	memset(titles, 0, sizeof titles);
+	ZERO(&titles);
 
+	WALLOC0(rd);
     rd->handle      = u->upload_handle;
     rd->range_start = u->range_start;
     rd->range_end   = u->range_end;
@@ -371,15 +382,20 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
         titles[c_ul_range] =  "...";
     else {
 		static gchar range_tmp[256];	/* MUST be static! */
+		char pct[20];
 
-        range_len = gm_snprintf(range_tmp, sizeof range_tmp, "%s%s",
-            short_size(u->range_end - u->range_start + 1,
-				show_metric_units()),
-			u->partial ? _(" (partial)") : "");
+		if (u->partial && u->file_size != 0) {
+			str_bprintf(ARYLEN(pct), " (%.2f%%)",
+				u->available * 100.0 / u->file_size);
+		} else {
+			pct[0] = '\0';
+		}
+        range_len = str_bprintf(ARYLEN(range_tmp), "%s%s%s",
+            short_size(u->range_end - u->range_start + 1, show_metric_units()),
+			u->shrunk_chunk ? "-" : "", pct);
 
         if (u->range_start)
-            range_len += gm_snprintf(
-                &range_tmp[range_len], sizeof range_tmp - range_len,
+            range_len += str_bprintf(ARYPOSLEN(range_tmp, range_len),
                 " @ %s", short_size(u->range_start, show_metric_units()));
 
         g_assert((guint) range_len < sizeof range_tmp);
@@ -387,8 +403,7 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
         titles[c_ul_range] = range_tmp;
     }
 
-	g_strlcpy(size_tmp, short_size(u->file_size, show_metric_units()),
-		sizeof size_tmp);
+	cstr_bcpy(ARYLEN(size_tmp), short_size(u->file_size, show_metric_units()));
     titles[c_ul_size] = size_tmp;
 
    	titles[c_ul_agent] = u->user_agent ? u->user_agent : "...";
@@ -411,10 +426,10 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
 		c_ul_fg, NULL,
 		c_ul_data, rd,
 		(-1));
-	g_hash_table_insert(upload_handles, GUINT_TO_POINTER(rd->handle), rd);
+	htable_insert(upload_handles, uint_to_pointer(rd->handle), rd);
 }
 
-static void
+static GtkTreeViewColumn *
 add_column(gint column_id, GtkTreeIterCompareFunc sortfunc, GtkType column_type)
 {
 	GtkTreeViewColumn *column;
@@ -439,19 +454,19 @@ add_column(gint column_id, GtkTreeIterCompareFunc sortfunc, GtkType column_type)
 			GTK_CELL_RENDERER_TEXT(renderer), 1);
 		g_object_set(renderer,
 			"foreground-set", TRUE,
-			(void *) 0);
+			NULL_PTR);
 		column = gtk_tree_view_column_new_with_attributes(
 					_(column_titles[column_id]), renderer,
 					"foreground-gdk", c_ul_fg,
 					"text", column_id,
-					(void *) 0);
+					NULL_PTR);
 	}
 
 	g_object_set(renderer,
 		"xalign", (gfloat) 0.0,
 		"xpad", xpad,
 		"ypad", GUI_CELL_RENDERER_YPAD,
-		(void *) 0);
+		NULL_PTR);
 
 	g_object_set(G_OBJECT(column),
 		"fixed-width", 1,
@@ -459,7 +474,7 @@ add_column(gint column_id, GtkTreeIterCompareFunc sortfunc, GtkType column_type)
 		"reorderable", TRUE,
 		"resizable", TRUE,
 		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
-		(void *) 0);
+		NULL_PTR);
 
 	gtk_tree_view_column_set_sort_column_id(column, column_id);
 	gtk_tree_view_append_column(treeview_uploads, column);
@@ -467,6 +482,8 @@ add_column(gint column_id, GtkTreeIterCompareFunc sortfunc, GtkType column_type)
 	if (NULL != sortfunc)
 		gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store_uploads),
 			column_id, sortfunc, GINT_TO_POINTER(column_id), NULL);
+
+	return column;
 }
 
 static GtkListStore *
@@ -476,9 +493,9 @@ create_uploads_model(void)
 	GtkListStore *store;
 	guint i;
 
-	STATIC_ASSERT(c_ul_num == G_N_ELEMENTS(columns));
+	STATIC_ASSERT(c_ul_num == N_ITEMS(columns));
 #define SET(c, x) case (c): columns[i] = (x); break
-	for (i = 0; i < G_N_ELEMENTS(columns); i++) {
+	for (i = 0; i < N_ITEMS(columns); i++) {
 		switch (i) {
 		SET(c_ul_filename, G_TYPE_STRING);
 		SET(c_ul_host, G_TYPE_STRING);
@@ -496,7 +513,7 @@ create_uploads_model(void)
 	}
 #undef SET
 
-	store = gtk_list_store_newv(G_N_ELEMENTS(columns), columns);
+	store = gtk_list_store_newv(N_ITEMS(columns), columns);
 	return GTK_LIST_STORE(store);
 }
 
@@ -505,12 +522,11 @@ free_row_data(upload_row_data_t *rd)
 {
 	atom_str_free_null(&rd->user_agent);
 	atom_str_free_null(&rd->name);
-	wfree(rd, sizeof *rd);
+	WFREE(rd);
 }
 
 static inline void
-free_handle(gpointer key, gpointer value,
-	gpointer user_data)
+free_handle(const void *key, void *value, void *user_data)
 {
 	(void) key;
 	(void) user_data;
@@ -530,16 +546,16 @@ remove_row(upload_row_data_t *rd, remove_row_ctx_t *ctx)
 }
 
 static inline void
-update_row(gpointer key, gpointer data, gpointer unused_udata)
+update_row(const void *key, void *value, void *unused_udata)
 {
 	time_t now;
-	upload_row_data_t *rd = data;
+	upload_row_data_t *rd = value;
 	gnet_upload_status_t status;
 	guint progress;
 
 	(void) unused_udata;
 	g_assert(NULL != rd);
-	g_assert(GPOINTER_TO_UINT(key) == rd->handle);
+	g_assert(pointer_to_uint(key) == rd->handle);
 
 	now = tm_time();
 	if (delta_time(now, rd->last_update) < 2)
@@ -578,7 +594,7 @@ uploads_gui_update_display(time_t now)
 	sl_removed_uploads = ctx.sl_remaining;
 
 	/* Update the status column for all active uploads. */
-	g_hash_table_foreach(upload_handles, update_row, NULL);
+	htable_foreach(upload_handles, update_row, NULL);
 	g_object_thaw_notify(G_OBJECT(treeview_uploads));
 
 	gtk_widget_set_sensitive(button_uploads_clear_completed,
@@ -648,6 +664,17 @@ uploads_gui_clear_completed(void)
 	}
 }
 
+/**
+ * Enforce a tri-state sorting.
+ */
+static void
+on_uploads_treeview_column_clicked(GtkTreeViewColumn *column, void *udata)
+{
+	(void) udata;
+
+	column_sort_tristate(column, &uploads_sort);
+}
+
 void
 uploads_gui_init(void)
 {
@@ -666,7 +693,7 @@ uploads_gui_init(void)
 	};
 	size_t i;
 
-	STATIC_ASSERT(G_N_ELEMENTS(cols) == UPLOADS_GUI_VISIBLE_COLUMNS);
+	STATIC_ASSERT(N_ITEMS(cols) == UPLOADS_GUI_VISIBLE_COLUMNS);
 	store_uploads = create_uploads_model();
 
 	button_uploads_clear_completed =
@@ -676,16 +703,26 @@ uploads_gui_init(void)
 	gtk_tree_view_set_model(treeview_uploads, GTK_TREE_MODEL(store_uploads));
 	tree_view_set_fixed_height_mode(treeview_uploads, TRUE);
 
-	for (i = 0; i < G_N_ELEMENTS(cols); i++) {
-		add_column(cols[i].id, cols[i].sortfunc,
+	gui_parent_widths_saveto(treeview_uploads, PROP_UPLOADS_COL_WIDTHS);
+
+	for (i = 0; i < N_ITEMS(cols); i++) {
+		GtkTreeViewColumn *column;
+
+		column = add_column(cols[i].id, cols[i].sortfunc,
 			c_ul_progress == cols[i].id
 				? GTK_TYPE_CELL_RENDERER_PROGRESS
 				: GTK_TYPE_CELL_RENDERER_TEXT);
+
+		column_sort_tristate_register(column,
+			on_uploads_treeview_column_clicked, NULL);
+
+		gui_column_map(column, treeview_uploads);	/* Capture resize events */
+
 	}
 	tree_view_restore_widths(treeview_uploads, PROP_UPLOADS_COL_WIDTHS);
 	tree_view_restore_visibility(treeview_uploads, PROP_UPLOADS_COL_VISIBLE);
 
-	upload_handles = g_hash_table_new(NULL, NULL);
+	upload_handles = htable_create(HASH_KEY_SELF, 0);
 
     guc_upload_add_upload_added_listener(upload_added);
     guc_upload_add_upload_removed_listener(upload_removed);
@@ -705,7 +742,6 @@ uploads_gui_shutdown(void)
 {
 	uploads_shutting_down = TRUE;
 
-	tree_view_save_widths(treeview_uploads, PROP_UPLOADS_COL_WIDTHS);
 	tree_view_save_visibility(treeview_uploads, PROP_UPLOADS_COL_VISIBLE);
 
     guc_upload_remove_upload_added_listener(upload_added);
@@ -714,12 +750,10 @@ uploads_gui_shutdown(void)
 
 	gtk_list_store_clear(store_uploads);
 
-	g_hash_table_foreach(upload_handles, free_handle, NULL);
-	g_hash_table_destroy(upload_handles);
-	upload_handles = NULL;
+	htable_foreach(upload_handles, free_handle, NULL);
+	htable_free_null(&upload_handles);
 	G_SLIST_FOREACH(sl_removed_uploads, free_row_data);
-	g_slist_free(sl_removed_uploads);
-	sl_removed_uploads = NULL;
+	gm_slist_free_null(&sl_removed_uploads);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
