@@ -1,6 +1,4 @@
 /*
- * $Id: gui.h 17399 2010-03-21 08:47:43Z rmanfredi $
- *
  * Copyright (c) 2004, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -34,8 +32,9 @@
 #define settings_gui_shutdown()
 
 #define main_gui_early_init(argc, argv, disable_xshm)
+#define main_gui_disable_ancient(v)
 #define main_gui_init()
-#define main_gui_run(x)
+#define main_gui_run(a, b)
 #define main_gui_exit(x)
 #define main_gui_timer(x)
 #define main_gui_shutdown()
@@ -85,7 +84,7 @@
 
 #define gui_signal_stop_emit_by_name(widget, name) \
 	gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), (name))
-	
+
 #endif /* USE_GTK1 */
 
 #ifdef USE_GTK2
@@ -103,7 +102,7 @@
 
 #define gui_signal_stop_emit_by_name(widget, name) \
 	g_signal_stop_emission_by_name((widget), (name))
-	
+
 #endif	/* Gtk+ 2.0 */
 
 /**
@@ -117,6 +116,11 @@ enum sorting_order {
 	SORT_NO_COL = 2		/**< No column chosen yet */
 };
 
+struct sorting_context {
+	int s_column;					/**< The column being sorted */
+	enum sorting_order s_order;		/**< Sorting order attached to column */
+};
+
 #endif	/* USE_TOPLESS */
 #endif /* _gtk_ui_h_ */
 
