@@ -1,6 +1,4 @@
 /*
- * $Id: main.c 17203 2009-11-07 13:24:28Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -36,13 +34,14 @@
 
 #include "gui.h"
 
-RCSID("$Id: main.c 17203 2009-11-07 13:24:28Z rmanfredi $")
-
 #ifdef I_PWD
 #include <pwd.h>
 #endif /* I_PWD */
 
-#include "gtk-gnutella.h"
+#ifdef HAVE_GTKOSXAPPLICATION
+#include <gtkmacintegration/gtkosxapplication.h>
+#include "settings_cb.h"
+#endif
 
 #include "notebooks.h"
 #include "main.h"
@@ -69,16 +68,40 @@ RCSID("$Id: main.c 17203 2009-11-07 13:24:28Z rmanfredi $")
 
 #include "if/bridge/ui2c.h"
 
+#include "lib/crash.h"
+#include "lib/entropy.h"
 #include "lib/glib-missing.h"
 #include "lib/halloc.h"
+#include "lib/hstrfn.h"
+#include "lib/omalloc.h"
+#include "lib/path.h"
+#include "lib/product.h"
+#include "lib/str.h"
 #include "lib/tm.h"
 #include "lib/utf8.h"
+
 #include "lib/override.h"			/* Must be the last header included */
 
 /***
  *** Windows
  ***/
 
+/*
+ * For each window xxx defined by glade through a create_xxx() call, we define
+ * 3 routines and a private variable.
+ *
+ * Variable xxx_protected_ contains the output of create_xxx(), and is
+ * initialized via a call to gui_xxx_set() done in main_gui_early_init().
+ * This variable is accessed only through gui_xxx() calls, never directly.
+ *
+ * The gui_xxx_lookup() routine is used to invoke lookup_widget() on a
+ * particular xxx window.
+ *
+ * gui_xxx()			public
+ * gui_xxx_set()		private
+ * gui_xxx_lookup()		public
+ */
+
 #define WIDGET(name) \
 static GtkWidget * name ## _protected_ ; \
  \
@@ -101,6 +124,7 @@ gui_ ## name ## _lookup(const gchar *id) \
 
 WIDGET(dlg_about)
 WIDGET(dlg_faq)
+WIDGET(dlg_glossary)
 WIDGET(dlg_ancient)
 WIDGET(dlg_prefs)
 WIDGET(dlg_quit)
@@ -123,18 +147,20 @@ static void
 gui_init_window_title(void)
 {
 	gchar title[256];
+	const char *revision = product_revision();
 
-#ifdef GTA_REVISION
-	gm_snprintf(title, sizeof(title), "gtk-gnutella %s %s",
-		GTA_VERSION_NUMBER, GTA_REVISION);
-#else
-	gm_snprintf(title, sizeof(title), "gtk-gnutella %s", GTA_VERSION_NUMBER);
-#endif
+	if (revision[0] != '\0') {
+		str_bprintf(ARYLEN(title), "gtk-gnutella %s %s",
+			product_version(), revision);
+	} else {
+		str_bprintf(ARYLEN(title), "gtk-gnutella %s", product_version());
+	}
 
 	gtk_window_set_title(GTK_WINDOW(gui_main_window()), title);
 }
 
 static GSList *visibility_listeners[nb_main_page_num];
+static GSList *main_visibility_listeners;
 static int notebook_main_current_page;
 static gboolean main_window_is_visible;
 
@@ -147,15 +173,18 @@ main_gui_window_visible(void)
 static void
 main_gui_page_visibility_change(int page_num, gboolean visible)
 {
-	GSList *iter;
+	GSList *sl;
 
 	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
 
-	iter = visibility_listeners[page_num];
-	for (/* NOTHING */; NULL != iter; iter = g_slist_next(iter)) {
-		main_gui_visibility_cb func = cast_pointer_to_func(iter->data);
+	/*
+	 * Process per-page visibility change handlers.
+	 */
 
-		g_assert(func);
+	GM_SLIST_FOREACH(visibility_listeners[page_num], sl) {
+		main_gui_visibility_cb func = cast_pointer_to_func(sl->data);
+
+		g_assert(func != NULL);
 		(*func)(visible);
 	}
 }
@@ -169,6 +198,8 @@ on_main_gui_map_event(GtkWidget *unused_widget,
 	(void) unused_widget;
 	(void) unused_udata;
 
+	entropy_harvest_single(PTRLEN(event));
+
 	switch (event->type) {
 	case GDK_MAP:
 		main_window_is_visible = TRUE;
@@ -189,6 +220,19 @@ on_main_gui_map_event(GtkWidget *unused_widget,
 		break;
 	}
 	if (was_visible != main_window_is_visible) {
+		GSList *sl;
+
+		/*
+		 * Process global visibility change handlers.
+		 */
+
+		GM_SLIST_FOREACH(main_visibility_listeners, sl) {
+			main_gui_visibility_cb func = cast_pointer_to_func(sl->data);
+
+			g_assert(func != NULL);
+			(*func)(main_window_is_visible);
+		}
+
 		main_gui_page_visibility_change(notebook_main_current_page,
 			main_window_is_visible);
 	}
@@ -207,15 +251,37 @@ on_notebook_main_switch_page(GtkNotebook *unused_notebook,
 
 	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
 
+	entropy_harvest_single(VARLEN(page_num));
+
 	old_page = notebook_main_current_page;
 	notebook_main_current_page = page_num;
 
+	gui_prop_set_guint32_val(PROP_MAIN_NOTEBOOK_TAB, page_num);
+
 	if (main_window_is_visible) {
 		main_gui_page_visibility_change(old_page, FALSE);
 		main_gui_page_visibility_change(notebook_main_current_page, TRUE);
 	}
 }
 
+void
+main_gui_add_visibility_listener(main_gui_visibility_cb func)
+{
+	g_return_if_fail(func);
+
+	main_visibility_listeners = g_slist_append(
+			main_visibility_listeners, cast_func_to_pointer(func));
+}
+
+void
+main_gui_remove_visibility_listener(main_gui_visibility_cb func)
+{
+	g_return_if_fail(func);
+
+	main_visibility_listeners = g_slist_remove(
+			main_visibility_listeners, cast_func_to_pointer(func));
+}
+
 void
 main_gui_add_page_visibility_listener(main_gui_visibility_cb func,
 	int page_num)
@@ -249,6 +315,9 @@ void
 main_gui_notebook_set_page(int page_num)
 {
 	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
+
+	entropy_harvest_single(VARLEN(page_num));
+
 	gtk_notebook_set_current_page(
 		GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")),
 		page_num);
@@ -346,7 +415,7 @@ gui_init_main_window(void)
 		on_main_gui_map_event, NULL);
 }
 
-static void 
+static void
 gui_init_dlg_prefs(void)
 {
 #ifdef USE_GTK2
@@ -469,13 +538,13 @@ gui_init_dlg_about(void)
     };
 	GtkWidget *text;
     guint i;
-	
+
 	text = gui_dlg_about_lookup("textview_about_contributors");
     gtk_label_set_text(
         GTK_LABEL(gui_dlg_about_lookup("label_about_title")),
         guc_version_get_version_string());
 
-  	for (i = 0; i < G_N_ELEMENTS(contributors); i++) {
+  	for (i = 0; i < N_ITEMS(contributors); i++) {
 		if (i > 0)
 			text_widget_append(GTK_WIDGET(text), "\n");
 		text_widget_append(GTK_WIDGET(text),
@@ -517,16 +586,16 @@ main_gui_gtkrc_init(void)
 	HFREE_NULL(userrc);
 
 	userrc = h_strconcat(guc_settings_home_dir(),
-		G_DIR_SEPARATOR_S, ".gtk", G_DIR_SEPARATOR_S, "gtkrc", (void *) 0);
+		G_DIR_SEPARATOR_S, ".gtk", G_DIR_SEPARATOR_S, "gtkrc", NULL_PTR);
 	gtk_rc_parse(userrc);
 	HFREE_NULL(userrc);
 
 #ifdef USE_GTK2
 	userrc = h_strconcat(guc_settings_home_dir(),
-		G_DIR_SEPARATOR_S, ".gtk2", G_DIR_SEPARATOR_S, "gtkrc", (void *) 0);
+		G_DIR_SEPARATOR_S, ".gtk2", G_DIR_SEPARATOR_S, "gtkrc", NULL_PTR);
 #else
 	userrc = h_strconcat(guc_settings_home_dir(),
-		G_DIR_SEPARATOR_S, ".gtk1", G_DIR_SEPARATOR_S, "gtkrc", (void *) 0);
+		G_DIR_SEPARATOR_S, ".gtk1", G_DIR_SEPARATOR_S, "gtkrc", NULL_PTR);
 #endif
 	gtk_rc_parse(userrc);
 	HFREE_NULL(userrc);
@@ -544,12 +613,241 @@ void
 main_gui_show_prefences(void)
 {
 	g_return_if_fail(gui_dlg_prefs());
-	
+
     gtk_widget_show(gui_dlg_prefs());
 	gui_restore_window(gui_dlg_prefs(), PROP_PREFS_DLG_COORDS);
 	gdk_window_raise(gui_dlg_prefs()->window);
 }
 
+#ifdef HAVE_GTKOSXAPPLICATION
+void main_gui_init_osx()
+{
+	GError *err = NULL;
+	GtkosxApplication *theApp = g_object_new(GTKOSX_TYPE_APPLICATION, NULL);
+	GtkUIManager *mgr = gtk_ui_manager_new();
+	GtkWidget *item;
+	GtkWidget *sep;
+	GtkWidget *menubar;
+
+	g_signal_connect(theApp, "NSApplicationBlockTermination",
+					 G_CALLBACK(on_button_quit_clicked), NULL);
+	g_signal_connect(theApp, "NSApplicationDidBecomeActive",
+					 G_CALLBACK(on_NSApplicationDidBecomeActive), NULL);
+	g_signal_connect(theApp, "NSApplicationOpenFile",
+					 G_CALLBACK(on_NSApplicationOpenFile), NULL);
+
+	gtk_ui_manager_add_ui_from_string(mgr,
+		"<ui>"
+		"<menubar name='MenuBar'>"
+		"<menu name='File' action='FileAction'>"
+		"  <menuitem action='CloseWindow'/>"
+		"  <menuitem name='Preferences' action='PreferencesAction'/>"
+		"  <menuitem name='Quit' action='QuitAction'/>"
+		"</menu>"
+		"<menu name='View' action='ViewAction'>"
+		"  <menuitem action='menu_searchbar_visible'/>"
+		"  <menuitem action='menu_sidebar_visible'/>"
+		"  <menuitem action='menu_menubar_visible'/>"
+		"  <menuitem action='menu_statusbar_visible'/>"
+		"  <separator/>"
+		"  <menu action='connection_counters1'>"
+		"    <menuitem action='menu_downloads_visible'/>"
+		"    <menuitem action='menu_uploads_visible'/>"
+		"    <menuitem action='menu_connections_visible'/>"
+		"  </menu>"
+
+		"  <menu action='menu_http_stats_visible'>"
+		"    <menuitem action='menu_bws_in_visible'/>"
+		"    <menuitem action='menu_bws_out_visible'/>"
+		"  </menu>"
+
+		"  <menu action='menu_gnet_stats_visible'>"
+		"    <menuitem action='menu_bws_gin_visible'/>"
+		"    <menuitem action='menu_bws_gout_visible'/>"
+		"  </menu>"
+
+		"  <menu action='menu_gnet_leaf_stats_visible'>"
+		"    <menuitem action='menu_autohide_bws_gleaf'/>"
+		"    <menuitem action='menu_bws_glin_visible'/>"
+		"    <menuitem action='menu_bws_glout_visible'/>"
+		"  </menu>"
+
+		"  <menu action='menu_dht_traffic_stats_visible'>"
+		"    <menuitem action='menu_autohide_bws_dht'/>"
+		"    <menuitem action='menu_bws_dht_in_visible'/>"
+		"    <menuitem action='menu_bws_dht_out_visible'/>"
+		"  </menu>"
+		"</menu>"
+		"<menu name='Help' action='HelpAction'>"
+		"  <menuitem action='FAQAction'/>"
+		"  <menuitem name='About' action='AboutAction'/>"
+		"</menu>"
+		"</menubar>"
+		"</ui>",
+		-1, &err);
+
+
+	if (err != NULL) {
+		g_error("%s", err->message);
+	}
+	static GtkActionEntry entries[] = {
+		{ "FileAction", NULL, "_File", NULL, NULL, NULL },
+		{ "PreferencesAction", GTK_STOCK_PREFERENCES, "Preferences",
+			NULL, "Set Viewing Preferences",
+			G_CALLBACK(on_menu_prefs_activate) },
+
+		{ "CloseWindow", NULL, "Close window",
+			"<meta>W", NULL,
+			G_CALLBACK(on_main_window_delete_event_hide) },
+
+		{ "QuitAction", GTK_STOCK_QUIT, "_Quit",
+			"<control>q", "Quit Gtk-Gnutella",
+			G_CALLBACK (on_button_quit_clicked) },
+
+		{ "ViewAction", NULL, "View", NULL, NULL, NULL },
+
+		{ "connection_counters1", NULL,
+			"Connection counters", NULL, NULL, NULL },
+
+		{ "menu_http_stats_visible", NULL,
+			"_HTTP traffic stats", NULL, NULL, NULL },
+
+		{ "menu_gnet_stats_visible", NULL,
+			"Gnutella _traffic stats", NULL, NULL, NULL },
+
+		{ "menu_gnet_leaf_stats_visible", NULL,
+			"Gnutella _leaf traffic stats", NULL, NULL, NULL },
+
+		{ "menu_dht_traffic_stats_visible", NULL,
+			"DHT traffic stats", NULL, NULL, NULL },
+
+
+		{ "HelpAction", NULL, "_Help", NULL, NULL, NULL },
+		{ "FAQAction", NULL, "_FAQ",
+			NULL, "Show Frequently Asked Questions",
+			G_CALLBACK(on_menu_faq_activate) },
+		{ "AboutAction", NULL,
+			"_About Gtk-Gnutella", "<control>a", "About Gtk-Gnutella",
+			G_CALLBACK(on_menu_about_activate) }
+	};
+
+	static const GtkToggleActionEntry toggle_entries[] = {
+		{ "menu_searchbar_visible", NULL,
+			"Show Search_bar", "F2", NULL,
+			G_CALLBACK(on_menu_searchbar_visible_activate), TRUE },
+		{ "menu_sidebar_visible", NULL,
+			"Show _Sidebar", "F8", NULL,
+			G_CALLBACK(on_menu_sidebar_visible_activate), TRUE },
+		{ "menu_menubar_visible", NULL,
+			"Show _Menubar", "F9", NULL,
+			G_CALLBACK(on_menu_menubar_visible_activate), TRUE },
+		{ "menu_statusbar_visible", NULL,
+			"Show _Statusbar", NULL, NULL,
+			G_CALLBACK(on_menu_statusbar_visible_activate), TRUE},
+
+		{ "menu_downloads_visible", NULL,
+			"Show _Downloads", NULL, NULL,
+			G_CALLBACK(on_menu_downloads_visible_activate), TRUE },
+		{ "menu_uploads_visible", NULL,
+			"Show _Uploads", NULL, NULL,
+			G_CALLBACK(on_menu_uploads_visible_activate), TRUE },
+		{ "menu_connections_visible", NULL,
+			"Show _Peers", NULL, NULL,
+			G_CALLBACK(on_menu_connections_visible_activate), TRUE },
+
+
+		{ "menu_bws_in_visible", NULL,
+			"Show _inbound HTTP traffic", NULL, NULL,
+			G_CALLBACK(on_menu_bws_in_visible_activate), TRUE },
+		{ "menu_bws_out_visible", NULL,
+			"Show _outbound HTTP traffic", NULL, NULL,
+			G_CALLBACK(on_menu_bws_out_visible_activate), TRUE },
+
+		{ "menu_bws_gin_visible", NULL,
+			"Show _inbound Gnutella traffic" ,NULL, NULL,
+			G_CALLBACK(on_menu_bws_gin_visible_activate), TRUE },
+		{ "menu_bws_gout_visible", NULL,
+			"Show _outbound Gnutella traffic",  NULL, NULL,
+			G_CALLBACK(on_menu_bws_gout_visible_activate), TRUE },
+
+		{ "menu_autohide_bws_gleaf", NULL,
+			"_Auto-hide leaf traffic stats", NULL, NULL,
+			G_CALLBACK(on_menu_autohide_bws_gleaf_activate), TRUE },
+		{ "menu_bws_glin_visible", NULL,
+			"Show _inbound leaf traffic", NULL, NULL,
+			G_CALLBACK(on_menu_bws_glin_visible_activate), TRUE },
+		{ "menu_bws_glout_visible", NULL,
+			"Show _outbound leaf traffic", NULL, NULL,
+			G_CALLBACK(on_menu_bws_glout_visible_activate), TRUE },
+
+		{ "menu_autohide_bws_dht", NULL,
+			"auto-hide DHT traffic stats", NULL,  NULL,
+			G_CALLBACK(on_menu_autohide_bws_dht_activate), TRUE },
+		{ "menu_bws_dht_in_visible", NULL,
+			"Show inbound DHT traffic", NULL, NULL,
+			G_CALLBACK(on_menu_bws_dht_in_visible_activate), TRUE },
+		{ "menu_bws_dht_out_visible", NULL,
+			"Show outbound DHT traffic", NULL, NULL,
+			G_CALLBACK(on_menu_bws_dht_out_visible_activate), TRUE },
+
+	};
+
+	GtkActionGroup *actions = gtk_action_group_new ("Actions");
+	gtk_action_group_add_actions (actions, entries,
+								  N_ITEMS (entries), NULL);
+	gtk_action_group_add_toggle_actions(actions, toggle_entries,
+										N_ITEMS (toggle_entries), NULL);
+	gtk_ui_manager_insert_action_group (mgr, actions, 0);
+
+	menubar = gtk_ui_manager_get_widget(mgr, "/MenuBar");
+
+
+	gtkosx_application_set_menu_bar(theApp, GTK_MENU_SHELL(menubar));
+	gtkosx_application_set_use_quartz_accelerators(theApp, TRUE);
+
+
+	item = gtk_ui_manager_get_widget(mgr, "/MenuBar/File/Quit");
+	gtk_widget_hide(GTK_WIDGET(item));
+
+	item = gtk_ui_manager_get_widget(mgr, "/MenuBar/View/menu_statusbar_visible");
+	gtk_widget_hide(GTK_WIDGET(item));
+
+	item = gtk_ui_manager_get_widget(mgr,"/MenuBar/Help/About");
+	gtkosx_application_insert_app_menu_item  (theApp, item, 0);
+	sep = gtk_separator_menu_item_new();
+	g_object_ref(sep);
+	gtkosx_application_insert_app_menu_item  (theApp, sep, 1);
+
+	item = gtk_ui_manager_get_widget(mgr,"/MenuBar/File/Preferences");
+	gtkosx_application_insert_app_menu_item  (theApp, item, 2);
+
+
+	GtkWidget *dock_menu = gtk_menu_new();
+
+	item = gtk_menu_item_new_with_label("Toon");
+	g_signal_connect_data (item, "activate", G_CALLBACK (on_NSApplicationDidBecomeActive),0,0, 0);
+	gtk_menu_append(dock_menu, item);
+
+	gtkosx_application_set_dock_menu(theApp, GTK_MENU_SHELL(dock_menu));
+
+	gtkosx_application_ready(theApp);
+}
+#endif
+
+static bool opt_ancien_is_disabled = FALSE;
+
+bool
+main_gui_ancient_is_disabled(void)
+{
+	return opt_ancien_is_disabled;
+}
+
+void
+main_gui_disable_ancient(bool v)
+{
+	opt_ancien_is_disabled = v;
+}
+
 /**
  * Some setup of the gui side which I wanted out of main.c but must be done
  * before the backend can be initialized since the core code is not free of
@@ -559,29 +857,63 @@ main_gui_show_prefences(void)
 void
 main_gui_early_init(gint argc, gchar **argv, gboolean disable_xshm)
 {
+	static const char pixmaps[] = "pixmaps";
+
 	/* Glade inits */
 
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
 
+#ifdef HAVE_GTKOSXAPPLICATION
+	main_gui_init_osx();
+#endif
+
 	if (disable_xshm)
 		gdk_set_use_xshm(FALSE);
 
-	add_pixmap_directory(PRIVLIB_EXP G_DIR_SEPARATOR_S "pixmaps");
+	add_pixmap_directory(native_path(PRIVLIB_EXP G_DIR_SEPARATOR_S "pixmaps"));
+
 #ifndef OFFICIAL_BUILD
-	add_pixmap_directory(PACKAGE_SOURCE_DIR G_DIR_SEPARATOR_S "pixmaps");
+	add_pixmap_directory(
+		native_path(PACKAGE_SOURCE_DIR G_DIR_SEPARATOR_S "pixmaps"));
 #endif
 
+	{
+		const char *path = get_folder_path(PRIVLIB_PATH);
+
+		if (path != NULL) {
+			char *tmp = h_strconcat(path, G_DIR_SEPARATOR_S, pixmaps, NULL_PTR);
+			add_pixmap_directory(native_path(tmp));
+			HFREE_NULL(tmp);
+		}
+	}
+
+#ifdef MINGW32
+	add_pixmap_directory(mingw_filename_nearby(pixmaps));
+#endif
+
+	/*
+	 * We need to initialize the GUI properties early (create the set)
+	 * to be able to process and validate properties given when creating
+	 * things that need to be saved within a GUI property (e.g. column
+	 * widths).
+	 * 		--RAM, 2017-10-26
+	 */
+
+	settings_gui_early_init();
+	misc_gui_early_init();			/* For column resize events capture */
+
     gui_main_window_set(create_main_window());
 	gui_init_main_window();
 
     gui_shutdown_window_set(create_shutdown_window());
     gui_dlg_quit_set(create_dlg_quit());
-	
+
     gui_dlg_about_set(create_dlg_about());
 	gui_init_dlg_about();
 
     gui_dlg_faq_set(create_dlg_faq());
+    gui_dlg_glossary_set(create_dlg_glossary());
     gui_dlg_ancient_set(create_dlg_ancient());
 
     gui_dlg_prefs_set(create_dlg_prefs());
@@ -613,6 +945,7 @@ main_gui_early_init(gint argc, gchar **argv, gboolean disable_xshm)
 void
 main_gui_exit(int n)
 {
+	exit_cleanup();
 	gtk_exit(n);
 }
 
@@ -657,28 +990,60 @@ main_gui_init(void)
     monitor_gui_init();
 	gui_update_files_scanned();
     gui_update_stats_frames();
+
+#ifdef HAVE_GTKOSXAPPLICATION
+	g_signal_handlers_disconnect_by_func((gpointer) gui_main_window(), G_CALLBACK (on_main_window_delete_event), NULL);
+
+	g_signal_connect ((gpointer) gui_main_window(), "delete_event",
+					  G_CALLBACK (on_main_window_delete_event_hide),
+					  NULL);
+
+	gui_prop_set_boolean_val(PROP_MENUBAR_VISIBLE, FALSE);
+#endif
 }
 
 void
-main_gui_run(const gchar *geometry_spec)
+main_gui_run(const gchar *geometry_spec, const gboolean minimized)
 {
 	time_t now = tm_time_exact();
 
-    gtk_widget_show_now(gui_main_window());		/* Display the main window */
+	/*
+	 * With GTK2, we can iconify the window before its widget is initialized
+	 * For GTK1, this will be a no-op but no need to #ifdef the code.
+	 */
+
+	if (minimized) {
+		gtk_window_iconify(GTK_WINDOW(gui_main_window()));
+	}
+
+	gtk_widget_show_now(gui_main_window());		/* Display the main window */
 	gtk_widget_map(gui_main_window());
 
 	if (geometry_spec) {
 		guint32 coord[4] = { 0, 0, 0, 0 };
 
-    	gui_prop_get_guint32(PROP_WINDOW_COORDS, coord, 0, G_N_ELEMENTS(coord));
+    	gui_prop_get_guint32(PROP_WINDOW_COORDS, coord, 0, N_ITEMS(coord));
 		if (0 == gui_parse_geometry_spec(geometry_spec, coord)) {
     		gui_prop_set_guint32(PROP_WINDOW_COORDS,
-				coord, 0, G_N_ELEMENTS(coord));
+				coord, 0, N_ITEMS(coord));
 		}
 	}
 	gui_restore_window(gui_main_window(), PROP_WINDOW_COORDS);
 
+#ifdef USE_GTK1
+	/*
+	 * With GTK1, we need to iconify after the window is properly displayed.
+	 * Users will see the window display briefly and then iconify itself, but
+	 * this is inevitable, halas.
+	 */
+	if (minimized) {
+		gtk_window_iconify(GTK_WINDOW(gui_main_window()));
+	}
+#endif
+
+#ifndef HAVE_GTKOSXAPPLICATION
     icon_init();
+#endif
     main_gui_timer(now);
 
  	gtk_widget_fix_width(
@@ -687,12 +1052,29 @@ main_gui_run(const gchar *geometry_spec)
         8, 8);
 
 	/*
-	 * Make sure the application starts in the Gnet pane.
+	 * Make sure the application starts in the Gnet pane by default, or
+	 * whereever they had left it in the previous session if it restarted
+	 * abnormally (after a crash).
 	 */
 
 	gui_signal_connect(GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")),
 		"switch-page", on_notebook_main_switch_page, NULL);
-	main_gui_notebook_set_page(nb_main_page_network);
+
+	{
+		uint32 page = nb_main_page_network;
+		bool clean_restart;
+
+		gnet_prop_get_boolean_val(PROP_CLEAN_RESTART, &clean_restart);
+
+		if (!clean_restart) {
+			gui_prop_get_guint32_val(PROP_MAIN_NOTEBOOK_TAB, &page);
+
+			if (page >= nb_main_page_num)
+				page = nb_main_page_network;
+		}
+
+		main_gui_notebook_set_page(page);
+	}
 
 	settings_gui_restore_panes();
     gtk_main();
@@ -778,12 +1160,11 @@ void
 main_gui_shutdown(void)
 {
 	unsigned i;
-	
+
 	gui_save_window(gui_main_window(), PROP_WINDOW_COORDS);
 
-	for (i = 0; i < G_N_ELEMENTS(visibility_listeners); i++) {
-		g_slist_free(visibility_listeners[i]);
-		visibility_listeners[i] = NULL;
+	for (i = 0; i < N_ITEMS(visibility_listeners); i++) {
+		gm_slist_free_null(&visibility_listeners[i]);
 	}
 	slist_free(&timers);
 
@@ -807,11 +1188,12 @@ main_gui_shutdown(void)
     fi_gui_shutdown();
     nodes_gui_shutdown();
     uploads_gui_shutdown();
-    upload_stats_common_gui_shutdown();
     upload_stats_gui_shutdown();
+    upload_stats_common_gui_shutdown();
 	gnet_stats_gui_shutdown();
     hcache_gui_shutdown();
     statusbar_gui_shutdown();
+	misc_gui_shutdown();
 }
 
 /* vi: set ts=4 sw=4 cindent: */
