@@ -1,6 +1,4 @@
 /*
- * $Id: nodes_common.c 17078 2009-10-16 21:12:34Z rmanfredi $
- *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -36,8 +34,6 @@
 
 #include "gui.h"
 
-RCSID("$Id: nodes_common.c 17078 2009-10-16 21:12:34Z rmanfredi $")
-
 #include "gtk-gnutella.h"
 
 #include "nodes_common.h"
@@ -51,8 +47,10 @@ RCSID("$Id: nodes_common.c 17078 2009-10-16 21:12:34Z rmanfredi $")
 #include "if/gui_property_priv.h"
 
 #include "lib/ascii.h"
-#include "lib/glib-missing.h"
+#include "lib/cstr.h"
 #include "lib/parse.h"
+#include "lib/random.h"
+#include "lib/str.h"
 
 #include "lib/halloc.h"
 #include "lib/walloc.h"
@@ -95,7 +93,7 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			size_t slen = 0;
 
 			if (!GUI_PROPERTY(node_show_detailed_info)) {
-				gm_snprintf(gui_tmp, sizeof(gui_tmp),
+				str_bprintf(ARYLEN(gui_tmp),
 					"TX=%u RX=%u Q=%u,%u%% %s",
 					n->sent, n->received,
 					n->mqueue_count, n->mqueue_percent_used,
@@ -106,11 +104,10 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			}
 
 			if (n->tx_compressed && GUI_PROPERTY(show_gnet_info_txc))
-				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TXc=%u,%d%%",
-						n->sent, (int) (n->tx_compression_ratio * 100));
+				slen += str_bprintf(ARYLEN(gui_tmp), "TXc=%u,%d%%",
+						n->sent, (int) (n->tx_compression_ratio * 100.0));
 			else
-				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TX=%u",
-						n->sent);
+				slen += str_bprintf(ARYLEN(gui_tmp), "TX=%u", n->sent);
 
 			if (
 				GUI_PROPERTY(show_gnet_info_tx_speed) ||
@@ -118,30 +115,28 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			) {
 				gboolean is_first = TRUE;
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					" (" /* ')' */);
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), " (" /* ')' */);
 
 				if (GUI_PROPERTY(show_gnet_info_tx_wire)) {
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					"%s", compact_size(n->tx_written, show_metric_units()));
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
+						"%s", compact_size(n->tx_written, show_metric_units()));
 					is_first = FALSE;
 				}
 
 				if (GUI_PROPERTY(show_gnet_info_tx_speed))
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%s%s", is_first ? "" : ", ",
 						compact_rate(n->tx_bps, show_metric_units()));
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					/* '(' */ ")");
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), /* '(' */ ")");
 			}
 
 			if (n->rx_compressed && GUI_PROPERTY(show_gnet_info_rxc))
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 					" RXc=%u,%d%%",
-					n->received, (int) (n->rx_compression_ratio * 100));
+					n->received, (int) (n->rx_compression_ratio * 100.0));
 			else
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 					" RX=%u", n->received);
 
 			if (
@@ -150,22 +145,20 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			) {
 				gboolean is_first = TRUE;
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					" (" /* ')' */);
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), " (" /* ')' */);
 
 				if (GUI_PROPERTY(show_gnet_info_rx_wire)) {
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%s", compact_size(n->rx_given, show_metric_units()));
 					is_first = FALSE;
 				}
 
 				if (GUI_PROPERTY(show_gnet_info_rx_speed))
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%s%s", is_first ? "" : ", ",
 						compact_rate(n->rx_bps, show_metric_units()));
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					/* '(' */ ")");
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), /* '(' */ ")");
 			}
 
 			if (
@@ -176,30 +169,28 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			) {
 				gboolean is_first = TRUE;
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					" Query(" /* ')' */);
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), " Query(" /* ')' */);
 
 				if (GUI_PROPERTY(show_gnet_info_gen_queries)) {
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"Gen=%u", n->squeue_sent);
 					is_first = FALSE;
 				}
 				if (GUI_PROPERTY(show_gnet_info_sq_queries)) {
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%sQ=%u", is_first ? "" : ", ", n->squeue_count);
 					is_first = FALSE;
 				}
 				if (GUI_PROPERTY(show_gnet_info_tx_queries)) {
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%sTX=%u", is_first ? "" : ", ", n->tx_queries);
 					is_first = FALSE;
 				}
 				if (GUI_PROPERTY(show_gnet_info_rx_queries))
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%sRX=%u", is_first ? "" : ", ", n->rx_queries);
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					/* '(' */ ")");
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), /* '(' */ ")");
 			}
 
 			if (
@@ -208,20 +199,18 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			) {
 				gboolean is_first = TRUE;
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					" QHit(" /* ')' */);
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), " QHit(" /* ')' */);
 
 				if (GUI_PROPERTY(show_gnet_info_tx_hits)) {
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"TX=%u", n->tx_qhits);
 					is_first = FALSE;
 				}
 				if (GUI_PROPERTY(show_gnet_info_rx_hits))
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%sRX=%u", is_first ? "" : ", ", n->rx_qhits);
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					/* '(' */ ")");
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), /* '(' */ ")");
 			}
 
 			if (
@@ -230,20 +219,18 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			) {
 				gboolean is_first = TRUE;
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					" Drop(" /* ')' */);
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), " Drop(" /* ')' */);
 
 				if (GUI_PROPERTY(show_gnet_info_tx_dropped)) {
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"TX=%u", n->tx_dropped);
 					is_first = FALSE;
 				}
 				if (GUI_PROPERTY(show_gnet_info_rx_dropped))
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%sRX=%u", is_first ? "" : ", ", n->rx_dropped);
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					/* '(' */ ")");
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), /* '(' */ ")");
 			}
 
 			if (
@@ -252,14 +239,13 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			) {
 				gboolean is_first = TRUE;
 
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					" Lib(" /* ')' */);
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), " Lib(" /* ')' */);
 
 				if (
 					GUI_PROPERTY(show_gnet_info_shared_size) &&
 					n->gnet_info_known
 				) {
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%s",
 						compact_kb_size(n->gnet_files_count
 							? n->gnet_kbytes_count : 0, show_metric_units()));
@@ -269,21 +255,21 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 					GUI_PROPERTY(show_gnet_info_shared_files) &&
 					n->gnet_info_known
 				) {
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						"%s#=%u", is_first ? "" : ", ", n->gnet_files_count);
 				}
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 					/* '(' */ "%s)", n->gnet_info_known ? "" : "?");
 			}
 
 			if (GUI_PROPERTY(show_gnet_info_qrp_stats)) {
 				if (n->has_qrp)
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						" QRP=%u%%",
 						(guint) (n->qrp_efficiency * 100.0));
 
 				if (n->qrt_slots != 0)
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						" QRT(%s, g=%u, f=%u%%, t=%u%%, e=%u%%)",
 						compact_size(n->qrt_slots, show_metric_units()),
 						n->qrt_generation,
@@ -292,25 +278,24 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			}
 
 			if (GUI_PROPERTY(show_gnet_info_dbw))
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 				" Dup=%u Bad=%u W=%u H=%u S=%u E=%u",
 				n->n_dups, n->n_bad, n->n_weird,
 				n->n_hostile, n->n_spam, n->n_evil);
 
 			if (GUI_PROPERTY(show_gnet_info_rt)) {
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 				" RT(avg=%u, last=%u", n->rt_avg, n->rt_last);	/* ) */
 				if (n->tcp_rtt)
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						", tcp=%u", n->tcp_rtt);
 				if (n->udp_rtt)
-					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+					slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 						", udp=%u", n->udp_rtt);
-				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					/* ( */ ")");
+				slen += str_bprintf(ARYPOSLEN(gui_tmp, slen), /* ( */ ")");
 			}
 
-			slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
+			slen += str_bprintf(ARYPOSLEN(gui_tmp, slen),
 				" Q=%u,%u%% %s",
 				n->mqueue_count, n->mqueue_percent_used,
 				n->in_tx_swift_control ? " [SW]" :
@@ -325,7 +310,7 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 
 	case GTA_NODE_SHUTDOWN:
 		{
-			gm_snprintf(gui_tmp, sizeof(gui_tmp),
+			str_bprintf(ARYLEN(gui_tmp),
 				_("Closing: %s [Stop in %us] RX=%u Q=%u,%u%%"),
 				n->message, n->shutdown_remain, n->received,
 				n->mqueue_count, n->mqueue_percent_used);
@@ -351,6 +336,7 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 struct add_node_context {
 	guint32 flags;
 	guint16 port;
+	bool g2;
 };
 
 static void
@@ -363,10 +349,15 @@ add_node_helper(const host_addr_t *addrs, size_t n, gpointer data)
 	g_assert(0 != ctx->port);
 
 	if (n > 0) {
-		guc_node_add(addrs[random_u32() % n], ctx->port, ctx->flags);
+		const host_addr_t addr = addrs[random_value(n - 1)];
+		if (ctx->g2) {
+			guc_node_g2_add(addr, ctx->port, ctx->flags);
+		} else {
+			guc_node_add(addr, ctx->port, ctx->flags);
+		}
 	}
 
-	wfree(ctx, sizeof *ctx);
+	WFREE(ctx);
 }
 
 /**
@@ -379,7 +370,7 @@ add_node_helper(const host_addr_t *addrs, size_t n, gpointer data)
  *       | <IPv4 address>[":" <port>]
  *		 | <IPv6 address>
  *		 | "[" <IPv6 address> "]:" <port>
- * peer = ["tls:"]<node>
+ * peer = ["tls:"]["g2:"]<node>
  *
  * If the port is omitted, the default port (GTA_PORT: 6346) is used.
  * The case-insensitive prefix "tls:" requests a TLS (encrypted) connection.
@@ -398,6 +389,7 @@ nodes_gui_common_connect_by_name(const gchar *line)
 		host_addr_t addr;
 		guint32 flags;
     	guint16 port;
+		bool g2;
 
 		q = skip_ascii_spaces(q);
 		if (',' == *q) {
@@ -418,8 +410,16 @@ nodes_gui_common_connect_by_name(const gchar *line)
 			q = endptr;
 		}
 
+		endptr = is_strcaseprefix(q, "g2:");
+		if (endptr) {
+			g2 = TRUE;
+			q = endptr;
+		} else {
+			g2 = FALSE;
+		}
+
 		if (!string_to_host_or_addr(q, &endptr, &addr)) {
-			g_message("Expected hostname or IP address");
+			g_message("expected hostname or IP address");
 			break;
 		}
 
@@ -435,7 +435,7 @@ nodes_gui_common_connect_by_name(const gchar *line)
 
 			port = parse_uint16(&q[1], &endptr, 10, &error);
 			if (error || 0 == port) {
-				g_message("Cannot parse port");
+				g_message("cannot parse port");
 				break;
 			}
 
@@ -443,13 +443,17 @@ nodes_gui_common_connect_by_name(const gchar *line)
 		} else {
 			q = skip_ascii_spaces(endptr);
 			if ('\0' != *q && ',' != *q) {
-				g_message("Expected \",\" or \":\"");
+				g_message("expected \",\" or \":\"");
 				break;
 			}
 		}
 
 		if (!hostname) {
-			guc_node_add(addr, port, flags);
+			if (g2) {
+				guc_node_g2_add(addr, port, flags);
+			} else {
+				guc_node_add(addr, port, flags);
+			}
 		} else {
 			struct add_node_context *ctx;
 			gchar *p;
@@ -461,13 +465,14 @@ nodes_gui_common_connect_by_name(const gchar *line)
 
 				g_assert(n > hostname_len);
 				p = halloc(n);
-				g_strlcpy(p, hostname, n);
+				cstr_lcpy(p, n, hostname);
 				hostname = p;
 			}
 
-			ctx = walloc(sizeof *ctx);
+			WALLOC(ctx);
 			ctx->port = port;
 			ctx->flags = flags;
+			ctx->g2 = g2;
 			guc_adns_resolve(hostname, add_node_helper, ctx);
 
 			HFREE_NULL(p);
@@ -485,10 +490,10 @@ nodes_gui_is_visible(void)
 void
 nodes_gui_timer(time_t now)
 {
-    static time_t last_update;
+	static time_t last_update;
 
-    if (last_update == now)
-        return;
+	if (last_update == now)
+		return;
 
 	/*
 	 * Usually don't perform updates if nobody is watching.  However,
@@ -502,7 +507,7 @@ nodes_gui_timer(time_t now)
 		nodes_gui_is_visible() ||
 		delta_time(now, last_update) >= UPDATE_MIN
 	) {
-    	last_update = now;
+		last_update = now;
 		nodes_gui_update_display(now);
 	}
 }
