@@ -1,6 +1,4 @@
 /*
- * $Id: statusbar.h 14389 2007-08-09 22:10:53Z cbiere $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -19,7 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with gtk-gnutella; if not, write to the Free Software
  *  Foundation, Inc.:
- *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *----------------------------------------------------------------------
  */
 
@@ -37,7 +35,7 @@
 void statusbar_gui_init(void);
 void statusbar_gui_shutdown(void);
 
-void statusbar_gui_set_default(const gchar *, ...) G_GNUC_PRINTF(1, 2);
+void statusbar_gui_set_default(const gchar *, ...) G_PRINTF(1, 2);
 void statusbar_gui_remove(statusbar_msgid_t);
 
 #endif /* _gtk_statusbar_h_ */
